@@ -12,6 +12,9 @@
 #undef DOMAIN
 @TOP@
 
+/* Mutt version info */
+#undef VERSION
+
 /* use dotlocking to lock mailboxes? */
 #undef USE_DOTLOCK
 
@@ -54,6 +57,18 @@
 /* Where is PGP located on your system? */
 #undef _PGPPATH
 
+/* Where is PGP 2.* located on your system? */
+#undef _PGPV2PATH
+
+/* Where is PGP 5 located on your system? */
+#undef _PGPV3PATH
+
+/* Do we have PGP 2.*? */
+#undef HAVE_PGP2
+
+/* Do we have PGP 5.0 or up? */
+#undef HAVE_PGP5
+
 /* Where to find ispell on your system? */
 #undef ISPELL
 
@@ -77,3 +92,13 @@
 
 /* The result of isprint() is unreliable? */
 #undef LOCALES_HACK
+
+/* Enable exact regeneration of email addresses as parsed?  NOTE: this requires
+   significant more memory when defined. */
+#undef EXACT_ADDRESS
+
+/* Does your system have the snprintf() call? */
+#undef HAVE_SNPRINTF
+
+/* Does your system have the vsnprintf() call? */
+#undef HAVE_VSNPRINTF
