@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,11 +17,14 @@
  */ 
 
 #include "mutt.h"
-#include "mutt_curses.h"
 #include "mutt_menu.h"
+#include "sort.h"
 
 #include <string.h>
 #include <stdlib.h>
+#include <ctype.h>
+
+#define RSORT(x) (SortAlias & SORT_REVERSE) ? -x : x
 
 typedef struct entry
 {
@@ -29,6 +32,13 @@ typedef struct entry
   ALIAS *alias;
 } ENTRY;
 
+static struct mapping_t AliasHelp[] = {
+  { "Exit",   OP_EXIT },
+  { "Select", OP_GENERIC_SELECT_ENTRY },
+  { "Help",   OP_HELP },
+  { NULL }
+};
+
 int alias_search (MUTTMENU *m, regex_t *re, int n)
 {
   ENTRY *table = (ENTRY *) m->data;
@@ -36,36 +46,116 @@ int alias_search (MUTTMENU *m, regex_t *re, int n)
   return (regexec (re, table[n].alias->name, 0, NULL, 0));
 }
 
+/* %a	alias
+   %n	number
+   %r	address
+   %t	tag */
+static void
+alias_fmt_str (char *d, size_t dlen, const char *s, ENTRY *table, int num)
+{
+  char *pd = d;
+  char *pfmt;
+  char fmt[SHORT_STRING];
+  char buf[SHORT_STRING];
+  char adr[SHORT_STRING];
+  char tmp[SHORT_STRING];
+  size_t len;
+
+  dlen--; /* save room for the trailing \0 */
+  while (*s && dlen > 0)
+  {
+    if (*s == '%')
+    {
+      s++;
+
+      pfmt = fmt;
+      while ((isdigit (*s) || *s == '.' || *s == '-') &&
+	     (pfmt - fmt < sizeof (fmt) - 1))
+	*pfmt++ = *s++;
+      *pfmt = 0;
+
+      switch (*s)
+      {
+	case '%':
+	  buf[0] = '%';
+	  buf[1] = 0;
+	  break;
+	case 'a':
+	  snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
+	  snprintf (buf, sizeof (buf), tmp, table[num].alias->name);
+	  break;
+	case 'r':
+	  adr[0] = 0;
+	  rfc822_write_address (adr, sizeof (adr), table[num].alias->addr);
+	  snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
+	  snprintf (buf, sizeof (buf), tmp, adr);
+	  break;
+	case 'n':
+	  snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
+	  snprintf (buf, sizeof (buf), tmp, num + 1);
+	  break;
+	case 't':
+	  buf[0] = table[num].tagged ? '*' : ' ';
+	  buf[1] = 0;
+	  break;
+      }
+      len = strlen (buf);
+      if (len > dlen)
+	len = dlen;
+      memcpy (pd, buf, len);
+      pd += len;
+      dlen -= len;
+    }
+    else
+    {
+      *pd++ = *s;
+      dlen--;
+    }
+    s++;
+  }
+  *pd = 0;
+}
+
 /* This is the callback routine from mutt_menuLoop() which is used to generate
  * a menu entry for the requested item number.
  */
 void alias_entry (char *s, size_t slen, MUTTMENU *m, int num)
 {
-  char buf[SHORT_STRING] = { 0 };
-  ENTRY *table = (ENTRY *) m->data;
-
-  rfc822_write_address (buf, sizeof (buf), table[num].alias->addr);
-  snprintf (s, slen, "%2d %c %-10s   %s", num+1,
-	    table[num].tagged ? '*' : ' ',
-	    table[num].alias->name,
-	    buf);
+  alias_fmt_str (s, slen, AliasFmt, (ENTRY *) m->data, num);
 }
 
+int alias_tag (MUTTMENU *menu, int n)
+{
+  return (((ENTRY *) menu->data)[n].tagged = !((ENTRY *) menu->data)[n].tagged);
+}
 
-static int addrbook_sort (const void *a, const void *b)
+static int alias_SortAlias (const void *a, const void *b)
+{
+  ALIAS *pa = ((ENTRY *) a)->alias;
+  ALIAS *pb = ((ENTRY *) b)->alias;
+  int r = strcasecmp (pa->name, pb->name);
+
+  return (RSORT (r));
+}
+
+static int alias_SortAddress (const void *a, const void *b)
 {
   ADDRESS *pa = ((ENTRY *) a)->alias->addr;
   ADDRESS *pb = ((ENTRY *) b)->alias->addr;
+  int r;
 
   if (pa->personal)
-  {
+  { 
     if (pb->personal)
-      return (strcasecmp (pa->personal, pb->personal));
-    return (-1);
+      r = strcasecmp (pa->personal, pb->personal);
+    else
+      r = 1;
   }
   else if (pb->personal)
-    return (1);
-  return (strcasecmp (pa->mailbox, pb->mailbox));
+    r = -1;
+  else
+    r = strcasecmp (pa->mailbox, pb->mailbox);
+  return (RSORT (r));
 }
 
 void mutt_alias_menu (char *buf, size_t buflen, ALIAS *aliases)
@@ -73,9 +163,9 @@ void mutt_alias_menu (char *buf, size_t buflen, ALIAS *aliases)
   ALIAS *aliasp;
   MUTTMENU *menu;
   ENTRY *AliasTable = NULL;
+  int t = -1;
   int i, done = 0;
   char helpstr[SHORT_STRING];
-  char tmp[16];
 
   if (!aliases)
   {
@@ -89,17 +179,10 @@ void mutt_alias_menu (char *buf, size_t buflen, ALIAS *aliases)
   menu = mutt_new_menu ();
   menu->make_entry = alias_entry;
   menu->search = alias_search;
+  menu->tag = alias_tag;
   menu->menu = MENU_ALIAS;
   menu->title = "Aliases";
-
-  helpstr[0] = 0;
-  mutt_make_help (tmp, sizeof (tmp), "Exit  ", MENU_ALIAS, OP_EXIT);
-  strcat (helpstr, tmp);
-  mutt_make_help (tmp, sizeof (tmp), "Select  ", MENU_ALIAS, OP_GENERIC_SELECT_ENTRY);
-  strcat (helpstr, tmp);
-  mutt_make_help (tmp, sizeof (tmp), "Help", MENU_ALIAS, OP_HELP);
-  strcat (helpstr, tmp);
-  menu->help = helpstr;
+  menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_ALIAS, AliasHelp);
 
   /* count the number of aliases */
   for (aliasp = aliases; aliasp; aliasp = aliasp->next)
@@ -110,41 +193,36 @@ void mutt_alias_menu (char *buf, size_t buflen, ALIAS *aliases)
   for (i = 0, aliasp = aliases; aliasp; aliasp = aliasp->next, i++)
     AliasTable[i].alias = aliasp;
 
-  qsort (AliasTable, i, sizeof (ENTRY), addrbook_sort);
+  if ((SortAlias & SORT_MASK) != SORT_ORDER)
+  {
+    qsort (AliasTable, i, sizeof (ENTRY),
+	 (SortAlias & SORT_MASK) == SORT_ADDRESS ? alias_SortAddress : alias_SortAlias);
+  }
 
   while (!done)
   {
     switch (mutt_menuLoop (menu))
     {
       case OP_GENERIC_SELECT_ENTRY:
-
-	AliasTable[menu->current].tagged = !AliasTable[menu->current].tagged;
-	if (option (OPTRESOLVE) && menu->current < menu->max - 1)
-	{
-	  menu->oldcurrent = menu->current++;
-	  if (menu->current >= menu->top + PAGELEN)
-	  {
-	    menu->top = menu->current;
-	    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	  }
-	  else
-	    menu->redraw = REDRAW_MOTION_RESYNCH;
-	}
-	else
-	  menu->redraw = REDRAW_CURRENT;
-	break;
-
+        t = menu->current;
       case OP_EXIT:
-
 	done = 1;
 	break;
     }
   }
 
   for (i = 0; i < menu->max; i++)
+  {
     if (AliasTable[i].tagged)
+    {
       rfc822_write_address (buf, buflen, AliasTable[i].alias->addr);
+      t = -1;
+    }
+  }
 
+  if(t != -1)
+    rfc822_write_address (buf, buflen, AliasTable[t].alias->addr);
+  
   mutt_menuDestroy (&menu);
   safe_free ((void **) &AliasTable);
 }
