@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,69 +17,65 @@
  */ 
 
 #include "mutt.h"
-#include "mutt_curses.h"
+#include "mutt_regex.h"
 
 #include <pwd.h>
 #include <string.h>
 
-static LIST *Alias_list = NULL;
-
 static ADDRESS *lookup_alias (const char *s)
 {
   ALIAS *t = Aliases;
 
-  while (t) {
-    if (!strcmp(s, t->name)) return(t->addr);
-    t = t->next;
-  }
+  for (; t; t = t->next)
+    if (!strcasecmp (s, t->name))
+      return (t->addr);
   return (NULL);   /* no such alias */
 }
 
-static ADDRESS *mutt_expand_aliases_r (ADDRESS *a)
+static ADDRESS *mutt_expand_aliases_r (ADDRESS *a, LIST **expn)
 {
-  ADDRESS *head = 0, *last = 0, *t, *w;
+  ADDRESS *head = NULL, *last = NULL, *t, *w;
   LIST *u;
   char i;
 
   while (a)
   {
-    if (a->host && *a->host == '@' && !a->personal)
+    if (!a->group && !a->personal && a->mailbox && strchr (a->mailbox, '@') == NULL)
     {
       t = lookup_alias (a->mailbox);
 
       if (t)
       {	
-        u = Alias_list;
         i = 0;
-        while (u)
+        for (u = *expn; u; u = u->next)
 	{
 	  if (strcmp (a->mailbox, u->data) == 0) /* alias already found */
 	  {
+	    dprint (1, (debugfile, "mutt_expand_aliases_r(): loop in alias found for '%s'\n", a->mailbox));
 	    i = 1;
 	    break;
 	  }
-	  u = u->next;
 	}
 
         if (!i)
 	{
           u = safe_malloc (sizeof (LIST));
           u->data = safe_strdup (a->mailbox);
-          u->next = Alias_list;
-          Alias_list = u;
+          u->next = *expn;
+          *expn = u;
 	  w = rfc822_cpy_adr (t);
-	  w = mutt_expand_aliases_r (w);
+	  w = mutt_expand_aliases_r (w, expn);
 	  if (head)
 	    last->next = w;
 	  else
 	    head = last = w;
-	  while (last->next)
+	  while (last && last->next)
 	    last = last->next;
         }
 	t = a;
 	a = a->next;
 	t->next = NULL;
-	mutt_free_address (&t);
+	rfc822_free_address (&t);
 	continue;
       }
       else
@@ -93,6 +89,10 @@ static ADDRESS *mutt_expand_aliases_r (ADDRESS *a)
 	  if ((p = strchr (buffer, ',')))
 	    *p = 0;
 	  a->personal = safe_strdup (buffer);
+#ifdef EXACT_ADDRESS
+	  free (a->val);
+	  a->val = NULL;
+#endif
 	}
       }
     }
@@ -108,17 +108,10 @@ static ADDRESS *mutt_expand_aliases_r (ADDRESS *a)
     last->next = NULL;
   }
 
-  if (option (OPTUSEDOMAIN))
+  if (option (OPTUSEDOMAIN) && Fqdn[0] != '@')
   {
     /* now qualify all local addresses */
-    for (t = head; t; t = t->next)
-    {
-      if (t->host && *t->host == '@')
-      {
-	safe_free ((void **) &t->host);
-	t->host = safe_strdup (Fqdn);
-      }
-    }
+    rfc822_qualify (head, Fqdn);
   }
 
   return (head);
@@ -127,17 +120,41 @@ static ADDRESS *mutt_expand_aliases_r (ADDRESS *a)
 ADDRESS *mutt_expand_aliases (ADDRESS *a)
 {
   ADDRESS *t;
+  LIST *expn = NULL; /* previously expanded aliases to avoid loops */
 
-  t = mutt_expand_aliases_r (a);
-  mutt_free_list (&Alias_list);
+  t = mutt_expand_aliases_r (a, &expn);
+  mutt_free_list (&expn);
   return (t);
 }
 
+/* if someone has an address like
+ *	From: Michael `/bin/rm -f ~` Elkins <me@cs.hmc.edu>
+ * and the user creates an alias for this, Mutt could wind up executing
+ * the backtics because it writes aliases like
+ *	alias me Michael `/bin/rm -f ~` Elkins <me@cs.hmc.edu>
+ * To avoid this problem, use a backslash (\) to quote any backtics.  We also
+ * need to quote backslashes as well, since you could defeat the above by
+ * doing
+ *	From: Michael \`/bin/rm -f ~\` Elkins <me@cs.hmc.edu>
+ * since that would get aliased as
+ *	alias me Michael \\`/bin/rm -f ~\\` Elkins <me@cs.hmc.edu>
+ * which still gets evaluated because the double backslash is not a quote.
+ */
+static void write_safe_address (FILE *fp, char *s)
+{
+  while (*s)
+  {
+    if (*s == '\\' || *s == '`')
+      fputc ('\\', fp);
+    fputc (*s, fp);
+    s++;
+  }
+}
+
 void mutt_create_alias (ENVELOPE *cur)
 {
   ALIAS *new, *t;
-  char buf[LONG_STRING];
-  char prompt[SHORT_STRING];
+  char buf[LONG_STRING], prompt[SHORT_STRING], *pc;
   FILE *rc;
   ADDRESS *adr = NULL;
 
@@ -157,12 +174,16 @@ void mutt_create_alias (ENVELOPE *cur)
 
   }
 
-  if(adr)
-    strfcpy(buf, adr->mailbox, sizeof(buf));
+  if (adr && adr->mailbox)
+  {
+    strfcpy (buf, adr->mailbox, sizeof (buf));
+    if ((pc = strchr (buf, '@')))
+      *pc = 0;
+  }
   else
     buf[0] = '\0';
   
-  if (ci_get_field ("Alias as: ", buf, sizeof (buf), 0) != 0 || !buf[0])
+  if (mutt_get_field ("Alias as: ", buf, sizeof (buf), 0) != 0 || !buf[0])
     return;
 
   /* check to see if the user already has an alias defined */
@@ -176,23 +197,23 @@ void mutt_create_alias (ENVELOPE *cur)
   new->name = safe_strdup (buf);
 
   if (adr)
-    mutt_simple_address (buf, sizeof (buf), adr);
+    strfcpy (buf, adr->mailbox, sizeof (buf));
   else
     buf[0] = 0;
 
-  if (ci_get_field ("Address: ", buf, sizeof (buf), 0) != 0 || !buf[0])
+  if (mutt_get_field ("Address: ", buf, sizeof (buf), 0) != 0 || !buf[0])
   {
     mutt_free_alias (&new);
     return;
   }
-  rfc822_parse_adrlist (&new->addr, buf, Fqdn);
+  new->addr = rfc822_parse_adrlist (new->addr, buf);
 
   if (adr && adr->personal && !mutt_is_mail_list (adr))
     strfcpy (buf, adr->personal, sizeof (buf));
   else
     buf[0] = 0;
 
-  if (ci_get_field ("Personal name: ", buf, sizeof (buf), 0) != 0)
+  if (mutt_get_field ("Personal name: ", buf, sizeof (buf), 0) != 0)
   {
     mutt_free_alias (&new);
     return;
@@ -218,14 +239,16 @@ void mutt_create_alias (ENVELOPE *cur)
     Aliases = new;
 
   strfcpy (buf, AliasFile, sizeof (buf));
-  if (ci_get_field ("Save to file: ", buf, sizeof (buf), M_FILE) != 0)
+  if (mutt_get_field ("Save to file: ", buf, sizeof (buf), M_FILE) != 0)
     return;
   mutt_expand_path (buf, sizeof (buf));
   if ((rc = fopen (buf, "a")))
   {
     buf[0] = 0;
-    rfc822_write_address (buf, sizeof(buf), new->addr);
-    fprintf (rc, "alias %s %s\n", new->name, buf);
+    rfc822_write_address (buf, sizeof (buf), new->addr);
+    fprintf (rc, "alias %s ", new->name);
+    write_safe_address (rc, buf);
+    fputc ('\n', rc);
     fclose (rc);
     mutt_message ("Alias added.");
   }
@@ -242,22 +265,18 @@ ADDRESS *alias_reverse_lookup (ADDRESS *a)
   ALIAS *t = Aliases;
   ADDRESS *ap;
 
-  if (!a || !a->mailbox || !a->host)
+  if (!a || !a->mailbox)
     return NULL;
 
-  while (t)
+  for (; t; t = t->next)
   {
     /* cycle through all addresses if this is a group alias */
-    ap = t->addr;
-    while (ap)
+    for (ap = t->addr; ap; ap = ap->next)
     {
-      if (ap->mailbox && ap->host &&
-	  strcasecmp (ap->mailbox, a->mailbox) == 0 &&
-	  strcasecmp (ap->host, a->host) == 0)
+      if (!ap->group && ap->mailbox &&
+	  strcasecmp (ap->mailbox, a->mailbox) == 0)
 	return ap;
-      ap = ap->next;
     }
-    t = t->next;
   }
   return 0;
 }
@@ -342,3 +361,37 @@ int mutt_alias_complete (char *s, size_t buflen)
 
   return 1;
 }
+
+/* returns TRUE if the given address belongs to the user. */
+int mutt_addr_is_user (ADDRESS *addr)
+{
+  char buf[LONG_STRING];
+
+  /* NULL address is assumed to be the user. */
+  if (!addr)
+    return 1;
+  if (!addr->mailbox)
+    return 0;
+  if (strcasecmp (addr->mailbox, Username) == 0)
+    return 1;
+  snprintf (buf, sizeof (buf), "%s@%s", Username, Hostname);
+  if (strcasecmp (addr->mailbox, buf) == 0)
+    return 1;
+  snprintf (buf, sizeof (buf), "%s@%s", Username, Fqdn);
+  if (strcasecmp (addr->mailbox, buf) == 0)
+    return 1;
+
+  if (Alternates.pattern &&
+      regexec (Alternates.rx, addr->mailbox, 0, NULL, 0) == 0)
+    return 1;
+  return 0;
+}
+
+/* returns 1 if the list of address contains a known mailing list */
+int mutt_is_list_recipient (ADDRESS *a)
+{
+  for (; a; a = a->next)
+    if (mutt_is_mail_list (a))
+      return 1;
+  return 0;
+}
