@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,21 +19,19 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "keymap.h"
+#include "mapping.h"
 
 #include <string.h>
 
 static struct mapping_t Menus[] = {
- { "index",	MENU_MAIN },
- { "pager",	MENU_PAGER },
- { "url",	MENU_URL },
- { "compose",	MENU_COMPOSE },
+ { "alias",	MENU_ALIAS },
  { "attach",	MENU_ATTACH },
  { "browser",	MENU_FOLDER },
- { "alias",	MENU_ALIAS },
+ { "compose",	MENU_COMPOSE },
  { "editor",	MENU_EDITOR },
-#ifdef _PGPPATH
- { "pgp",	MENU_PGP },
-#endif /* _PGPPATH */
+ { "generic",	MENU_GENERIC },
+ { "index",	MENU_MAIN },
+ { "pager",	MENU_PAGER },
  { NULL,	0 }
 };
 
@@ -48,9 +46,10 @@ static const char *parse_keymap (int *menu,
 				 size_t errlen)
 {
   char buf[SHORT_STRING];
+  char expn[SHORT_STRING];
 
   /* menu name */
-  s = mutt_extract_token (buf, sizeof (buf), s);
+  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
 
   if (s)
   {
@@ -61,7 +60,7 @@ static const char *parse_keymap (int *menu,
     }
 
     /* key sequence */
-    s = mutt_extract_token (key, keylen, s);
+    s = mutt_extract_token (key, keylen, s, expn, sizeof (expn), 0);
 
     if (s)
       return s;
@@ -71,13 +70,28 @@ static const char *parse_keymap (int *menu,
   return (NULL);
 }
 
+static int
+try_bind (char *key, int menu, char *func, struct binding_t *bindings)
+{
+  int i;
+  
+  for (i = 0; bindings[i].name; i++)
+    if (strcmp (func, bindings[i].name) == 0)
+    {
+      km_bindkey (key, menu, bindings[i].op, NULL);
+      return (0);
+    }
+  return (-1);
+}
+
 /* bind menu-name '<key_sequence>' function-name */
-int mutt_parse_bind (const char *s, void *data, char *err, size_t errlen)
+int mutt_parse_bind (const char *s, unsigned long data, char *err, size_t errlen)
 {
   struct binding_t *bindings = NULL;
   char buf[SHORT_STRING];
   char key[SHORT_STRING];
-  int menu, i;
+  char expn[SHORT_STRING];
+  int menu;
 
   if ((s = parse_keymap (&menu, key, sizeof (key), s, err, errlen)) == NULL)
     return (-1);
@@ -87,44 +101,34 @@ int mutt_parse_bind (const char *s, void *data, char *err, size_t errlen)
     case MENU_MAIN:
       bindings = OpMain;
       break;
-
+    case MENU_GENERIC:
+      bindings = OpGeneric;
+      break;
     case MENU_COMPOSE:
       bindings = OpCompose;
       break;
-
     case MENU_PAGER:
       bindings = OpPager;
       break;
-
     case MENU_POST:
       bindings = OpPost;
       break;
-
     case MENU_FOLDER:
       bindings = OpBrowser;
       break;
-
     case MENU_ATTACH:
       bindings = OpAttach;
       break;
-
     case MENU_EDITOR:
       bindings = OpEditor;
       break;
-
     case MENU_ALIAS:
       bindings = OpAlias;
       break;
-
-#ifdef _PGPPATH
-    case MENU_PGP:
-      bindings = OpPgp;
-      break;
-#endif /* _PGPPATH */
   }
 
   /* function to execute */
-  s = mutt_extract_token (buf, sizeof (buf), s);
+  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
   if (s)
   {
     strfcpy (err, "too many arguments", errlen);
@@ -136,63 +140,34 @@ int mutt_parse_bind (const char *s, void *data, char *err, size_t errlen)
     km_bindkey (key, menu, OP_NULL, NULL);
     return 0;
   }
-  else if (menu == MENU_PAGER || menu == MENU_MAIN || menu == MENU_EDITOR)
-  {
-    /* These menus have keymaps of their own and do not refer to the
-     * generic map.
-     */
-    for (i = 0; bindings[i].name; i++)
-      if (strcmp (buf, bindings[i].name) == 0)
-      {
-	km_bindkey (key, menu, bindings[i].op, NULL);
-	return (0);
-      }
-  }
-  else
-  {
-    /*
-     * First check the "generic" list of commands.
-     */
-    for (i=0; OpGeneric[i].name; i++)
-    {
-      if (strcmp (buf, OpGeneric[i].name) == 0)
-      {
-	km_bindkey (key, menu, OpGeneric[i].op, NULL);
-	return (0);
-      }
-    }
 
-    /*
-     * Now check the menu-specific list of commands (if they exist).
-     */
-    if (bindings)
-    {
-      for (i=0; bindings[i].name; i++)
-      {
-	if (strcmp (buf, bindings[i].name) == 0)
-	{
-	  km_bindkey (key, menu, bindings[i].op, NULL);
-	  return (0);
-	}
-      }
-    }
+  if (menu != MENU_PAGER && menu != MENU_EDITOR && menu != MENU_GENERIC)
+  {
+    /* First check the "generic" list of commands.  */
+    if (try_bind (key, menu, buf, OpGeneric) == 0)
+      return 0;
   }
 
+  /* Now check the menu-specific list of commands (if they exist).  */
+  if (bindings && try_bind (key, menu, buf, bindings) == 0)
+    return 0;
+
   snprintf (err, errlen, "%s: no such function in map", buf);
   return (-1);
 }
 
 /* macro <menu> <key> <macro> */
-int mutt_parse_macro (const char *s, void *data, char *err, size_t errlen)
+int mutt_parse_macro (const char *s, unsigned long data, char *err, size_t errlen)
 {
   int menu;
   char key[SHORT_STRING];
   char buf[SHORT_STRING];
+  char expn[SHORT_STRING];
 
   if ((s = parse_keymap (&menu, key, sizeof (key), s, err, errlen)) == NULL)
     return (-1);
 
-  s = _mutt_extract_token (buf, sizeof (buf), s, M_CONDENSE);
+  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), M_CONDENSE);
   if (s)
   {
     strfcpy (err, "too many arguments", errlen);
