@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -18,15 +18,22 @@
 
 #include "mutt.h"
 #include "mutt_curses.h"
+#include "mapping.h"
 
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
 
+/* globals */
+int *ColorQuote;
+int ColorQuoteUsed;
 int ColorDefs[MT_COLOR_MAX];
 COLOR_LINE *ColorHdrList = NULL;
 COLOR_LINE *ColorBodyList = NULL;
 
+/* local to this file */
+static int ColorQuoteSize;
+
 #ifdef HAVE_COLOR
 
 #define COLOR_DEFAULT (-2)
@@ -42,7 +49,6 @@ typedef struct color_list
 
 static COLOR_LIST *ColorList = NULL;
 static int UserColors = 0;
-static int ColorQuoteSize;
 
 static struct mapping_t Colors[] =
 {
@@ -98,6 +104,7 @@ void ci_start_color (void)
   ColorDefs[MT_COLOR_STATUS] = A_REVERSE;
   ColorDefs[MT_COLOR_INDICATOR] = A_REVERSE;
   ColorDefs[MT_COLOR_SEARCH] = A_REVERSE;
+  ColorDefs[MT_COLOR_MARKERS] = A_REVERSE;
   /* special meaning: toggle the relevant attribute */
   ColorDefs[MT_COLOR_BOLD] = 0;
   ColorDefs[MT_COLOR_UNDERLINE] = 0;
@@ -281,7 +288,7 @@ static int add_pattern (COLOR_LINE **top, const char *s, int sensitive,
     int r;
 
     tmp = mutt_new_color_line ();
-    if ((r = regcomp (&tmp->rx, s, REG_EXTENDED | (sensitive ? mutt_which_case (s) : REG_ICASE))) != 0)
+    if ((r = REGCOMP (&tmp->rx, s, (sensitive ? mutt_which_case (s) : REG_ICASE))) != 0)
     {
       regerror (r, &tmp->rx, err, errlen);
       regfree (&tmp->rx);
@@ -345,11 +352,12 @@ static int parse_color_name (const char *s, int *col, int *attr, int brite,
 }
 
 /* usage: color <object> <fg> <bg> [ <regexp> ] */
-int mutt_parse_color (const char *s, void *data, char *errmsg, size_t errlen)
+int mutt_parse_color (const char *s, unsigned long data, char *errmsg, size_t errlen)
 {
   int object = 0, bold = 0, fg = 0, bg = 0, q_level = 0;
   int r = 0;
   char buf[SHORT_STRING];
+  char expn[SHORT_STRING];
   char *eptr;
 
   /* don't parse curses command if we're not using the screen */
@@ -360,7 +368,7 @@ int mutt_parse_color (const char *s, void *data, char *errmsg, size_t errlen)
   if (!has_colors ())
     return 0;
   
-  s = mutt_extract_token (buf, sizeof (buf), s);
+  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
 
   if (strncmp (buf, "quoted", 6) == 0)
   {
@@ -387,7 +395,7 @@ int mutt_parse_color (const char *s, void *data, char *errmsg, size_t errlen)
     strfcpy (errmsg, "too few arguments", errlen);
     return (-1);
   }
-  s = mutt_extract_token (buf, sizeof (buf), s);
+  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
 
   if (parse_color_name (buf, &fg, &bold, A_BOLD, errmsg, errlen) != 0)
     return (-1);
@@ -398,7 +406,7 @@ int mutt_parse_color (const char *s, void *data, char *errmsg, size_t errlen)
     strfcpy (errmsg, "too few arguments", errlen);
     return (-1);
   }
-  s = mutt_extract_token (buf, sizeof (buf), s);
+  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
 
   /* A_BLINK turns the background color brite on some terms */
   if (parse_color_name (buf, &bg, &bold, A_BLINK, errmsg, errlen) != 0)
@@ -412,7 +420,7 @@ int mutt_parse_color (const char *s, void *data, char *errmsg, size_t errlen)
       return (-1);
     }
 
-    s = mutt_extract_token (buf, sizeof (buf), s);
+    s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
     if (s)
     {
       strfcpy (errmsg, "too many arguments", errlen);
@@ -466,13 +474,14 @@ int mutt_parse_color (const char *s, void *data, char *errmsg, size_t errlen)
  *
  * set attribute for an object when using a terminal with no color support
  */
-int mutt_parse_mono (const char *s, void *data, char *err, size_t errlen)
+int mutt_parse_mono (const char *s, unsigned long data, char *err, size_t errlen)
 {
   int r = 0;
   int object;
   int q_level = 0;
   int attr = A_NORMAL;
   char buf[SHORT_STRING];
+  char expn[SHORT_STRING];
   char *eptr;
 
   if (option (OPTNOCURSES))
@@ -484,7 +493,7 @@ int mutt_parse_mono (const char *s, void *data, char *err, size_t errlen)
     return 0;
 #endif
 
-  s = mutt_extract_token (buf, sizeof (buf), s);
+  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
 
   if (strncmp (buf, "quoted", 6) == 0)
   {
@@ -511,7 +520,7 @@ int mutt_parse_mono (const char *s, void *data, char *err, size_t errlen)
     return (-1);
   }
 
-  s = mutt_extract_token (buf, sizeof (buf), s);
+  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
 
   if (strcasecmp ("bold", buf) == 0)
     attr |= A_BOLD;
@@ -523,6 +532,8 @@ int mutt_parse_mono (const char *s, void *data, char *err, size_t errlen)
     attr |= A_REVERSE;
   else if (strcasecmp ("standout", buf) == 0)
     attr |= A_STANDOUT;
+  else if (strcasecmp ("normal", buf) == 0)
+    attr = A_NORMAL; /* needs use = instead of |= to clear other bits */
   else
   {
     snprintf (err, errlen, "%s: no such attribute", s);
@@ -537,7 +548,7 @@ int mutt_parse_mono (const char *s, void *data, char *err, size_t errlen)
       return (-1);
     }
 
-    s = mutt_extract_token (buf, sizeof (buf), s);
+    s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
     if (s)
     {
       strfcpy (err, "too many arguments", errlen);
