@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,8 +19,10 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
-#include "send.h"
 #include "rfc1524.h"
+#include "mime.h"
+#include "attach.h"
+#include "mapping.h"
 
 #include <string.h>
 #include <sys/stat.h>
@@ -30,26 +32,38 @@
 
 #define CHECK_COUNT if (idxlen == 0) { mutt_error ("There are no attachments."); break; }
 
-#define HDR_TO 1
-#define HDR_CC 2
-#define HDR_BCC 3
-#define HDR_SUBJECT 4
-#define HDR_REPLYTO 5
-#define HDR_FCC 6
-#define HDR_PGP 7
 
-/* where to start printing the attachments */
-#define HDR_ATTACH 10
+
+enum
+{
+  HDR_FROM  = 1,
+  HDR_TO,
+  HDR_CC,
+  HDR_BCC,
+  HDR_SUBJECT,
+  HDR_REPLYTO,
+  HDR_FCC,
+
+
+  
+  
+
+  HDR_ATTACH  = (HDR_FCC + 5) /* where to start printing the attachments */
+};
 
 #define HDR_XOFFSET 10
 
-/* don't put these in "protos.h" since we don't want to include
- * "mutt_menu.h" everywhere
- */
-void mutt_pipe_attachment_list (FILE *, int, MUTTMENU *, ATTACHPTR **, int);
-void mutt_print_attachment_list (FILE *, int, MUTTMENU *, ATTACHPTR **);
-void mutt_save_attachment_list (FILE *, int, MUTTMENU *, ATTACHPTR **, int);
-void mutt_tag_attachment (int, MUTTMENU *, ATTACHPTR **); 
+static struct mapping_t ComposeHelp[] = {
+  { "Send",    OP_COMPOSE_SEND_MESSAGE },
+  { "Abort",   OP_EXIT },
+  { "To",      OP_COMPOSE_EDIT_TO },
+  { "CC",      OP_COMPOSE_EDIT_CC },
+  { "Subj",    OP_COMPOSE_EDIT_SUBJECT },
+  { "Attach",  OP_COMPOSE_ATTACH_FILE },
+  { "Descrip", OP_COMPOSE_EDIT_DESCRIPTION },
+  { "Help",    OP_HELP },
+  { NULL }
+};
 
 void snd_entry (char *b, size_t blen, MUTTMENU *menu, int num)
 {
@@ -79,84 +93,47 @@ void snd_entry (char *b, size_t blen, MUTTMENU *menu, int num)
   snprintf (t, sizeof (t), "[%.7s/%.10s, %.6s, %s]",
 	    TYPE (m->type), m->subtype, ENCODING (m->encoding), size);
 
-  snprintf (b, blen, "%c%c %2d %-34.34s %s%s %s",
+  snprintf (b, blen, "%c%c%2d %-34.34s %s%s <%s>",
 	    m->unlink ? '-' : ' ',
-	    idx[num]->tagged ? '*' : ' ',
+	    m->tagged ? '*' : ' ',
 	    num + 1,
 	    t,
 	    idx[num]->tree ? idx[num]->tree : "",
 	    tmp,
-	    m->description ? m->description : "<no description>");
+	    m->description ? m->description : "no description");
 }
 
-#ifdef _PGPPATH
-#include "pgp.h"
 
-static int pgp_send_menu (int bits)
-{
-  int c;
-  char *p;
 
-  mvaddstr (LINES-1, 0, "(e)ncrypt, (s)ign, sign (a)s, (b)oth or (f)orget it? ");
-  clrtoeol ();
-  do
-  {
-    refresh ();
-    if ((c = ci_getch ()) == ERR)
-      break;
-    if (c == 'a')
-    {
-      if ((p = pgp_ask_for_key (1, NULL, "Sign as: ")))
-      {
-	snprintf (PgpSignAs, sizeof (PgpSignAs), "0x%s", p);
-	pgp_void_passphrase (); /* probably need a different passphrase */
-	free (p);
-	bits |= PGPSIGN;
-      }
-    }
-    else if (c == 'e')
-      bits |= PGPENCRYPT;
-    else if (c == 's')
-      bits |= PGPSIGN;
-    else if (c == 'b')
-      bits = PGPENCRYPT | PGPSIGN;
-    else if (c == 'f')
-      bits = 0;
-    else
-    {
-      BEEP ();
-      c = 0;
-    }
-  }
-  while (c == 0);
-  CLEARLINE (LINES-1);
-  refresh ();
-  return (bits);
-}
-#endif /* _PGPPATH */
 
 static void draw_envelope (HEADER *msg, char *fcc)
 {
   char buf[STRING];
+  int w = COLS - HDR_XOFFSET;
+
+  mvaddstr (HDR_FROM, 0,    "    From: ");
+  buf[0] = 0;
+  rfc822_write_address (buf, sizeof (buf), msg->env->from);
+  printw ("%-*.*s", w, w, buf);
 
   mvaddstr (HDR_TO, 0,      "      To: ");
   buf[0] = 0;
   rfc822_write_address (buf, sizeof (buf), msg->env->to);
-  printw ("%-*.*s", COLS-HDR_XOFFSET, COLS-HDR_XOFFSET, buf);
+  printw ("%-*.*s", w, w, buf);
 
   mvaddstr (HDR_CC, 0,      "      Cc: ");
   buf[0] = 0;
   rfc822_write_address (buf, sizeof (buf), msg->env->cc);
-  printw ("%-*.*s", COLS-HDR_XOFFSET, COLS-HDR_XOFFSET, buf);
+  printw ("%-*.*s", w, w, buf);
 
   mvaddstr (HDR_BCC, 0,     "     Bcc: ");
   buf[0] = 0;
   rfc822_write_address (buf, sizeof (buf), msg->env->bcc);
-  printw ("%-*.*s", COLS-HDR_XOFFSET, COLS-HDR_XOFFSET, buf);
+  printw ("%-*.*s", w, w, buf);
 
   mvaddstr (HDR_SUBJECT, 0, " Subject: ");
   if (msg->env->subject)
-    printw ("%-*.*s", COLS-HDR_XOFFSET, COLS-HDR_XOFFSET, msg->env->subject);
+    printw ("%-*.*s", w, w, msg->env->subject);
   else
     clrtoeol ();
 
@@ -165,7 +142,7 @@ static void draw_envelope (HEADER *msg, char *fcc)
   {
     buf[0] = 0;
     rfc822_write_address (buf, sizeof (buf), msg->env->reply_to);
-    printw ("%-*.*s", COLS - HDR_XOFFSET, COLS - HDR_XOFFSET, buf);
+    printw ("%-*.*s", w, w, buf);
   }
   else
     clrtoeol ();
@@ -173,24 +150,8 @@ static void draw_envelope (HEADER *msg, char *fcc)
   mvaddstr (HDR_FCC, 0,     "     Fcc: ");
   addstr (fcc);
 
-#ifdef _PGPPATH
-  mvaddstr (HDR_PGP, 0,     "     PGP: ");
-  if ((msg->pgp & (PGPENCRYPT | PGPSIGN)) == (PGPENCRYPT | PGPSIGN))
-    addstr ("Sign, Encrypt");
-  else if (msg->pgp & PGPENCRYPT)
-    addstr ("Encrypt");
-  else if (msg->pgp & PGPSIGN)
-  {
-    addstr ("Sign");
-    if (PgpSignAs[0])
-      printw (" as %s", PgpSignAs);
-  }
-  else
-    addstr ("Clear");
-  clrtoeol ();
-#endif /* _PGPPATH */
 
-  mvaddstr (HDR_ATTACH-1, 0, "===== Attachments =====");
+  mvaddstr (HDR_ATTACH - 1, 0, "===== Attachments =====");
 }
 
 static int edit_address_list (int line, ENVELOPE *env)
@@ -201,41 +162,36 @@ static int edit_address_list (int line, ENVELOPE *env)
 
   switch (line)
   {
+    case HDR_FROM:
+      prompt = "From: ";
+      addr = &env->from;
+      break;
     case HDR_TO:
-
       prompt = "To: ";
       addr = &env->to;
       break;
-
     case HDR_CC:
-
       prompt = "Cc: ";
       addr = &env->cc;
       break;
-
     case HDR_BCC:
-
       prompt = "Bcc: ";
       addr = &env->bcc;
       break;
-
     case HDR_REPLYTO:
-
       prompt = "Reply-To: ";
       addr = &env->reply_to;
       break;
-
     default:
-
       return 0;
   }
 
   rfc822_write_address (buf, sizeof (buf), *addr);
-  if (ci_get_field (prompt, buf, sizeof (buf), M_ALIAS) != 0)
+  if (mutt_get_field (prompt, buf, sizeof (buf), M_ALIAS) != 0)
     return 0;
 
-  mutt_free_address (addr);
-  mutt_parse_adrlist (addr, buf, "@");
+  rfc822_free_address (addr);
+  *addr = mutt_parse_adrlist (*addr, buf);
   *addr = mutt_expand_aliases (*addr);
 
   if (option (OPTNEEDREDRAW))
@@ -253,31 +209,6 @@ static int edit_address_list (int line, ENVELOPE *env)
   return 0;
 }
 
-static void update_unlink (ATTACHPTR **idx, short idxlen)
-{
-  BODY *tmp;
-  int x;
-
-  mutt_clear_markers (idx, idxlen);
-  for (tmp = idx[0]->content; tmp; tmp = tmp->next)
-  {
-    if (!tmp->empty)
-    {
-      if (tmp->unlink)
-	mutt_mark_parts (tmp, 1);
-      else
-	mutt_mark_parts (tmp, 0);
-    }
-  }
-  for (x = 0; x < idxlen; x++)
-  {
-    if (idx[x]->content->marked)
-      idx[x]->content->unlink = 1;
-    else
-      idx[x]->content->unlink = 0;
-  }
-}
-
 static int delete_attachment (MUTTMENU *menu, short *idxlen, int x)
 {
   ATTACHPTR **idx = (ATTACHPTR **) menu->data;
@@ -288,7 +219,7 @@ static int delete_attachment (MUTTMENU *menu, short *idxlen, int x)
   if (x == 0 && menu->max == 1)
   {
     mutt_error ("You may not delete the only attachment.");
-    idx[x]->tagged = 0;
+    idx[x]->content->tagged = 0;
     return (-1);
   }
 
@@ -328,118 +259,54 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
   char buf[LONG_STRING];
   char fname[_POSIX_PATH_MAX];
   MUTTMENU *menu;
-  BODY *tmp;
   ATTACHPTR **idx = NULL;
   short idxlen = 0;
   short idxmax = 0;
-  int tag = 0;		/* has the tag-prefix command been pressed? */
   int i;
   int r = -1;		/* return value */
   int op = 0;
   int loop = 1;
   int fccSet = 0;	/* has the user edited the Fcc: field ? */
-  int found;
-  int x;
 
-  idx = mutt_gen_attach_list (msg->content, &idxlen, &idxmax, 1);
-  update_unlink (idx, idxlen);
-
-  for (x = 0; x < idxlen; x++)
-      idx[x]->content->empty = 1;
-  for (tmp = idx[0]->content; tmp; tmp = tmp->next)
-    tmp->empty = 0;
+  idx = mutt_gen_attach_list (msg->content, idx, &idxlen, &idxmax, 0, 1);
 
   menu = mutt_new_menu ();
   menu->menu = MENU_COMPOSE;
   menu->offset = HDR_ATTACH;
   menu->max = idxlen;
   menu->make_entry = snd_entry;
+  menu->tag = mutt_tag_attach;
   menu->title = "Compose";
   menu->data = idx;
-
-  helpstr[0] = 0;
-  mutt_make_help (buf, sizeof (buf), "Send  ", MENU_COMPOSE, OP_COMPOSE_SEND_MESSAGE);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Abort  ", MENU_COMPOSE, OP_EXIT);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "To  ", MENU_COMPOSE, OP_COMPOSE_EDIT_TO);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "CC  ", MENU_COMPOSE, OP_COMPOSE_EDIT_CC);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Subj  ", MENU_COMPOSE, OP_COMPOSE_EDIT_SUBJECT);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Attach  ", MENU_COMPOSE, OP_COMPOSE_ATTACH_FILE);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Descrip  ", MENU_COMPOSE, OP_COMPOSE_EDIT_DESCRIPTION);
-  strcat (helpstr, buf);
-#ifdef _PGPPATH
-  mutt_make_help (buf, sizeof (buf), "PGP  ", MENU_COMPOSE, OP_COMPOSE_PGP_MENU);
-  strcat (helpstr, buf);
-#endif
-  mutt_make_help (buf, sizeof (buf), "Help", MENU_COMPOSE, OP_HELP);
-  strcat (helpstr, buf);
-  menu->help = helpstr;
-
+  menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_COMPOSE, ComposeHelp);
+  
   while (loop)
   {
-    tag = 0; /* clear the tag-prefix */
-    op = mutt_menuLoop (menu);
-
-    if (op == OP_ATTACH_TAG_PREFIX) 
-    {
-      CLEARLINE (LINES-1);
-      addstr ("tag-");
-      op = mutt_menuLoop (menu);
-      if (op == OP_ATTACH_TAG_PREFIX) 
-      {
-	CLEARLINE (LINES-1);
-	continue;
-      }
-      tag = 1;
-    }
-    else if (option (OPTAUTOTAG))
-    {
-      for (x = 0; x < menu->max ; x++)
-      {
-	if (idx[x]->tagged)
-	{
-	  tag = 1;
-	  break;
-	}
-      }
-    }
-
-    switch (op)
+    switch (op = mutt_menuLoop (menu))
     {
       case OP_REDRAW:
-
 	draw_envelope (msg, fcc);
 	menu->offset = HDR_ATTACH;
-	menu->pagelen = LINES - 12;
+	menu->pagelen = LINES - HDR_ATTACH - 2;
+	break;
+      case OP_COMPOSE_EDIT_FROM:
+	menu->redraw = edit_address_list (HDR_FROM, msg->env);
 	break;
-
       case OP_COMPOSE_EDIT_TO:
-
 	menu->redraw = edit_address_list (HDR_TO, msg->env);
 	break;
-
       case OP_COMPOSE_EDIT_BCC:
-
 	menu->redraw = edit_address_list (HDR_BCC, msg->env);
 	break;
-
       case OP_COMPOSE_EDIT_CC:
-
 	menu->redraw = edit_address_list (HDR_CC, msg->env);
 	break;
-
       case OP_COMPOSE_EDIT_SUBJECT:
-
 	if (msg->env->subject)
 	  strfcpy (buf, msg->env->subject, sizeof (buf));
 	else
 	  buf[0] = 0;
-	if (ci_get_field ("Subject: ", buf, sizeof (buf), 0) == 0)
+	if (mutt_get_field ("Subject: ", buf, sizeof (buf), 0) == 0)
 	{
 	  safe_free ((void **) &msg->env->subject);
 	  msg->env->subject = safe_strdup (buf);
@@ -447,20 +314,15 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	  clrtoeol ();
 	  if (msg->env->subject)
 	    printw ("%-*.*s", COLS-HDR_XOFFSET, COLS-HDR_XOFFSET,
-						      msg->env->subject);
+		    msg->env->subject);
 	}
 	break;
-
       case OP_COMPOSE_EDIT_REPLY_TO:
-
 	menu->redraw = edit_address_list (HDR_REPLYTO, msg->env);
 	break;
-
       case OP_COMPOSE_EDIT_FCC:
-
-	buf[0] = 0;
 	strfcpy (buf, fcc, sizeof (buf));
-	if (ci_get_field ("Fcc: ", buf, sizeof (buf), M_FILE | M_CLEAR) == 0)
+	if (mutt_get_field ("Fcc: ", buf, sizeof (buf), M_FILE | M_CLEAR) == 0)
 	{
 	  strfcpy (fcc, buf, _POSIX_PATH_MAX);
 	  mutt_pretty_mailbox (fcc);
@@ -470,46 +332,49 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	}
 	MAYBE_REDRAW (menu->redraw);
 	break;
-
       case OP_COMPOSE_EDIT_MESSAGE:
-
-	if (strcmp ("builtin", Editor) == 0)
+	if (strcmp ("builtin", Editor) != 0 && !option (OPTEDITHDRS))
 	{
-	  mutt_builtin_editor (msg->content->filename, &msg->env, cur);
-	  menu->redraw = REDRAW_FULL;
+	  mutt_edit_file (Editor, msg->content->filename);
+	  mutt_update_encoding (msg->content);
+	  menu->redraw = REDRAW_CURRENT;
+	  break;
 	}
-	else if (option (OPTEDITHDRS))
+	/* fall through */
+      case OP_COMPOSE_EDIT_HEADERS:
+	if (op == OP_COMPOSE_EDIT_HEADERS ||
+	    (op == OP_COMPOSE_EDIT_MESSAGE && option (OPTEDITHDRS)))
 	{
-	  mutt_edit_headers (Editor, msg->content->filename, &msg->env,
-							      fcc, fcclen);
-	  menu->redraw = REDRAW_FULL; /* recipients could have changed */
+	  mutt_edit_headers (strcmp ("builtin", Editor) == 0 ? Visual : Editor,
+			     msg->content->filename, msg, fcc, fcclen);
 	}
 	else
 	{
-	  mutt_edit_file (Editor, msg->content->filename);
-	  menu->redraw = REDRAW_CURRENT;
+	  /* this is grouped with OP_COMPOSE_EDIT_HEADERS because the
+	     attachment list could change if the user invokes ~v to edit
+	     the message with headers, in which we need to execute the
+	     code below to regenerate the index array */
+	  mutt_builtin_editor (msg->content->filename, msg, cur);
 	}
 	mutt_update_encoding (msg->content);
-	break;
 
-      case OP_COMPOSE_EDIT_HEADERS:
+	/* attachments may have been added */
+	if (idxlen && idx[idxlen - 1]->content->next)
+	{
+	  for (i = 0; i < idxlen; i++)
+	    safe_free ((void **) &idx[i]);
+	  idxlen = 0;
+	  idx = mutt_gen_attach_list (msg->content, idx, &idxlen, &idxmax, 0, 1);
+	  menu->max = idxlen;
+	}
 
-	mutt_edit_headers (strcmp ("builtin", Editor) == 0 ? Visual : Editor,
-			   msg->content->filename, &msg->env, fcc, fcclen);
-        mutt_update_encoding (msg->content);
 	menu->redraw = REDRAW_FULL;
 	break;
 
-      case OP_ATTACH_TAG:
-
-	mutt_tag_attachment (tag, menu, idx);
-	break;
-
       case OP_COMPOSE_ATTACH_FILE:
-
 	fname[0] = 0;
 	if (mutt_enter_fname ("Attach file", fname, sizeof (fname),
-							&menu->redraw) == -1)
+			      &menu->redraw, 0) == -1)
 	  break;
 	if (!fname[0])
 	  continue;
@@ -532,11 +397,10 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	if ((idx[idxlen]->content = mutt_make_attach (fname)) != NULL)
 	{
 	  idx[idxlen]->level = (idxlen > 0) ? idx[idxlen-1]->level : 0;
-	  tmp = idx[0]->content;
-	  while (tmp && tmp->next)
-	    tmp = tmp->next;
-	  if (tmp)
-	    tmp->next = idx[idxlen]->content;
+
+	  if (idxlen)
+	    idx[idxlen - 1]->content->next = idx[idxlen]->content;
+
 	  menu->current = idxlen++;
 	  mutt_update_tree (idx, idxlen);
 	  menu->max = idxlen;
@@ -550,45 +414,10 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	break;
 
       case OP_DELETE:
-
 	CHECK_COUNT;
-	mutt_clear_markers (idx, idxlen);
-	if (tag)
-	{
-	  found = 0;
-	  for (x = 0; x < menu->max; x++)
-	  {
-	    if (idx[x]->tagged && !idx[x]->content->marked)
-	    {
-	      found = 1;
-	      mutt_mark_parts (idx[x]->content, 1);
-	    }
-	  }
-	  if (!found)
-	  {
-	    mutt_error ("No tagged attachments!");
-	    break;
-	  }
-	}
-	else
-	  mutt_mark_parts (idx[menu->current]->content, 1);
-
-	x = 0;
-	while (x < menu->max)
-	{
-	  if (idx[x]->content->marked)
-	  {
-	    if (delete_attachment (menu, &idxlen, x) == -1)
-	      break;
-	    if (x < menu->current)
-	      (menu->current)--;
-	  }
-	  else
-	    x++;
-	}
-
+	if (delete_attachment (menu, &idxlen, menu->current) == -1)
+	  break;
 	mutt_update_tree (idx, idxlen);
-
 	if (idxlen)
 	{
 	  if (menu->current > idxlen - 1)
@@ -602,13 +431,12 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	break;
 
       case OP_COMPOSE_EDIT_DESCRIPTION:
-
 	CHECK_COUNT;
 	strfcpy (buf,
 		 idx[menu->current]->content->description ?
 		 idx[menu->current]->content->description : "",
 		 sizeof (buf));
-	if (ci_get_field ("Description: ", buf, sizeof (buf), 0) == 0)
+	if (mutt_get_field ("Description: ", buf, sizeof (buf), 0) == 0)
 	{
 	  safe_free ((void **) &idx[menu->current]->content->description);
 	  idx[menu->current]->content->description = safe_strdup (buf);
@@ -617,12 +445,11 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	break;
 
       case OP_COMPOSE_EDIT_TYPE:
-
 	CHECK_COUNT;
 	snprintf (buf, sizeof (buf), "%s/%s",
 		  TYPE (idx[menu->current]->content->type),
 		  idx[menu->current]->content->subtype);
-	if (ci_get_field ("Content-Type: ", buf, sizeof (buf), 0) == 0 && buf[0])
+	if (mutt_get_field ("Content-Type: ", buf, sizeof (buf), 0) == 0 && buf[0])
 	{
 	  char *p = strchr (buf, '/');
 
@@ -641,11 +468,10 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	break;
 
       case OP_COMPOSE_EDIT_ENCODING:
-
 	CHECK_COUNT;
 	strfcpy (buf, ENCODING (idx[menu->current]->content->encoding),
 							      sizeof (buf));
-	if (ci_get_field ("Content-Transfer-Encoding: ", buf,
+	if (mutt_get_field ("Content-Transfer-Encoding: ", buf,
 					    sizeof (buf), 0) == 0 && buf[0])
 	{
 	  if ((i = mutt_check_encoding (buf)) != ENCOTHER)
@@ -659,7 +485,6 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	break;
 
       case OP_COMPOSE_SEND_MESSAGE:
-
 	if (!fccSet && *fcc)
 	{
 	  if ((i = query_quadoption (OPT_COPY, "Save a copy of this message?"))
@@ -674,7 +499,6 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	break;
 
       case OP_COMPOSE_EDIT_FILE:
-
 	CHECK_COUNT;
 	mutt_edit_file (strcmp ("builtin", Editor) == 0 ? Visual : Editor,
 			idx[menu->current]->content->filename);
@@ -683,40 +507,20 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	break;
 
       case OP_COMPOSE_TOGGLE_UNLINK:
-
 	CHECK_COUNT;
-	found = 0;
-	if (tag)
-	{
-	  for (x = 0; x < menu->max; x++)
-	  {
-	    if (idx[x]->tagged)
-	    {
-	      found = 1;
-	      idx[x]->content->unlink = !idx[x]->content->unlink;
-	    }
-	  }
-	  if (!found)
-	    mutt_error ("No tagged attachments!");
-	}
-	else
-	  idx[menu->current]->content->unlink =
-				      !idx[menu->current]->content->unlink;
-	update_unlink (idx, idxlen);
+	idx[menu->current]->content->unlink = !idx[menu->current]->content->unlink;
 	menu->redraw = REDRAW_INDEX;
 	break;
 
       case OP_COMPOSE_RENAME_FILE:
-
-	if (idx[menu->current]->content->empty)
-	  break;
+	CHECK_COUNT;
 	strfcpy (fname, idx[menu->current]->content->filename, sizeof (fname));
 	mutt_pretty_mailbox (fname);
-	if (ci_get_field ("Rename to: ", fname, sizeof (fname), M_FILE) == 0
+	if (mutt_get_field ("Rename to: ", fname, sizeof (fname), M_FILE) == 0
 								  && fname[0])
 	{
 	  mutt_expand_path (fname, sizeof (fname));
-	  if (mutt_rename_file (idx[menu->current]->content->filename, fname))
+	  if (!mutt_rename_file (idx[menu->current]->content->filename, fname))
 	  {
 	    safe_free ((void **) &idx[menu->current]->content->filename);
 	    idx[menu->current]->content->filename = safe_strdup (fname);
@@ -726,7 +530,6 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	break;
 
       case OP_COMPOSE_NEW_MIME:
-
 	{
 	  char type[STRING];
 	  char *p;
@@ -735,14 +538,14 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 
 	  CLEARLINE (LINES-1);
 	  fname[0] = 0;
-	  if (ci_get_field ("New file: ", fname, sizeof (fname), M_FILE) != 0 ||
+	  if (mutt_get_field ("New file: ", fname, sizeof (fname), M_FILE) != 0 ||
 								    !fname[0])
 	    continue;
 	  mutt_expand_path (fname, sizeof (fname));
 
 	  /* Call to lookup_mime_type () ?  maybe later */
 	  type[0] = 0;
-	  if (ci_get_field ("Content-Type: ", type, sizeof (type), 0) != 0 ||
+	  if (mutt_get_field ("Content-Type: ", type, sizeof (type), 0) != 0 ||
 								    !type[0])
 	    continue;
 
@@ -765,7 +568,7 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 
 	  idx[idxlen] = (ATTACHPTR *) safe_calloc (1, sizeof (ATTACHPTR));
 	  /* Touch the file */
-	  if (!(fp = fopen (fname, "w")))
+	  if (!(fp = safe_fopen (fname, "w")))
 	  {
 	    mutt_error ("Can't create file %s", fname);
 	    safe_free ((void **) &idx[idxlen]);
@@ -779,13 +582,10 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	    continue;
 	  }
 	  
-	  tmp = idx[0]->content;
-	  while (tmp && tmp->next)
-	    tmp = tmp->next;
-	  if (tmp)
-	    tmp->next = idx[idxlen]->content;
-	    
 	  idx[idxlen]->level = (idxlen > 0) ? idx[idxlen-1]->level : 0;
+	  if (idxlen)
+	    idx[idxlen - 1]->content->next = idx[idxlen]->content;
+	  
 	  menu->current = idxlen++;
 	  mutt_update_tree (idx, idxlen);
 	  menu->max = idxlen;
@@ -798,7 +598,6 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	}
 
       case OP_COMPOSE_EDIT_MIME:
-
 	CHECK_COUNT;
 	if (mutt_edit_attachment (idx[menu->current]->content, 0))
 	{
@@ -807,60 +606,34 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	}
 	break;
 
-      case OP_COMPOSE_VIEW_ATTACH:
+      case OP_VIEW_ATTACH:
       case OP_DISPLAY_HEADERS:
-
 	CHECK_COUNT;
-	if (idx[menu->current]->content->empty)
-	  break;
-
-	{
-	  int rc;
-
-	  if (op == OP_DISPLAY_HEADERS)
-	    unset_option (OPTWEED);
-	  while ((rc = mutt_view_attachment (NULL,
-			      idx[menu->current]->content, M_REGULAR)) == 2)
-	  {
-	    if (option (OPTWEED))
-	      unset_option (OPTWEED);
-	    else
-	      set_option (OPTWEED);
-	  }
-	  if (rc)
-	  {
-	    menu->redraw = REDRAW_FULL;
-	    mutt_clear_error ();
-	  }
-	  set_option (OPTWEED);		/* turn header weeding back on. */
-	}
+	mutt_attach_display_loop (menu, op, NULL, idx);
+	menu->redraw = REDRAW_FULL;
+	mutt_clear_error ();
 	break;
 
       case OP_SAVE:
-      case OP_ATTACH_DECODE_SAVE:
-
 	CHECK_COUNT;
-	mutt_save_attachment_list (NULL, tag, menu, idx,
-					      op == OP_ATTACH_DECODE_SAVE);
+	mutt_save_attachment_list (NULL, menu->tagprefix, menu->tagprefix ? msg->content : idx[menu->current]->content);
 	break;
 
       case OP_PRINT:
-
 	CHECK_COUNT;
-	mutt_print_attachment_list (NULL, tag, menu, idx);
+	mutt_print_attachment_list (NULL, menu->tagprefix, menu->tagprefix ? msg->content : idx[menu->current]->content);
 	break;
 
       case OP_PIPE:
       case OP_FILTER:
-
-	CHECK_COUNT;
-	mutt_pipe_attachment_list (NULL, tag, menu, idx, op == OP_FILTER);
+        CHECK_COUNT;
+	mutt_pipe_attachment_list (NULL, menu->tagprefix, menu->tagprefix ? msg->content : idx[menu->current]->content, op == OP_FILTER);
+	if (op == OP_FILTER)
+	  menu->redraw = REDRAW_CURRENT; /* cte might have changed */
 	break;
 
       case OP_EXIT:
-
-	i = query_quadoption (OPT_POSTPONE, "Postpone this message?");
-	if (i == M_NO)
+	if ((i = query_quadoption (OPT_POSTPONE, "Postpone this message?")) == M_NO)
 	{
 	  while (idxlen-- > 0)
 	  {
@@ -884,31 +657,16 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 	/* fall through to postpone! */
 
       case OP_COMPOSE_POSTPONE_MESSAGE:
-
 	loop = 0;
 	r = 1;
 	break;
 
       case OP_COMPOSE_ISPELL:
-
 	endwin ();
 	snprintf (buf, sizeof (buf), "%s -x %s", Ispell, msg->content->filename);
 	mutt_system (buf);
 	break;
 
-#ifdef _PGPPATH
-      case OP_COMPOSE_PGP_MENU:
-
-	msg->pgp = pgp_send_menu (msg->pgp);
-	menu->redraw = REDRAW_FULL;
-	break;
-
-      case OP_FORGET_PASSPHRASE:
-
-	mutt_forget_passphrase ();
-	break;
-
-#endif /* _PGPPATH */
     }
   }
 
@@ -916,17 +674,14 @@ int mutt_send_menu (HEADER *msg,   /* structure for new message */
 
   if (idxlen)
   {
-    for (i = 0; i < idxlen - 1; i++)
-      idx[i]->content->next = idx[i+1]->content;
-    idx[i]->content->next = NULL;
     msg->content = idx[0]->content;
+    for (i = 0; i < idxlen; i++)
+      safe_free ((void **) &idx[i]);
   }
   else
     msg->content = NULL;
 
   safe_free ((void **) &idx);
-  idxlen = 0;
-  idxmax = 0;
 
   return (r);
 }
