@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
+#include "mailbox.h"
+#include "sort.h"
+#include "buffy.h"
+
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -51,11 +55,11 @@
 #define CURHDR Context->hdrs[Context->v2r[menu->current]]
 #define OLDHDR Context->hdrs[Context->v2r[menu->oldcurrent]]
 
-static char LastSearch[SHORT_STRING] = "";
+extern const char *ReleaseDate;
 
 void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
 {
-  int flag = 0;
+  int flag = M_MAKEPRINT;
   HEADER *h = Context->hdrs[Context->v2r[num]];
 
   if ((Sort & SORT_MASK) == SORT_THREADS &&
@@ -63,16 +67,16 @@ void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
       h->tree)
   {
     if (h->display_subject)
-      flag = M_FORCESUBJ;
+      flag = M_FORCESUBJ | M_MAKEPRINT;
     else if (Sort & SORT_REVERSE)
     {
       if ((h->parent->msgno > menu->top + menu->pagelen - 1) && 
 	  (!h->prev || (h->prev->msgno > menu->top + menu->pagelen - 1)))
-	flag = M_FORCESUBJ;
+	flag = M_FORCESUBJ | M_MAKEPRINT;
     }
     else if ((h->parent->msgno < menu->top) &&
 	     (!h->prev || (h->prev->msgno < menu->top)))
-      flag = M_FORCESUBJ;
+      flag = M_FORCESUBJ | M_MAKEPRINT;
 
     flag |= M_TREE; /* display the thread tree */
   }
@@ -109,22 +113,20 @@ static int ci_first_message (void)
 
   if (Context && Context->msgcount)
   {
-    if (option (OPTPOINTNEW))
+    for (i=0; i < Context->vcount; i++)
     {
-      for (i=0; i < Context->vcount; i++)
+      if (! Context->hdrs[Context->v2r[i]]->read &&
+	  ! Context->hdrs[Context->v2r[i]]->deleted)
       {
-	if (! Context->hdrs[Context->v2r[i]]->read &&
-	    ! Context->hdrs[Context->v2r[i]]->deleted)
-	{
-	  if (! Context->hdrs[Context->v2r[i]]->old)
-	    return (i);
-	  else if (old == -1)
-	    old = i;
-	}
+	if (! Context->hdrs[Context->v2r[i]]->old)
+	  return (i);
+	else if (old == -1)
+	  old = i;
       }
-      if (old != -1)
-	return (old);
     }
+    if (old != -1)
+      return (old);
+
     /* If Sort is reverse and not threaded, the latest message is first.
      * If Sort is threaded, the latest message is first iff exactly one
      * of Sort and SortAux are reverse.
@@ -165,13 +167,25 @@ static int mx_toggle_write (CONTEXT *ctx)
   return 0;
 }
 
+struct mapping_t IndexHelp[] = {
+  { "Quit",  OP_QUIT },
+  { "Del",   OP_DELETE },
+  { "Undel", OP_UNDELETE },
+  { "Save",  OP_SAVE },
+  { "Mail",  OP_MAIL },
+  { "Reply", OP_REPLY },
+  { "Group", OP_GROUP_REPLY },
+  { "Help",  OP_HELP },
+  { NULL }
+};
+
 /* This function handles the message index window as well as commands returned
  * from the pager (MENU_PAGER).
  */
 void mutt_index_menu (void)
 {
   char buf[LONG_STRING], helpstr[SHORT_STRING];
-  int op = -1;                 /* function to execute */
+  int op = OP_NULL;                 /* function to execute */
   int done = 0;                /* controls when to exit the "event" loop */
   int i = 0, j;
   int tag = 0;                 /* has the tag-prefix command been pressed? */
@@ -180,7 +194,8 @@ void mutt_index_menu (void)
   int rc = -1;
   MUTTMENU *menu;
   char *cp;                    /* temporary variable. */
-  unsigned short index_hint;   /* used to restore cursor position */
+  int index_hint;   /* used to restore cursor position */
+  int do_buffy_notify = 1;
 
   menu = mutt_new_menu ();
   menu->menu = MENU_MAIN;
@@ -188,32 +203,15 @@ void mutt_index_menu (void)
   menu->pagelen = LINES - 3;
   menu->make_entry = index_make_entry;
   menu->current = ci_first_message ();
-
-  helpstr[0] = 0;
-  mutt_make_help (buf, sizeof (buf), "Quit  ", MENU_MAIN, OP_QUIT);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Del  ", MENU_MAIN, OP_DELETE);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Undel  ", MENU_MAIN, OP_UNDELETE);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Save  ", MENU_MAIN, OP_SAVE);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Mail  ", MENU_MAIN, OP_MAIL);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Reply  ", MENU_MAIN, OP_REPLY);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Group  ", MENU_MAIN, OP_GROUP_REPLY);
-  strcat (helpstr, buf);
-  mutt_make_help (buf, sizeof (buf), "Help", MENU_MAIN, OP_HELP);
-  strcat (helpstr, buf);
-  menu->help = helpstr;
+  menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_MAIN, IndexHelp);
+  
+  mutt_buffy_check(1); /* force the buffy check after we enter the folder */
 
   FOREVER
   {
     tag = 0; /* clear the tag-prefix */
 
     menu->max = Context ? Context->vcount : 0;
-
     oldcount = Context ? Context->msgcount : 0;
 
     if (Context)
@@ -224,7 +222,7 @@ void mutt_index_menu (void)
        */
       index_hint = (Context->vcount) ? CURHDR->index : 0;
 
-      if ((i = _mx_check_mailbox (Context, &index_hint)) < 0)
+      if ((i = mx_check_mailbox (Context, &index_hint)) < 0)
       {
 	if (!Context->path)
 	{
@@ -235,7 +233,7 @@ void mutt_index_menu (void)
 
 	set_option (OPTSEARCHINVALID);
       }
-      else if (i > 0)
+      else if (i == M_NEW_MAIL ||  i == M_REOPENED)
       {
 	/* take note of the current message */
 	if (oldcount)
@@ -246,7 +244,8 @@ void mutt_index_menu (void)
 	    oldcount = 0; /* invalid message number! */
 	}
 
-	mutt_sort_headers (Context);
+	/* if the mailbox was reopened, need to rethread from scratch */
+	mutt_sort_headers (Context, (i == M_REOPENED));
 
 	menu->current = -1;
 	if (oldcount)
@@ -266,10 +265,16 @@ void mutt_index_menu (void)
 	  menu->current = ci_first_message ();
 
 	/* notify the user of new mail */
-	if (i == 2)
-	  mutt_error ("Mailbox was externally modified. Flags may be wrong.");
+	if (i == M_REOPENED)
+	  mutt_error ("Mailbox was externally modified.  Flags may be wrong.");
 	else
-	  mutt_message ("New mail in this folder.");
+	{
+	  mutt_message ("New mail in this mailbox.");
+	  if (option (OPTBEEPNEW))
+	    beep ();
+	}
+	/* avoid the message being overwritten by buffy */
+	do_buffy_notify = 0;
 
 	menu->redraw = REDRAW_FULL;
 	menu->max = Context->vcount;
@@ -280,8 +285,15 @@ void mutt_index_menu (void)
 
     /* check for new mail in the incoming folders */
     oldcount = newcount;
-    if ((newcount = mutt_buffy_check ()) != oldcount)
+    if ((newcount = mutt_buffy_check (0)) != oldcount)
       menu->redraw |= REDRAW_STATUS;
+    if (do_buffy_notify)
+    {
+      if (mutt_buffy_notify () && option (OPTBEEPNEW))
+	beep ();
+    }
+    else
+      do_buffy_notify = 1;
 
     mutt_curs_set (0);
 
@@ -291,39 +303,33 @@ void mutt_index_menu (void)
       mutt_show_error ();
     }
 
-    if (menu->redraw & REDRAW_STATUS)
-    {
-      mutt_status_line (buf, sizeof (buf), Status);
-      CLEARLINE (option (OPTSTATUSONTOP) ? 0 : LINES-2);
-      SETCOLOR (MT_COLOR_STATUS);
-      printw ("%-*.*s", COLS, COLS, buf);
-      SETCOLOR (MT_COLOR_NORMAL);
-      menu->redraw &= ~REDRAW_STATUS;
-    }
-
     if (menu->menu == MENU_MAIN)
     {
       if (Context)
       {
-	if (menu->current >= menu->top + menu->pagelen)
-	{
-	  while (menu->top + menu->pagelen  - 1 < menu->current) menu->top += menu->pagelen;
-	  menu->redraw = REDRAW_INDEX;
-	}
-	else if (menu->current < menu->top)
-	{
-	  while (menu->top > menu->current) menu->top -= menu->pagelen;
-	  if (menu->top < 0) menu->top = 0;
-	  menu->redraw = REDRAW_INDEX;
-	}
+	menu_check_recenter (menu);
 
-	if (menu->redraw == REDRAW_INDEX)
+	if (menu->redraw & REDRAW_INDEX)
+	{
 	  menu_redraw_index (menu);
+	  menu->redraw |= REDRAW_STATUS;
+	}
 	else if (menu->redraw & (REDRAW_MOTION_RESYNCH | REDRAW_MOTION))
 	  menu_redraw_motion (menu);
-	else if (menu->redraw == REDRAW_CURRENT)
+	else if (menu->redraw & REDRAW_CURRENT)
 	  menu_redraw_current (menu);
       }
+
+      if (menu->redraw & REDRAW_STATUS) 
+      {
+	menu_status_line (buf, sizeof (buf), menu, Status);
+	CLEARLINE (option (OPTSTATUSONTOP) ? 0 : LINES-2);
+	SETCOLOR (MT_COLOR_STATUS);
+	printw ("%-*.*s", COLS, COLS, buf);
+	SETCOLOR (MT_COLOR_NORMAL);
+	menu->redraw &= ~REDRAW_STATUS;
+      }
+
       menu->redraw = 0;
       menu->oldcurrent = menu->current;
 
@@ -331,14 +337,22 @@ void mutt_index_menu (void)
 	move (menu->current - menu->top + menu->offset, 2);
       else
 	move (menu->current - menu->top + menu->offset, COLS - 1);
-      refresh ();
+      mutt_refresh ();
 
-      if (Timeout)
+      if (Timeout > 0)
+      {
 	timeout (Timeout * 1000); /* milliseconds */      
-      op = km_dokey (MENU_MAIN);
-      mutt_curs_set (1);
-      if (Timeout)
+	op = mutt_getch ();
 	timeout (-1); /* restore blocking operation */
+	if (op != -1)
+	{
+	  mutt_ungetch (op);
+	  op = km_dokey (MENU_MAIN);
+	}
+      }
+      else
+	op = km_dokey (MENU_MAIN);
+      mutt_curs_set (1);
 
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
       if (Signals & S_SIGWINCH)
@@ -357,7 +371,7 @@ void mutt_index_menu (void)
 	continue; /* either user abort or timeout */
 
       /* special handling for the tag-prefix function */
-      if (op == OP_MAIN_TAG_PREFIX)
+      if (op == OP_TAG_PREFIX)
       {
 	if (!Context)
 	{
@@ -377,7 +391,7 @@ void mutt_index_menu (void)
 	clrtoeol ();
 
 	/* get the real command */
-	if ((op = km_dokey (MENU_MAIN)) == OP_MAIN_TAG_PREFIX)
+	if ((op = km_dokey (MENU_MAIN)) == OP_TAG_PREFIX)
 	{
 	  /* abort tag sequence */
 	  CLEARLINE (LINES-1);
@@ -400,67 +414,54 @@ void mutt_index_menu (void)
        */
 
       case OP_BOTTOM_PAGE:
-
-	CHECK_MSGCOUNT;
 	menu_bottom_page (menu);
 	break;
-
       case OP_FIRST_ENTRY:
-
-	CHECK_MSGCOUNT;
 	menu_first_entry (menu);
 	break;
-
       case OP_MIDDLE_PAGE:
-
-	CHECK_MSGCOUNT;
 	menu_middle_page (menu);
 	break;
-
+      case OP_HALF_UP:
+	menu_half_up (menu);
+	break;
+      case OP_HALF_DOWN:
+	menu_half_down (menu);
+	break;
       case OP_NEXT_LINE:
-	
-	CHECK_MSGCOUNT;
-	
 	menu_next_line (menu);
 	break;
-	
       case OP_PREV_LINE:
-	
-	CHECK_MSGCOUNT;
-	
 	menu_prev_line (menu);
 	break;
-	
       case OP_NEXT_PAGE:
-
-	CHECK_MSGCOUNT;
 	menu_next_page (menu);
 	break;
-
       case OP_PREV_PAGE:
-
-	CHECK_MSGCOUNT;
 	menu_prev_page (menu);
 	break;
-
       case OP_LAST_ENTRY:
-
-	CHECK_MSGCOUNT;
 	menu_last_entry (menu);
 	break;
-
       case OP_TOP_PAGE:
-
-	CHECK_MSGCOUNT;
 	menu_top_page (menu);
 	break;
+      case OP_CURRENT_TOP:
+	menu_current_top (menu);
+	break;
+      case OP_CURRENT_MIDDLE:
+	menu_current_middle (menu);
+	break;
+      case OP_CURRENT_BOTTOM:
+	menu_current_bottom (menu);
+	break;
 
       case OP_JUMP:
 
 	CHECK_MSGCOUNT;
 	mutt_ungetch (LastKey);
 	buf[0] = 0;
-	if (ci_get_field ("Jump to message: ", buf, sizeof (buf), 0) != 0 ||
+	if (mutt_get_field ("Jump to message: ", buf, sizeof (buf), 0) != 0 ||
 	    !buf[0])
 	  break;
 
@@ -573,6 +574,7 @@ void mutt_index_menu (void)
       case OP_SEARCH:
       case OP_SEARCH_REVERSE:
       case OP_SEARCH_NEXT:
+      case OP_SEARCH_OPPOSITE:
 
 	CHECK_MSGCOUNT;
 	if ((menu->current = mutt_search_command (menu->current, op)) == -1)
@@ -589,7 +591,7 @@ void mutt_index_menu (void)
 	  if (Context && Context->msgcount)
 	  {
 	    menu->oldcurrent = CURHDR->index;
-	    mutt_sort_headers (Context);
+	    mutt_sort_headers (Context, 0);
 
 	    /* try to restore the current message */
 	    for (i = 0; i < Context->vcount; i++)
@@ -637,7 +639,7 @@ void mutt_index_menu (void)
 	menu->redraw = REDRAW_FULL;
 	break;
 
-      case OP_TAG_MESSAGE:
+      case OP_TAG:
 
 	CHECK_MSGCOUNT;
 	if (tag && !option (OPTAUTOTAG))
@@ -682,19 +684,6 @@ void mutt_index_menu (void)
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	break;
 
-      case OP_MAIN_SEARCH_BODY:
-
-	CHECK_MSGCOUNT;
-	if (ci_get_field ("Search messages for: ", LastSearch, sizeof (LastSearch), M_CLEAR) == -1
-	    || !LastSearch[0])
-	  break;
-
-	if ((menu->current = mutt_body_search (menu->current, LastSearch)) == -1)
-	  menu->current = menu->oldcurrent;
-	else
-	  menu->redraw = REDRAW_MOTION;
-	break;
-
 	/* --------------------------------------------------------------------
 	 * The following operations can be performed inside of the pager.
 	 */
@@ -716,7 +705,7 @@ void mutt_index_menu (void)
 	buf[0] = '\0';
 	mutt_buffy (buf);
 
-	if (mutt_enter_fname (cp, buf, sizeof (buf), &menu->redraw) == -1)
+	if (mutt_enter_fname (cp, buf, sizeof (buf), &menu->redraw, 1) == -1)
 	  break;
 	if (!buf[0])
 	{
@@ -758,6 +747,8 @@ void mutt_index_menu (void)
 	  menu->current = 0;
 
 	mutt_clear_error ();
+	mutt_buffy_check(1); /* force the buffy check after we have changed
+			      the folder */
 	menu->redraw = REDRAW_FULL;
 	set_option (OPTSEARCHINVALID);
 	break;
@@ -771,15 +762,10 @@ void mutt_index_menu (void)
 	 * again while reading the message.
 	 */
 	if (op == OP_DISPLAY_HEADERS)
-	{
-	  if (option (OPTWEED))
-	    unset_option (OPTWEED);
-	  else
-	    set_option (OPTWEED);
-	}
+	  toggle_option (OPTWEED);
 
 	unset_option (OPTNEEDRESORT);
-	
+
 	if ((op = mutt_display_message (CURHDR)) == -1)
 	{
 	  unset_option (OPTNEEDRESORT);
@@ -789,7 +775,7 @@ void mutt_index_menu (void)
 	if (option (OPTNEEDRESORT) && Context && Context->msgcount)
 	{
 	  menu->oldcurrent = CURHDR->index;
-	  mutt_sort_headers (Context);
+	  mutt_sort_headers (Context, 0);
 
 	  /* try to restore the current message */
 	  for (i = 0; i < Context->vcount; i++)
@@ -799,13 +785,12 @@ void mutt_index_menu (void)
 	  }
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	}
-	unset_option (OPTNEEDRESORT);
 
 	menu->menu = MENU_PAGER;
 	menu->oldcurrent = menu->current;
 	continue;
 
-      case OP_MAIN_EXIT:
+      case OP_EXIT:
 
 	if (menu->menu == MENU_MAIN)
 	{
@@ -830,7 +815,8 @@ void mutt_index_menu (void)
 	if ((menu->current = ci_next_undeleted (menu->current)) == -1)
 	{
 	  menu->current = menu->oldcurrent;
-	  if (menu->menu == MENU_MAIN) mutt_error ("No undeleted messages.");
+	  if (menu->menu == MENU_MAIN)
+	    mutt_error ("No undeleted messages.");
 	}
 	else if (menu->menu == MENU_PAGER)
 	{
@@ -846,7 +832,8 @@ void mutt_index_menu (void)
 	CHECK_MSGCOUNT;
 	if (menu->current >= Context->vcount - 1)
 	{
-	  if (menu->menu == MENU_MAIN) mutt_error ("You are on the last message.");
+	  if (menu->menu == MENU_MAIN)
+	    mutt_error ("You are on the last message.");
 	  break;
 	}
 	menu->current++;
@@ -870,7 +857,8 @@ void mutt_index_menu (void)
 	if ((menu->current = ci_previous_undeleted (menu->current)) == -1)
 	{
 	  menu->current = menu->oldcurrent;
-	  if (menu->menu == MENU_MAIN) mutt_error ("No undeleted messages.");
+	  if (menu->menu == MENU_MAIN)
+	    mutt_error ("No undeleted messages.");
 	}
 	else if (menu->menu == MENU_PAGER)
 	{
@@ -905,18 +893,11 @@ void mutt_index_menu (void)
       case OP_DECODE_SAVE:
 
 	CHECK_MSGCOUNT;
-	j = 0;
-	if (op == OP_DECODE_COPY)
-	{
-	  op = OP_COPY_MESSAGE;
-	  j = 1;
-	}
-	else if (op == OP_DECODE_SAVE)
-	{
-	  op = OP_SAVE;
-	  j = 1;
-	}
-	if (mutt_save_message (tag ? NULL : CURHDR, (op == OP_SAVE), j, &menu->redraw) == 0 && op == OP_SAVE)
+	if (mutt_save_message (tag ? NULL : CURHDR,
+			       (op == OP_SAVE || op == OP_DECODE_SAVE),
+			       (op == OP_DECODE_SAVE || op == OP_DECODE_COPY),
+			       &menu->redraw) == 0 &&
+	    (op == OP_SAVE || op == OP_DECODE_SAVE))
 	{
 	  if (tag)
 	    menu->redraw |= REDRAW_INDEX;
@@ -1143,16 +1124,10 @@ void mutt_index_menu (void)
 	ci_bounce_message (tag ? NULL : CURHDR, &menu->redraw);
 	break;
 
-      case OP_BROWSE_URL:
-
-	CHECK_MSGCOUNT;
-	mutt_browse_url (CURHDR);
-	menu->redraw = REDRAW_FULL;
-	break;
-
       case OP_CREATE_ALIAS:
 
-	mutt_create_alias (Context && Context->msgcount ? CURHDR->env : NULL);
+	mutt_create_alias (Context && Context->vcount ? CURHDR->env : NULL);
+	MAYBE_REDRAW (menu->redraw);
 	break;
 
       case OP_DELETE:
@@ -1194,10 +1169,8 @@ void mutt_index_menu (void)
 	CHECK_MSGCOUNT;
 	CHECK_READONLY;
 
-	if (op == OP_DELETE_THREAD)
-	  rc = mutt_thread_set_flag (CURHDR, M_DELETE, 1);
-	else
-	  rc = mutt_subthread_set_flag (CURHDR, M_DELETE, 1);
+	rc = mutt_thread_set_flag (CURHDR, M_DELETE, 1,
+				   op == OP_DELETE_THREAD ? 0 : 1);
 
 	if (rc != -1)
 	{
@@ -1216,14 +1189,12 @@ void mutt_index_menu (void)
 
       case OP_ENTER_COMMAND:
 
-	unset_option (OPTNEEDRESORT);
-
 	mutt_enter_command ();
 
 	if (option (OPTNEEDRESORT) && Context && Context->msgcount)
 	{
 	  menu->oldcurrent = CURHDR->index;
-	  mutt_sort_headers (Context);
+	  mutt_sort_headers (Context, 0);
 
 	  /* try to restore the current message */
 	  for (i = 0; i < Context->vcount; i++)
@@ -1238,44 +1209,38 @@ void mutt_index_menu (void)
 	  menu->redraw = REDRAW_FULL;
 	unset_option (OPTFORCEREDRAWINDEX);
 	unset_option (OPTFORCEREDRAWPAGER);
-	unset_option (OPTNEEDRESORT);
 	
 	break;
 
       case OP_FORWARD_MESSAGE:
 
 	CHECK_MSGCOUNT;
-	ci_send_message (SENDFORWARD, NULL, NULL, tag ? NULL : CURHDR);
+	ci_send_message (SENDFORWARD, NULL, NULL, Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
 
-#ifdef _PGPPATH
-      case OP_FORGET_PASSPHRASE:
-
-	mutt_forget_passphrase ();
-	break;
-#endif /* _PGPPATH */
 
       case OP_GROUP_REPLY:
 
 	CHECK_MSGCOUNT;
-	ci_send_message (SENDREPLY|SENDGROUPREPLY, NULL, NULL, tag ? NULL : CURHDR);
+	ci_send_message (SENDREPLY|SENDGROUPREPLY, NULL, NULL, Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_LIST_REPLY:
 
 	CHECK_MSGCOUNT;
-	ci_send_message (SENDREPLY|SENDLISTREPLY, NULL, NULL, tag ? NULL : CURHDR);
+	ci_send_message (SENDREPLY|SENDLISTREPLY, NULL, NULL, Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_MAIL:
 
-	ci_send_message (0, NULL, NULL, NULL);
+	ci_send_message (0, NULL, NULL, NULL, NULL);
 	menu->redraw = REDRAW_FULL;
 	break;
 
+
       case OP_PIPE:
 
 	CHECK_MSGCOUNT;
@@ -1294,10 +1259,8 @@ void mutt_index_menu (void)
 	CHECK_MSGCOUNT;
 	CHECK_READONLY;
 
-	if (op == OP_MAIN_READ_THREAD)
-	  rc = mutt_thread_set_flag (CURHDR, M_READ, 1);
-	else
-	  rc = mutt_subthread_set_flag (CURHDR, M_READ, 1);
+	rc = mutt_thread_set_flag (CURHDR, M_READ, 1,
+				   op == OP_MAIN_READ_THREAD ? 0 : 1);
 
 	if (rc != -1)
 	{
@@ -1312,14 +1275,14 @@ void mutt_index_menu (void)
 
       case OP_RECALL_MESSAGE:
 
-	ci_send_message (SENDPOSTPONED, NULL, NULL, NULL);
+	ci_send_message (SENDPOSTPONED, NULL, NULL, Context, NULL);
 	menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_REPLY:
 
 	CHECK_MSGCOUNT;
-	ci_send_message (SENDREPLY, NULL, NULL, tag ? NULL : CURHDR);
+	ci_send_message (SENDREPLY, NULL, NULL, Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
 
@@ -1329,6 +1292,26 @@ void mutt_index_menu (void)
 	MAYBE_REDRAW (menu->redraw);
 	break;
 
+      case OP_TAG_THREAD:
+      case OP_TAG_SUBTHREAD:
+
+	CHECK_MSGCOUNT;
+	rc = mutt_thread_set_flag (CURHDR, M_TAG, !CURHDR->tagged,
+				   op == OP_TAG_THREAD ? 0 : 1);
+	
+	if (rc != -1)
+	{
+	  if (option (OPTRESOLVE))
+	  {
+	    menu->current = mutt_next_thread (CURHDR);
+
+	    if (menu->current == -1)
+	      menu->current = menu->oldcurrent;
+	  }
+	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	}
+	break;
+
       case OP_UNDELETE:
 
 	CHECK_MSGCOUNT;
@@ -1358,22 +1341,8 @@ void mutt_index_menu (void)
 	CHECK_MSGCOUNT;
 	CHECK_READONLY;
 
-	if ((Sort & SORT_MASK) != SORT_THREADS)
-	{
-	  mutt_error ("Threading is not enabled.");
-	  break;
-	}
-
-	if (op == OP_UNDELETE_THREAD)
-	  rc = mutt_thread_set_flag (CURHDR, M_DELETE, 0);
-	else
-	{
-	  HEADER *hdr = CURHDR;
-
-	  while (hdr->parent && hdr->parent->deleted)
-	    hdr = hdr->parent;
-	  rc = mutt_subthread_set_flag (hdr, M_DELETE, 0);
-	}
+	rc = mutt_thread_set_flag (CURHDR, M_DELETE, 0,
+				   op == OP_UNDELETE_THREAD ? 0 : 1);
 
 	if (rc != -1)
 	{
@@ -1392,19 +1361,16 @@ void mutt_index_menu (void)
 	break;
 
       case OP_VERSION:
-
-	mutt_message (MuttVersion);
+	mutt_version ();
 	break;
 
       case OP_VIEW_ATTACHMENTS:
-
 	CHECK_MSGCOUNT;
 	mutt_view_attachments (CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
 
       default:
-
 	if (menu->menu == MENU_MAIN)
 	  km_error_key (MENU_MAIN);
     }
