@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -18,9 +18,14 @@
 
 #include "mutt.h"
 #include "mutt_curses.h"
+#include "sort.h"
 
 void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
 {
+  int changed = h->changed;
+  int deleted = ctx->deleted;
+  int tagged = ctx->tagged;
+
   if (ctx->readonly && flag != M_TAG)
     return; /* don't modify anything if we are read-only */
 
@@ -47,9 +52,13 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
       {
 	if (h->read || h->old)
 	{
-	  h->read = 0;
 	  h->old = 0;
 	  ctx->new++;
+	  if (h->read)
+	  {
+	    h->read = 0;
+	    ctx->unread++;
+	  }
 	  h->changed = 1;
 	  ctx->changed = 1;
 	}
@@ -59,6 +68,7 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
 	if (!h->old)
 	  ctx->new--;
 	h->read = 1;
+	ctx->unread--;
 	h->changed = 1;
 	ctx->changed = 1;
       }
@@ -92,6 +102,7 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
 	if (!h->read)
 	{
 	  h->read = 1;
+	  ctx->unread--;
 	  if (!h->old)
 	    ctx->new--;
 	  h->changed = 1;
@@ -101,6 +112,7 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
       else if (h->read)
       {
 	h->read = 0;
+	ctx->unread++;
 	if (!h->old)
 	  ctx->new++;
 	h->changed = 1;
@@ -117,6 +129,7 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
 	  if (!h->read)
 	  {
 	    h->read = 1;
+	    ctx->unread--;
 	    if (!h->old)
 	      ctx->new--;
 	  }
@@ -133,7 +146,6 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
       break;
 
     case M_FLAG:
-
       if (bf)
       {
 	if (!h->flagged)
@@ -154,7 +166,6 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
       break;
 
     case M_TAG:
-
       if (bf)
       {
 	if (!h->tagged)
@@ -170,6 +181,13 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
       }
       break;
   }
+
+  /* if the message status has changed, we need to invalidate the cached
+   * search results so that any future search will match the current status
+   * of this message and not what it was at the time it was last searched.
+   */
+  if (h->searched && (changed != h->changed || deleted != ctx->deleted || tagged != ctx->tagged))
+    h->searched = 0;
 }
 
 void mutt_tag_set_flag (int flag, int bf)
@@ -181,42 +199,43 @@ void mutt_tag_set_flag (int flag, int bf)
       mutt_set_flag (Context, Context->hdrs[Context->v2r[j]], flag, bf);
 }
 
-static void thread_set_flag_r (HEADER *h, int flag, int bf)
-{
-  for (; h; h = h->next)
-  {
-    mutt_set_flag (Context, h, flag, bf);
-    if (h->child)
-      thread_set_flag_r (h->child, flag, bf);
-  }
-}
-
-int mutt_thread_set_flag (HEADER *cur, int flag, int bf)
+int mutt_thread_set_flag (HEADER *cur, int flag, int bf, int subthread)
 {
+  HEADER *start;
+  
   if ((Sort & SORT_MASK) != SORT_THREADS)
   {
     mutt_error ("Threading is not enabled.");
     return (-1);
   }
 
-  while (cur->parent)
-    cur = cur->parent;
+  if (!subthread)
+    while (cur->parent)
+      cur = cur->parent;
+  start = cur;
+  
   mutt_set_flag (Context, cur, flag, bf);
-  thread_set_flag_r (cur->child, flag, bf);
-  return (0);
-}
-
-int mutt_subthread_set_flag (HEADER *cur, int flag, int bf)
-{
-  if ((Sort & SORT_MASK) != SORT_THREADS)
+  if ((cur = cur->child) == NULL)
+    return (0);
+  FOREVER
   {
-    mutt_error ("Threading is not enabled.");
-    return (-1);
+    mutt_set_flag (Context, cur, flag, bf);
+    if (cur->child)
+      cur = cur->child;
+    else if (cur->next)
+      cur = cur->next;
+    else 
+    {
+      while (!cur->next)
+      {
+	cur = cur->parent;
+	if (cur == start)
+	  return (0);
+      }
+      cur = cur->next;
+    }
   }
-
-  mutt_set_flag (Context, cur, flag, bf);
-  thread_set_flag_r (cur->child, flag, bf);
-  return (0);
+  /* not reached */
 }
 
 int mutt_change_flag (HEADER *h, int bf)
@@ -226,7 +245,7 @@ int mutt_change_flag (HEADER *h, int bf)
   mvprintw (LINES - 1, 0, "%s flag? (D/N/O/r/*/!): ", bf ? "Set" : "Clear");
   clrtoeol ();
 
-  if ((i = ci_getch ()) == ERR)
+  if ((i = mutt_getch ()) == ERR)
   {
     CLEARLINE (LINES-1);
     return (-1);
