@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -68,21 +68,45 @@ time_t is_from (const char *s, char *path, size_t pathlen)
     return 0;
 
   s = next_word (s); /* skip over the From part. */
-  if (!*s) return 0;
+  if (!*s)
+    return 0;
 
   dprint (3, (debugfile, "\nis_from(): parsing: %s", s));
 
   if (!is_day_name (s))
   {
-    char *p;
+    const char *p;
     size_t len;
 
     /* looks like we got the return-path, so extract it  */
-    if ((p = strchr (s, ' ')) == 0) return 0;
-    len = (size_t)(p-s) + 2;
-    if (len > pathlen) len = pathlen;
-    strfcpy(path, s, len);
-    path[len-1] = 0;
+    if (*s == '"')
+    {
+      /* sometimes we see bogus addresses like
+       *	From "/foo/bar baz/"@dumbdar.com Sat Nov 22 15:29:32 PST 1997
+       */
+      p = s;
+      p++; /* skip over the quote */
+      do
+      {
+	if (!(p = strpbrk (p, "\\\"")))
+	  return 0;
+	if (*p == '\\')
+	  p += 2;
+      }
+      while (*p != '"');
+      while (*p && !ISSPACE (*p))
+	p++;
+    }
+    else
+    {
+      if ((p = strchr (s, ' ')) == NULL)
+	return 0;
+    }
+    len = (size_t) (p - s);
+    if (len + 1 > pathlen)
+      len = pathlen - 1;
+    memcpy (path, s, len);
+    path[len] = 0;
 
     s = p + 1;
     SKIPWS (s);
@@ -159,5 +183,5 @@ time_t is_from (const char *s, char *path, size_t pathlen)
   tm.tm_yday = 0;
   tm.tm_wday = 0;
 
-  return (mktime (&tm));
+  return (mutt_mktime (&tm, 0));
 }
