@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -31,31 +31,41 @@
  */
 
 struct binding_t OpGeneric[] = {
-  { "select-entry",	OP_GENERIC_SELECT_ENTRY,M_ENTER_S },
+  { "top-page",		OP_TOP_PAGE,		"H" },
   { "next-entry",	OP_NEXT_ENTRY,		"j" },
   { "previous-entry",	OP_PREV_ENTRY,		"k" },
-  { "last-entry",	OP_LAST_ENTRY,		"*" },
-  { "first-entry",	OP_FIRST_ENTRY,		"=" },
+  { "bottom-page",	OP_BOTTOM_PAGE,		"L" },
+  { "refresh",		OP_REDRAW,		"\014" },
+  { "middle-page",	OP_MIDDLE_PAGE,		"M" },
+  { "search-next",	OP_SEARCH_NEXT,		"n" },
+  { "exit",		OP_EXIT,		"q" },
+  { "tag-entry",	OP_TAG,			"t" },
   { "next-page",	OP_NEXT_PAGE,		"z" },
   { "previous-page",	OP_PREV_PAGE,		"Z" },
-  { "help",		OP_HELP,		"?" },
-  { "refresh",		OP_REDRAW,		"\014" },
-  { "top-page",		OP_TOP_PAGE,		"H" },
-  { "middle-page",	OP_MIDDLE_PAGE,		"M" },
-  { "bottom-page",	OP_BOTTOM_PAGE,		"L" },
-  { "exit",		OP_EXIT,		"q" },
+  { "last-entry",	OP_LAST_ENTRY,		"*" },
+  { "first-entry",	OP_FIRST_ENTRY,		"=" },
   { "enter-command",	OP_ENTER_COMMAND,	":" },
   { "next-line",	OP_NEXT_LINE,		">" },
   { "previous-line",	OP_PREV_LINE,		"<" },
+  { "half-up",		OP_HALF_UP,		"[" },
+  { "half-down", 	OP_HALF_DOWN,		"]" },
+  { "help",		OP_HELP,		"?" },
+  { "tag-prefix",	OP_TAG_PREFIX,		";" },
+  { "shell-escape",	OP_SHELL_ESCAPE,	"!" },
+  { "select-entry",	OP_GENERIC_SELECT_ENTRY,M_ENTER_S },
+  { "search",		OP_SEARCH,		"/" },
+  { "search-reverse",	OP_SEARCH_REVERSE,	"\033/" },
+  { "search-opposite",	OP_SEARCH_OPPOSITE,	NULL },
   { "jump",		OP_JUMP,		NULL },
+  { "current-top",      OP_CURRENT_TOP,		NULL },
+  { "current-middle",   OP_CURRENT_MIDDLE,	NULL },
+  { "current-bottom",   OP_CURRENT_BOTTOM,	NULL },
   { NULL,		0,			NULL }
 };
 
 struct binding_t OpMain[] = {
   { "create-alias",		OP_CREATE_ALIAS,		"a" },
   { "bounce-message",		OP_BOUNCE_MESSAGE,		"b" },
-  { "bottom-page",		OP_BOTTOM_PAGE,			"B" },
-  { "browse-url",		OP_BROWSE_URL,			"\002" },
   { "change-folder",		OP_MAIN_CHANGE_FOLDER,		"c" },
   { "change-folder-readonly",	OP_MAIN_CHANGE_FOLDER_READONLY,	"\033c" },
   { "copy-message",		OP_COPY_MESSAGE,		"C" },
@@ -67,27 +77,18 @@ struct binding_t OpMain[] = {
   { "delete-subthread",		OP_DELETE_SUBTHREAD,		"\033d" },
   { "forward-message",		OP_FORWARD_MESSAGE,		"f" },
   { "flag-message",		OP_FLAG_MESSAGE,		"F" },
-#ifdef _PGPPATH
-  { "forget-passphrase",	OP_FORGET_PASSPHRASE,		"\006" },
-#endif
   { "group-reply",		OP_GROUP_REPLY,			"g" },
 #ifdef USE_POP
   { "fetch-mail",		OP_MAIN_FETCH_MAIL,		"G" },
 #endif
   { "display-headers",		OP_DISPLAY_HEADERS,		"h" },
-  { "top-page",			OP_TOP_PAGE,			"H" },
   { "next-undeleted",		OP_MAIN_NEXT_UNDELETED,		"j" },
-  { "next-message",		OP_NEXT_ENTRY,			"J" },
   { "previous-undeleted",	OP_MAIN_PREV_UNDELETED,		"k" },
-  { "previous-message",		OP_PREV_ENTRY,			"K" },
   { "limit",			OP_MAIN_LIMIT,			"l" },
-  { "redraw-screen",		OP_REDRAW,			"\014" },
   { "list-reply",		OP_LIST_REPLY,			"L" },
   { "mail",			OP_MAIL,			"m" },
-  { "middle-page",		OP_MIDDLE_PAGE,			"M" },
   { "toggle-new",		OP_TOGGLE_NEW,			"N" },
   { "toggle-write",		OP_TOGGLE_WRITE,		"%" },
-  { "search-next",		OP_SEARCH_NEXT,			"n" },
   { "next-thread",		OP_MAIN_NEXT_THREAD,		"\016" },
   { "next-subthread",		OP_MAIN_NEXT_SUBTHREAD,		"\033n" },
   { "quit",			OP_QUIT,			"q" },
@@ -101,9 +102,9 @@ struct binding_t OpMain[] = {
   { "read-thread",		OP_MAIN_READ_THREAD,		"\022" },
   { "read-subthread",		OP_MAIN_READ_SUBTHREAD,		"\033r" },
   { "save-message",		OP_SAVE,			"s" },
-  { "search-body",		OP_MAIN_SEARCH_BODY,		"S" },
-  { "tag-message",		OP_TAG_MESSAGE,			"t" },
   { "tag-pattern",		OP_MAIN_TAG_PATTERN,		"T" },
+  { "tag-subthread",		OP_TAG_SUBTHREAD,		NULL },
+  { "tag-thread",		OP_TAG_THREAD,			"\033t" },
   { "untag-pattern",		OP_MAIN_UNTAG_PATTERN,		"\024" },
   { "undelete-message",		OP_UNDELETE,			"u" },
   { "undelete-pattern",		OP_MAIN_UNDELETE_PATTERN,	"U"},
@@ -113,60 +114,47 @@ struct binding_t OpMain[] = {
   { "show-version",		OP_VERSION,			"V" },
   { "set-flag",			OP_MAIN_SET_FLAG,		"w" },
   { "clear-flag",		OP_MAIN_CLEAR_FLAG,		"W" },
-  { "exit",			OP_MAIN_EXIT,			"x" },
-  { "next-page",		OP_NEXT_PAGE,			"z" },
-  { "previous-page",		OP_PREV_PAGE,			"Z" },
   { "display-message",		OP_DISPLAY_MESSAGE,		M_ENTER_S },
-  { "first-message",		OP_FIRST_ENTRY,			"=" },
-  { "last-message",		OP_LAST_ENTRY,			"*" },
-  { "enter-command",		OP_ENTER_COMMAND,		":" },
   { "sync-mailbox",		OP_MAIN_SYNC_FOLDER,		"$" },
-  { "tag-prefix",		OP_MAIN_TAG_PREFIX,		";" },
-  { "search",			OP_SEARCH,			"/" },
-  { "search-reverse",		OP_SEARCH_REVERSE,		"\033/" },
-  { "help",			OP_HELP,			"?" },
-  { "shell-escape",		OP_SHELL_ESCAPE,		"!" },
   { "display-address",		OP_DISPLAY_ADDRESS,		"@" },
   { "pipe-message",		OP_PIPE,			"|" },
   { "next-new",			OP_MAIN_NEXT_NEW,		"\t" },
   { "previous-new",		OP_MAIN_PREV_NEW,		"\033\t" },
-  { "next-line",        	OP_NEXT_LINE,           	">" },
-  { "previous-line",    	OP_PREV_LINE,           	"<" },
-  { "jump",			OP_JUMP,			NULL },
   { "next-unread",		OP_MAIN_NEXT_UNREAD,		NULL },
   { "previous-unread",		OP_MAIN_PREV_UNREAD,		NULL },
+
+
+
+
+
+
   { NULL,			0,				NULL }
 };
 
 struct binding_t OpPager[] = {
   { "create-alias",	OP_CREATE_ALIAS,		"a" },
   { "bounce-message",	OP_BOUNCE_MESSAGE,		"b" },
-  { "browse-url",	OP_BROWSE_URL,			"\002" },
   { "change-folder",	OP_MAIN_CHANGE_FOLDER,		"c" },
   { "change-folder-readonly",	OP_MAIN_CHANGE_FOLDER_READONLY,	"\033c" },
   { "copy-message",	OP_COPY_MESSAGE,		"C" },
   { "decode-copy",	OP_DECODE_COPY,			"\033C" },
-  { "decode-save",	OP_DECODE_SAVE,			"\033s" },
   { "delete-message",	OP_DELETE,			"d" },
   { "delete-thread",	OP_DELETE_THREAD,		"\004" },
   { "delete-subthread",	OP_DELETE_SUBTHREAD,		"\033d" },
   { "forward-message",	OP_FORWARD_MESSAGE,		"f" },
   { "flag-message",	OP_FLAG_MESSAGE,		"F" },
-#ifdef _PGPPATH
-  { "forget-passphrase",OP_FORGET_PASSPHRASE,		"\006" },
-#endif
   { "group-reply",	OP_GROUP_REPLY,			"g" },
   { "display-headers",	OP_DISPLAY_HEADERS,		"h" },
   { "exit",		OP_PAGER_EXIT,			"i" },
   { "next-undeleted",	OP_MAIN_NEXT_UNDELETED,		"j" },
-  { "next-message",	OP_NEXT_ENTRY,			"J" },
+  { "next-entry",	OP_NEXT_ENTRY,			"J" },
   { "previous-undeleted",OP_MAIN_PREV_UNDELETED,	"k" },
-  { "previous-message",	OP_PREV_ENTRY,			"K" },
+  { "previous-entry",	OP_PREV_ENTRY,			"K" },
   { "list-reply",	OP_LIST_REPLY,			"L" },
   { "redraw-screen",	OP_REDRAW,			"\014" },
   { "mail",		OP_MAIL,			"m" },
-  { "search-next",      OP_SEARCH_NEXT,         	"n" },
   { "mark-as-new",	OP_TOGGLE_NEW,			"N" },
+  { "search-next",	OP_SEARCH_NEXT,			"n" },
   { "next-thread",	OP_MAIN_NEXT_THREAD,		"\016" },
   { "next-subthread",	OP_MAIN_NEXT_SUBTHREAD,		"\033n" },
   { "print-message",	OP_PRINT,			"p" },
@@ -178,7 +166,9 @@ struct binding_t OpPager[] = {
   { "read-thread",	OP_MAIN_READ_THREAD,		"\022" },
   { "read-subthread",	OP_MAIN_READ_SUBTHREAD,		"\033r" },
   { "save-message",	OP_SAVE,			"s" },
-  { "tag-message",	OP_TAG_MESSAGE,			"t" },
+  { "skip-quoted",	OP_PAGER_SKIP_QUOTED,		"S" },
+  { "decode-save",	OP_DECODE_SAVE,			"\033s" },
+  { "tag-message",	OP_TAG,				"t" },
   { "toggle-quoted",	OP_PAGER_HIDE_QUOTED,		"T" },
   { "undelete-message",	OP_UNDELETE,			"u" },
   { "undelete-subthread",OP_UNDELETE_SUBTHREAD,		"\033u" },
@@ -195,24 +185,65 @@ struct binding_t OpPager[] = {
   { "top",		OP_PAGER_TOP,			"^" },
   { "bottom",		OP_PAGER_BOTTOM,		"$" },
   { "shell-escape",	OP_SHELL_ESCAPE,		"!" },
+  { "enter-command",	OP_ENTER_COMMAND,		":" },
   { "search",		OP_SEARCH,			"/" },
   { "search-reverse",	OP_SEARCH_REVERSE,		"\033/" },
-  { "enter-command",	OP_ENTER_COMMAND,		":" },
+  { "search-opposite",	OP_SEARCH_OPPOSITE,		NULL },
   { "next-line",	OP_NEXT_LINE,			M_ENTER_S },
   { "jump",		OP_JUMP,			NULL },
   { "next-unread",	OP_MAIN_NEXT_UNREAD,		NULL },
   { "previous-new",	OP_MAIN_PREV_NEW,		NULL },
   { "previous-unread",	OP_MAIN_PREV_UNREAD,		NULL },
-  { "half-up",		OP_PAGER_HALF_UP,		NULL },
-  { "half-down",	OP_PAGER_HALF_DOWN,		NULL },
+  { "half-up",		OP_HALF_UP,			NULL },
+  { "half-down",	OP_HALF_DOWN,			NULL },
   { "previous-line",	OP_PREV_LINE,			NULL },
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   { NULL,		0,				NULL }
 };
 
 struct binding_t OpAttach[] = {
   { "bounce-message",	OP_BOUNCE_MESSAGE,		"b" },
-  { "decode-copy",	OP_ATTACH_DECODE_COPY,		"\033C" },
-  { "decode-save",	OP_ATTACH_DECODE_SAVE,		"\033s" },
   { "display-headers",	OP_DISPLAY_HEADERS,		"h" },
   { "print-entry",	OP_PRINT,			"p" },
   { "save-entry",	OP_SAVE,			"s" },
@@ -222,9 +253,14 @@ struct binding_t OpAttach[] = {
   { "group-reply",	OP_GROUP_REPLY,			"g" },
   { "list-reply",	OP_LIST_REPLY,			"L" },
   { "forward-message",	OP_FORWARD_MESSAGE,		"f" },
-  { "tag-attachment",	OP_ATTACH_TAG,			"t" },
-  { "tag-prefix",     	OP_ATTACH_TAG_PREFIX,		";" },
   { "view-text",	OP_ATTACH_VIEW_TEXT,		"T" },
+  { "view-attach",	OP_VIEW_ATTACH,			M_ENTER_S },
+
+
+
+
+
+
   { NULL,		0,				NULL }
 };
 
@@ -240,33 +276,26 @@ struct binding_t OpCompose[] = {
   { "edit-headers",	OP_COMPOSE_EDIT_HEADERS,	"E" },
   { "edit-file",	OP_COMPOSE_EDIT_FILE,		"\030e" },
   { "edit-encoding",	OP_COMPOSE_EDIT_ENCODING,	"\005" },
+  { "edit-from",	OP_COMPOSE_EDIT_FROM,		"\033f" },
   { "edit-fcc",		OP_COMPOSE_EDIT_FCC,		"f" },
-  { "filter-entry",	OP_FILTER,			"|" }, 
-#ifdef _PGPPATH
-  { "forget-passphrase",OP_FORGET_PASSPHRASE,		"\006" },
-#endif
+  { "filter-entry",	OP_FILTER,			"F" }, 
   { "ispell",		OP_COMPOSE_ISPELL,		"i" },
   { "print-entry",	OP_PRINT,			"l" },
   { "redraw-screen",	OP_REDRAW,			"\014" },
   { "edit-mime",	OP_COMPOSE_EDIT_MIME,		"m" },
   { "new-mime",		OP_COMPOSE_NEW_MIME,		"n" },
-  { "pgp-menu",		OP_COMPOSE_PGP_MENU,		"p" },
   { "postpone-message",	OP_COMPOSE_POSTPONE_MESSAGE,	"P" },
-  { "exit",		OP_EXIT,			"q" },
   { "edit-reply-to",	OP_COMPOSE_EDIT_REPLY_TO,	"r" },
   { "rename-file",	OP_COMPOSE_RENAME_FILE,		"R" },
   { "edit-subject",	OP_COMPOSE_EDIT_SUBJECT,	"s" },
-  { "decode-save",	OP_ATTACH_DECODE_SAVE,		"\033s" },
-  { "tag-attachment",	OP_ATTACH_TAG,			"T" },
   { "edit-to",		OP_COMPOSE_EDIT_TO,		"t" },
   { "edit-type",	OP_COMPOSE_EDIT_TYPE,		"\024" },
   { "toggle-unlink",	OP_COMPOSE_TOGGLE_UNLINK,	"u" },
-  { "view-attach",	OP_COMPOSE_VIEW_ATTACH,		M_ENTER_S },
+  { "view-attach",	OP_VIEW_ATTACH,			M_ENTER_S },
   { "send-message",	OP_COMPOSE_SEND_MESSAGE,	"y" },
-  { "enter-command",	OP_ENTER_COMMAND,		":" },
-  { "tag-prefix",	OP_ATTACH_TAG_PREFIX,		";" },
-  { "help",		OP_HELP,			"?" },
-  { "pipe-entry",	OP_PIPE,			"\033|" },
+  { "pipe-entry",	OP_PIPE,			"|" },
+
+
   { NULL,		0,				NULL }
 };
 
@@ -280,13 +309,10 @@ struct binding_t OpPost[] = {
 struct binding_t OpBrowser[] = {
   { "change-dir",	OP_CHANGE_DIRECTORY,	"c" },
   { "enter-mask",	OP_ENTER_MASK,		"m" },
-  { "search-next",	OP_SEARCH_NEXT,		"n" },
   { "sort",		OP_SORT,		"o" },
   { "sort-reverse",	OP_SORT_REVERSE,	"O" },
   { "select-new",	OP_BROWSER_NEW_FILE,	"N" },
   { "check-new",	OP_CHECK_NEW,		"\t" },
-  { "search",		OP_SEARCH,		"/" },
-  { "search-reverse",	OP_SEARCH_REVERSE,	"\033/" },
   { NULL,		0,			NULL }
 };
 
@@ -299,24 +325,14 @@ struct binding_t OpEditor[] = {
   { "backspace",	OP_EDITOR_BACKSPACE,		"\010" },
   { "kill-eol",		OP_EDITOR_KILL_EOL,		"\013" },
   { "kill-line",	OP_EDITOR_KILL_LINE,		"\025" },
+  { "quote-char",	OP_EDITOR_QUOTE_CHAR,		"\026" },
   { "kill-word",	OP_EDITOR_KILL_WORD,		"\027" },
+  { "complete",		OP_EDITOR_COMPLETE,		"\t" },
+  { "buffy-cycle",	OP_EDITOR_BUFFY_CYCLE,		" " },
   { "history-up",	OP_EDITOR_HISTORY_UP,		NULL },
   { "history-down",	OP_EDITOR_HISTORY_DOWN,		NULL },
   { NULL,		0,				NULL }
 };
 
-struct binding_t OpAlias[] = {
-  { "search-next",	OP_SEARCH_NEXT,		"n" },
-  { "search",		OP_SEARCH,		"/" },
-  { "search-reverse",	OP_SEARCH_REVERSE,	"\033/" },
-  { NULL,		0,				NULL }
-};
 
-#ifdef _PGPPATH
-struct binding_t OpPgp[] = {
-  { "search-next",	OP_SEARCH_NEXT,		"n" },
-  { "search",		OP_SEARCH,		"/" },
-  { "search-reverse",	OP_SEARCH_REVERSE,	"\033/" },
-  { NULL,		0,				NULL }
-};
-#endif /* _PGPPATH */
+
