@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -30,10 +30,12 @@ WHERE CONTEXT *Context;
 WHERE char Errorbuf[SHORT_STRING];
 
 WHERE char AliasFile[_POSIX_PATH_MAX];
+WHERE char AliasFmt[SHORT_STRING];
 WHERE char AttachSep[SHORT_STRING] INITVAL({0});
 WHERE char Attribution[SHORT_STRING];
 WHERE char Charset[SHORT_STRING];
 WHERE char DecodeFmt[SHORT_STRING];
+WHERE char DefaultHook[SHORT_STRING];
 WHERE char DateFmt[SHORT_STRING];
 WHERE char DsnNotify[SHORT_STRING] INITVAL({0});
 WHERE char DsnReturn[SHORT_STRING] INITVAL({0});
@@ -47,7 +49,6 @@ WHERE char InReplyTo[SHORT_STRING];
 WHERE char Inbox[_POSIX_PATH_MAX];
 WHERE char Ispell[_POSIX_PATH_MAX];
 WHERE char LastFolder[_POSIX_PATH_MAX];
-WHERE char LocalSig[_POSIX_PATH_MAX] INITVAL({0});
 WHERE char Locale[SHORT_STRING];
 WHERE char MailcapPath[LONG_STRING];
 WHERE char Maildir[_POSIX_PATH_MAX];
@@ -56,20 +57,12 @@ WHERE char Muttrc[_POSIX_PATH_MAX] INITVAL({0});
 WHERE char Outbox[_POSIX_PATH_MAX] INITVAL({0});
 WHERE char Pager[_POSIX_PATH_MAX];
 WHERE char PagerFmt[SHORT_STRING];
-#ifdef _PGPPATH
-WHERE char Pgp[_POSIX_PATH_MAX];
-WHERE char PgpSignAs[SHORT_STRING] INITVAL ({0});
-#endif
 WHERE char PipeSep[SHORT_STRING] INITVAL({0});
-WHERE char PopHost[SHORT_STRING] INITVAL({0});
-WHERE char PopUser[SHORT_STRING] INITVAL({0});
-WHERE char PopPass[SHORT_STRING] INITVAL({0});
 WHERE char PostIndentString[SHORT_STRING] INITVAL({0});
 WHERE char Postponed[_POSIX_PATH_MAX];
 WHERE char Prefix[SHORT_STRING];
 WHERE char PrintCmd[_POSIX_PATH_MAX];
 WHERE char Realname[SHORT_STRING];
-WHERE char RemoteSig[_POSIX_PATH_MAX] INITVAL({0});
 WHERE char Sendmail[_POSIX_PATH_MAX];
 WHERE char SendmailBounce[_POSIX_PATH_MAX];
 WHERE char Shell[_POSIX_PATH_MAX];
@@ -82,20 +75,18 @@ WHERE char Tempdir[_POSIX_PATH_MAX];
 WHERE char Tochars[8]; /* " +TCF" */
 WHERE char Username[SHORT_STRING];
 WHERE char Visual[_POSIX_PATH_MAX];
-WHERE char WebBrowser[_POSIX_PATH_MAX] INITVAL({0});
 
 WHERE LIST *AutoViewList INITVAL(0);
 WHERE LIST *HeaderOrderList INITVAL(0);
 WHERE LIST *Ignore INITVAL(0);
 WHERE LIST *UnIgnore INITVAL(0);
 WHERE LIST *MailLists INITVAL(0);
-WHERE BUFFY *Incoming INITVAL(0);
-WHERE REGEXP Alternates;
-WHERE REGEXP Localsites;
-WHERE REGEXP Mask;
-WHERE REGEXP QuoteRegexp;
-WHERE REGEXP ReplyRegexp;
-WHERE REGEXP UrlRegexp;
+
+#ifdef USE_POP
+WHERE char PopHost[SHORT_STRING] INITVAL({0});
+WHERE char PopPass[SHORT_STRING] INITVAL({0});
+WHERE char PopUser[SHORT_STRING] INITVAL({0});
+#endif
 
 /* bit vector for boolean variables */
 #ifdef MAIN_C
@@ -109,19 +100,13 @@ WHERE unsigned long QuadOptions INITVAL (0);
 
 WHERE char EscChar[2] INITVAL ("~"); /* $escape, used for $editor=builtin */
 
-WHERE short BrowserSort INITVAL (SORT_SUBJECT);
 WHERE unsigned short Counter INITVAL (0);
-WHERE short DefaultMagic INITVAL (M_MBOX);
 WHERE short HistSize INITVAL (10);
 WHERE short PagerContext INITVAL (0);
 WHERE short PagerIndexLines INITVAL (0);
-#ifdef _PGPPATH
-WHERE short PgpTimeout INITVAL (300); /* 5 minutes */
-#endif
 WHERE short PopPort INITVAL (110);
 WHERE short ReadInc INITVAL (10);
-WHERE short Sort INITVAL (SORT_DATE);
-WHERE short SortAux INITVAL (SORT_DATE); /* auxiallary sorting method */
+WHERE short ScoreDefault INITVAL (10);
 WHERE short Timeout INITVAL (600); /* seconds */
 WHERE short TrimRef INITVAL (10);
 WHERE short WriteInc INITVAL (10);
@@ -129,9 +114,6 @@ WHERE short WriteInc INITVAL (10);
 /* vector to store received signals */
 WHERE short Signals INITVAL (0);
 
-WHERE int *ColorQuote;
-WHERE int ColorQuoteUsed;
-
 WHERE ALIAS *Aliases INITVAL (0);
 WHERE LIST *UserHeader INITVAL (0);
 
@@ -149,14 +131,17 @@ WHERE gid_t UserGid;
 const char *Weekdays[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
 const char *Months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "ERR" };
 
-const char *body_types[] = { "x-unknown", "audio", "application", "image", "message", "multipart", "text", "video" };
-const char *body_encodings[] = { "x-unknown", "7bit", "8bit", "quoted-printable", "base64", "binary" };
+const char *BodyTypes[] = { "x-unknown", "audio", "application", "image", "message", "multipart", "text", "video" };
+const char *BodyEncodings[] = { "x-unknown", "7bit", "8bit", "quoted-printable", "base64", "binary" };
 #else
 extern const char *Weekdays[];
 extern const char *Months[];
-extern const char *body_types[];
-extern const char *body_encodings[];
-extern char MuttVersion[];
 #endif
 
-extern const struct mapping_t SortMethods[];
+#ifdef MAIN_C
+/* so that global vars get included */ 
+#include "mx.h"
+#include "mutt_regex.h"
+#include "buffy.h"
+#include "sort.h"
+#endif /* MAIN_C */
