@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -27,10 +27,13 @@
 #include "mutt_curses.h"
 #include "rfc1524.h"
 #include "keymap.h"
+#include "mime.h"
+#include "copy.h"
+
+
+
+
 
-#ifdef _PGPPATH
-#include "pgp.h"
-#endif
 
 typedef void handler_f (BODY *, STATE *);
 typedef handler_f *handler_t;
@@ -242,6 +245,7 @@ struct enriched_state
   size_t buff_len;
   size_t line_len;
   size_t line_used;
+  size_t line_max;
   size_t indent_len;
   size_t word_len;
   size_t buff_used;
@@ -371,9 +375,14 @@ static void enriched_flush (struct enriched_state *stte, int wrap)
   if (stte->buff_used)
   {
     stte->buffer[stte->buff_used] = '\0';
-    strncat (stte->line, stte->buffer, stte->buff_len);
-    stte->line_len += stte->word_len;
     stte->line_used += stte->buff_used;
+    if (stte->line_used > stte->line_max)
+    {
+      stte->line_max = stte->line_used;
+      safe_realloc ((void **) &stte->line, stte->line_max + 1);
+    }
+    strcat (stte->line, stte->buffer);
+    stte->line_len += stte->word_len;
     stte->word_len = 0;
     stte->buff_used = 0;
   }
@@ -412,7 +421,7 @@ static void enriched_putc (int c, struct enriched_state *stte)
   }
   else
   {
-    if (stte->s->displaying)
+    if (stte->s->flags & M_DISPLAY)
     {
       if (stte->tag_level[RICH_BOLD])
       {
@@ -487,7 +496,7 @@ static void enriched_set_flags (const char *tag, struct enriched_state *stte)
     {
       if (stte->tag_level[j]) /* make sure not to go negative */
 	stte->tag_level[j]--;
-      if (stte->s->displaying && j == RICH_PARAM && stte->tag_level[RICH_COLOR])
+      if ((stte->s->flags & M_DISPLAY) && j == RICH_PARAM && stte->tag_level[RICH_COLOR])
       {
 	stte->param[stte->param_len] = '\0';
 	if (!strcasecmp(stte->param, "black"))
@@ -525,7 +534,7 @@ static void enriched_set_flags (const char *tag, struct enriched_state *stte)
 	stte->param_len = 0;
 	stte->param[0] = '\0';
       }
-      if (stte->s->displaying && j == RICH_COLOR)
+      if ((stte->s->flags & M_DISPLAY) && j == RICH_COLOR)
       {
 	enriched_puts("\033[0m", stte);
       }
@@ -552,9 +561,10 @@ void text_enriched_handler (BODY *a, STATE *s)
 
   memset (&stte, 0, sizeof (stte));
   stte.s = s;
-  stte.WrapMargin = (s->displaying ? (COLS-4) : ((COLS-4)<72)?(COLS-4):72);
-  stte.line = (char *) safe_calloc(1, stte.WrapMargin * 4);
-  stte.param = (char *) safe_calloc(1, STRING);
+  stte.WrapMargin = ((s->flags & M_DISPLAY) ? (COLS-4) : ((COLS-4)<72)?(COLS-4):72);
+  stte.line_max = stte.WrapMargin * 4;
+  stte.line = (char *) safe_calloc (1, stte.line_max + 1);
+  stte.param = (char *) safe_calloc (1, STRING);
 
   if (s->prefix)
   {
@@ -664,41 +674,92 @@ void text_enriched_handler (BODY *a, STATE *s)
 
 void alternative_handler (BODY *a, STATE *s)
 {
-  BODY *choice = 0;
+  BODY *choice = NULL;
+  BODY *b;
+  char buf[STRING];
   int type = 0;
 
-  if (a && a->parts) a = a->parts;
-  while (a)
+  /* First, look for an autoviewable type */
+  if (a && a->parts) 
+    b = a->parts;
+  else
+    b = a;
+  while (b && !choice)
   {
-    if (a->type == TYPETEXT)
+    snprintf (buf, sizeof (buf), "%s/%s", TYPE (b->type), b->subtype);
+    if (mutt_is_autoview (buf))
     {
-      if (strcasecmp ("plain", a->subtype) == 0)
+      rfc1524_entry *entry = rfc1524_new_entry ();
+
+      if (rfc1524_mailcap_lookup (b, buf, entry, M_AUTOVIEW))
       {
-        choice = a;
-        type = TXTPLAIN;
-      }
-      else if (strcasecmp ("enriched", a->subtype) == 0)
-      {
-        if (type == 0 || type > TXTENRICHED)
-	{
-          choice = a;
-          type = TXTENRICHED;
-        }
-      }
-      else if (strcasecmp ("html", a->subtype) == 0)
-      {
-        if (type == 0)
-	{
-          choice = a;
-          type = TXTHTML;
-        }
+	choice = b;
       }
+      rfc1524_free_entry (&entry);
+    }
+    b = b->next;
+  }
+
+  /* Then, look for a text entry */
+  if (!choice)
+  {
+    if (a && a->parts) 
+      b = a->parts;
+    else
+      b = a;
+    while (b)
+    {
+      if (b->type == TYPETEXT)
+      {
+	if (strcasecmp ("plain", b->subtype) == 0)
+	{
+	  choice = b;
+	  type = TXTPLAIN;
+	}
+	else if (strcasecmp ("enriched", b->subtype) == 0)
+	{
+	  if (type == 0 || type > TXTENRICHED)
+	  {
+	    choice = b;
+	    type = TXTENRICHED;
+	  }
+	}
+	else if (strcasecmp ("html", b->subtype) == 0)
+	{
+	  if (type == 0)
+	  {
+	    choice = b;
+	    type = TXTHTML;
+	  }
+	}
+      }
+      b = b->next;
+    }
+  }
+  /* Finally, look for other possibilities */
+  if (!choice)
+  {
+    if (a && a->parts) 
+      b = a->parts;
+    else
+      b = a;
+    while (b && !choice)
+    {
+      if (mutt_can_decode (b))
+	choice = b;
+      b = b->next;
     }
-    a = a->next;
   }
   if (choice)
+  {
+    if (s->flags & M_DISPLAY && !option (OPTWEED))
+    {
+      fseek (s->fpin, choice->hdr_offset, 0);
+      mutt_copy_bytes(s->fpin, s->fpout, choice->offset-choice->hdr_offset);
+    }
     mutt_body_handler (choice, s);
-  else if (s->displaying)
+  }
+  else if (s->flags & M_DISPLAY)
   {
     /* didn't find anything that we could display! */
     state_puts("[-- Error:  Could not display any parts of Multipart/Alternative! --]\n", s);
@@ -726,8 +787,8 @@ void message_handler (BODY *a, STATE *s)
   if (b->parts)
   {
     mutt_copy_hdr (s->fpin, s->fpout, off_start, b->parts->offset,
-	((s->displaying && option (OPTWEED)) ? (CH_WEED | CH_REORDER) : 0) |
-	(s->prefix ? CH_PREFIX : 0) | CH_DECODE | CH_FROM);
+	(((s->flags & M_DISPLAY) && option (OPTWEED)) ? (CH_WEED | CH_REORDER) : 0) |
+	(s->prefix ? CH_PREFIX : 0) | CH_DECODE | CH_FROM, s->prefix);
 
     if (s->prefix)
       state_puts (s->prefix, s);
@@ -746,7 +807,7 @@ int mutt_can_decode (BODY *a)
   char type[STRING];
 
   snprintf (type, sizeof (type), "%s/%s", TYPE (a->type), a->subtype);
-  if (!option (OPTNOAUTOVIEW) && mutt_is_autoview (type))
+  if (mutt_is_autoview (type))
     return (rfc1524_mailcap_lookup (a, type, NULL, M_AUTOVIEW));
   else if (a->type == TYPETEXT)
     return (1);
@@ -754,34 +815,16 @@ int mutt_can_decode (BODY *a)
     return (1);
   else if (a->type == TYPEMULTIPART)
   {
-#ifdef _PGPPATH
-    if (strcasecmp (a->subtype, "signed") == 0 ||
-	strcasecmp (a->subtype, "encrypted") == 0)
-      return (1);
-    else
-#endif
     {
-      BODY *p = a->parts;
+      BODY *p;
 
-      while (p)
+      for (p = a->parts; p; p = p->next)
       {
 	if (mutt_can_decode (p))
 	  return (1);
-	p = p->next;
       }
-      return (0);
     }
   }
-  else if (a->type == TYPEAPPLICATION)
-  {
-    if (strcasecmp (a->subtype, "pgp") == 0 ||
-	strcasecmp (a->subtype, "x-pgp-message") == 0 ||
-	strcasecmp (a->subtype, "pgp-signed") == 0 ||
-	strcasecmp (a->subtype, "pgp-keys") == 0)
-      return (1);
-    else
-      return (0);
-  }
   return (0);
 }
 
@@ -807,7 +850,7 @@ void multipart_handler (BODY *a, STATE *s)
 
   for (p = b->parts, count = 1; p; p = p->next, count++)
   {
-    if (s->displaying)
+    if (s->flags & M_DISPLAY)
     {
       snprintf (buffer, sizeof (buffer), "[-- Attachment #%d", count);
       state_puts (buffer, s);
@@ -822,9 +865,16 @@ void multipart_handler (BODY *a, STATE *s)
       mutt_pretty_size (length, sizeof (length), p->length);
 
       snprintf (buffer, sizeof (buffer),
-		"[-- Type: %s/%s, Encoding: %s, Size: %s --]\n\n",
+		"[-- Type: %s/%s, Encoding: %s, Size: %s --]\n",
 	       TYPE (p->type), p->subtype, ENCODING (p->encoding), length);
       state_puts (buffer, s);
+      if (!option (OPTWEED))
+      {
+	fseek (s->fpin, p->hdr_offset, 0);
+	mutt_copy_bytes(s->fpin, s->fpout, p->offset-p->hdr_offset);
+      }
+      else
+	state_putc ('\n', s);
     }
     else
     {
@@ -873,7 +923,7 @@ void autoview_handler (BODY *a, STATE *s)
     /* rfc1524_expand_command returns 0 if the file is required */
     piped = rfc1524_expand_command (a, tempfile, type, command, sizeof (command));
 
-    if (s->displaying)
+    if (s->flags & M_DISPLAY)
     {
       char mesg[STRING];
 
@@ -890,7 +940,7 @@ void autoview_handler (BODY *a, STATE *s)
     }
     else
     {
-      if ((fpin = fopen (tempfile,"w")) == NULL)
+      if ((fpin = safe_fopen (tempfile, "w")) == NULL)
       {
         mutt_perror ("fopen");
 	rfc1524_free_entry (&entry);
@@ -912,7 +962,7 @@ void autoview_handler (BODY *a, STATE *s)
       /* check for data on stderr */
       if (fgets (buffer, sizeof(buffer), fperr)) 
       {
-	if (s->displaying) 
+	if (s->flags & M_DISPLAY) 
 	{
 	  char mesg[STRING];
 
@@ -935,7 +985,7 @@ void autoview_handler (BODY *a, STATE *s)
       /* Check for stderr messages */
       if (fgets (buffer, sizeof(buffer), fperr))
       {
-	if (s->displaying)
+	if (s->flags & M_DISPLAY)
 	{
 	  char mesg[STRING];
 
@@ -952,7 +1002,7 @@ void autoview_handler (BODY *a, STATE *s)
     fclose (fperr);
     mutt_wait_filter (thepid);
     mutt_unlink (tempfile);
-    if (s->displaying) 
+    if (s->flags & M_DISPLAY) 
       mutt_clear_error ();
   }
   rfc1524_free_entry (&entry);
@@ -989,7 +1039,7 @@ void mutt_body_handler (BODY *b, STATE *s)
   /* first determine which handler to use to process this part */
 
   snprintf (type, sizeof (type), "%s/%s", TYPE (b->type), b->subtype);
-  if (!option (OPTNOAUTOVIEW) && mutt_is_autoview (type))
+  if (mutt_is_autoview (type))
   {
     rfc1524_entry *entry = rfc1524_new_entry ();
 
@@ -1020,65 +1070,14 @@ void mutt_body_handler (BODY *b, STATE *s)
   }
   else if (b->type == TYPEMULTIPART)
   {
-#ifdef _PGPPATH
-    char *p;
-#endif /* _PGPPATH */
 
     if (strcasecmp ("alternative", b->subtype) == 0)
       handler = alternative_handler;
 
-#ifdef _PGPPATH
-    else if (strcasecmp ("signed", b->subtype) == 0)
-    {
-      p = mutt_get_parameter ("protocol", b->parameter);
-
-      if (!p)
-        mutt_error ("Error: Multipart/Signed has no protocol.");
-      else if (strcasecmp ("application/pgp-signature", p) == 0)
-      {
-	if (option (OPTVERIFYSIG))
-	  handler = pgp_signed_handler;
-      }
-    }
-    else if (strcasecmp ("encrypted", b->subtype) == 0)
-    {
-      p = mutt_get_parameter ("protocol", b->parameter);
-
-      if (!p)
-        mutt_error ("Error: Multipart/Encrypted has no protocol parameter!");
-      else if (strcasecmp ("application/pgp-encrypted", p) == 0)
-        handler = pgp_encrypted_handler;
-    }
-#endif /* _PGPPATH */
 
     if (!handler)
       handler = multipart_handler;
   }
-#ifdef _PGPPATH
-  else if (b->type == TYPEAPPLICATION)
-  {
-    if (strcasecmp ("pgp", b->subtype) == 0 ||
-	strcasecmp ("x-pgp-message", b->subtype) == 0 ||
-	strcasecmp ("pgp-signed", b->subtype) == 0)
-    {
-      char *p = mutt_get_parameter ("x-action", b->parameter);
-      char *q = mutt_get_parameter ("format", b->parameter);
-
-      if(q && strcasecmp (q,"mime") == 0) 
-	handler = application_pgp_handler;
-      else if (q && !strcasecmp (q, "keys-only") && option (OPTNOAUTOVIEW))
-	plaintext = 1;
-      else if (option (OPTVERIFYSIG) || !p ||
-	       (strcasecmp (p, "sign") != 0 && 
-		strcasecmp (p, "signclear") != 0))
-	handler = application_pgp_handler;
-      else
-	plaintext = 1;
-    }
-    else if (strcasecmp ("pgp-keys", b->subtype) == 0)
-      plaintext = 1;
-  }
-#endif /* _PGPPATH */
 
   if (plaintext || handler)
   {
@@ -1157,7 +1156,7 @@ void mutt_body_handler (BODY *b, STATE *s)
       }
     }
   }
-  else if (s->displaying)
+  else if (s->flags & M_DISPLAY)
   {
     fprintf (s->fpout, "[-- %s/%s is unsupported ", TYPE (b->type), b->subtype);
     if (!option (OPTVIEWATTACH))
