@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,9 +19,6 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 
-#ifdef _PGPPATH
-#include "pgp.h"
-#endif
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -30,15 +27,13 @@
 
 int mutt_is_mail_list (ADDRESS *addr)
 {
-  LIST *p = MailLists;
+  LIST *p;
 
-  if (!addr->mailbox)
-    return 0;
-  while (p)
+  if (addr->mailbox)
   {
-    if (strncasecmp (addr->mailbox, p->data, strlen (p->data)) == 0)
-      return 1;
-    p = p->next;
+    for (p = MailLists; p; p = p->next)
+      if (strncasecmp (addr->mailbox, p->data, strlen (p->data)) == 0)
+	return 1;
   }
   return 0;
 }
@@ -46,55 +41,17 @@ int mutt_is_mail_list (ADDRESS *addr)
 static int
 check_for_mailing_list (ADDRESS *adr, char *pfx, char *buf, int buflen)
 {
-  ADDRESS *list;
-  int len = strlen (pfx);
-
   for (; adr; adr = adr->next)
   {
     if (mutt_is_mail_list (adr))
     {
-      strfcpy (buf, pfx, buflen);
-      len = strlen (pfx);
-      buflen -= len;
-      buf += len;
-
-      if (option (OPTREVALIAS) && (list = alias_reverse_lookup (adr)) && list->personal)
-	strfcpy (buf, list->personal, buflen);
-      else
-	strfcpy (buf, adr->mailbox, buflen);
+      snprintf (buf, buflen, "%s%s", pfx, mutt_get_name (adr));
       return 1;
     }
   }
   return 0;
 }
 
-static void print_addr (char *pfx, char *s, size_t l, ADDRESS *a)
-{
-  ADDRESS *t;
-
-  if (pfx)
-  {
-    strfcpy (s, pfx, l);
-    while (*s)
-    {
-      s++;
-      l--;
-    }
-  }
-
-  if (option (OPTREVALIAS) && (t = alias_reverse_lookup (a)) && t->personal)
-    strfcpy (s, t->personal, l);
-  else if (a->personal)
-    strfcpy (s, a->personal, l);
-  else
-  {
-    t = a->next;
-    a->next = NULL;
-    rfc822_write_address (s, l, a);
-    a->next = t;
-  }
-}
-
 static void make_from (ENVELOPE *hdr, char *buf, size_t len, int do_lists)
 {
   int me;
@@ -109,17 +66,25 @@ static void make_from (ENVELOPE *hdr, char *buf, size_t len, int do_lists)
       return;
   }
 
-  *buf = 0;
   if (me && hdr->to)
-    print_addr ("To ", buf, len, hdr->to);
+    snprintf (buf, len, "To %s", mutt_get_name (hdr->to));
   else if (me && hdr->cc)
-    print_addr ("Cc ", buf, len, hdr->cc);
+    snprintf (buf, len, "Cc %s", mutt_get_name (hdr->cc));
   else if (hdr->from)
-    print_addr (NULL, buf, len, hdr->from);
+    strfcpy (buf, mutt_get_name (hdr->from), len);
+  else
+    *buf = 0;
 }
 
-/*
- * Return values:
+int mutt_user_is_recipient (ADDRESS *a)
+{
+  for (; a; a = a->next)
+    if (mutt_addr_is_user (a))
+      return 1;
+  return 0;
+}
+
+/* Return values:
  * 0: user is not in list
  * 1: user is unique recipient
  * 2: user is in the TO list
@@ -128,27 +93,20 @@ static void make_from (ENVELOPE *hdr, char *buf, size_t len, int do_lists)
  */
 static int user_is_recipient (ENVELOPE *hdr)
 {
-  ADDRESS *a;
-
   if (mutt_addr_is_user (hdr->from))
     return 4;
 
-  for (a = hdr->to; a; a = a->next)
+  if (mutt_user_is_recipient (hdr->to))
   {
-    if (mutt_addr_is_user (a))
-    {
-      if (!a->next && !hdr->cc && hdr->to == a)
-	return (1);
-      else
-	return (2);
-    }
+    if (hdr->to->next || hdr->cc)
+      return 2; /* non-unique recipient */
+    else
+      return 1; /* unique recipient */
   }
 
-  for (a = hdr->cc; a; a = a->next)
-  {
-    if (mutt_addr_is_user (a))
-      return (3);
-  }
+  if (mutt_user_is_recipient (hdr->cc))
+    return 3;
+
   return (0);
 }
 
@@ -158,12 +116,13 @@ static int user_is_recipient (ENVELOPE *hdr)
  * %C = current message number
  * %d = date and time of message (using strftime)
  * %f = entire from line
- * %F = entire from line, unless from self
+ * %F = like %n, unless from self
  * %i = message-id
  * %l = number of lines in the message
- * %L = list-from function.
+ * %L = like %F, except `lists' are displayed first
  * %m = number of messages in the mailbox
  * %n = name of author
+ * %N = score
  * %s = subject
  * %S = short message status (e.g., N/O/D/!/r/-)
  * %t = `to:' field (recipients)
@@ -214,7 +173,6 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
       switch (*s)
       {
 	case '>': /* right justify the rest of the line */
-
 	  s++;
 	  ch = *s++;
 	  _mutt_make_string (buf, sizeof (buf), s, hdr, flags);
@@ -230,7 +188,6 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 	  break;
 
 	case '|': /* pad to end of line */
-
 	  s++;
 	  ch = *s++;
 	  if (destlen > COLS)
@@ -240,21 +197,17 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 	  break;
 
 	case 'a':
-
-	  mutt_simple_address (buf2, sizeof (buf2), hdr->env->from);
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	  snprintf (buf, sizeof (buf), fmt, buf2);
+	  snprintf (buf, sizeof (buf), fmt, hdr->env->from->mailbox);
 	  break;
 
 	case 'c':
-
 	  mutt_pretty_size (buf2, sizeof (buf2), (long) hdr->content->length);
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	  snprintf (buf, sizeof (buf), fmt, buf2);
 	  break;
 
 	case 'C':
-
 	  snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
 	  snprintf (buf, sizeof (buf), fmt, hdr->msgno + 1);
 	  break;
@@ -262,6 +215,7 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 	case 'd':
 	case '{':
 	case '[':
+	case '(':
 
 	  /* preprocess $date_format to handle %Z */
 	  {
@@ -278,13 +232,14 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 
 	    len = sizeof (buf) - 1;
 	    while (len > 0 && ((*s == 'd' && *cp) ||
-			      (*s == '{' && *cp != '}') || 
-			      (*s == '[' && *cp != ']')))
+			       (*s == '{' && *cp != '}') || 
+			       (*s == '[' && *cp != ']') ||
+			       (*s == '(' && *cp != ')')))
 	    {
 	      if (*cp == '%')
 	      {
 		cp++;
-		if (*cp == 'Z' && *s != '[')
+		if (*cp == 'Z' && *s != '[' && *s != '(')
 		{
 		  if (len >= 5)
 		  {
@@ -326,6 +281,8 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 
 	    if (*s == '[')
 	      tm = localtime (&hdr->date_sent);
+	    else if (*s == '(')
+	      tm = localtime (&hdr->received);
 	    else
 	    {
 	      time_t T;
@@ -352,7 +309,6 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 	  break;
 
 	case 'f':
-
 	  buf2[0] = 0;
 	  rfc822_write_address (buf2, sizeof (buf2), hdr->env->from);
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
@@ -360,65 +316,50 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 	  break;
 
 	case 'F':
-
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	  make_from (hdr->env, buf2, sizeof (buf2), 0);
 	  snprintf (buf, sizeof (buf), fmt, buf2);
 	  break;
 
 	case 'i':
-
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	  snprintf (buf, sizeof (buf), fmt, hdr->env->message_id ? hdr->env->message_id : "<no.id>");
 	  break;
 
 	case 'l':
-
 	  snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
 	  snprintf (buf, sizeof (buf), fmt, (int) hdr->lines);
 	  break;
 
 	case 'L':
-
 	  make_from (hdr->env, buf2, sizeof (buf2), 1);
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	  snprintf (buf, sizeof (buf), fmt, buf2);
 	  break;
 
 	case 'm':
-
 	  snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
 	  snprintf (buf, sizeof (buf), fmt, Context->msgcount);
 	  break;
 
 	case 'n':
-
-	  {
-	    ADDRESS *a;
-	    
-	    if (hdr->env->from && hdr->env->from->personal)
-	      strfcpy (buf2, hdr->env->from->personal, sizeof (buf2));
-	    else if (option (OPTREVALIAS) && hdr->env->from &&
-		    (a = alias_reverse_lookup (hdr->env->from)) && a->personal)
-	      strfcpy (buf2, a->personal, sizeof (buf2));
-	    else
-	    {
-	      buf2[0] = 0;
-	      rfc822_write_address (buf2, sizeof (buf2), hdr->env->from);
-	    }
-	  }
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	  snprintf (buf, sizeof (buf), fmt, buf2);
+	  snprintf (buf, sizeof (buf), fmt, mutt_get_name (hdr->env->from));
+	  break;
+
+	case 'N':
+	  snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+	  snprintf (buf, sizeof (buf), fmt, hdr->score);
 	  break;
 
 	case 's':
-
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	  if (flags & M_TREE)
 	  {
 	    if (flags & M_FORCESUBJ)
 	    {
-	      snprintf (buf2, sizeof (buf2), "%s%s", hdr->tree, hdr->env->subject);
+	      snprintf (buf2, sizeof (buf2), "%s%s", hdr->tree,
+			hdr->env->subject ? hdr->env->subject : "");
 	      snprintf (buf, sizeof (buf), fmt, buf2);
 	    }
 	    else
@@ -432,7 +373,6 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 	  break;
 
 	case 'S':
-
 	  if (hdr->deleted)
 	    ch = 'D';
 	  else if (hdr->tagged)
@@ -451,44 +391,40 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 	  break;
 
 	case 't':
-
 	  buf2[0] = 0;
 	  if (!check_for_mailing_list (hdr->env->to, "To ", buf2, sizeof (buf2)) &&
 	      !check_for_mailing_list (hdr->env->cc, "Cc ", buf2, sizeof (buf2)))
 	  {
 	    if (hdr->env->to)
-	      print_addr ("To ", buf2, sizeof (buf2), hdr->env->to);
-	    else
-	      print_addr ("Cc ", buf2, sizeof (buf2), hdr->env->cc);
+	      snprintf (buf2, sizeof (buf2), "To %s", mutt_get_name (hdr->env->to));
+	    else if (hdr->env->cc)
+	      snprintf (buf2, sizeof (buf2), "Cc %s", mutt_get_name (hdr->env->cc));
 	  }
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	  snprintf (buf, sizeof (buf), fmt, buf2);
 	  break;
 
 	case 'T':
-
 	  snprintf (fmt, sizeof (fmt), "%%%sc", prefix);
 	  snprintf (buf, sizeof (buf), fmt, Tochars[user_is_recipient (hdr->env)]);
 	  break;
 
 	case 'u':
-
+	  if (hdr->env->from && hdr->env->from->mailbox)
+	  {
+	    strfcpy (buf2, hdr->env->from->mailbox, sizeof (buf));
+	    if ((cp = strpbrk (buf2, "%@")))
+	      *cp = 0;
+	  }
+	  else
+	    buf2[0] = 0;
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	  snprintf (buf, sizeof (buf), fmt, hdr->env->from->mailbox);	
+	  snprintf (buf, sizeof (buf), fmt, buf2);
 	  break;
 
 	case 'Z':
-	  
 	  if (hdr->mailcap)
 	    ch = 'M';
-#ifdef _PGPPATH
-	  else if (hdr->pgp & PGPENCRYPT)
-	    ch = 'P';
-	  else if (hdr->pgp & PGPSIGN)
-	    ch = 'S';
-	  else if (hdr->pgp & PGPKEY)
-	    ch = 'K';
-#endif
 	  else
 	    ch = ' ';
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
@@ -504,7 +440,6 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
 	  break;
 
 	default:
-
 	  snprintf (buf, sizeof (buf), "%%%s%c", prefix, *s);
 	  break;
       }
@@ -547,4 +482,14 @@ _mutt_make_string (char *dest, size_t destlen, char *s, HEADER *hdr, int flags)
     s++;
   }
   *wptr = 0;
+
+  if (flags & M_MAKEPRINT)
+  {
+    /* Make sure that the string is printable by changing all non-printable
+       chars to dots, or spaces for non-printable whitespace */
+    for (cp = dest ; *cp ; cp++)
+      if (!IsPrint (*cp) &&
+	  !((flags & M_TREE) && (*cp <= 8)))
+	*cp = isspace ((unsigned char) *cp) ? ' ' : '.';
+  }
 }
