@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "keymap.h"
+#include "pager.h"
+#include "mapping.h"
 
 #include <ctype.h>
 #include <string.h>
@@ -30,52 +32,21 @@ static struct binding_t *help_lookupFunction (int op, int menu)
   int i;
   struct binding_t *map;
 
-  switch (menu)
+  if (menu != MENU_PAGER)
   {
-    case MENU_MAIN:
-      map = OpMain;
-      break;
-
-    case MENU_PAGER:
-      map = OpPager;
-      break;
-
-    default:
-      for (i=0; OpGeneric[i].name; i++)
-	if (OpGeneric[i].op == op)
-	  return (&OpGeneric[i]);    
-
-      switch (menu)
-      {
-	case MENU_ATTACH:
-	  map = OpAttach;
-	  break;
-	case MENU_POST:
-	  map = OpPost;
-	  break;
-	case MENU_FOLDER:
-	  map = OpBrowser;
-	  break;
-	case MENU_COMPOSE:
-	  map = OpCompose;
-	  break;
-#ifdef _PGPPATH
-        case MENU_PGP:
-	  map = OpPgp;
-	  break;
-#endif
-        case MENU_ALIAS:
-	  map = OpAlias;
-	  break;
-	
-        default:
-	  return NULL;
-      }
-
+    /* first look in the generic map for the function */
+    for (i = 0; OpGeneric[i].name; i++)
+      if (OpGeneric[i].op == op)
+	return (&OpGeneric[i]);    
   }
-  for (i=0; map[i].name; i++)
-    if (map[i].op == op)
-      return (&map[i]);
+
+  if ((map = km_get_table(menu)))
+  {
+    for (i = 0; map[i].name; i++)
+      if (map[i].op == op)
+	return (&map[i]);
+  }
+  
   return (NULL);
 }
 
@@ -83,26 +54,59 @@ void mutt_make_help (char *d, size_t dlen, char *txt, int menu, int op)
 {
   char buf[SHORT_STRING];
 
-  if (km_expand_key (buf, sizeof(buf), km_find_func (menu, op)))
+  if (km_expand_key (buf, sizeof (buf), km_find_func (menu, op)) ||
+      km_expand_key (buf, sizeof (buf), km_find_func (MENU_GENERIC, op)))
     snprintf (d, dlen, "%s:%s", buf, txt);
   else
     d[0] = 0;
 }
 
+char *
+mutt_compile_help (char *buf, size_t buflen, int menu, struct mapping_t *items)
+{
+  int i;
+  size_t len;
+  char *pbuf = buf;
+  
+  for (i = 0; items[i].name && buflen > 2; i++)
+  {
+    if (i)
+    {
+      *pbuf++ = ' ';
+      *pbuf++ = ' ';
+      buflen -= 2;
+    }
+    mutt_make_help (pbuf, buflen, items[i].name, menu, items[i].value);
+    len = strlen (pbuf);
+    pbuf += len;
+    buflen -= len;
+  }
+  return buf;
+}
+
 static void print_macro (FILE *f, const char *macro)
 {
-  for (; *macro; macro++)
+  int i;
+
+  for (i = 0; *macro && i < COLS - 34; macro++, i++)
   {
     switch (*macro)
     {
+      case '\033':
+	fputs ("\\e", f);
+	i++;
+	break;
       case '\n':
 	fputs ("\\n", f);
+	i++;
 	break;
       case '\r':
 	fputs ("\\r", f);
+	i++;
 	break;
       case '\t':
 	fputs ("\\t", f);
+	i++;
 	break;
       default:
 	fputc (*macro, f);
@@ -111,70 +115,16 @@ static void print_macro (FILE *f, const char *macro)
   }
 }
 
-void mutt_help (int menu)
+static void dump_menu (FILE *f, int menu)
 {
-  char t[_POSIX_PATH_MAX];
-  char buf[SHORT_STRING];
-  char *desc;
   struct keymap_t *map;
   struct binding_t *b;
-  FILE *f;
-
-  mutt_mktemp (t);
-  if ((f = fopen (t, "w")) == NULL)
-  {
-    mutt_perror (t);
-    return;
-  }
-
-  switch (menu)
-  {
-    case MENU_PAGER:
-      desc = "pager";
-      break;
-
-    case MENU_MAIN:
-      desc = "index";
-      break;
-
-    case MENU_ATTACH:
-      desc = "attach";
-      break;
-
-    case MENU_COMPOSE:
-      desc = "compose";
-      break;
-
-    case MENU_FOLDER:
-      desc = "browser";
-      break;
-
-    case MENU_ALIAS:
-      desc = "alias";
-      break;
-
-    case MENU_URL:
-      desc = "url";
-      break;
-
-    case MENU_POST:
-      desc = "postponed";
-      break;
-
-    case MENU_PGP:
-      desc = "pgp";
-      break;
-
-    default:
-      desc = "<UNKNOWN MENU>";
-  }
-
-  map = Keymaps[menu];
+  char buf[SHORT_STRING];
 
   /* browse through the keymap table */
-  while (map)
+  for (map = Keymaps[menu]; map; map = map->next)
   {
-    km_expand_key (buf, sizeof(buf), map);
+    km_expand_key (buf, sizeof (buf), map);
 
     if (map->op == OP_MACRO)
     {
@@ -189,12 +139,67 @@ void mutt_help (int menu)
 	      b ? b->name : "UNKNOWN",
 	      b ? HelpStrings[b->op] : "ERROR: please report this bug");
     }
-
-    map = map->next;
   }
+}
+
+static int is_bound (struct keymap_t *map, int op)
+{
+  for (; map; map = map->next)
+    if (map->op == op)
+      return 1;
+  return 0;
+}
+
+static void dump_unbound (FILE *f,
+			  struct binding_t *funcs,
+			  struct keymap_t *map,
+			  struct keymap_t *aux)
+{
+  int i;
+
+  for (i = 0; funcs[i].name; i++)
+  {
+    if (! is_bound (map, funcs[i].op) &&
+	(!aux || ! is_bound (aux, funcs[i].op)))
+      fprintf (f, "%s\t\t%s\n", funcs[i].name, HelpStrings[funcs[i].op]);
+  }
+}
+
+void mutt_help (int menu)
+{
+  char t[_POSIX_PATH_MAX];
+  char buf[SHORT_STRING];
+  char *desc;
+  FILE *f;
+  struct binding_t *funcs;
+
+  mutt_mktemp (t);
+  if ((f = safe_fopen (t, "w")) == NULL)
+  {
+    mutt_perror (t);
+    return;
+  }
+
+  funcs = km_get_table (menu);
+  desc = mutt_getnamebyvalue (menu, Menus);
+  if (!desc)
+    desc = "<UNKNOWN>";
+  
+  dump_menu (f, menu);
+  if (menu != MENU_EDITOR && menu != MENU_PAGER)
+  {
+    fputs ("\nGeneric bindings:\n\n", f);
+    dump_menu (f, MENU_GENERIC);
+  }
+
+  fputs ("\nUnbound functions:\n\n", f);
+  if (funcs)
+    dump_unbound (f, funcs, Keymaps[menu], NULL);
+  if (menu != MENU_PAGER)
+    dump_unbound (f, OpGeneric, Keymaps[MENU_GENERIC], Keymaps[menu]);
 
   fclose (f);
 
   snprintf (buf, sizeof (buf), "Help for %s", desc);
-  mutt_do_pager (buf, t, 0);
+  mutt_do_pager (buf, t, 0, NULL);
 }
