@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mx.h"
+#include "mailbox.h"
 
 #include <unistd.h>
 #include <netinet/in.h>
@@ -146,7 +147,7 @@ static time_t imap_parse_date (char *s)
   if (s[0] == '+')
     tz = -tz;
 
-  return (mutt_mktime (&t) + tz);
+  return (mutt_mktime (&t, 0) + tz);
 }
 
 static int imap_parse_fetch (HEADER *h, char *s)
@@ -532,7 +533,7 @@ int imap_open_mailbox (CONTEXT *ctx)
 
   if (!ImapUser[0])
     strfcpy (ImapUser, Username, sizeof (ImapUser));
-  if (ci_get_field ("IMAP Username: ", ImapUser, sizeof (ImapUser), 0) != 0 ||
+  if (mutt_get_field ("IMAP Username: ", ImapUser, sizeof (ImapUser), 0) != 0 ||
       !ImapUser[0])
   {
     ImapUser[0] = 0;
@@ -541,7 +542,7 @@ int imap_open_mailbox (CONTEXT *ctx)
 
   snprintf (buf, sizeof (buf), "Password for %s@%s: ", ImapUser, host);
   ImapPass[0] = 0;
-  if (ci_get_field (buf, ImapPass, sizeof (ImapPass), M_PASS) != 0 ||
+  if (mutt_get_field (buf, ImapPass, sizeof (ImapPass), M_PASS) != 0 ||
       !ImapPass[0])
   {
     return (-1);
@@ -632,7 +633,7 @@ int imap_open_mailbox (CONTEXT *ctx)
   mutt_message ("Fetching message headers...");
   ctx->hdrmax = count;
   ctx->hdrs = safe_malloc (count * sizeof (HEADER *));
-  ctx->v2r = safe_malloc (count * sizeof (short));
+  ctx->v2r = safe_malloc (count * sizeof (int));
   for (ctx->msgcount = 0; ctx->msgcount < count; )
   {
     ctx->hdrs[ctx->msgcount] = mutt_new_header ();
@@ -899,7 +900,7 @@ int imap_close_mailbox (CONTEXT *ctx)
 }
 
 /* use the NOOP command to poll for new mail */
-int imap_check_mailbox (CONTEXT *ctx, unsigned short *index_hint)
+int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
 {
   char seq[8];
   char buf[LONG_STRING];
