@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -47,24 +47,22 @@ void km_error_key (int);
 
 enum
 {
+  MENU_ALIAS,
+  MENU_ATTACH,
+  MENU_COMPOSE,
+  MENU_EDITOR,
+  MENU_FOLDER,
+  MENU_GENERIC,
   MENU_MAIN,
   MENU_PAGER,
-  MENU_ATTACH,
-  MENU_FOLDER,
-  MENU_ALIAS,
-  MENU_COMPOSE,
   MENU_POST,
-  MENU_URL,
-  MENU_PGP,
-  MENU_EDITOR,
-  MENU_MAX
-};
 
-struct binding_t
-{
-  char *name;	/* name of the function */
-  short op;	/* function id number */
-  char *seq;	/* default key binding */
+
+
+
+
+
+  MENU_MAX
 };
 
 /* the keymap trees (one for each menu) */
@@ -73,6 +71,17 @@ extern struct keymap_t *Keymaps[];
 /* dokey() records the last real key pressed  */
 extern int LastKey;
 
+extern struct mapping_t Menus[];
+
+struct binding_t
+{
+  char *name;	/* name of the function */
+  int op;	/* function id number */
+  char *seq;	/* default key binding */
+};
+
+struct binding_t *km_get_table (int menu);
+
 extern struct binding_t OpGeneric[];
 extern struct binding_t OpPost[];
 extern struct binding_t OpMain[];
@@ -81,10 +90,7 @@ extern struct binding_t OpPager[];
 extern struct binding_t OpCompose[];
 extern struct binding_t OpBrowser[];
 extern struct binding_t OpEditor[];
-extern struct binding_t OpAlias[];
-#ifdef _PGPPATH
-extern struct binding_t OpPgp[];
-#endif /* _PGPPATH */
+
 
 #include "keymap_defs.h"
 
