@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  */ 
 
 #include "mutt.h"
-#include "mutt_regex.h"
 #include "mutt_curses.h"
+#include "mime.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -99,6 +99,25 @@ void mutt_free_parameter (PARAMETER **p)
   *p = 0;
 }
 
+LIST *mutt_add_list (LIST *head, const char *data)
+{
+  LIST *tmp;
+
+  for (tmp = head; tmp && tmp->next; tmp = tmp->next)
+    ;
+  if (tmp)
+  {
+    tmp->next = safe_malloc (sizeof (LIST));
+    tmp = tmp->next;
+  }
+  else
+    head = tmp = safe_malloc (sizeof (LIST));
+
+  tmp->data = safe_strdup (data);
+  tmp->next = NULL;
+  return head;
+}
+
 void mutt_free_list (LIST **list)
 {
   LIST *p;
@@ -134,12 +153,10 @@ void mutt_free_header (HEADER **h)
 /* returns true if the header contained in "s" is in list "t" */
 int mutt_matches_ignore (const char *s, LIST *t)
 {
-  while (t)
+  for (; t; t = t->next)
   {
-    if (!strncasecmp (s, t->data, strlen (t->data)) || *t->data == '*' ||
-	(strcasecmp ("From_", t->data) == 0 && strncasecmp ("From ", s, 5) == 0))
-      return (1);
-    t = t->next;
+    if (!strncasecmp (s, t->data, strlen (t->data)) || *t->data == '*')
+      return 1;
   }
   return 0;
 }
@@ -220,40 +237,6 @@ char *mutt_expand_path (char *s, size_t slen)
   return (s);
 }
 
-/* returns TRUE if the given address belongs to the user. */
-int mutt_addr_is_user (ADDRESS *addr)
-{
-  char buf[LONG_STRING];
-
-  /* NULL address is assumed to be the user. */
-  if (!addr)
-    return 1;
-
-  if (!addr->mailbox)
-    return 0;
-
-  if (!addr->host || *addr->host == '@')
-  {
-    if (strcasecmp (addr->mailbox, Username) == 0)
-      return (1);
-    return (0);
-  }
-
-  if (strcasecmp (addr->mailbox, Username) == 0 &&
-      (strcasecmp (addr->host, Hostname) == 0 ||
-       strcasecmp (addr->host, Fqdn) == 0))
-    return 1;
-
-  if (Alternates.pattern)
-  {
-    buf[0] = 0;
-    mutt_simple_address (buf, sizeof (buf), addr);
-    if (regexec ((regex_t *) Alternates.rx, buf, 0, NULL, 0) == 0)
-      return (1);
-  }
-  return (0);
-}
-
 void *safe_calloc (size_t nmemb, size_t size)
 {
   void *p;
@@ -337,10 +320,10 @@ char *safe_strdup (const char *s)
   return (p);
 }
 
-char *skip_whitespace (char *p)
+char *mutt_skip_whitespace (char *p)
 {
-  while (*p && (*p == ' ' || *p == '\t')) p++;
-  return (p);
+  SKIPWS (p);
+  return p;
 }
 
 int mutt_copy_bytes (FILE *in, FILE *out, size_t size)
@@ -364,22 +347,6 @@ int mutt_copy_bytes (FILE *in, FILE *out, size_t size)
   return 0;
 }
 
-void mutt_free_address (ADDRESS **p)
-{
-  ADDRESS *t;
-
-  while (*p)
-  {
-    t = *p;
-    safe_free ((void **) &t->personal);
-    safe_free ((void **) &t->mailbox);
-    safe_free ((void **) &t->adl);
-    safe_free ((void **) &t->host);
-    *p = (*p)->next;
-    safe_free ((void **) &t);
-  }
-}
-
 char *mutt_get_parameter (const char *s, PARAMETER *p)
 {
   for (; p; p = p->next)
@@ -402,15 +369,6 @@ int mutt_needs_mailcap (BODY *m)
 	return 0;
       break;
 
-#ifdef _PGPPATH
-    case TYPEAPPLICATION:
-
-      if (!strcasecmp ("pgp", m->subtype) ||
-	  !strcasecmp ("pgp-signed", m->subtype) ||
-	  !strcasecmp ("x-pgp-message", m->subtype))
-	return 0;
-      break;
-#endif /* _PGPPATH */
 
     case TYPEMULTIPART:
     case TYPEMESSAGE:
@@ -432,13 +390,6 @@ int mutt_is_text_type (int t, char *s)
       return 1;
   }
 
-#ifdef _PGPPATH
-  if (t == TYPEAPPLICATION)
-  {
-    if (!strcasecmp ("pgp-keys", s))
-      return 1;
-  }
-#endif /* _PGPPATH */
 
   return 0;
 }
@@ -446,13 +397,14 @@ int mutt_is_text_type (int t, char *s)
 void mutt_free_envelope (ENVELOPE **p)
 {
   if (!*p) return;
-  mutt_free_address (&(*p)->return_path);
-  mutt_free_address (&(*p)->to);
-  mutt_free_address (&(*p)->cc);
-  mutt_free_address (&(*p)->bcc);
-  mutt_free_address (&(*p)->sender);
-  mutt_free_address (&(*p)->from);
-  mutt_free_address (&(*p)->reply_to);
+  rfc822_free_address (&(*p)->return_path);
+  rfc822_free_address (&(*p)->to);
+  rfc822_free_address (&(*p)->cc);
+  rfc822_free_address (&(*p)->bcc);
+  rfc822_free_address (&(*p)->sender);
+  rfc822_free_address (&(*p)->from);
+  rfc822_free_address (&(*p)->reply_to);
+  rfc822_free_address (&(*p)->mail_followup_to);
   safe_free ((void **) &(*p)->subject);
   safe_free ((void **) &(*p)->message_id);
   mutt_free_list (&(*p)->references);
@@ -504,35 +456,6 @@ int mutt_strcmp (const char *s1, const char *s2)
     return ((s2 == NULL) ? 0 : -1);
 }
 
-/* write an address without the personal name */
-void mutt_simple_address (char *buf, size_t buflen, ADDRESS *addr)
-{
-  int len;
-  
-  *buf = 0;
-  if (!addr || !addr->mailbox)
-    return;
-  strfcpy (buf, addr->mailbox, buflen);
-  len = strlen (buf);
-  buflen -= len;
-  buf += len;
-  if (addr->host && *addr->host != '@')
-    snprintf (buf, buflen, "@%s", addr->host);
-}
-
-/* returns 1 if ALL addresses in list are local */
-int is_local_site (ADDRESS *addr)
-{
-  for (; addr; addr = addr->next)
-  {
-    if (!addr->host || *addr->host == '@' || strcasecmp (addr->host, Fqdn) == 0)
-      continue;
-    if (!Localsites.pattern || regexec ((regex_t *) Localsites.rx, addr->host, 0, NULL, 0) != 0)
-      return (0);
-  }
-  return (1);
-}
-
 void mutt_free_alias (ALIAS **p)
 {
   ALIAS *t;
@@ -542,7 +465,7 @@ void mutt_free_alias (ALIAS **p)
     t = *p;
     *p = (*p)->next;
     safe_free ((void **) &t->name);
-    mutt_free_address (&t->addr);
+    rfc822_free_address (&t->addr);
     free (t);
   }
 }
@@ -730,7 +653,7 @@ int mutt_check_overwrite (const char *attname, const char *path,
     if (!attname || !attname[0])
     {
       tmp[0] = 0;
-      if (ci_get_field ("File under directory: ", tmp, sizeof (tmp),
+      if (mutt_get_field ("File under directory: ", tmp, sizeof (tmp),
 				      M_FILE | M_CLEAR) != 0 || !tmp[0])
 	return (-1);
       snprintf (fname, flen, "%s/%s", path, tmp);
@@ -763,7 +686,7 @@ void mutt_pretty_size (char *s, size_t len, long n)
     strfcpy (s, "0.1K", len);
   else if (n < 10189) /* 0.1K - 9.9K */
     snprintf (s, len, "%3.1fK", n / 1024.0);
-  else if (n < 1024000) /* 10K - 1000K */
+  else if (n < 1023949) /* 10K - 999K */
   {
     /* 51 is magic which causes 10189/10240 to be rounded up to 10 */
     snprintf (s, len, "%ldK", (n + 51) / 1024);
@@ -781,16 +704,75 @@ void mutt_safe_path (char *s, size_t l, ADDRESS *a)
 {
   char *p;
 
-  if (option (OPTSAVEADDRESS))
-    mutt_simple_address (s, l, a);
-  else
+  if (a && a->mailbox)
   {
     strfcpy (s, a->mailbox, l);
-    if ((p = strchr (s, '%')))
+    if (!option (OPTSAVEADDRESS))
+    {
+    if ((p = strpbrk (s, "%@")))
       *p = 0;
+    }
   }
+  else
+    *s = 0;
   mutt_strlower (s);
   for (p = s; *p; p++)
     if (*p == '/' || ISSPACE (*p) || !IsPrint ((unsigned char) *p))
       *p = '_';
 }
+
+/* Read a line from ``fp'' into the dynamically allocated ``s'',
+ * increasing ``s'' if necessary. The ending "\n" or "\r\n" is removed.
+ * If a line ends with "\", this char and the linefeed is removed,
+ * and the next line is read too.
+ */
+char *mutt_read_line (char *s, size_t *size, FILE *fp, int *line)
+{
+  size_t offset = 0;
+  char *ch;
+
+  if (!s)
+  {
+    s = safe_malloc (STRING);
+    *size = STRING;
+  }
+
+  FOREVER
+  {
+    if (fgets (s + offset, *size - offset, fp) == NULL)
+    {
+      free (s);
+      return NULL;
+    }
+    if ((ch = strchr (s + offset, '\n')) != NULL)
+    {
+      (*line)++;
+      *ch = 0;
+      if (ch > s && *(ch - 1) == '\r')
+	*--ch = 0;
+      if (ch == s || *(ch - 1) != '\\')
+	return s;
+      offset = ch - s - 1;
+    }
+    else
+    {
+      /* There wasn't room for the line -- increase ``s'' */
+      offset = *size - 1; /* overwrite the terminating 0 */
+      *size += STRING;
+      safe_realloc ((void **) &s, *size);
+    }
+  }
+}
+
+char *
+mutt_substrcpy (char *dest, const char *beg, const char *end, size_t destlen)
+{
+  size_t len;
+
+  len = end - beg;
+  if (len > destlen - 1)
+    len = destlen - 1;
+  memcpy (dest, beg, len);
+  dest[len] = 0;
+  return dest;
+}
