@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "keymap.h"
-#include "send.h"
-#include "version.h"
+#include "mailbox.h"
+#include "reldate.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -33,13 +33,13 @@
 #include <sys/utsname.h>
 
 const char Notice[] = "\
-Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>\n\
+Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>\n\
 Mutt comes with ABSOLUTELY NO WARRANTY; for details type `mutt -vv'.\n\
 Mutt is free software, and you are welcome to redistribute it\n\
 under certain conditions; type `mutt -vv' for details.\n";
 
 const char Copyright[] = "\
-Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>\n\
+Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>\n\
 \n\
     This program is free software; you can redistribute it and/or modify\n\
     it under the terms of the GNU General Public License as published by\n\
@@ -63,30 +63,33 @@ void mutt_exit (int code)
 
 static void mutt_usage (void)
 {
-  puts ("Usage:\n\
-mutt [ -nzZ ] [ -F <muttrc> ] [ -m <type> ] [ -f <mailbox> ]\n\
-mutt [ -nx ] [ -a <file> ] [ -F <muttrc> ] [ -H <file> ] [ -i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
-mutt [ -n ] [ -F <muttrc> ] -p\n\
-mutt -v\n\
+  printf ("Mutt %s (%s)\n", VERSION, ReleaseDate);
+  puts (
+"usage: mutt [ -nRzZ ] [ -e <cmd> ] [ -F <file> ] [ -m <type> ] [ -f <file> ]\n\
+       mutt [ -nx ] [ -e <cmd> ] [ -a <file> ] [ -F <file> ] [ -H <file> ] [ -i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
+       mutt [ -n ] [ -e <cmd> ] [ -F <file> ] -p\n\
+       mutt -v[v]\n\
 \n\
--a <file>\tattach a file to the message\n\
--b <address>\tspecify a blind carbon-copy (BCC) address\n\
--c <address>\tspecify a carbon-copy (CC) address\n\
--f <mailbox>\tspecify which mailbox to read\n\
--F <muttrc>\tspecify an alternate muttrc file\n\
--H <file>\tspecify a draft file to read header from\n\
--i <file>\tspecify a file which Mutt should include in the reply\n\
--m <type>\tspecify a default mailbox type\n\
--n\t\tcauses Mutt not to read the system Muttrc\n\
--p\t\trecall a postponed message\n\
--R\t\topen mailbox in read-only mode\n\
--s <subj>\tspecify a subject (must be in quotes if it has spaces)\n\
--v\t\tshow version and compile-time definitions\n\
--x\t\tsimulate the mailx send mode\n\
--y\t\tselect a mailbox specified in your `mailboxes' list\n\
--z\t\texit immediately if there are no messages in the mailbox\n\
--Z\t\topen the first folder with new message, exit immediately if none\n\
--h\t\tthis help message");
+options:\n\
+  -a <file>\tattach a file to the message\n\
+  -b <address>\tspecify a blind carbon-copy (BCC) address\n\
+  -c <address>\tspecify a carbon-copy (CC) address\n\
+  -e <command>\tspecify a command to be executed after initialization\n\
+  -f <file>\tspecify which mailbox to read\n\
+  -F <file>\tspecify an alternate muttrc file\n\
+  -H <file>\tspecify a draft file to read header from\n\
+  -i <file>\tspecify a file which Mutt should include in the reply\n\
+  -m <type>\tspecify a default mailbox type\n\
+  -n\t\tcauses Mutt not to read the system Muttrc\n\
+  -p\t\trecall a postponed message\n\
+  -R\t\topen mailbox in read-only mode\n\
+  -s <subj>\tspecify a subject (must be in quotes if it has spaces)\n\
+  -v\t\tshow version and compile-time definitions\n\
+  -x\t\tsimulate the mailx send mode\n\
+  -y\t\tselect a mailbox specified in your `mailboxes' list\n\
+  -z\t\texit immediately if there are no messages in the mailbox\n\
+  -Z\t\topen the first folder with new message, exit immediately if none\n\
+  -h\t\tthis help message");
 
   exit (0);
 }
@@ -95,17 +98,17 @@ static void show_version (void)
 {
   struct utsname uts;
 
-  printf ("%s, ", MuttVersion);
+  printf ("Mutt %s (%s)\n", VERSION, ReleaseDate);
   puts (Notice);
 
   uname (&uts);
 
 #ifdef _AIX
-  printf ("System info: %s %s.%s", uts.sysname, uts.version, uts.release);
+  printf ("System: %s %s.%s", uts.sysname, uts.version, uts.release);
 #elif defined (SCO)
-  printf ("System info: SCO %s", uts.release);
+  printf ("System: SCO %s", uts.release);
 #else
-  printf ("System info: %s %s", uts.sysname, uts.release);
+  printf ("System: %s %s", uts.sysname, uts.release);
 #endif
 
 #ifdef NCURSES_VERSION
@@ -114,7 +117,7 @@ static void show_version (void)
   printf (" [using slang %d]", SLANG_VERSION);
 #endif
 
-  puts ("\n\nCompile time definitions:");
+  puts ("\nCompile options:");
 
 #ifdef DOMAIN
   printf ("DOMAIN=\"%s\"\n", DOMAIN);
@@ -122,72 +125,92 @@ static void show_version (void)
   puts ("-DOMAIN");
 #endif
 
+  puts (
 #ifdef HIDDEN_HOST
-  fputs ("+HIDDEN_HOST  ", stdout);
+	"+HIDDEN_HOST  "
 #else
-  fputs ("-HIDDEN_HOST  ", stdout);
+	"-HIDDEN_HOST  "
 #endif
 
 #ifdef HOMESPOOL
-  fputs ("+HOMESPOOL  ", stdout);
+	"+HOMESPOOL  "
 #else
-  fputs ("-HOMESPOOL  ", stdout);
+	"-HOMESPOOL  "
 #endif
- 
+
 #ifdef USE_SETGID
-  fputs ("+USE_SETGID  ", stdout);
+	"+USE_SETGID  "
 #else
-  fputs ("-USE_SETGID  ", stdout);
+	"-USE_SETGID  "
 #endif
 
 #ifdef USE_DOTLOCK
-  fputs ("+USE_DOTLOCK  ", stdout);
+	"+USE_DOTLOCK  "
 #else
-  fputs ("-USE_DOTLOCK  ", stdout);
+	"-USE_DOTLOCK  "
 #endif
 
 #ifdef USE_FCNTL
-  fputs ("+USE_FCNTL  ", stdout);
+	"+USE_FCNTL  "
 #else
-  fputs ("-USE_FCNTL  ", stdout);
+	"-USE_FCNTL  "
 #endif
 
 #ifdef USE_FLOCK
-  puts ("+USE_FLOCK");
+	"+USE_FLOCK"
 #else
-  puts ("-USE_FLOCK");
+	"-USE_FLOCK"
 #endif
+	);
 
+  puts (
 #ifdef USE_POP
-  fputs ("+USE_POP  ", stdout);
+	"+USE_POP  "
 #else
-  fputs ("-USE_POP  ", stdout);
+	"-USE_POP  "
 #endif
 
 #ifdef HAVE_REGCOMP
-  fputs ("+HAVE_REGCOMP  ", stdout);
+	"+HAVE_REGCOMP  "
 #else
-  fputs ("-HAVE_REGCOMP  ", stdout);
+	"-HAVE_REGCOMP  "
 #endif
 
 #ifdef USE_GNU_RX
-  fputs ("+USE_GNU_RX  ", stdout);
+	"+USE_GNU_RX  "
 #else
-  fputs ("-USE_GNU_RX  ", stdout);
+	"-USE_GNU_RX  "
 #endif
 
 #ifdef HAVE_COLOR
-  fputs ("+HAVE_COLOR  ", stdout);
+	"+HAVE_COLOR  "
 #else
-  fputs ("-HAVE_COLOR  ", stdout);
+	"-HAVE_COLOR  "
+#endif
+
+#ifdef _PGPPATH
+#ifdef HAVE_PGP5
+	"+HAVE_PGP5  "
+#endif
+#ifdef HAVE_PGP2
+	"+HAVE_PGP2  "
+#endif
 #endif
 
 #ifdef BUFFY_SIZE
-  puts ("+BUFFY_SIZE");
+	"+BUFFY_SIZE  "
 #else
-  puts ("-BUFFY_SIZE");
+	"-BUFFY_SIZE  "
 #endif
 
+#ifdef EXACT_ADDRESS
+	"+"
+#else
+	"-"
+#endif
+	"EXACT_ADDRESS"
+	);
+
   printf ("SENDMAIL=\"%s\"\n", SENDMAIL);
   printf ("MAILPATH=\"%s\"\n", MAILPATH);
   printf ("SHAREDIR=\"%s\"\n", SHAREDIR);
@@ -200,9 +223,15 @@ static void show_version (void)
 
 #ifdef _PGPPATH
   printf ("_PGPPATH=\"%s\"\n", _PGPPATH);
+# ifdef _PGPV2PATH
+  printf ("_PGPV2PATH=\"%s\"\n", _PGPV2PATH);
+# endif
+# ifdef _PGPV3PATH
+  printf ("_PGPV3PATH=\"%s\"\n", _PGPV3PATH);
+# endif
 #endif
 
-  mutt_print_patchlist ();
+  puts ("\nMail bug reports along with this output to <mutt-users@cs.hmc.edu>.");
 
   exit (0);
 }
@@ -213,6 +242,7 @@ static void start_curses (void)
 
 #ifdef USE_SLANG_CURSES
   SLtt_Ignore_Beep = 1; /* don't do that #*$@^! annoying visual beep! */
+  SLsmg_Display_Eight_Bit = 128; /* characters above this are printable */
 #endif
   if (initscr () == NULL)
   {
@@ -238,6 +268,26 @@ static void start_curses (void)
 #define M_RO      (1<<3)	/* -R */
 #define M_SELECT  (1<<4)	/* -y */
 
+#ifdef USE_SLANG_CURSES
+
+void check_slang(void)
+{
+  char *term = getenv("TERM");
+
+  /* slang < 1.2x */
+  
+  if(SLANG_VERSION < 12000)
+  {
+    if(term && strlen(term) > 100)
+    {
+      fputs("Sorry, your TERM variable's value is too long.\n", stderr);
+      exit(1);
+    }
+  }
+}
+
+#endif
+
 int main (int argc, char **argv)
 {
   char folder[_POSIX_PATH_MAX] = "";
@@ -247,6 +297,7 @@ int main (int argc, char **argv)
   char *newMagic = NULL;
   HEADER *msg = NULL;
   LIST *attach = NULL;
+  LIST *commands = NULL;
   int sendflags = 0;
   int flags = 0;
   int version = 0;
@@ -254,9 +305,6 @@ int main (int argc, char **argv)
   extern char *optarg;
   extern int optind;
 
-  SRAND (time (NULL));
-  setlocale (LC_CTYPE, "");
-  umask (077);
 
 #ifdef USE_SETGID
   /* Determine the user's default gid and the gid to use for locking the spool
@@ -280,48 +328,43 @@ int main (int argc, char **argv)
   }
 #endif
 
+#ifdef USE_SLANG_CURSES
+  
+  check_slang();
+  
+#endif
+  
+  SRAND (time (NULL));
+  setlocale (LC_CTYPE, "");
+  umask (077);
+
   memset (Options, 0, sizeof (Options));
 
-  while ((i = getopt (argc, argv, "a:b:F:f:c:d:H:s:i:hm:npRvxyzZ")) != EOF)
+  while ((i = getopt (argc, argv, "a:b:F:f:c:d:e:H:s:i:hm:npRvxyzZ")) != EOF)
     switch (i)
     {
       case 'a':
-	{
-	  LIST *t;
-
-	  t = attach;
-	  while (t && t->next)
-	    t = t->next;
-	  if (t)
-	  {
-	    t->next = mutt_new_list ();
-	    t = t->next;
-	  }
-	  else
-	    attach = t = mutt_new_list ();
-	  t->data = safe_strdup (optarg);
-	}
+	attach = mutt_add_list (attach, optarg);
 	break;
 
       case 'F':
-
 	strfcpy (Muttrc, optarg, sizeof (Muttrc));
 	break;
 
       case 'f':
-
 	strfcpy (folder, optarg, sizeof (folder));
 	break;
 
       case 'b':
       case 'c':
-
 	if (!msg)
 	  msg = mutt_new_header ();
 	if (!msg->env)
 	  msg->env = mutt_new_envelope ();
-	rfc822_parse_adrlist (i == 'b' ? &msg->env->bcc : &msg->env->cc,
-			      optarg, "@");
+	if (i == 'b')
+	  msg->env->bcc = rfc822_parse_adrlist (msg->env->bcc, optarg);
+	else
+	  msg->env->cc = rfc822_parse_adrlist (msg->env->cc, optarg);
 	break;
 
       case 'd':
@@ -333,82 +376,73 @@ int main (int argc, char **argv)
 #endif
 	break;
 
-      case 'H':
+      case 'e':
+	commands = mutt_add_list (commands, optarg);
+	break;
 
+      case 'H':
 	draftFile = optarg;
 	break;
 
       case 'i':
-
 	includeFile = optarg;
 	break;
 
       case 'm':
-
 	/* should take precedence over .muttrc setting, so save it for later */
 	newMagic = optarg; 
 	break;
 
       case 'n':
-
 	flags |= M_NOSYSRC;
 	break;
 
       case 'p':
-
 	sendflags |= SENDPOSTPONED;
 	break;
 
       case 'R':
-
 	flags |= M_RO; /* read-only mode */
 	break;
 
       case 's':
-
 	subject = optarg;
 	break;
 
       case 'v':
-
 	version++;
 	break;
 
       case 'x': /* mailx compatible send mode */
-
 	sendflags |= SENDMAILX;
 	break;
 
       case 'y': /* My special hack mode */
-
 	flags |= M_SELECT;
 	break;
 
       case 'z':
-
 	flags |= M_IGNORE;
 	break;
 
       case 'Z':
-
 	flags |= M_BUFFY | M_IGNORE;
 	break;
 
       default:
-
 	mutt_usage ();
     }
 
   switch (version)
   {
+    case 0:
+      break;
     case 1:
-
       show_version ();
       break;
-
-    case 2:
-
-      printf ("%s, %s", MuttVersion, Copyright);
+    default:
+      printf ("Mutt %s (%s)\n", VERSION, ReleaseDate);
+      puts (Copyright);
       exit (0);
   }
 
@@ -427,7 +461,8 @@ int main (int argc, char **argv)
     start_curses ();
 
   /* set defaults and read init files */
-  mutt_init (flags & M_NOSYSRC);
+  mutt_init (flags & M_NOSYSRC, commands);
+  mutt_free_list (&commands);
 
   if (newMagic)
     mx_set_magic (newMagic);
@@ -442,7 +477,7 @@ int main (int argc, char **argv)
   {
     if (!option (OPTNOCURSES))
       mutt_flushinp ();
-    ci_send_message (SENDPOSTPONED, NULL, NULL, NULL);
+    ci_send_message (SENDPOSTPONED, NULL, NULL, NULL, NULL);
     mutt_endwin (NULL);
   }
   else if (subject || msg || sendflags || draftFile || includeFile || attach ||
@@ -450,7 +485,7 @@ int main (int argc, char **argv)
   {
     FILE *fin = NULL;
     char buf[LONG_STRING];
-    char *tempfile = NULL;
+    char *tempfile = NULL, *infile = NULL;
 
     if (!option (OPTNOCURSES))
       mutt_flushinp ();
@@ -458,13 +493,15 @@ int main (int argc, char **argv)
     if (!msg)
       msg = mutt_new_header ();
 
-    if (!draftFile)
+    if (draftFile)
+      infile = draftFile;
+    else
     {
       if (!msg->env)
 	msg->env = mutt_new_envelope ();
 
       for (i = optind; i < argc; i++)
-	rfc822_parse_adrlist (&msg->env->to, argv[i], "@");
+	msg->env->to = rfc822_parse_adrlist (msg->env->to, argv[i]);
 
       if (!msg->env->to && !msg->env->cc)
       {
@@ -475,47 +512,47 @@ int main (int argc, char **argv)
       }
 
       msg->env->subject = safe_strdup (subject);
-
+      
       if (includeFile)
-      {
-	mutt_mktemp (buf);
-	tempfile = safe_strdup (buf);
-	fin = fopen (includeFile, "r");
-      }
+	infile = includeFile;
     }
-    else
+
+    if (infile)
     {
-      if (strcmp ("-", draftFile) == 0)
+      if (strcmp ("-", infile) == 0)
 	fin = stdin;
-      else if ((fin = fopen (draftFile, "r")) == NULL)
+      else if ((fin = fopen (infile, "r")) == NULL)
       {
 	if (!option (OPTNOCURSES))
 	  mutt_endwin (NULL);
-	perror (draftFile);
+	perror (infile);
 	exit (1);
       }
       mutt_mktemp (buf);
       tempfile = safe_strdup (buf);
 
-      msg->env = mutt_read_rfc822_header (fin, NULL);
-    }
+      if (draftFile)
+	msg->env = mutt_read_rfc822_header (fin, NULL);
 
-    if (tempfile)
-    {
-      FILE *fout;
-
-      if ((fout = fopen (tempfile, "w")) == NULL)
+      if (tempfile)
       {
-	if (!option (OPTNOCURSES))
-	  mutt_endwin (NULL);
-	perror (tempfile);
-	fclose (fin);
-	free (tempfile);
-      }
+	FILE *fout;
 
-      mutt_copy_stream (fin, fout);
-      fclose (fout);
-      fclose (fin);
+	if ((fout = safe_fopen (tempfile, "w")) == NULL)
+	{
+	  if (!option (OPTNOCURSES))
+	    mutt_endwin (NULL);
+	  perror (tempfile);
+	  fclose (fin);
+	  free (tempfile);
+	  exit (1);
+	}
+
+	mutt_copy_stream (fin, fout);
+	fclose (fout);
+	if (fin != stdin)
+	  fclose (fin);
+      }
     }
 
     if (attach)
@@ -545,7 +582,7 @@ int main (int argc, char **argv)
       mutt_free_list (&attach);
     }
 
-    ci_send_message (sendflags, msg, tempfile, NULL);
+    ci_send_message (sendflags, msg, tempfile, NULL, NULL);
 
     if (!option (OPTNOCURSES))
       mutt_endwin (NULL);
@@ -554,7 +591,7 @@ int main (int argc, char **argv)
   {
     if (flags & M_BUFFY)
     {
-      if (mutt_buffy_check () == 0)
+      if (!mutt_buffy_check (0))
       {
 	mutt_endwin ("No mailbox with new mail.");
 	exit (1);
