@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,10 @@
 /* This file contains code to parse ``mbox'' and ``mmdf'' style mailboxes */
 
 #include "mutt.h"
+#include "mailbox.h"
 #include "mx.h"
+#include "sort.h"
+#include "copy.h"
 
 #include <sys/stat.h>
 #include <dirent.h>
@@ -91,7 +94,12 @@ static int can_dotlock (const char *path)
 }
 #endif
 
-int mbox_lock_mailbox (CONTEXT *ctx, int excl)
+/* parameters:
+ * ctx - context to lock
+ * excl - exclusive lock?
+ * retry - should retry if unable to lock?
+ */
+int mbox_lock_mailbox (CONTEXT *ctx, int excl, int retry)
 {
   int r = 0;
 
@@ -115,7 +123,7 @@ int mbox_lock_mailbox (CONTEXT *ctx, int excl)
 #endif /* USE_SETGID */
 #endif /* USE_DOTLOCK */
 
-  if ((r = mx_lock_file (ctx->path, fileno (ctx->fp), excl, r)) == 0)
+  if ((r = mx_lock_file (ctx->path, fileno (ctx->fp), excl, r, retry)) == 0)
     ctx->locked = 1;
 
 #ifdef USE_SETGID
@@ -155,7 +163,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
   char buf[HUGE_STRING];
   char return_path[LONG_STRING];
   int lines;
-  time_t t;
+  time_t t, tz;
   long loc, tmploc;
   HEADER *hdr;
   struct stat sb;
@@ -180,6 +188,10 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
   }
 #endif
 
+  /* precompute the local timezone to speed up calculation of the
+     received time */
+  tz = mutt_local_tz ();
+
   buf[sizeof (buf) - 1] = 0;
   FOREVER
   {
@@ -208,7 +220,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
       if (!t)
 	fseek (ctx->fp, loc, 0);
       else
-	hdr->received = t;
+	hdr->received = t + tz;
 
       hdr->env = mutt_read_rfc822_header (ctx->fp, hdr);
 
@@ -249,7 +261,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
       }
 
       if (!hdr->env->return_path && return_path[0])
-	rfc822_parse_adrlist (&hdr->env->return_path, return_path, "@");
+	hdr->env->return_path = rfc822_parse_adrlist (hdr->env->return_path, return_path);
 
       if (!hdr->env->from)
 	hdr->env->from = rfc822_cpy_adr (hdr->env->return_path);
@@ -278,7 +290,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
   struct stat sb;
   char buf[HUGE_STRING], return_path[STRING];
   HEADER *curhdr;
-  time_t t;
+  time_t t, tz;
   int count = 0, lines = 0;
   long loc;
 #ifdef NFS_ATTRIBUTE_HACK
@@ -307,8 +319,12 @@ int mbox_parse_mailbox (CONTEXT *ctx)
   if (!ctx->readonly)
     ctx->readonly = access (ctx->path, W_OK) ? 1 : 0;
 
+  /* precompute the local timezone to speed up calculation of the
+     date received */
+  tz = mutt_local_tz ();
+
   loc = ftell (ctx->fp);
-  while (fgets (buf, LONG_STRING, ctx->fp) != NULL)
+  while (fgets (buf, sizeof (buf), ctx->fp) != NULL)
   {
     if ((t = is_from (buf, return_path, sizeof (return_path))))
     {
@@ -337,7 +353,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 	mx_alloc_memory (ctx);
       
       curhdr = ctx->hdrs[ctx->msgcount] = mutt_new_header ();
-      curhdr->received = t;
+      curhdr->received = t + tz;
       curhdr->offset = loc;
       curhdr->index = ctx->msgcount;
 	
@@ -404,7 +420,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
       mx_update_context (ctx);
 
       if (!curhdr->env->return_path && return_path[0])
-	rfc822_parse_adrlist (&curhdr->env->return_path, return_path, "@");
+	curhdr->env->return_path = rfc822_parse_adrlist (curhdr->env->return_path, return_path);
 
       if (!curhdr->env->from)
 	curhdr->env->from = rfc822_cpy_adr (curhdr->env->return_path);
@@ -452,7 +468,7 @@ int mbox_open_mailbox (CONTEXT *ctx)
     return (-1);
   }
   mutt_block_signals ();
-  if (mbox_lock_mailbox (ctx, 0) == -1)
+  if (mbox_lock_mailbox (ctx, 0, 1) == -1)
   {
     mutt_unblock_signals ();
     return (-1);
@@ -476,9 +492,7 @@ static int strict_addrcmp (const ADDRESS *a, const ADDRESS *b)
   while (a && b)
   {
     if (mutt_strcmp (a->mailbox, b->mailbox) ||
-	mutt_strcmp (a->host, b->host) ||
-	mutt_strcmp (a->personal, b->personal) ||
-	mutt_strcmp (a->adl, b->adl))
+	mutt_strcmp (a->personal, b->personal))
       return (0);
 
     a = a->next;
@@ -589,16 +603,16 @@ int mbox_strict_cmp_headers (const HEADER *h1, const HEADER *h2)
   }
 }
 
-/*
- * check to see if the mailbox has changed on disk.
+/* check to see if the mailbox has changed on disk.
  *
  * return values:
- *	2	mailbox has been reopened
- *	1	new mail has arrived!
- *	0	no change
- *	-1	error
+ *	M_REOPENED	mailbox has been reopened
+ *	M_NEW_MAIL	new mail has arrived!
+ *	M_LOCKED	couldn't lock the file
+ *	0		no change
+ *	-1		error
  */
-int mbox_check_mailbox (CONTEXT *ctx, unsigned short *index_hint)
+int mbox_check_mailbox (CONTEXT *ctx, int *index_hint)
 {
   struct stat st;
   char buffer[LONG_STRING];
@@ -623,10 +637,14 @@ int mbox_check_mailbox (CONTEXT *ctx, unsigned short *index_hint)
       if (!ctx->locked)
       {
 	mutt_block_signals ();
-	if (mbox_lock_mailbox (ctx, 0) == -1)
+	if (mbox_lock_mailbox (ctx, 0, 0) == -1)
 	{
 	  mutt_unblock_signals ();
-	  return (-1);
+	  /* we couldn't lock the mailbox, but nothing serious happened:
+	   * probably the new mail arrived: no reason to wait till we can
+	   * parse it: we'll get it on the next pass
+	   *  */
+	  return (M_LOCKED);
 	}
 	unlock = 1;
       }
@@ -660,7 +678,7 @@ int mbox_check_mailbox (CONTEXT *ctx, unsigned short *index_hint)
 	    mutt_unblock_signals ();
 	  }
 
-	  return (1); /* signal that new mail arrived */
+	  return (M_NEW_MAIL); /* signal that new mail arrived */
 	}
 	else
 	  modified = 1;
@@ -684,7 +702,7 @@ int mbox_check_mailbox (CONTEXT *ctx, unsigned short *index_hint)
 	mbox_unlock_mailbox (ctx);
 	mutt_unblock_signals ();
       }
-      return (2);
+      return (M_REOPENED);
     }
   }
 
@@ -719,7 +737,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
   {
     save_sort = Sort;
     Sort = SORT_ORDER;
-    mutt_sort_headers (ctx);
+    mutt_sort_headers (ctx, 0);
   }
 
   /* need to open the file for writing in such a way that it does not truncate
@@ -734,7 +752,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
 
   mutt_block_signals ();
 
-  if (mbox_lock_mailbox (ctx, 1) == -1)
+  if (mbox_lock_mailbox (ctx, 1, 1) == -1)
   {
     mutt_unblock_signals ();
     mutt_error ("Unable to lock mailbox!");
@@ -742,10 +760,10 @@ int mbox_sync_mailbox (CONTEXT *ctx)
   }
 
   /* Check to make sure that the file hasn't changed on disk */
-  if ((i = mbox_check_mailbox (ctx, NULL)) > 0)
+  if ((i = mbox_check_mailbox (ctx, NULL)) == M_NEW_MAIL ||  i == M_REOPENED)
   {
-    /* new mail arrived */
-    need_sort = 1;
+    /* new mail arrived, or mailbox reopened */
+    need_sort = i;
     goto bail;
   }
   else if (i < 0)
@@ -799,7 +817,8 @@ int mbox_sync_mailbox (CONTEXT *ctx)
     {
       j++;
       if (!ctx->quiet && WriteInc && ((j % WriteInc) == 0 || j == 1))
-	mutt_message ("Writing messages... %d", j);
+	mutt_message ("Writing messages... %d (%d%%)", j,
+		      ftell (ctx->fp) / (ctx->size / 100 + 1));
 
       if (ctx->magic == M_MMDF)
       {
@@ -813,7 +832,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
        */
       newOffset[i - first].hdr = ftell (fp) + offset;
 
-      if (mx_copy_message (ctx->hdrs[i], ctx->fp, fp) == -1)
+      if (mutt_copy_message (fp, ctx, ctx->hdrs[i], 0, CH_FROM | CH_UPDATE | CH_UPDATE_LEN) == -1)
 	goto bail;
 
       /* Since messages could have been deleted, the offsets stored in memory
@@ -956,7 +975,9 @@ bail:  /* Come here in case of disaster */
   if (need_sort || save_sort != Sort)
   {
     Sort = save_sort;
-    mutt_sort_headers (ctx);
+    /* if the mailbox was reopened, the thread tree will be invalid so make
+     * sure to start threading from scratch.  */
+    mutt_sort_headers (ctx, (need_sort == M_REOPENED));
   }
 
   return (-1);
