@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -219,7 +219,7 @@ void menu_redraw_motion (MUTTMENU *menu)
     SETCOLOR (MT_COLOR_NORMAL);
     BKGDSET (MT_COLOR_NORMAL);
   }
-  menu->redraw = 0;
+  menu->redraw &= REDRAW_STATUS;
 }
 
 void menu_redraw_current (MUTTMENU *menu)
@@ -249,7 +249,31 @@ void menu_redraw_current (MUTTMENU *menu)
     SETCOLOR (MT_COLOR_NORMAL);
     BKGDSET (MT_COLOR_NORMAL);
   }
-  menu->redraw = 0;
+  menu->redraw &= REDRAW_STATUS;
+}
+
+void menu_check_recenter (MUTTMENU *menu)
+{
+  if (menu->current >= menu->top + menu->pagelen)
+  {
+    if (option (OPTMENUSCROLL))
+      menu->top = menu->current - menu->pagelen + 1;
+    else
+      menu->top += menu->pagelen * ((menu->current - menu->top) / menu->pagelen);
+    menu->redraw = REDRAW_INDEX;
+  }
+  else if (menu->current < menu->top)
+  {
+    if (option (OPTMENUSCROLL))
+      menu->top = menu->current;
+    else
+    {
+      menu->top -= menu->pagelen * ((menu->top + menu->pagelen - 1 - menu->current) / menu->pagelen);
+      if (menu->top < 0)
+	menu->top = 0;
+    }
+    menu->redraw = REDRAW_INDEX;
+  }
 }
 
 void menu_jump (MUTTMENU *menu)
@@ -257,57 +281,52 @@ void menu_jump (MUTTMENU *menu)
   int n;
   char buf[SHORT_STRING];
 
-  mutt_ungetch (LastKey);
-  buf[0] = 0;
-  if (ci_get_field ("Jump to: ", buf, sizeof (buf), 0) == 0 && buf[0])
+  if (menu->max)
   {
-    n = atoi (buf) - 1;
-    if (n >= 0 && n < menu->max)
+    mutt_ungetch (LastKey);
+    buf[0] = 0;
+    if (mutt_get_field ("Jump to: ", buf, sizeof (buf), 0) == 0 && buf[0])
     {
-      menu->oldcurrent = menu->current;
-      menu->current = n;
-      if (menu->current < menu->top)
+      n = atoi (buf) - 1;
+      if (n >= 0 && n < menu->max)
       {
-	while (menu->top > menu->current) menu->top -= menu->pagelen;
-	if (menu->top < 0) menu->top = 0;
-	menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-      }
-      else if (menu->current >= menu->top + menu->pagelen)
-      {
-	do {
-	  menu->top += menu->pagelen;
-	} while (menu->top + menu->pagelen < menu->current);
-	menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	menu->current = n;
+	menu->redraw = REDRAW_MOTION;
       }
       else
-	menu->redraw = REDRAW_MOTION;
+	mutt_error ("Invalid index number.");
     }
-    else
-      mutt_error ("Invalid index number.");
   }
+  else
+    mutt_error ("No entries.");
 }
 
 void menu_next_line (MUTTMENU *menu)
 {
-  if (menu->top < menu->max - 1)
+  if (menu->max)
   {
-    (menu->top)++;
-    if (menu->current < menu->top)
-      (menu->current)++;
-    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+    if (menu->top < menu->max - 1)
+    {
+      menu->top++;
+      if (menu->current < menu->top)
+	menu->current++;
+      menu->redraw = REDRAW_INDEX;
+    }
+    else
+      mutt_error ("You cannot scroll down farther.");
   }
   else
-    mutt_error ("You cannot scroll down farther.");
+    mutt_error ("No entries.");
 }
 
 void menu_prev_line (MUTTMENU *menu)
 {
   if (menu->top > 0)
   {
-    (menu->top)--;
+    menu->top--;
     if (menu->current >= menu->top + menu->pagelen)
-      (menu->current)--;
-    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+      menu->current--;
+    menu->redraw = REDRAW_INDEX;
   }
   else
     mutt_error ("You cannot scroll up farther.");
@@ -315,19 +334,25 @@ void menu_prev_line (MUTTMENU *menu)
 
 void menu_next_page (MUTTMENU *menu)
 {
-  if (menu->top + menu->pagelen <= menu->max - 1)
+  if (menu->max)
   {
-    menu->top += menu->pagelen;
-    menu->current = menu->top;
-    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-  }
-  else if (menu->current != menu->max - 1)
-  {
-    menu->current = menu->max - 1;
-    menu->redraw = REDRAW_INDEX;
+    if (menu->top + menu->pagelen < menu->max)
+    {
+      menu->top += menu->pagelen;
+      if (menu->current < menu->top)
+	menu->current = menu->top;
+      menu->redraw = REDRAW_INDEX;
+    }
+    else if (menu->current != menu->max - 1)
+    {
+      menu->current = menu->max - 1;
+      menu->redraw = REDRAW_MOTION;
+    }
+    else
+      mutt_error ("You are on the last page.");
   }
   else
-    mutt_error ("You are on the last page.");
+    mutt_error ("No entries.");
 }
 
 void menu_prev_page (MUTTMENU *menu)
@@ -336,74 +361,184 @@ void menu_prev_page (MUTTMENU *menu)
   {
     if ((menu->top -= menu->pagelen) < 0)
       menu->top = 0;
-    if (menu->current > menu->top + menu->pagelen - 1)
+    if (menu->current >= menu->top + menu->pagelen)
       menu->current = menu->top + menu->pagelen - 1;
-    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+    menu->redraw = REDRAW_INDEX;
   }
   else if (menu->current)
   {
-    menu->current = menu->top;
-    menu->redraw = REDRAW_INDEX;
+    menu->current = 0;
+    menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error("You are on the first page.");
+    mutt_error ("You are on the first page.");
 }
 
 void menu_top_page (MUTTMENU *menu)
 {
-  menu->oldcurrent = menu->current;
-  menu->current = menu->top;
-  menu->redraw = REDRAW_MOTION;
+  if (menu->current != menu->top)
+  {
+    menu->current = menu->top;
+    menu->redraw = REDRAW_MOTION;
+  }
 }
 
 void menu_bottom_page (MUTTMENU *menu)
 {
-  menu->oldcurrent = menu->current;
-  menu->current = menu->top + menu->pagelen - 1;
-  if (menu->current > menu->max - 1) menu->current = menu->max - 1;
-  menu->redraw = REDRAW_MOTION;
+  if (menu->max)
+  {
+    menu->current = menu->top + menu->pagelen - 1;
+    if (menu->current > menu->max - 1)
+      menu->current = menu->max - 1;
+    menu->redraw = REDRAW_MOTION;
+  }
+  else
+    mutt_error ("No entries.");
 }
 
 void menu_middle_page (MUTTMENU *menu)
 {
   int i;
 
-  menu->oldcurrent = menu->current;
-  i = menu->top + menu->pagelen;
-  if (i > menu->max - 1) i = menu->max - 1;
-  menu->current = menu->top + (i - menu->top) / 2;
-  menu->redraw = REDRAW_MOTION;
+  if (menu->max)
+  {
+    i = menu->top + menu->pagelen;
+    if (i > menu->max - 1)
+      i = menu->max - 1;
+    menu->current = menu->top + (i - menu->top) / 2;
+    menu->redraw = REDRAW_MOTION;
+  }
+  else
+    mutt_error ("No entries.");
 }
 
 void menu_first_entry (MUTTMENU *menu)
 {
-  menu->oldcurrent = menu->current;
-  if (menu->top > 0)
-    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-  else
+  if (menu->max)
+  {
+    menu->current = 0;
     menu->redraw = REDRAW_MOTION;
-  menu->top = menu->current = 0;
+  }
+  else
+    mutt_error ("No entries.");
 }
 
 void menu_last_entry (MUTTMENU *menu)
 {
-  menu->oldcurrent = menu->current;
-  menu->current = menu->max - 1;
-  if (menu->current > menu->top + menu->pagelen - 1)
+  if (menu->max)
   {
-    while (menu->top + menu->pagelen  - 1 < menu->current)
-      menu->top += menu->pagelen;
-    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+    menu->current = menu->max - 1;
+    menu->redraw = REDRAW_MOTION;
   }
   else
+    mutt_error ("No entries.");
+}
+
+void menu_half_up (MUTTMENU *menu)
+{
+  if (menu->top > 0)
+  {
+    if ((menu->top -= menu->pagelen / 2) < 0)
+      menu->top = 0;
+    if (menu->current >= menu->top + menu->pagelen)
+      menu->current = menu->top + menu->pagelen - 1;
+    menu->redraw = REDRAW_INDEX;
+  }
+  else if (menu->current)
+  {
+    menu->current = 0;
     menu->redraw = REDRAW_MOTION;
+  }
+  else
+    mutt_error ("First entry is shown.");
+}
+
+void menu_half_down (MUTTMENU *menu)
+{
+  if (menu->max)
+  {
+    if (menu->top + menu->pagelen < menu->max)
+    {
+      menu->top += menu->pagelen / 2;
+      if (menu->current < menu->top)
+	menu->current = menu->top;
+      menu->redraw = REDRAW_INDEX;
+    }
+    else if (menu->current != menu->max - 1)
+    {
+      menu->current = menu->max - 1;
+      menu->redraw = REDRAW_INDEX;
+    }
+    else
+      mutt_error ("Last entry is shown.");
+  }
+  else
+    mutt_error ("No entries.");
+}
+
+void menu_current_top (MUTTMENU *menu)
+{
+  if (menu->max)
+  {
+    menu->top = menu->current;
+    menu->redraw = REDRAW_INDEX;
+  }
+  else
+    mutt_error ("No entries.");
+}
+
+void menu_current_middle (MUTTMENU *menu)
+{
+  if (menu->max)
+  {
+    menu->top = menu->current - menu->pagelen / 2;
+    if (menu->top < 0)
+      menu->top = 0;
+    menu->redraw = REDRAW_INDEX;
+  }
+  else
+    mutt_error ("No entries.");
+}
+
+void menu_current_bottom (MUTTMENU *menu)
+{
+  if (menu->max)
+  {
+    menu->top = menu->current - menu->pagelen + 1;
+    if (menu->top < 0)
+      menu->top = 0;
+    menu->redraw = REDRAW_INDEX;
+  }
+  else
+    mutt_error ("No entries.");
+}
+
+void menu_next_entry (MUTTMENU *menu)
+{
+  if (menu->current < menu->max - 1)
+  {
+    menu->current++;
+    menu->redraw = REDRAW_MOTION;
+  }
+  else
+    mutt_error ("You are on the last entry.");
+}
+
+void menu_prev_entry (MUTTMENU *menu)
+{
+  if (menu->current)
+  {
+    menu->current--;
+    menu->redraw = REDRAW_MOTION;
+  }
+  else
+    mutt_error ("You are on the first entry.");
 }
 
 MUTTMENU *mutt_new_menu (void)
 {
-  MUTTMENU *p = (MUTTMENU *) safe_malloc (sizeof (MUTTMENU));
+  MUTTMENU *p = (MUTTMENU *) safe_calloc (1, sizeof (MUTTMENU));
 
-  memset (p, 0, sizeof (MUTTMENU));
   p->offset = 1;
   p->redraw = REDRAW_FULL;
   p->pagelen = PAGELEN;
@@ -426,22 +561,28 @@ static int menu_search (MUTTMENU *menu, int op)
   regex_t re;
   char buf[SHORT_STRING];
 
-  if (op != OP_SEARCH_NEXT)
+  if (op != OP_SEARCH_NEXT && op != OP_SEARCH_OPPOSITE)
   {
     strfcpy (buf, menu->searchBuf ? menu->searchBuf : "", sizeof (buf));
-    if (ci_get_field ("Search for: ", buf, sizeof (buf), M_CLEAR) != 0 || !buf[0])
+    if (mutt_get_field ("Search for: ", buf, sizeof (buf), M_CLEAR) != 0 || !buf[0])
       return (-1);
     safe_free ((void **) &menu->searchBuf);
     menu->searchBuf = safe_strdup (buf);
     menu->searchDir = (op == OP_SEARCH) ? M_SEARCH_DOWN : M_SEARCH_UP;
   }
-  else if (!menu->searchBuf)
+  else 
   {
-    mutt_error ("No search pattern.");
-    return (-1);
+    if (!menu->searchBuf)
+    {
+      mutt_error ("No search pattern.");
+      return (-1);
+    }
+
+    if (op == OP_SEARCH_OPPOSITE)
+      searchDir = -searchDir;
   }
 
-  if ((r = REGCOMP (re, menu->searchBuf, mutt_which_case (menu->searchBuf))) != 0)
+  if ((r = REGCOMP (&re, menu->searchBuf, REG_NOSUB | mutt_which_case (menu->searchBuf))) != 0)
   {
     regerror (r, &re, buf, sizeof (buf));
     regfree (&re);
@@ -468,7 +609,7 @@ static int menu_search (MUTTMENU *menu, int op)
 
 int mutt_menuLoop (MUTTMENU *menu)
 {
-  int i;
+  int i = OP_NULL;
   char buf[STRING];
 
   FOREVER
@@ -483,19 +624,13 @@ int mutt_menuLoop (MUTTMENU *menu)
       return (OP_REDRAW);
     }
 
+    menu_check_recenter (menu);
+
     if (menu->redraw & REDRAW_STATUS)
     {
-      char tmp[STRING];
-      int l;
-
       snprintf (buf, sizeof (buf), M_MODEFMT, menu->title);
-      snprintf (tmp, sizeof (tmp), " -- %d/%d --", menu->top + 1, menu->max);
-      l = strlen (tmp);
-
       SETCOLOR (MT_COLOR_STATUS);
-      move (option (OPTSTATUSONTOP) ? 0 : LINES - 2, 0);
-      printw ("%-*.*s", COLS - l, COLS - l, buf);
-      addstr (tmp);
+      mvprintw (option (OPTSTATUSONTOP) ? 0 : LINES - 2, 0, "%-*.*s", COLS, COLS, buf);
       SETCOLOR (MT_COLOR_NORMAL);
       menu->redraw &= ~REDRAW_STATUS;
     }
@@ -507,12 +642,26 @@ int mutt_menuLoop (MUTTMENU *menu)
     else if (menu->redraw == REDRAW_CURRENT)
       menu_redraw_current (menu);
 
+    menu->oldcurrent = menu->current;
+
     /* move the cursor out of the way */
     move (menu->current - menu->top + menu->offset,
 	  (option (OPTARROWCURSOR) ? 2 : COLS-1));
 
-    refresh ();
+    mutt_refresh ();
     i = km_dokey (menu->menu);
+    if (i == OP_TAG_PREFIX)
+    {
+      mvaddstr (LINES - 1, 0, "Tag-");
+      i = km_dokey (menu->menu);
+      menu->tagprefix = 1;
+      CLEARLINE (LINES - 1);
+    }
+    else if (menu->tagged && option (OPTAUTOTAG))
+      menu->tagprefix = 1;
+    else
+      menu->tagprefix = 0;
+
     mutt_curs_set (1);
 
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
@@ -532,109 +681,62 @@ int mutt_menuLoop (MUTTMENU *menu)
     switch (i)
     {
       case OP_NEXT_ENTRY:
-
-	if (menu->current < menu->max - 1)
-	{
-	  menu->oldcurrent = menu->current;
-	  menu->current++;
-	  if (menu->current >= menu->top + menu->pagelen)
-	  {
-	    menu->top = menu->current;
-	    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	  }
-	  else
-	    menu->redraw = REDRAW_MOTION;
-	}
-	else
-	  mutt_error ("You are on the last entry.");
+	menu_next_entry (menu);
 	break;
-
       case OP_PREV_ENTRY:
-
-	if (menu->current > 0)
-	{
-	  menu->oldcurrent = menu->current;
-	  if (--menu->current < menu->top)
-	  {
-	    if ((menu->top = menu->current - menu->pagelen + 1) < 0)
-	      menu->top = 0;
-	    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	  }
-	  else
-	    menu->redraw = REDRAW_MOTION;
-	}
-	else
-	  mutt_error("You are on the first entry.");
+	menu_prev_entry (menu);
+	break;
+      case OP_HALF_DOWN:
+	menu_half_down (menu);
+	break;
+      case OP_HALF_UP:
+	menu_half_up (menu);
 	break;
-
       case OP_NEXT_PAGE:
-
 	menu_next_page (menu);
 	break;
-
       case OP_PREV_PAGE:
-
 	menu_prev_page (menu);
 	break;
-	
       case OP_NEXT_LINE:
-	
 	menu_next_line (menu);
 	break;
-	
       case OP_PREV_LINE:
-	
 	menu_prev_line (menu);
 	break;
-
       case OP_FIRST_ENTRY:
-
 	menu_first_entry (menu);
 	break;
-
       case OP_LAST_ENTRY:
-
 	menu_last_entry (menu);
 	break;
-
       case OP_TOP_PAGE:
-
 	menu_top_page (menu);
 	break;
-
       case OP_MIDDLE_PAGE:
-
 	menu_middle_page (menu);
 	break;
-
       case OP_BOTTOM_PAGE:
-
 	menu_bottom_page (menu);
 	break;
-
+      case OP_CURRENT_TOP:
+	menu_current_top (menu);
+	break;
+      case OP_CURRENT_MIDDLE:
+	menu_current_middle (menu);
+	break;
+      case OP_CURRENT_BOTTOM:
+	menu_current_bottom (menu);
+	break;
       case OP_SEARCH:
       case OP_SEARCH_REVERSE:
       case OP_SEARCH_NEXT:
-	
+      case OP_SEARCH_OPPOSITE:
 	if (menu->search)
 	{
 	  menu->oldcurrent = menu->current;
 	  if ((menu->current = menu_search (menu, i)) != -1)
-	  {
-	    if (menu->top + menu->pagelen - 1 < menu->current)
-	    {
-	      while (menu->top + menu->pagelen - 1 < menu->current)
-		menu->top += menu->pagelen;
-	      menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	    }
-	    else if (menu->current < menu->top)
-	    {
-	      while (menu->top > menu->current) menu->top -= menu->pagelen;
-	      menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	    }
-	    else
-	      menu->redraw = REDRAW_MOTION;
-	  }
+	    menu->redraw = REDRAW_MOTION;
 	  else
 	    menu->current = menu->oldcurrent;
 	}
@@ -643,12 +745,10 @@ int mutt_menuLoop (MUTTMENU *menu)
 	break;
 
       case OP_JUMP:
-	
 	menu_jump (menu);
 	break;
 
       case OP_ENTER_COMMAND:
-
 	mutt_enter_command ();
 	if (option (OPTFORCEREDRAWINDEX))
 	{
@@ -658,20 +758,46 @@ int mutt_menuLoop (MUTTMENU *menu)
 	}
 	break;
 
-      case OP_REDRAW:
+      case OP_TAG:
+	if (menu->tag)
+	{
+	  if (menu->max)
+	  {
+	    if (menu->tag (menu, menu->current))
+	      menu->tagged++;
+	    else
+	      menu->tagged--;
+	    if (option (OPTRESOLVE) && menu->current < menu->max - 1)
+	    {
+	      menu->current++;
+	      menu->redraw = REDRAW_MOTION_RESYNCH;
+	    }
+	    else
+	      menu->redraw = REDRAW_CURRENT;
+	  }
+	  else
+	    mutt_error ("No entries.");
+	}
+	else
+	  mutt_error ("Tagging is not supported.");
+	break;
 
+      case OP_SHELL_ESCAPE:
+	mutt_shell_escape ();
+	MAYBE_REDRAW (menu->redraw);
+	break;
+
+      case OP_REDRAW:
 	clearok (stdscr, TRUE);
 	menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_HELP:
-
 	mutt_help (menu->menu);
 	menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_NULL:
-
 	km_error_key (menu->menu);
 	break;
 
