@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -25,111 +25,16 @@
 #include <stdarg.h>
 
 #include "rfc822.h"
+#include "hash.h"
 
 #define TRUE 1
 #define FALSE 0
 
-#define EPOCH 1970
-
-#ifdef HAVE_SRAND48
-#define LRAND lrand48
-#define SRAND srand48
-#define DRAND drand48
-#else
-#define LRAND rand
-#define SRAND srand
-#define DRAND (double)rand
-#endif /* HAVE_SRAND48 */
-
-#ifdef HAVE_SETEGID
-#define SETEGID setegid
-#else
-#define SETEGID setgid
-#endif
-
-#define whitespace(c) ((c) == ' ' || (c) == '\t' || (c) == '\r' || (c) == '\n')
-
-#define FOREVER while (1)
-
-#ifdef DEBUG
-#define dprint(N,X) if(debuglevel>=N) fprintf X
-#else
-#define dprint(N,X) 
-#endif
-
-#define strfcpy(A,B,C) strncpy(A,B,C), *(A+(C)-1)=0
-
-#define SKIPWS(c) while (*(c) && isspace ((unsigned char) *(c))) c++;
-
-#ifdef LOCALES_HACK
-#define IsPrint(c) (isprint((unsigned char)(c)) || \
-	(((unsigned char)(c) > 0xa0) && ((unsigned char)(c) < 0xff)))
-#else
-#define IsPrint(c) (isprint((unsigned char)(c)) || \
-	(option (OPTLOCALES) ? 0 : \
-	((unsigned char)(c) > 0xa0 && (unsigned char)(c) < 0xff)))
-#endif
-
 #define HUGE_STRING	5120
 #define LONG_STRING     1024
 #define STRING          256
 #define SHORT_STRING    128
 
-/* Content-Disposition values */
-#define DISPINLINE 0
-#define DISPATTACH 1
-#define DISPFORMDATA 2
-
-#define TYPE(X) body_types[(X)]
-#define ENCODING(X) body_encodings[(X)]
-
-#define is_multipart(x) \
-    ((x)->type == TYPEMULTIPART \
-     || ((x)->type == TYPEMESSAGE && (!strcasecmp((x)->subtype, "rfc822") \
-				      || !strcasecmp((x)->subtype, "news"))))
-
-/* Content-Type */
-enum
-{
-  TYPEOTHER,
-  TYPEAUDIO,
-  TYPEAPPLICATION,
-  TYPEIMAGE,
-  TYPEMESSAGE,
-  TYPEMULTIPART,
-  TYPETEXT,
-  TYPEVIDEO
-};
-
-/* Content-Transfer-Encoding */
-enum
-{
-  ENCOTHER,
-  ENC7BIT,
-  ENC8BIT,
-  ENCQUOTEDPRINTABLE,
-  ENCBASE64,
-  ENCBINARY
-};
-
-/* MIME encoding/decoding global vars */
-extern int Index_hex[];
-extern int Index_64[];
-extern char Base64_chars[];
-
-#define hexval(c) Index_hex[(int)(c)]
-#define base64val(c) Index_64[(int)(c)]
-
-/* supported mailbox formats */
-enum
-{
-  M_MBOX = 1,
-  M_MMDF,
-  M_MH,
-  M_MAILDIR,
-  M_IMAP
-};
-
 /* flags for mutt_copy_header() */
 #define CH_UPDATE	1      /* update the status and x-status fields? */
 #define CH_WEED		(1<<1) /* weed the headers? */
@@ -139,44 +44,12 @@ enum
 #define CH_PREFIX	(1<<5) /* use Prefix string? */
 #define CH_NOSTATUS	(1<<6) /* supress the status and x-status fields */
 #define CH_REORDER	(1<<7) /* Re-order output of headers */
-#define CH_MIME		(1<<8) /* update MIME headers */
+#define CH_NONEWLINE	(1<<8) /* don't output terminating newline */
+#define CH_MIME		(1<<9) /* ignore MIME fields */
+#define CH_UPDATE_LEN	(1<<10) /* update Lines: and Content-Length: */
+#define CH_TXTPLAIN	(1<<11) /* generate text/plain MIME headers */
 
-typedef int sort_t (const void *, const void *);
-sort_t *mutt_get_sort_func (int);
-
-#define REDRAW_INDEX		(1)
-#define REDRAW_MOTION		(1<<1)
-#define REDRAW_MOTION_RESYNCH	(1<<2)
-#define REDRAW_CURRENT		(1<<3)
-#define REDRAW_STATUS		(1<<4)
-#define REDRAW_FULL		(1<<5)
-
-/* forced redraw types */
-#define R_NONE			0
-#define R_INDEX			(1<<0)
-#define R_PAGER			(1<<1)
-#define R_BOTH			(R_INDEX | R_PAGER)
-
-/* These must be in the same order as in the global var "Sort_methods" */
-#define SORT_DATE     1   /* the date the mail was sent. */
-#define SORT_SIZE     2
-#define SORT_SUBJECT  3
-#define SORT_FROM     4
-#define SORT_ORDER    5   /* the order the messages appear in the mailbox. */
-#define SORT_THREADS  6
-#define SORT_RECEIVED 7   /* when the message were delivered locally */
-#define SORT_MAX      7   /* FOO - needs to be changed if more methods */
-#define SORT_MASK     0x7
-#define SORT_REVERSE  8
-#define SORT_LAST     16
-
-/* flags for mutt_open_mailbox() */
-#define M_NOSORT	(1<<0) /* do not sort the mailbox after opening it */
-#define M_APPEND	(1<<1) /* open mailbox for appending messages */
-#define M_READONLY	(1<<2) /* open in read-only mode */
-#define M_QUIET		(1<<3) /* do not print any messages */
-
-/* flags for ci_enter_string() */
+/* flags for mutt_enter_string() */
 #define  M_ALIAS 1      /* do alias "completion" by calling up the alias-menu */
 #define  M_FILE  (1<<1) /* do file completion */
 #define  M_EFILE (1<<2) /* do file completion, plus incoming folders */
@@ -187,7 +60,11 @@ sort_t *mutt_get_sort_func (int);
 /* flags for mutt_extract_token() */
 #define M_EQUAL		1	/* treat '=' as a special */
 #define M_CONDENSE	(1<<1)	/* ^(char) to control chars (macros) */
-#define M_SPACE		(1<<2)
+#define M_SPACE		(1<<2)  /* don't treat whitespace as a terminator */
+#define M_QUOTE		(1<<3)	/* don't interpret quotes */
+#define M_PATTERN	(1<<4)	/* !)|~ are terminators (for patterns) */
+#define M_COMMENT	(1<<5)	/* don't reap comments */
+#define M_NULL		(1<<6)  /* don't return NULL when \0 is reached */
 
 /* flags for _mutt_system() */
 #define M_DETACH_PROCESS	1	/* detach subprocess from group */
@@ -195,18 +72,19 @@ sort_t *mutt_get_sort_func (int);
 /* flags for _mutt_make_string() */
 #define M_FORCESUBJ	(1<<0) /* print the subject even if it didn't change */
 #define M_TREE		(1<<1) /* draw the thread tree */
+#define M_MAKEPRINT	(1<<2) /* make sure that all chars are printable */
+
+/* types for mutt_add_hook() */
+#define M_FOLDERHOOK	1
+#define M_MBOXHOOK	(1<<1)
+#define M_SENDHOOK	(1<<2)
+#define M_FCCHOOK	(1<<3)
+#define M_SAVEHOOK	(1<<4)
 
 enum
 {
-  /* types for mutt_add_hook() */
-  M_FOLDERHOOK = 1,
-  M_FCCHOOK,
-  M_SAVEHOOK,
-  M_MBOXHOOK,
-  M_SENDHOOK,
-
   /* modes for mutt_view_attachment() */
-  M_REGULAR,
+  M_REGULAR = 1,
   M_MAILCAP,
   M_AS_TEXT,
 
@@ -225,8 +103,12 @@ enum
   M_TAG,
   M_UNTAG,
   M_LIMIT,
+  M_EXPIRED,
+  M_SUPERSEDED,
 
-  /* actions for doString() / mutt_pattern_function() */
+  /* actions for mutt_pattern_comp/mutt_pattern_exec */
+  M_AND,
+  M_OR,
   M_TO,
   M_CC,
   M_SUBJECT,
@@ -238,7 +120,13 @@ enum
   M_HEADER,
   M_SENDER,
   M_MESSAGE,
-  M_THREAD,
+  M_SCORE,
+  M_REFERENCE,
+  M_RECIPIENT,
+  M_LIST,
+  M_PERSONAL_RECIP,
+  M_PERSONAL_FROM,
+  M_ADDRESS,
 
   /* Options for Mailcap lookup */
   M_EDIT,
@@ -262,7 +150,8 @@ enum
 /* quad-option vars */
 enum
 {
-  OPT_VERIFYSIG, /* verify PGP signatures */
+
+
   OPT_USEMAILCAP,
   OPT_PRINT,
   OPT_INCLUDE,
@@ -284,6 +173,7 @@ enum
 #define SENDPOSTPONED	(1<<4)
 #define SENDBATCH	(1<<5)
 #define SENDMAILX	(1<<6)
+#define SENDKEY		(1<<7)
 
 /* boolean vars */
 enum
@@ -292,13 +182,11 @@ enum
   OPTSTATUSONTOP,
   OPTALLOW8BIT,
   OPTASCIICHARS,
-  OPTHOLD,
   OPTMETOO,
   OPTEDITHDRS,
   OPTARROWCURSOR,
   OPTASKCC,
   OPTHEADER,
-  OPTPOINTNEW,
   OPTREVALIAS,
   OPTREVNAME,
   OPTFORCENAME,
@@ -307,14 +195,6 @@ enum
   OPTSIGDASHES,
   OPTASKBCC,
   OPTAUTOEDIT,
-#ifdef _PGPPATH
-  OPTPGPAUTOSIGN,
-  OPTPGPAUTOENCRYPT,
-  OPTPGPREPLYENCRYPT,
-  OPTPGPREPLYSIGN,
-  OPTPGPENCRYPTSELF,
-  OPTPGPSTRICTENC,
-#endif
   OPTMARKOLD,
   OPTCONFIRMCREATE,
   OPTCONFIRMAPPEND,
@@ -328,7 +208,6 @@ enum
   OPTATTACHSPLIT,
   OPTPIPESPLIT,
   OPTPIPEDECODE,
-  OPTFORWDECODE,
   OPTREADONLY,
   OPTRESOLVE,
   OPTSTRICTTHREADS,
@@ -348,74 +227,72 @@ enum
   OPTUSEDOMAIN,
   OPTUSEFROM,
   OPTUSE8BITMIME,
+  OPTFORWDECODE,
+  OPTFORWQUOTE,
+  OPTBEEPNEW,
+  OPTMENUSCROLL,	/* scroll menu instead of implicit next-page */
+  OPTMETAKEY,		/* interpret ALT-x as ESC-x */
+  OPTAUXSORT,		/* (pseudo) using auxillary sort function */
+  OPTFORCEREFRESH,	/* (pseudo) refresh even during macros */
   OPTLOCALES,		/* (pseudo) set if user has valid locale definition */
-  OPTFORWARD,		/* (pseudo) set when forwarding messages */
-  OPTVERIFYSIG,		/* (pseudo) controlled by M_VERIFYSIG */
   OPTNOCURSES,		/* (pseudo) when sending in batch mode */
   OPTNEEDREDRAW,	/* (pseudo) to notify caller of a submenu */
   OPTSEARCHREVERSE,	/* (pseudo) used by ci_search_command */
   OPTMSGERR,		/* (pseudo) used by mutt_error/mutt_message */
   OPTSEARCHINVALID,	/* (pseudo) used to invalidate the search pat */
-  OPTNOAUTOVIEW, 	/* (pseudo) used by pipe_msg */
   OPTSIGNALSBLOCKED,	/* (pseudo) using by mutt_block_signals () */
   OPTNEEDRESORT,	/* (pseudo) used to force a re-sort */
   OPTVIEWATTACH,	/* (pseudo) signals that we are viewing attachments */
   OPTFORCEREDRAWINDEX,	/* (pseudo) used to force a redraw in the main index */
   OPTFORCEREDRAWPAGER,	/* (pseudo) used to force a redraw in the pager */
+  OPTSORTSUBTHREADS,	/* (pseudo) used when $sort_aux changes */
+  OPTNEEDRESCORE,	/* (pseudo) set when the `score' command is used */
+
+
+
+
+
+
   OPTMAX
 };
 
-#define set_option(x) Options[(x)/8] |= (1 << ((x) % 8))
-#define unset_option(x) Options[(x)/8] &= ~(1 << ((x) % 8))
-#define toggle_option(x) Options[(x)/8] ^= (1 << ((x) % 8))
-#define option(x) (Options[(x)/8] & (1 << ((x) % 8)))
+#define mutt_bit_alloc(n) calloc ((n + 7) / 8, sizeof (char))
+#define mutt_bit_set(v,n) v[n/8] |= (1 << (n % 8))
+#define mutt_bit_unset(v,n) v[n/8] &= ~(1 << (n % 8))
+#define mutt_bit_toggle(v,n) v[n/8] ^= (1 << (n % 8))
+#define mutt_bit_isset(v,n) (v[n/8] & (1 << (n % 8)))
+
+#define set_option(x) mutt_bit_set(Options,x)
+#define unset_option(x) mutt_bit_unset(Options,x)
+#define toggle_option(x) mutt_bit_toggle(Options,x)
+#define option(x) mutt_bit_isset(Options,x)
 
 /* Bit fields for ``Signals'' */
 #define S_INTERRUPT (1<<1)
 #define S_SIGWINCH  (1<<2)
 
-struct mapping_t
-{
-  char *name;
-  int value;
-};
-
-typedef struct
-{
-  FILE *fpin;
-  FILE *fpout;
-  char *prefix;
-  short displaying; /* nonzero iff sending the output to a pager. */
-} STATE;
-
-typedef struct alias
-{
-  char *name;
-  ADDRESS *addr;
-  struct alias *next;
-} ALIAS;
-
 typedef struct list_t
 {
   char *data;
   struct list_t *next;
 } LIST;
 
-typedef struct
-{
-  char *pattern;
-  void *rx; /* avoid including "mutt_regex.h" everywhere... */
-} REGEXP;
+#define mutt_new_list() safe_calloc (1, sizeof (LIST))
+void mutt_add_to_list (LIST **, const char *);
+void mutt_free_list (LIST **);
+int mutt_matches_ignore (const char *, LIST *);
 
-typedef struct buffy_t
+/* add an element to a list */
+LIST *mutt_add_list (LIST *, const char *);
+
+void mutt_init (int, LIST *);
+
+typedef struct alias
 {
-  char *path;
-#ifdef BUFFY_SIZE
-  long size;
-#endif /* BUFFY_SIZE */
-  int new;
-  struct buffy_t *next;
-} BUFFY;
+  char *name;
+  ADDRESS *addr;
+  struct alias *next;
+} ALIAS;
 
 typedef struct envelope
 {
@@ -426,11 +303,13 @@ typedef struct envelope
   ADDRESS *bcc;
   ADDRESS *sender;
   ADDRESS *reply_to;
+  ADDRESS *mail_followup_to;
   char *subject;
-  char *real_subj;   /* offset of the real subject */
+  char *real_subj;		/* offset of the real subject */
   char *message_id;
-  LIST *references;  /* message references (in reverse order) */
-  LIST *userhdrs;    /* user defined headers */
+  char *supersedes;
+  LIST *references;		/* message references (in reverse order) */
+  LIST *userhdrs;		/* user defined headers */
 } ENVELOPE;
 
 typedef struct parameter
@@ -443,30 +322,18 @@ typedef struct parameter
 /* Information that helps in determing the Content-* of an attachment */
 typedef struct content
 {
-  unsigned int space : 1;  /* whitespace at the end of lines? */
-  unsigned int binary : 1; /* long lines, or CR not in CRLF pair */
-  unsigned int from : 1;   /* has a line beginning with "From "? */
-  unsigned int dot : 1;    /* has a line consisting of a single dot? */
   long hibin;              /* 8-bit characters */
   long lobin;              /* unprintable 7-bit chars (eg., control chars) */
   long ascii;              /* number of ascii chars */
   long linemax;            /* length of the longest line in the file */
+  unsigned int space : 1;  /* whitespace at the end of lines? */
+  unsigned int binary : 1; /* long lines, or CR not in CRLF pair */
+  unsigned int from : 1;   /* has a line beginning with "From "? */
+  unsigned int dot : 1;    /* has a line consisting of a single dot? */
 } CONTENT;
 
 typedef struct body
 {
-  unsigned int type : 3;        /* content-type primary type */
-  unsigned int encoding : 3;    /* content-transfer-encoding */
-  unsigned int disposition : 2; /* content-disposition */
-  unsigned int use_disp : 1;    /* Content-Disposition field printed? */
-  unsigned int unlink : 1;      /* flag to indicate the the file named by
-				 * "filename" should be unlink()ed before
-				 * free()ing this structure
-				 */
-  unsigned int marked : 1;      /* used for computing subtree spans */
-  unsigned int empty : 1;       /* attachment without underlying file in 
-				 * the compose menu
-				 */
   char *subtype;                /* content-type subtype */
   PARAMETER *parameter;         /* parameters of the content-type */
   char *description;            /* content-description */
@@ -481,6 +348,11 @@ typedef struct body
   char *filename;               /* when sending a message, this is the file
 				 * to which this structure refers
 				 */
+  char *d_filename;		/* filename to be used for the 
+				 * content-disposition header.
+				 * If NULL, filename is used 
+				 * instead.
+				 */
   CONTENT *content;             /* structure used to store detailed info about
 				 * the content of the attachment.  this is used
 				 * to determine what content-transfer-encoding
@@ -489,6 +361,17 @@ typedef struct body
   struct body *next;            /* next attachment in the list */
   struct body *parts;           /* parts of a multipart or message/rfc822 */
   struct header *hdr;		/* header information for message/rfc822 */
+
+  unsigned int type : 3;        /* content-type primary type */
+  unsigned int encoding : 3;    /* content-transfer-encoding */
+  unsigned int disposition : 2; /* content-disposition */
+  unsigned int use_disp : 1;    /* Content-Disposition field printed? */
+  unsigned int unlink : 1;      /* flag to indicate the the file named by
+				 * "filename" should be unlink()ed before
+				 * free()ing this structure
+				 */
+  unsigned int tagged : 1;
+
 } BODY;
 
 typedef struct header
@@ -501,27 +384,33 @@ typedef struct header
   unsigned int changed : 1;
   unsigned int old : 1;
   unsigned int read : 1;
+  unsigned int expired : 1; /* already expired? */
+  unsigned int superseded : 1; /* got superseded? */
 
   unsigned int pgp : 3;
   unsigned int replied : 1;
   unsigned int subject_changed : 1; /* used for threading */
   unsigned int display_subject : 1; /* used for threading */
-  unsigned int fake_thread : 1;     /* used for threading */
-  unsigned int xxx : 1;             /* unused */
+  unsigned int fake_thread : 1;     /* no ref matched, but subject did */
+  unsigned int threaded : 1;        /* message has been threaded */
 
   /* timezone of the sender of this message */
   unsigned int zhours : 5;
   unsigned int zminutes : 6;
   unsigned int zoccident : 1;
-  unsigned int yyy : 4;		/* unused */
+
+  /* bits used for caching when searching */
+  unsigned int searched : 1;
+  unsigned int matched : 1;
 
   time_t date_sent;     /* time when the message was sent (UTC) */
   time_t received;      /* time when the message was placed in the mailbox */
   long offset;          /* where in the stream does this message begin? */
-  unsigned short lines; /* how many lines in the body of this message? */
-  unsigned short index; /* the absolute (unsorted) message number */
-  unsigned short msgno;	/* number displayed to the user */
-  short virtual;        /* virtual message number */
+  int lines;		/* how many lines in the body of this message? */
+  int index;		/* the absolute (unsorted) message number */
+  int msgno;		/* number displayed to the user */
+  int virtual;		/* virtual message number */
+  int score;
   ENVELOPE *env;	/* envelope information */
   BODY *content;	/* list of MIME parts */
   char *path;
@@ -543,16 +432,26 @@ typedef struct
   time_t mtime;
   off_t size;
   HEADER **hdrs;
-  short *v2r;			/* mapping from virtual to real msgno */
-  unsigned short hdrmax;	/* number of pointers in hdrs */
-  unsigned short msgcount;	/* number of messages in the mailbox */
-  unsigned short vcount;	/* the number of virtual messages */
-  unsigned short tagged;	/* how many messages are tagged? */
-  unsigned short new;		/* how many new messages? */
-  unsigned short deleted;	/* how many deleted messages */
-  unsigned short flagged;	/* how many flagged messages */
+  HEADER *tree;			/* top of thread tree */
+  HASH *id_hash;		/* hash table by msg id */
+  HASH *subj_hash;		/* hash table by subject */
+  int *v2r;			/* mapping from virtual to real msgno */
+  int hdrmax;			/* number of pointers in hdrs */
+  int msgcount;			/* number of messages in the mailbox */
+  int vcount;			/* the number of virtual messages */
+  int tagged;			/* how many messages are tagged? */
+  int new;			/* how many new messages? */
+  int unread;			/* how many unread messages? */
+  int deleted;			/* how many deleted messages */
+  int flagged;			/* how many flagged messages */
+  int msgnotreadyet;		/* which msg "new" in pager, -1 if none */
+#ifdef USE_IMAP
+  void *data;			/* driver specific data */
+  int fd;
+#endif /* USE_IMAP */
+
   short magic;			/* mailbox type */
-  short msgnotreadyet;		/* which msg "new" in pager, -1 if none */
+
   unsigned int locked : 1;	/* is the mailbox locked? */
   unsigned int changed : 1;	/* mailbox has been modified */
   unsigned int readonly : 1;    /* don't allow changes to the mailbox */
@@ -562,31 +461,32 @@ typedef struct
   unsigned int quiet : 1;	/* inhibit status messages? */
   unsigned int revsort : 1;	/* mailbox sorted in reverse? */
 
-#ifdef USE_IMAP
-  void *data;		/* driver specific data */
-  int fd;
-#endif /* USE_IMAP */
-
 } CONTEXT;
 
-typedef struct
-{
-  FILE *fp;	/* pointer to the message data */
-#ifdef USE_IMAP
-  char *path;	/* path to temp file */
-#endif /* USE_IMAP */
-  short magic;	/* type of mailbox this message belongs to */
-  short write;	/* nonzero if message is open for writing */
-} MESSAGE;
-
 typedef struct attachptr
 {
   BODY *content;
   char *tree;
   int level;
-  int tagged;
-  struct attachptr *next;
 } ATTACHPTR;
 
+typedef struct
+{
+  FILE *fpin;
+  FILE *fpout;
+  char *prefix;
+  int flags;
+} STATE;
+
+/* flags for the STATE struct */
+#define M_DISPLAY	(1<<0) /* output is displayed to the user */
+
+
+
+
+
+#define state_puts(x,y) fputs(x,(y)->fpout)
+#define state_putc(x,y) fputc(x,(y)->fpout)
+
 #include "protos.h"
 #include "globals.h"
