@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -70,21 +70,24 @@
 #define BKGDSET(x)
 #endif
 
+#if (defined(USE_SLANG_CURSES) || defined(HAVE_CURS_SET))
+void mutt_curs_set (int);
+#else
+#define mutt_curs_set(x)
+#endif
 #define PAGELEN (LINES-3)
 
 #define ctrl(c) ((c)-'@')
 
 #define CI_is_return(c) ((c) == '\r' || (c) == '\n' || (c) == KEY_ENTER)
 
-int ci_getch (void);
-int ci_parse_bind (char *, char *);
-int ci_enter_string (unsigned char *, size_t, int, int, int);
-int ci_get_field (char *, char *, size_t, int);
+int mutt_getch (void);
 
 void mutt_endwin (const char *);
-void mutt_ungetch (int);
 void mutt_flushinp (void);
+void mutt_refresh (void);
 void mutt_resize_screen (void);
+void mutt_ungetch (int);
 
 /* ----------------------------------------------------------------------------
  * Support for color
@@ -122,6 +125,8 @@ typedef struct color_line
   struct color_line *next;
 } COLOR_LINE;
 
+extern int *ColorQuote;
+extern int ColorQuoteUsed;
 extern int ColorDefs[];
 extern COLOR_LINE *ColorHdrList;
 extern COLOR_LINE *ColorBodyList;
