@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -23,17 +23,26 @@
 #include "keymap.h"
 #include "mutt_regex.h"
 
+#define REDRAW_INDEX		(1)
+#define REDRAW_MOTION		(1<<1)
+#define REDRAW_MOTION_RESYNCH	(1<<2)
+#define REDRAW_CURRENT		(1<<3)
+#define REDRAW_STATUS		(1<<4)
+#define REDRAW_FULL		(1<<5)
+#define REDRAW_BODY		(1<<6)
+
 typedef struct menu_t
 {
   char *title;   /* the title of this menu */
   char *help;    /* quickref for the current menu */
   void *data;    /* extra data for the current menu */
-  short current; /* current entry */
-  short redraw;
-  short menu;    /* menu definition for keymap entries. */
-  short max;     /* the number of entries in the menu */
-  short offset;  /* which screen row to start the index */
-  short pagelen; /* number of entries per screen */
+  int current;   /* current entry */
+  int max;       /* the number of entries in the menu */
+  int redraw;	/* when to redraw the screen */
+  int menu;	/* menu definition for keymap entries. */
+  int offset;	/* which screen row to start the index */
+  int pagelen;	/* number of entries per screen */
+  int tagprefix;
   
   /* callback to generate an index line for the requested element */
   void (*make_entry) (char *, size_t, struct menu_t *, int);
@@ -41,11 +50,14 @@ typedef struct menu_t
   /* how to search the menu */
   int (*search) (struct menu_t *, regex_t *re, int n);
 
+  int (*tag) (struct menu_t *, int i);
+
   /* the following are used only by mutt_menuLoop() */
-  short top;		/* entry that is the top of the current page */
-  short oldcurrent;	/* for driver use only. */
+  int top;		/* entry that is the top of the current page */
+  int oldcurrent;	/* for driver use only. */
   char *searchBuf;	/* last search pattern */
   int searchDir;	/* direction of search */
+  int tagged;		/* number of tagged entries */
 } MUTTMENU;
 
 void menu_jump (MUTTMENU *);
@@ -62,6 +74,13 @@ void menu_next_page (MUTTMENU *);
 void menu_prev_page (MUTTMENU *);
 void menu_next_line (MUTTMENU *);
 void menu_prev_line (MUTTMENU *);
+void menu_half_up (MUTTMENU *);
+void menu_half_down (MUTTMENU *);
+void menu_current_top (MUTTMENU *);
+void menu_current_middle (MUTTMENU *);
+void menu_current_bottom (MUTTMENU *);
+void menu_check_recenter (MUTTMENU *);
+void menu_status_line (char *, size_t, MUTTMENU *, const char *);
 
 MUTTMENU *mutt_new_menu (void);
 void mutt_menuDestroy (MUTTMENU **);
