@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -29,7 +29,26 @@
 #include "rxposix.h"
 #endif
 
-#define REGCOMP(X,Y,Z) regcomp(&X, Y, REG_NOSUB|REG_EXTENDED|(Z))
+/* this is a non-standard option supported by Solaris 2.5.x which allows
+ * patterns of the form \<...\>
+ */
+#ifndef REG_WORDS
+#define REG_WORDS 0
+#endif
+
+#define REGCOMP(X,Y,Z) regcomp(X, Y, REG_WORDS|REG_EXTENDED|(Z))
 #define REGEXEC(X,Y) regexec(&X, Y, (size_t)0, (regmatch_t *)0, (int)0)
 
+typedef struct
+{
+  char *pattern;	/* printable version */
+  void *rx; 		/* avoid including "mutt_regex.h" everywhere... */
+  int not;		/* do not match */
+} REGEXP;
+
+WHERE REGEXP Alternates;
+WHERE REGEXP Mask;
+WHERE REGEXP QuoteRegexp;
+WHERE REGEXP ReplyRegexp;
+
 #endif /* MUTT_REGEX_H */
