@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -18,11 +18,14 @@
 
 #include "mutt.h"
 #include "mx.h"
-#include "send.h"
 #include "rfc2047.h"
+#include "sort.h"
+#include "mailbox.h"
+#include "copy.h"
 
-#ifdef _PGPPATH
-#include "pgp.h"
+
+#ifdef BUFFY_SIZE
+#include "buffy.h"
 #endif
 
 #include <dirent.h>
@@ -34,7 +37,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#ifndef BUFFY_SIZE
 #include <utime.h>
+#endif
 
 /* HP-UX and ConvexOS don't have this macro */
 #ifndef S_ISLNK
@@ -46,14 +51,20 @@
 #define MAXLOCKATTEMPT 5
 
 #ifdef USE_DOTLOCK
-static int dotlock_file (const char *path)
+/* parameters: 
+ * path - file to lock
+ * retry - should retry if unable to lock?
+ */
+static int dotlock_file (const char *path, int retry)
 {
   const char *pathptr = path;
   char lockfile[_POSIX_PATH_MAX];
   char nfslockfile[_POSIX_PATH_MAX];
   char realpath[_POSIX_PATH_MAX];
   struct stat sb;
+  size_t prev_size = 0;
   int count = 0;
+  int attempt = 0;
   int fd;
 
   /* if the file is a symlink, find the real file to which it refers */
@@ -111,20 +122,30 @@ static int dotlock_file (const char *path)
 
     if (sb.st_nlink == 2)
       break;
+    
+    if (stat (path, &sb) != 0)
+      sb.st_size = 0;
 
-    if (++count == MAXLOCKATTEMPT)
+    if (count == 0)
+      prev_size = sb.st_size;
+
+    /* only try to remove the lock if the file is not changing */
+    if (prev_size == sb.st_size && ++count >= retry ? MAXLOCKATTEMPT : 0)
     {
-      if (mutt_yesorno ("Lock count exceeded, remove lock?", 1) == 1)
+      if (retry && mutt_yesorno ("Lock count exceeded, remove lock?", 1) == 1)
       {
 	unlink (lockfile);
 	count = 0;
+	attempt = 0;
 	continue;
       }
       else
 	return (-1);
     }
 
-    mutt_message ("Waiting for lock attempt #%d...", count);
+    prev_size = sb.st_size;
+
+    mutt_message ("Waiting for lock attempt #%d...", ++attempt);
     sleep (1);
   }
 
@@ -180,24 +201,30 @@ static int undotlock_file (const char *path)
 /* Args:
  *	excl		if excl != 0, request an exclusive lock
  *	dot		if dot != 0, try to dotlock the file
+ *	timeout 	should retry locking?
  */
-int mx_lock_file (const char *path, int fd, int excl, int dot)
+int mx_lock_file (const char *path, int fd, int excl, int dot, int timeout)
 {
 #if defined (USE_FCNTL) || defined (USE_FLOCK)
   int count;
+  int attempt;
+  struct stat prev_sb;
 #endif
   int r = 0;
 
 #ifdef USE_FCNTL
   struct flock lck;
+  
 
   memset (&lck, 0, sizeof (struct flock));
   lck.l_type = excl ? F_WRLCK : F_RDLCK;
   lck.l_whence = SEEK_SET;
 
   count = 0;
+  attempt = 0;
   while (fcntl (fd, F_SETLK, &lck) == -1)
   {
+    struct stat sb;
     dprint(1,(debugfile, "mx_lock_file(): fcntl errno %d.\n", errno));
     if (errno != EAGAIN && errno != EACCES)
     {
@@ -205,21 +232,33 @@ int mx_lock_file (const char *path, int fd, int excl, int dot)
       return (-1);
     }
 
-    if (++count == MAXLOCKATTEMPT)
+    if (fstat (fd, &sb) != 0)
+     sb.st_size = 0;
+    
+    if (count == 0)
+      prev_sb = sb;
+
+    /* only unlock file if it is unchanged */
+    if (prev_sb.st_size == sb.st_size && ++count >= timeout?MAXLOCKATTEMPT:0)
     {
-      mutt_error ("Timeout exceeded while attempting fcntl lock!");
+      if (timeout)
+	mutt_error ("Timeout exceeded while attempting fcntl lock!");
       return (-1);
     }
 
-    mutt_message ("Waiting for fcntl lock... %d", count);
+    prev_sb = sb;
+
+    mutt_message ("Waiting for fcntl lock... %d", ++attempt);
     sleep (1);
   }
 #endif /* USE_FCNTL */
 
 #ifdef USE_FLOCK
   count = 0;
+  attempt = 0;
   while (flock (fd, (excl ? LOCK_EX : LOCK_SH) | LOCK_NB) == -1)
   {
+    struct stat sb;
     if (errno != EWOULDBLOCK)
     {
       mutt_perror ("flock");
@@ -227,21 +266,31 @@ int mx_lock_file (const char *path, int fd, int excl, int dot)
       break;
     }
 
-    if (++count == MAXLOCKATTEMPT)
+    if (fstat(fd,&sb) != 0 )
+     sb.st_size=0;
+    
+    if (count == 0)
+      prev_sb=sb;
+
+    /* only unlock file if it is unchanged */
+    if (prev_sb.st_size == sb.st_size && ++count >= timeout?MAXLOCKATTEMPT:0)
     {
-      mutt_error ("Timeout exceeded while attempting flock lock!");
+      if (timeout)
+	mutt_error ("Timeout exceeded while attempting flock lock!");
       r = -1;
       break;
     }
 
-    mutt_message ("Waiting for flock attempt... %d", count);
+    prev_sb = sb;
+
+    mutt_message ("Waiting for flock attempt... %d", ++attempt);
     sleep (1);
   }
 #endif /* USE_FLOCK */
 
 #ifdef USE_DOTLOCK
   if (r == 0 && dot)
-    r = dotlock_file (path);
+    r = dotlock_file (path,timeout);
 #endif /* USE_DOTLOCK */
 
   if (r == -1)
@@ -290,9 +339,9 @@ FILE *mx_open_file_lock (const char *path, const char *mode)
 {
   FILE *f;
 
-  if ((f = fopen (path, mode)) != NULL)
+  if ((f = safe_fopen (path, mode)) != NULL)
   {
-    if (mx_lock_file (path, fileno (f), *mode != 'r', 1) != 0)
+    if (mx_lock_file (path, fileno (f), *mode != 'r', 1, 1) != 0)
     {
       fclose (f);
       f = NULL;
@@ -350,11 +399,16 @@ int mx_get_magic (const char *path)
   else if (st.st_size == 0)
   {
     /* hard to tell what zero-length files are, so assume the default magic */
-    return (DefaultMagic);
+    if (DefaultMagic == M_MBOX || DefaultMagic == M_MMDF)
+      return (DefaultMagic);
+    else
+      return (M_MBOX);
   }
   else if ((f = fopen (path, "r")) != NULL)
   {
+#ifndef BUFFY_SIZE
     struct utimbuf times;
+#endif
 
     fgets (tmp, sizeof (tmp), f);
     if (strncmp ("From ", tmp, 5) == 0)
@@ -362,6 +416,7 @@ int mx_get_magic (const char *path)
     else if (strcmp (MMDF_SEP, tmp) == 0)
       magic = M_MMDF;
     fclose (f);
+#ifndef BUFFY_SIZE
     /* need to restore the times here, the file was not really accessed,
      * only the type was accessed.  This is important, because detection
      * of "new mail" depends on those times set correctly.
@@ -369,6 +424,7 @@ int mx_get_magic (const char *path)
     times.actime = st.st_atime;
     times.modtime = st.st_mtime;
     utime (path, &times);
+#endif
   }
   else
   {
@@ -400,6 +456,112 @@ int mx_set_magic (const char *s)
   return 0;
 }
 
+static int mx_open_mailbox_append (CONTEXT *ctx)
+{
+  ctx->append = 1;
+  if (access (ctx->path, W_OK) == 0)
+  {
+    switch (ctx->magic = mx_get_magic (ctx->path))
+    {
+      case 0:
+	mutt_error ("%s is not a mailbox.", ctx->path);
+	/* fall through */
+      case -1:
+	return (-1);
+    }
+  }
+  else if (errno == ENOENT)
+  {
+    ctx->magic = DefaultMagic;
+
+    if (ctx->magic == M_MH || ctx->magic == M_MAILDIR)
+    {
+      char tmp[_POSIX_PATH_MAX];
+
+      if (mkdir (ctx->path, S_IRWXU))
+      {
+	mutt_perror (tmp);
+	return (-1);
+      }
+
+      if (ctx->magic == M_MAILDIR)
+      {
+	snprintf (tmp, sizeof (tmp), "%s/cur", ctx->path);
+	if (mkdir (tmp, S_IRWXU))
+	{
+	  mutt_perror (tmp);
+	  rmdir (ctx->path);
+	  return (-1);
+	}
+
+	snprintf (tmp, sizeof (tmp), "%s/new", ctx->path);
+	if (mkdir (tmp, S_IRWXU))
+	{
+	  mutt_perror (tmp);
+	  snprintf (tmp, sizeof (tmp), "%s/cur", ctx->path);
+	  rmdir (tmp);
+	  rmdir (ctx->path);
+	  return (-1);
+	}
+	snprintf (tmp, sizeof (tmp), "%s/tmp", ctx->path);
+	if (mkdir (tmp, S_IRWXU))
+	{
+	  mutt_perror (tmp);
+	  snprintf (tmp, sizeof (tmp), "%s/cur", ctx->path);
+	  rmdir (tmp);
+	  snprintf (tmp, sizeof (tmp), "%s/new", ctx->path);
+	  rmdir (tmp);
+	  rmdir (ctx->path);
+	  return (-1);
+	}
+      }
+      else
+      {
+	int i;
+
+	snprintf (tmp, sizeof (tmp), "%s/.mh_sequences", ctx->path);
+	if ((i = creat (tmp, S_IRWXU)) == -1)
+	{
+	  mutt_perror (tmp);
+	  rmdir (ctx->path);
+	  return (-1);
+	}
+	close (i);
+      }
+    }
+  }
+  else
+  {
+    mutt_perror (ctx->path);
+    return (-1);
+  }
+
+  switch (ctx->magic)
+  {
+    case M_MBOX:
+    case M_MMDF:
+      if ((ctx->fp = fopen (ctx->path, "a")) == NULL ||
+	  mbox_lock_mailbox (ctx, 1, 1) != 0)
+      {
+	if (!ctx->fp)
+	  mutt_perror (ctx->path);
+	return (-1);
+      }
+      fseek (ctx->fp, 0, 2);
+      break;
+
+    case M_MH:
+    case M_MAILDIR:
+      /* nothing to do */
+      break;
+
+    default:
+      return (-1);
+  }
+
+  return 0;
+}
+
 /*
  * open a mailbox and parse it
  *
@@ -429,188 +591,90 @@ CONTEXT *mx_open_mailbox (const char *path, int flags, CONTEXT *pctx)
 
   if (flags & M_APPEND)
   {
-    /* open mailbox in append-mode (write) */
-
-    ctx->append = 1;
-    if (access (path, W_OK) == 0)
+    if (mx_open_mailbox_append (ctx) != 0)
     {
-      switch (ctx->magic = mx_get_magic (path))
-      {
-	case 0:
-	  mutt_error ("%s is not a mailbox.", path);
-	  /* fall through */
-	case -1:
-	  mx_fastclose_mailbox (ctx);
-	  if (!pctx)
-	    free (ctx);
-	  return (NULL);
-      }
-    }
-    else if (errno == ENOENT)
-    {
-      ctx->magic = DefaultMagic;
-
-      if (ctx->magic == M_MH || ctx->magic == M_MAILDIR)
-      {
-	char tmp[_POSIX_PATH_MAX];
-
-	if (mkdir (path, S_IRWXU))
-	{
-	  mutt_perror (tmp);
-	  goto bail;
-	}
-
-	if (ctx->magic == M_MAILDIR)
-	{
-	  snprintf (tmp, sizeof (tmp), "%s/cur", path);
-	  if (mkdir (tmp, S_IRWXU))
-	  {
-	    mutt_perror (tmp);
-	    rmdir (path);
-	    goto bail;
-	  }
-
-	  snprintf (tmp, sizeof (tmp), "%s/new", path);
-	  if (mkdir (tmp, S_IRWXU))
-	  {
-	    mutt_perror (tmp);
-	    snprintf (tmp, sizeof (tmp), "%s/cur", path);
-	    rmdir (tmp);
-	    rmdir (path);
-	    goto bail;
-	  }
-	  snprintf (tmp, sizeof (tmp), "%s/tmp", path);
-	  if (mkdir (tmp, S_IRWXU))
-	  {
-	    mutt_perror (tmp);
-	    snprintf (tmp, sizeof (tmp), "%s/cur", path);
-	    rmdir (tmp);
-	    snprintf (tmp, sizeof (tmp), "%s/new", path);
-	    rmdir (tmp);
-	    rmdir (path);
-	    goto bail;
-	  }
-	}
-	else
-	{
-	  int i;
-
-	  snprintf (tmp, sizeof (tmp), "%s/.mh_sequences", path);
-	  if ((i = creat (tmp, S_IRWXU)) == -1)
-	  {
-	    mutt_perror (tmp);
-	    rmdir (path);
-	    goto bail;
-	  }
-	  close (i);
-	}
-      }
-    }
-    else
-    {
-      mutt_perror (path);
-bail:
       mx_fastclose_mailbox (ctx);
       if (!pctx)
 	safe_free ((void **) &ctx);
+      return NULL;
+    }
+    return ctx;
+  }
+
+  switch (ctx->magic = mx_get_magic (path))
+  {
+    case 0:
+      mutt_error ("%s is not a mailbox.", path);
+      /* fall through */
+
+    case -1:
+      mx_fastclose_mailbox (ctx);
+      if (!pctx)
+	free (ctx);
       return (NULL);
-    }
+  }
 
-    switch (ctx->magic)
+  /* if the user has a `push' command in their .muttrc, or in a folder-hook,
+   * it will cause the progress messages not to be displayed because
+   * mutt_refresh() will think we are in the middle of a macro.  so set a
+   * flag to indicate that we should really refresh the screen.
+   */
+  set_option (OPTFORCEREFRESH);
+
+  /* create hash tables */
+  ctx->id_hash = hash_create (257);
+  ctx->subj_hash = hash_create (257);
+
+  if (!ctx->quiet)
+    mutt_message ("Reading %s...", ctx->path);
+
+  switch (ctx->magic)
+  {
+    case M_MH:
+      rc = mh_read_dir (ctx, NULL);
+      break;
+
+    case M_MAILDIR:
+      rc = maildir_read_dir (ctx);
+      break;
+
+    case M_MMDF:
+    case M_MBOX:
+      rc = mbox_open_mailbox (ctx);
+      break;
+
+#ifdef USE_IMAP
+    case M_IMAP:
+      rc = imap_open_mailbox (ctx);
+      break;
+#endif /* USE_IMAP */
+
+    default:
+      rc = -1;
+      break;
+  }
+
+  if (rc == 0)
+  {
+    if ((flags & M_NOSORT) == 0)
     {
-      case M_MBOX:
-      case M_MMDF:
-
-	if ((ctx->fp = fopen (ctx->path, "a")) == NULL ||
-	    mbox_lock_mailbox (ctx, 1) != 0)
-	{
-	  if (ctx->fp == NULL)
-	    mutt_perror (ctx->path);
-	  mx_fastclose_mailbox (ctx);
-	  if (!pctx)
-	    safe_free ((void **) &ctx);
-	  return (NULL);
-	}
-	fseek (ctx->fp, 0, 2);
-	break;
-
-      case M_MH:
-      case M_MAILDIR:
-
-	/* nothing to do */
-	break;
-
-      default:
-
-	mx_fastclose_mailbox (ctx);
-	if (!pctx)
-	  safe_free ((void **) &ctx);
-	break;
+      /* avoid unnecessary work since the mailbox is completely unthreaded
+	 to begin with */
+      unset_option (OPTSORTSUBTHREADS);
+      unset_option (OPTNEEDRESCORE);
+      mutt_sort_headers (ctx, 1);
     }
+    if (!ctx->quiet)
+      mutt_clear_error ();
   }
   else
   {
-    /* read-mode */
-
-    switch (ctx->magic = mx_get_magic (path))
-    {
-      case 0:
-
-	mutt_error ("%s is not a mailbox.", path);
-	/* fall through */
-
-      case -1:
-
-	mx_fastclose_mailbox (ctx);
-	if (!pctx)
-	  free (ctx);
-	return (NULL);
-    }
-
-    if (!ctx->quiet)
-      mutt_message ("Reading %s...", ctx->path);
-
-    switch (ctx->magic)
-    {
-      case M_MH:
-	rc = mh_read_dir (ctx, NULL);
-	break;
-
-      case M_MAILDIR:
-	rc = maildir_read_dir (ctx);
-	break;
-
-      case M_MMDF:
-      case M_MBOX:
-	rc = mbox_open_mailbox (ctx);
-	break;
-
-#ifdef USE_IMAP
-      case M_IMAP:
-	rc = imap_open_mailbox (ctx);
-	break;
-#endif /* USE_IMAP */
-
-      default:
-	rc = -1;
-	break;
-    }
-    
-    if (rc == 0)
-    {
-      if ((flags & M_NOSORT) == 0)
-	mutt_sort_headers (ctx);
-      if (!ctx->quiet)
-	mutt_clear_error ();
-    }
-    else
-    {
-      mx_fastclose_mailbox (ctx);
-      if (!pctx)
-	safe_free ((void **) &ctx);
-    }
+    mx_fastclose_mailbox (ctx);
+    if (!pctx)
+      safe_free ((void **) &ctx);
   }
 
+  unset_option (OPTFORCEREFRESH);
   return (ctx);
 }
 
@@ -623,6 +687,10 @@ void mx_fastclose_mailbox (CONTEXT *ctx)
   if (ctx->magic == M_IMAP)
     imap_fastclose_mailbox (ctx);
 #endif /* USE_IMAP */
+  if (ctx->subj_hash)
+    hash_destroy (&ctx->subj_hash, NULL);
+  if (ctx->id_hash)
+    hash_destroy (&ctx->id_hash, NULL);
   for (i = 0; i < ctx->msgcount; i++)
     mutt_free_header (&ctx->hdrs[i]);
   safe_free ((void **) &ctx->hdrs);
@@ -647,7 +715,6 @@ static int sync_mailbox (CONTEXT *ctx)
   {
     case M_MBOX:
     case M_MMDF:
-    
       rc = mbox_sync_mailbox (ctx);
 #ifdef BUFFY_SIZE
       tmp = mutt_find_mailbox (ctx->path);
@@ -656,20 +723,19 @@ static int sync_mailbox (CONTEXT *ctx)
       
     case M_MH:
     case M_MAILDIR:
-    
       rc = mh_sync_mailbox (ctx);
       break;
       
 #ifdef USE_IMAP
     case M_IMAP:
-    
       rc = imap_sync_mailbox (ctx);
       break;
 #endif /* USE_IMAP */
   }
 
 #ifdef BUFFY_SIZE
-  mutt_update_mailbox (tmp);
+  if (tmp && tmp->new == 0)
+    mutt_update_mailbox (tmp);
 #endif
   return rc;
 }
@@ -706,7 +772,7 @@ int mx_close_mailbox (CONTEXT *ctx)
       read_msgs++;
   }
 
-  if (read_msgs && !option (OPTHOLD))
+  if (read_msgs && quadoption (OPT_MOVE) != M_NO)
   {
     char *p;
 
@@ -728,8 +794,6 @@ int mx_close_mailbox (CONTEXT *ctx)
       if ((move_messages = query_quadoption (OPT_MOVE, buf)) == -1)
 	return (-1);
     }
-    else
-      read_msgs = 0; /* no messages will be moved */
   }
 
   if (ctx->deleted)
@@ -738,10 +802,13 @@ int mx_close_mailbox (CONTEXT *ctx)
       return (-1);
   }
 
-  for (i = 0; i < ctx->msgcount; i++)
+  if (option (OPTMARKOLD))
   {
-    if (!ctx->hdrs[i]->deleted && !ctx->hdrs[i]->old && option (OPTMARKOLD))
-      mutt_set_flag (ctx, ctx->hdrs[i], M_OLD, 1);
+    for (i = 0; i < ctx->msgcount; i++)
+    {
+      if (!ctx->hdrs[i]->deleted && !ctx->hdrs[i]->old)
+	mutt_set_flag (ctx, ctx->hdrs[i], M_OLD, 1);
+    }
   }
 
   if (move_messages)
@@ -755,7 +822,7 @@ int mx_close_mailbox (CONTEXT *ctx)
     {
       if (ctx->hdrs[i]->read && !ctx->hdrs[i]->deleted)
       {
-	mx_append_message (ctx, i, &f);
+	mutt_append_message (&f, ctx, ctx->hdrs[i], 0, CH_UPDATE_LEN);
 	ctx->hdrs[i]->deleted = 1;
 	ctx->deleted++;
       }
@@ -844,7 +911,8 @@ int mx_sync_mailbox (CONTEXT *ctx)
   if ((rc = sync_mailbox (ctx)) == 0)
   {
     mutt_message ("%d kept, %d deleted.", ctx->msgcount - ctx->deleted,
-		ctx->deleted);
+		  ctx->deleted);
+    sleep (1); /* allow the user time to read the message */
 
     if (ctx->msgcount == ctx->deleted &&
 	(ctx->magic == M_MBOX || ctx->magic == M_MMDF) &&
@@ -860,9 +928,10 @@ int mx_sync_mailbox (CONTEXT *ctx)
     ctx->tagged = 0;
     ctx->deleted = 0;
     ctx->new = 0;
+    ctx->unread = 0;
     ctx->changed = 0;
     ctx->flagged = 0;
-    for (i=0, j=0; i<ctx->msgcount; i++)
+    for (i = 0, j = 0; i < ctx->msgcount; i++)
     {
       if (!ctx->hdrs[i]->deleted)
       {
@@ -882,16 +951,27 @@ int mx_sync_mailbox (CONTEXT *ctx)
 	  ctx->tagged++;
 	if (ctx->hdrs[j]->flagged)
 	  ctx->flagged++;
-	if (!ctx->hdrs[j]->read && !ctx->hdrs[j]->old)
-	  ctx->new++;
+	if (!ctx->hdrs[j]->read)
+	{ 
+	  ctx->unread++;
+	  if (!ctx->hdrs[j]->old)
+	    ctx->new++;
+	} 
 	j++;
       }
       else
+      {
+	/* remove message from the hash tables */
+	if (ctx->hdrs[i]->env->real_subj)
+	  hash_delete (ctx->subj_hash, ctx->hdrs[i]->env->real_subj, ctx->hdrs[i], NULL);
+	if (ctx->hdrs[i]->env->message_id)
+	  hash_delete (ctx->id_hash, ctx->hdrs[i]->env->message_id, ctx->hdrs[i], NULL);
 	mutt_free_header (&ctx->hdrs[i]);
+      }
     }
     ctx->msgcount = j;
 
-    mutt_sort_headers (ctx);
+    mutt_sort_headers (ctx, 1); /* rethread from scratch */
   }
 
   return (rc);
@@ -900,8 +980,7 @@ int mx_sync_mailbox (CONTEXT *ctx)
 int mh_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
 {
   int hi = 1;
-  int fd;
-  int n;
+  int fd, n;
   char *cp;
   char path[_POSIX_PATH_MAX];
   DIR *dirp;
@@ -966,8 +1045,6 @@ int maildir_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
 int mbox_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
 {
   msg->fp = dest->fp;
-  if (dest->magic == M_MMDF)
-    fputs (MMDF_SEP, msg->fp);
   return 0;
 }
 
@@ -976,10 +1053,12 @@ int mbox_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
  *	hdr	message being copied (required for maildir support, because
  *		the filename depends on the message flags)
  */
-MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr)
+MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
 {
   MESSAGE *msg;
   int (*func) (MESSAGE *, CONTEXT *, HEADER *);
+  ADDRESS *p = NULL;
+  time_t t;
 
   switch (dest->magic)
   {
@@ -987,15 +1066,12 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr)
     case M_MBOX:
       func = mbox_open_new_message;
       break;
-
     case M_MAILDIR:
       func = maildir_open_new_message;
       break;
-
     case M_MH:
       func = mh_open_new_message;
       break;
-
     default:
       dprint (1, (debugfile, "mx_open_new_message(): function unimplemented for mailbox type %d.\n",
 		  dest->magic));
@@ -1006,33 +1082,15 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr)
   msg->magic = dest->magic;
   msg->write = 1;
 
-  if (func (msg, dest, hdr) != 0)
-    safe_free ((void **) &msg);
-
-  return msg;
-}
-
-/* Called from mx_append_message () */
-int mutt_append_message (FILE *fp, HEADER *hdr, int has_from, CONTEXT *dest)
-{
-  MESSAGE *destmsg;               /* new message stream */
-  int chflags;                    /* flags to mutt_copy_header */
-  int r = -1;                     /* return value */
-
-  chflags = option (OPTFORWARD) ? (CH_XMIT | CH_MIME) : CH_UPDATE;
-  if ((destmsg = mx_open_new_message (dest, hdr)) != NULL)
+  if (func (msg, dest, hdr) == 0)
   {
-    if (!option (OPTFORWARD) && (dest->magic == M_MMDF || dest->magic == M_MBOX))
-    {
-      if (!has_from)
-      {
-	char tmp[LONG_STRING];
-	ADDRESS *p;
+    if (dest->magic == M_MMDF)
+      fputs (MMDF_SEP, msg->fp);
 
-	/*
-	 * need to generate a "From " line because these formats do not
-	 * use one.
-	 */
+    if (msg->magic != M_MAILDIR && (flags & M_ADD_FROM))
+    {
+      if (hdr)
+      {
 	if (hdr->env->return_path)
 	  p = hdr->env->return_path;
 	else if (hdr->env->sender)
@@ -1040,38 +1098,27 @@ int mutt_append_message (FILE *fp, HEADER *hdr, int has_from, CONTEXT *dest)
 	else
 	  p = hdr->env->from;
 
-	tmp[0] = 0;
-	mutt_simple_address (tmp, sizeof (tmp), p);
-
 	if (!hdr->received)
 	  hdr->received = time (NULL);
-	fprintf (destmsg->fp, "From %s %s", tmp[0] ? tmp : "-", ctime (&hdr->received));
+	t = hdr->received;
       }
       else
-	chflags |= CH_FROM;
+	t = time (NULL);
+
+      fprintf (msg->fp, "From %s %s", p ? p->mailbox : Username, ctime (&t));
     }
-    else if (dest->magic == M_MAILDIR)
-      chflags |= CH_NOSTATUS;
-
-    mutt_copy_header (fp, hdr, destmsg->fp, chflags);
-    /* mutt_copy_header() doesn't add '\n' with CH_XMIT */
-    if (option (OPTFORWARD))
-      fputc ('\n', destmsg->fp);
-    fseek (fp, hdr->content->offset, 0);
-    mutt_copy_bytes (fp, destmsg->fp, hdr->content->length);
-
-    mx_close_message (&destmsg);
-    r = 0;
   }
+  else
+    safe_free ((void **) &msg);
 
-  return (r);
+  return msg;
 }
 
-int mutt_reopen_mailbox (CONTEXT *ctx, unsigned short *index_hint)
+int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
 {
   int (*cmp_headers) (const HEADER *, const HEADER *) = NULL;
   HEADER **old_hdrs;
-  unsigned short old_msgcount;
+  int old_msgcount;
   int msg_mod = 0;
   int index_hint_set;
   int i, j;
@@ -1089,7 +1136,7 @@ int mutt_reopen_mailbox (CONTEXT *ctx, unsigned short *index_hint)
 
     old_sort = Sort;
     Sort = SORT_ORDER;
-    mutt_sort_headers (ctx);
+    mutt_sort_headers (ctx, 1);
     Sort = old_sort;
   }
 
@@ -1098,17 +1145,17 @@ int mutt_reopen_mailbox (CONTEXT *ctx, unsigned short *index_hint)
   old_hdrs = ctx->hdrs;
 
   /* simulate a close */
-  for (i = 0; i < ctx->msgcount; i++)
-  {
-    safe_free ((void **) &ctx->v2r);
-    if (ctx->readonly)
-      mutt_free_header (&(ctx->hdrs[i])); /* nothing to do! */
-    else
-      ctx->hdrs = NULL;
-  }
-
+  hash_destroy (&ctx->id_hash, NULL);
+  hash_destroy (&ctx->subj_hash, NULL);
+  safe_free ((void **) &ctx->v2r);
   if (ctx->readonly)
-    safe_free ((void **) &ctx->hdrs);
+  {
+    for (i = 0; i < ctx->msgcount; i++)
+      mutt_free_header (&(ctx->hdrs[i])); /* nothing to do! */
+      safe_free ((void **) &ctx->hdrs);
+  }
+  else
+    ctx->hdrs = NULL;
 
   ctx->hdrmax = 0;	/* force allocation of new headers */
   ctx->msgcount = 0;
@@ -1117,11 +1164,13 @@ int mutt_reopen_mailbox (CONTEXT *ctx, unsigned short *index_hint)
   ctx->deleted = 0;
   ctx->new = 0;
   ctx->flagged = 0;
+  ctx->changed = 0;
+  ctx->id_hash = hash_create (257);
+  ctx->subj_hash = hash_create (257);
 
   switch (ctx->magic)
   {
     case M_MBOX:
-
       fseek (ctx->fp, 0, 0);
       cmp_headers = mbox_strict_cmp_headers;
       rc = mbox_parse_mailbox (ctx);
@@ -1203,68 +1252,26 @@ int mutt_reopen_mailbox (CONTEXT *ctx, unsigned short *index_hint)
 	/* this is best done here */
 	if (!index_hint_set && *index_hint == j)
 	  *index_hint = i;
-	if (old_hdrs[j]->changed || old_hdrs[j]->deleted)
+
+	if (old_hdrs[j]->changed)
 	{
 	  /* Only update the flags if the old header was changed;
 	   * otherwise, the header may have been modified externally,
 	   * and we don't want to lose _those_ changes
 	   */
-	  if (ctx->hdrs[i]->flagged != old_hdrs[j]->flagged)
-	  {
-	    ctx->hdrs[i]->flagged = old_hdrs[j]->flagged;
-	    if (old_hdrs[j]->flagged)
-	      ctx->flagged++;
-	    else
-	      ctx->flagged--;
-	    ctx->hdrs[i]->changed = 1;
-	    ctx->changed = 1;
-	  }
-	  if (ctx->hdrs[i]->old != old_hdrs[j]->old)
-	  {
-	    ctx->hdrs[i]->old = old_hdrs[j]->old;
-	    ctx->hdrs[i]->changed = 1;
-	    ctx->changed = 1;
-	  }
-	  if (ctx->hdrs[i]->read != old_hdrs[j]->read)
-	  {
-	    ctx->hdrs[i]->read = old_hdrs[j]->read;
-	    ctx->hdrs[i]->changed = 1;
-	    ctx->changed = 1;
-	  }
-	  if (ctx->hdrs[i]->replied != old_hdrs[j]->replied)
-	  {
-	    ctx->hdrs[i]->replied = old_hdrs[j]->replied;
-	    ctx->hdrs[i]->changed = 1;
-	    ctx->changed = 1;
-	  }
-	  if (!ctx->hdrs[i]->deleted && old_hdrs[j]->deleted)
-	  {
-	    ctx->hdrs[i]->deleted = 1;
-	    ctx->deleted++;
-	    ctx->changed = 1;
-	  }
-	}
-	
-	/* tagged messages need to be restored anyway */
-	if (old_hdrs[j]->tagged)
-	{
-	  ctx->hdrs[i]->tagged = 1;
-	  ctx->tagged++;
+	  mutt_set_flag (ctx, ctx->hdrs[i], M_FLAG, old_hdrs[j]->flagged);
+	  mutt_set_flag (ctx, ctx->hdrs[i], M_REPLIED, old_hdrs[j]->replied);
+	  mutt_set_flag (ctx, ctx->hdrs[i], M_OLD, old_hdrs[j]->old);
+	  mutt_set_flag (ctx, ctx->hdrs[i], M_READ, old_hdrs[j]->read);
 	}
+	mutt_set_flag (ctx, ctx->hdrs[i], M_DELETE, old_hdrs[j]->deleted);
+	mutt_set_flag (ctx, ctx->hdrs[i], M_TAG, old_hdrs[j]->tagged);
 
 	/* we don't need this header any more */
 	mutt_free_header (&(old_hdrs[j]));
       }
     }
 
-    /* update the "new" count and the "changed" flag */
-    ctx->new = 0;
-    for (i = 0; i < ctx->msgcount; i++)
-    {
-      if (!ctx->hdrs[i]->old && (!ctx->hdrs[i]->read || option (OPTMARKOLD)))
-	ctx->new++;
-    }
-
     /* free the remaining old headers */
     for (j = 0; j < old_msgcount; j++)
     {
@@ -1279,211 +1286,11 @@ int mutt_reopen_mailbox (CONTEXT *ctx, unsigned short *index_hint)
 
   ctx->quiet = 0;
 
-  /* 2 = folder modified
-   * 1 = new mail has arrived
-   */
-  rc = (ctx->changed || msg_mod) ? 2 : 1;
-  return (rc);
-}
-
-/* append a message to another mailbox */
-int mx_append_message (CONTEXT *src, int msgno, CONTEXT *dest)
-{
-  MESSAGE *msg;			/* source message stream */
-  int r = -1;			/* return value */
-
-  if ((msg = mx_open_message (src, msgno)) != NULL)
-  {
-    mutt_parse_mime_message (src->hdrs[msgno]);
-    r = mutt_append_message (msg->fp, src->hdrs[msgno],
-		      (src->magic == M_MBOX || src->magic == M_MMDF), dest);
-    mx_close_message (&msg);
-  }
-  return r;
-}
-
-/* Called from mx_append_decoded() and mutt_view_attachments()
- * to build a new message with a given "flat" body
- */
-int mutt_append_decoded (FILE *fph, FILE *fpb, HEADER *hdr, BODY *b, CONTEXT *dest)
-{
-  char buffer[LONG_STRING];
-  char tempfile[_POSIX_PATH_MAX];
-  char attfile[_POSIX_PATH_MAX];
-  HEADER *hn;			/* destination header */
-  MESSAGE *msg;			/* new message stream */
-  STATE s;
-  FILE *tempfp = NULL;
-  struct stat st;
-  int r = -1;			/* return value */
-
-#ifdef _PGPPATH
-  if(hdr->pgp & PGPENCRYPT && !pgp_valid_passphrase())
-    return r;
-  unset_option (OPTVERIFYSIG);
-#endif
-
-  /* create the temporary file now in order to avoid memory cleanup
-   * if the operation fails
-   */
-  if (dest->magic == M_MMDF || dest->magic == M_MBOX)
-  {
-    mutt_mktemp (tempfile);
-    if ((tempfp = safe_fopen (tempfile, "w+")) == NULL)
-    {
-      mutt_perror (tempfile);
-      return r;
-    }
-  }
-
-  /* build a header for the new message */
-  hn = mutt_new_header ();
-  fseek (fph, hdr->offset, 0);
-  /* recover the relevant fields from the old header */
-  hn->env = _mutt_read_rfc822_header (fph, hn, 1);
-
-  /* we need to make sure we don't write lines with more than 72 characters,
-   * so we rely on _mutt_read_rfc822_header() to decode the headers, and
-   * we encode them back here
-   */
-  rfc2047_encode_adrlist (hn->env->from);
-  rfc2047_encode_adrlist (hn->env->to);
-  rfc2047_encode_adrlist (hn->env->cc);
-  rfc2047_encode_adrlist (hn->env->reply_to);
-  rfc2047_encode_adrlist (hn->env->return_path);
-  rfc2047_encode_adrlist (hn->env->sender);
-
-  if (hn->env->subject)
-  {
-    rfc2047_encode_string (buffer, (unsigned char *) hn->env->subject,
-                           sizeof (buffer) - 1);
-    safe_free ((void **) &hn->env->subject);
-    hn->env->subject = safe_strdup (buffer);
-  }
-
-  /* generate a new Message-Id: */
-  hn->env->message_id = mutt_gen_msgid ();
-
-  hn->pgp = 0;
-  if (dest->magic == M_MAILDIR)
-    hn->read = 1;
-
-  /* decode the attachments */
-  memset (&s, 0, sizeof (s));
-  s.fpin = fpb;
-  mutt_mktemp (attfile);
-  s.fpout = safe_fopen (attfile, "w+");
-
-  mutt_body_handler (b, &s);
-  fclose (s.fpout);
-
-  /* build a body for the new message */
-  hn->content->use_disp = 0;
-  hn->content->filename = safe_strdup (attfile);
-  /* the new message can still contain accented characters */
-  mutt_update_encoding (hn->content);
-
-  if ((msg = mx_open_new_message (dest, hn)) != NULL)
-  {
-    if (!option (OPTFORWARD) && (dest->magic == M_MMDF || dest->magic == M_MBOX))
-    {
-      /* we need a "From " line */
-      time_t now;
-      ADDRESS *p = NULL;
-
-      /* try something that makes sense first */
-      if (hdr->env->from)
-	p = hdr->env->from;
-      else if (hdr->env->sender)
-	p = hdr->env->sender;
-      else if (hdr->env->return_path)
-	p = hdr->env->return_path;
-
-      buffer[0] = 0;
-      mutt_simple_address (buffer, sizeof (buffer), p);
-      now = time (NULL);
-
-      fprintf (msg->fp, "From %s %s", buffer[0] ? buffer : Username, ctime (&now));
-    }
-
-    /* write the new header */
-    mutt_write_rfc822_header (msg->fp, hn->env, hn->content, 0);
-    if (!option (OPTFORWARD) && dest->magic != M_MAILDIR)
-      fputs ("Status: RO\n", msg->fp);
-
-    mutt_make_string (buffer, sizeof (buffer), DecodeFmt, hdr);
-
-    /* add a "Content-Length:" field */
-    if (!option (OPTFORWARD) && (dest->magic == M_MMDF || dest->magic == M_MBOX))
-    {
-      /* write the ID string */
-      fprintf (tempfp, "%s\n", buffer);
-      
-      mutt_write_mime_body (hn->content, tempfp);
-      
-      /* make sure the last line ends with a newline.  Emacs doesn't ensure
-       * this will happen, and it can cause problems parsing the mailbox
-       * later.
-       */
-      fseek (tempfp, -1, 2);
-      if (fgetc (tempfp) != '\n')
-      {
-	fseek (tempfp, 0, 2);
-	fputc ('\n', tempfp);
-      }
-      
-      fflush (tempfp);
-      rewind (tempfp);
-      fstat (fileno (tempfp), &st);
-      fprintf (msg->fp, "Content-Length: %ld\n\n", (long) st.st_size);
-
-      /* copy the body */
-      mutt_copy_stream (tempfp, msg->fp);
-    }
-    else
-    {
-      /* finish off the header and write the ID string */
-      fprintf (msg->fp, "\n%s\n", buffer);
-      
-      /* write the new body */
-      mutt_write_mime_body (hn->content, msg->fp);
-    }
-
-
-    mx_close_message (&msg);
-    r = 0;
-  }
-  
-  if (tempfp)
-  {
-    fclose (tempfp);
-    unlink (tempfile);
-  }
-
-  unlink (hn->content->filename);
-  mutt_free_header (&hn);
-
-  return (r);
-}
-
-/* append a decoded version of a message to another mailbox */
-int mx_append_decoded (CONTEXT *src, int msgno, CONTEXT *dest)
-{
-  MESSAGE *msg;			/* source message stream */
-  int r = -1;			/* return value */
-  
-  if ((msg = mx_open_message (src, msgno)) != NULL)
-  {
-    mutt_parse_mime_message (src->hdrs[msgno]);
-    r = mutt_append_decoded (msg->fp, msg->fp, src->hdrs[msgno],
-				      src->hdrs[msgno]->content, dest);
-    mx_close_message (&msg);
-  }
-  return r;
+  return ((ctx->changed || msg_mod) ? M_REOPENED : M_NEW_MAIL);
 }
 
 /* check for new mail */
-int _mx_check_mailbox (CONTEXT *ctx, unsigned short *index_hint)
+int mx_check_mailbox (CONTEXT *ctx, int *index_hint)
 {
   if (ctx)
   {
@@ -1590,12 +1397,12 @@ void mx_alloc_memory (CONTEXT *ctx)
   if (ctx->hdrs)
   {
     safe_realloc ((void **) &ctx->hdrs, sizeof (HEADER *) * (ctx->hdrmax += 25));
-    safe_realloc ((void **) &ctx->v2r, sizeof (short) * ctx->hdrmax);
+    safe_realloc ((void **) &ctx->v2r, sizeof (int) * ctx->hdrmax);
   }
   else
   {
     ctx->hdrs = safe_malloc (sizeof (HEADER *) * (ctx->hdrmax += 25));
-    ctx->v2r = safe_malloc (sizeof (short) * ctx->hdrmax);
+    ctx->v2r = safe_malloc (sizeof (int) * ctx->hdrmax);
   }
   for (i = ctx->msgcount ; i < ctx->hdrmax ; i++)
   {
@@ -1611,19 +1418,18 @@ void mx_update_context (CONTEXT *ctx)
 {
   HEADER *h = ctx->hdrs[ctx->msgcount];
 
-#ifdef _PGPPATH
-  /* NOTE: this _must_ be done before the check for mailcap! */
-  h->pgp = pgp_query (h->content);
-  if (!h->pgp)
-#endif /* _PGPPATH */
     if (mutt_needs_mailcap (h->content))
       h->mailcap = 1;
   if (h->flagged)
     ctx->flagged++;
   if (h->deleted)
     ctx->deleted++;
-  if (!h->read && !h->old)
-    ctx->new++;
+  if (!h->read)
+  {
+    ctx->unread++;
+    if (!h->old)
+      ctx->new++;
+  }
   if (ctx->vcount == ctx->msgcount)
   {
     ctx->v2r[ctx->vcount] = ctx->msgcount;
@@ -1633,12 +1439,24 @@ void mx_update_context (CONTEXT *ctx)
     h->virtual = -1;
   h->msgno = ctx->msgcount;
   ctx->msgcount++;
-}
 
-int mx_copy_message (HEADER *h, FILE *in, FILE *out)
-{
-  if (mutt_copy_header (in, h, out, CH_FROM | CH_UPDATE) == -1)
-    return (-1);
+  if (h->env->supersedes)
+  {
+    HEADER *h2 = hash_find (ctx->id_hash, h->env->supersedes);
 
-  return (mutt_copy_bytes (in, out, h->content->length));
+    /* safe_free (&h->env->supersedes); should I ? */
+    if (h2)
+    {
+      h2->superseded = 1;
+      mutt_score_message (h2);
+    }
+  }
+
+  /* add this message to the hash tables */
+  if (h->env->message_id)
+    hash_insert (ctx->id_hash, h->env->message_id, h, 0);
+  if (h->env->real_subj)
+    hash_insert (ctx->subj_hash, h->env->real_subj, h, 1);
+
+  mutt_score_message (h);
 }
