@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -21,9 +21,11 @@
 #include "mutt_regex.h"
 #include "keymap.h"
 #include "mutt_menu.h"
-#ifdef _PGPPATH
-#include "pgp.h"
-#endif
+#include "sort.h"
+#include "pager.h"
+#include "attach.h"
+
+
 
 #include <sys/stat.h>
 #include <ctype.h>
@@ -41,7 +43,10 @@
 
 #define ISHEADER(x) ((x) == MT_COLOR_HEADER || (x) == MT_COLOR_HDEFAULT)
 
-#define CHECK_MODE	if (!hdr) \
+#define IsAttach(x) (x && (x)->bdy)
+#define IsHeader(x) (x && (x)->hdr)
+
+#define CHECK_MODE(x)	if (!(x)) \
 			{ \
 			  	mutt_flushinp (); \
 				mutt_error ("Not available in this menu."); \
@@ -1225,13 +1230,12 @@ upNLines (int nlines, struct line_t *info, int cur, int hiding)
 }
 
 /* This pager is actually not so simple as it once was.  It now operates in
- * two modes: one for viewing messages and the other for viewing help.  These
- * can be distinguished by whether or not ``hdr'' is NULL.  The ``hdr'' arg
- * is there so that we can do operations on the current message without the
- * need to pop back out to the main-menu.
- */
+   two modes: one for viewing messages and the other for viewing help.  These
+   can be distinguished by whether or not ``hdr'' is NULL.  The ``hdr'' arg
+   is there so that we can do operations on the current message without the
+   need to pop back out to the main-menu.  */
 int 
-mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
+mutt_pager (const char *banner, const char *fname, int do_color, pager_t *extra)
 {
   static char searchbuf[STRING];
   char buffer[LONG_STRING];
@@ -1242,14 +1246,14 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
   int i, j, ch = 0, rc = -1, hideQuoted = 0, q_level = 0, force_redraw = 0;
   int lines = 0, curline = 0, topline = 0, oldtopline = 0, err, first = 1;
   int r = -1;
-  short redraw = REDRAW_FULL;
+  int redraw = REDRAW_FULL;
   FILE *fp = NULL;
   long last_pos = 0, last_offset = 0;
   int old_smart_wrap, old_markers;
   struct stat sb;
   regex_t SearchRE;
   int SearchCompiled = 0, SearchFlag = 0, SearchBack = 0;
-  int has_types = (hdr || do_color); /* main message or rfc822 attachment */
+  int has_types = (IsHeader(extra) || do_color); /* main message or rfc822 attachment */
 
   int bodyoffset = 1;			/* offset of first line of real text */
   int statusoffset = 0; 		/* offset for the status bar */
@@ -1263,6 +1267,20 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
   int old_PagerIndexLines;		/* some people want to resize it
   					 * while inside the pager... */
 
+  static struct mapping_t PagerHelp[] = {
+    { "Exit",	OP_PAGER_EXIT },
+    { "PrevPg",	OP_PREV_PAGE },
+    { "NextPg  ", OP_NEXT_PAGE },
+    { NULL,	0 }
+  };
+  static struct mapping_t PagerHelpExtra[] = {
+    { "Attach",	OP_VIEW_ATTACHMENTS },
+    { "Del",	OP_DELETE },
+    { "Reply",	OP_REPLY },
+    { "Next  ",	OP_MAIN_NEXT_UNDELETED },
+    { NULL,	0 }
+  };
+
   do_color = do_color ? M_SHOWCOLOR : M_SHOWFLAT;
 
   if ((fp = fopen (fname, "r")) == NULL)
@@ -1281,10 +1299,10 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 
   /* Initialize variables */
 
-  if (hdr && !hdr->read)
+  if (IsHeader (extra) && !extra->hdr->read)
   {
-    Context->msgnotreadyet = hdr->msgno;
-    mutt_set_flag (Context, hdr, M_READ, 1);
+    Context->msgnotreadyet = extra->hdr->msgno;
+    mutt_set_flag (Context, extra->hdr, M_READ, 1);
   }
 
   lineInfo = safe_malloc (sizeof (struct line_t) * (maxLine = LINES));
@@ -1297,26 +1315,12 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
     (lineInfo[i].syntax)[0].first = (lineInfo[i].syntax)[0].last = -1;
   }
 
-  helpstr[0] = 0;
-  mutt_make_help (buffer, sizeof (buffer), "Exit  ", MENU_PAGER, OP_PAGER_EXIT);
-  strcat (helpstr, buffer);
-  mutt_make_help (buffer, sizeof (buffer), "PrevPg  ", MENU_PAGER, OP_PREV_PAGE);
-  strcat (helpstr, buffer);
-  mutt_make_help (buffer, sizeof (buffer), "NextPg  ", MENU_PAGER, OP_NEXT_PAGE);
-  strcat (helpstr, buffer);
-  
-  if (hdr)
+  mutt_compile_help (helpstr, sizeof (helpstr), MENU_PAGER, PagerHelp);
+  if (IsHeader (extra))
   {
-    mutt_make_help (buffer, sizeof (buffer), "Attach  ", MENU_PAGER, OP_VIEW_ATTACHMENTS);
-    strcat (helpstr, buffer);
-    mutt_make_help (buffer, sizeof (buffer), "Del  ", MENU_PAGER, OP_DELETE);
-    strcat (helpstr, buffer);
-    mutt_make_help (buffer, sizeof (buffer), "Reply  ", MENU_PAGER, OP_REPLY);
-    strcat (helpstr, buffer);
-    mutt_make_help (buffer, sizeof (buffer), "Next  ", MENU_PAGER, OP_MAIN_NEXT_UNDELETED);
+    mutt_compile_help (buffer, sizeof (buffer), MENU_PAGER, PagerHelpExtra);
     strcat (helpstr, buffer);
   }
-
   if (!InHelp)
   {
     mutt_make_help (buffer, sizeof (buffer), "Help", MENU_PAGER, OP_HELP);
@@ -1334,7 +1338,7 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
       move (0, 0);
       clrtobot ();
 
-      if (hdr != NULL && Context->vcount < PagerIndexLines - 1)
+      if (IsHeader (extra) && Context->vcount + 1 < PagerIndexLines)
 	indexlen = Context->vcount + 1;
       else
 	indexlen = PagerIndexLines;
@@ -1344,7 +1348,7 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
       if (option (OPTSTATUSONTOP))
       {
 	indexoffset = 0;
-	statusoffset = hdr ? indexlen : 0;
+	statusoffset = IsHeader (extra) ? indexlen : 0;
 	bodyoffset = statusoffset + 1;
 	helpoffset = LINES - 2;
 	bodylen = helpoffset - bodyoffset;
@@ -1358,7 +1362,7 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	statusoffset = LINES - 2;
 	if (!option (OPTHELP))
 	  indexoffset = 0;
-	bodyoffset = indexoffset + (hdr ? indexlen : 0);
+	bodyoffset = indexoffset + (IsHeader (extra) ? indexlen : 0);
 	bodylen = statusoffset - bodyoffset;
       }
 
@@ -1369,7 +1373,7 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	SETCOLOR (MT_COLOR_NORMAL);
       }
 
-      if (hdr && PagerIndexLines)
+      if (IsHeader (extra) && PagerIndexLines)
       {
 	if (index == NULL)
 	{
@@ -1379,7 +1383,7 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	  index->menu = MENU_MAIN;
 	  index->make_entry = index_make_entry;
 	  index->max = Context->vcount;
-	  index->current = hdr->virtual;
+	  index->current = extra->hdr->virtual;
 	}
 
 	SETCOLOR (MT_COLOR_NORMAL);
@@ -1398,14 +1402,11 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	menu_redraw_index(index);
       }
 
-      redraw &= ~REDRAW_FULL;
-      redraw |= REDRAW_INDEX | REDRAW_STATUS;
+      redraw |= REDRAW_BODY | REDRAW_INDEX | REDRAW_STATUS;
       mutt_show_error ();
     }
 
-    /* REDRAW_INDEX does NOT refer to the pager index.	We use it 
-       to redraw the body, because we don't have a REDRAW_BODY */
-    if ((redraw & REDRAW_INDEX) || topline != oldtopline)
+    if ((redraw & REDRAW_BODY) || topline != oldtopline)
     {
       do {
 	move (bodyoffset, 0);
@@ -1434,50 +1435,51 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	addch ('\n');
 	lines++;
       }
-      BKGDSET (MT_COLOR_NORMAL);
+      /* We are going to update the pager status bar, so it isn't
+       * necessary to reset to normal color now. */
 
-      redraw &= ~REDRAW_INDEX;
       redraw |= REDRAW_STATUS; /* need to update the % seen */
     }
 
     if (redraw & REDRAW_STATUS)
     {
-      /* redraw the pager_index indicator, because the
-       * flags for this message might have changed. */
-      if (PagerIndexLines && index)
-	menu_redraw_current (index);
-
       /* print out the pager status bar */
       SETCOLOR (MT_COLOR_STATUS);
       BKGDSET (MT_COLOR_STATUS);
       CLEARLINE (statusoffset);
-      if (hdr)
+      if (IsHeader (extra))
       {
-	mutt_make_string (buffer,
-			  COLS-9 < sizeof (buffer) ? COLS-9 : sizeof (buffer),
-			  PagerFmt, hdr);
+	_mutt_make_string (buffer,
+			   COLS-9 < sizeof (buffer) ? COLS-9 : sizeof (buffer),
+			   PagerFmt, extra->hdr, M_MAKEPRINT);
       }
-      printw ("%-*.*s -- (", COLS-10, COLS-10, hdr ? buffer : banner);
+      printw ("%-*.*s -- (", COLS-10, COLS-10, IsHeader (extra) ? buffer : banner);
       if (last_pos < sb.st_size - 1)
 	printw ("%d%%)", (int) (100 * last_offset / sb.st_size));
       else
 	addstr (topline == 0 ? "all)" : "end)");
-	
-      /* print out the index status bar */
-      if (hdr && PagerIndexLines)
-      {
-	mutt_status_line (buffer, sizeof(buffer), Status);
-	SETCOLOR (MT_COLOR_STATUS);
-	CLEARLINE (indexoffset + (option(OPTSTATUSONTOP) ? 0 : (indexlen - 1)));
-	printw ("%-*.*s", COLS, COLS, buffer);
-      }
       BKGDSET (MT_COLOR_NORMAL);
       SETCOLOR (MT_COLOR_NORMAL);
-      redraw &= ~REDRAW_STATUS;
     }
 
+    if ((redraw & REDRAW_INDEX) && index)
+    {
+      /* redraw the pager_index indicator, because the
+       * flags for this message might have changed. */
+      menu_redraw_current (index);
+
+      /* print out the index status bar */
+      menu_status_line (buffer, sizeof (buffer), index, Status);
+      move (indexoffset + (option (OPTSTATUSONTOP) ? 0 : (indexlen - 1)), 0);
+      SETCOLOR (MT_COLOR_STATUS);
+      printw ("%-*.*s", COLS, COLS, buffer);
+      SETCOLOR (MT_COLOR_NORMAL);
+    }
+
+    redraw = 0;
+
     move (statusoffset, COLS-1);
-    refresh ();
+    mutt_refresh ();
     ch = km_dokey (MENU_PAGER);
     mutt_clear_error ();
     mutt_curs_set (1);
@@ -1533,13 +1535,11 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
     switch (ch)
     {
       case OP_PAGER_EXIT:
-
 	rc = -1;
 	ch = -1;
 	break;
 
       case OP_NEXT_PAGE:
-
 	if (lineInfo[curline].offset < sb.st_size-1)
 	{
 	  topline = upNLines (PagerContext, lineInfo, curline, hideQuoted);
@@ -1558,7 +1558,6 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_PREV_PAGE:
-
 	if (topline != 0)
 	{
 	  topline = upNLines (bodylen-PagerContext, lineInfo, topline, hideQuoted);
@@ -1568,7 +1567,6 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_NEXT_LINE:
-
 	if (lineInfo[curline].offset < sb.st_size-1)
 	{
 	  topline++;
@@ -1584,7 +1582,6 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_PREV_LINE:
-
 	if (topline)
 	  topline = upNLines (1, lineInfo, topline, hideQuoted);
 	else
@@ -1592,20 +1589,17 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_PAGER_TOP:
-
 	topline = 0;
 	break;
 
-      case OP_PAGER_HALF_UP:
-
+      case OP_HALF_UP:
 	if (topline)
 	  topline = upNLines (bodylen/2, lineInfo, topline, hideQuoted);
 	else
 	  mutt_error ("Top of message is shown.");
 	break;
 
-      case OP_PAGER_HALF_DOWN:
-
+      case OP_HALF_DOWN:
 	if (lineInfo[curline].offset < sb.st_size-1)
 	{
 	  topline = upNLines (bodylen/2, lineInfo, curline, hideQuoted);
@@ -1624,10 +1618,11 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_SEARCH_NEXT:
-
+      case OP_SEARCH_OPPOSITE:
 	if (SearchCompiled)
 	{
-	  if (!SearchBack)
+	  if ((!SearchBack && ch==OP_SEARCH_NEXT) ||
+	      (SearchBack &&ch==OP_SEARCH_OPPOSITE))
 	  {
 	    /* searching forward */
 	    for (i = topline + 1; i < lastLine; i++)
@@ -1668,14 +1663,13 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 
       case OP_SEARCH:
       case OP_SEARCH_REVERSE:
-
 	/* leave SearchBack alone if ch == OP_SEARCH_NEXT */
 	if (ch == OP_SEARCH)
 	  SearchBack = 0;
 	else if (ch == OP_SEARCH_REVERSE)
 	  SearchBack = 1;
 
-	if (ci_get_field ((SearchBack ? "Reverse search: " : "Search: "), 
+	if (mutt_get_field ((SearchBack ? "Reverse search: " : "Search: "), 
 			  searchbuf, sizeof (searchbuf), M_CLEAR) != 0 || 
 			  !searchbuf[0])
 	  break;
@@ -1691,7 +1685,7 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	  }
 	}
 
-	if ((err = regcomp (&SearchRE, searchbuf, REG_EXTENDED | mutt_which_case (searchbuf))) != 0)
+	if ((err = REGCOMP (&SearchRE, searchbuf, REG_NEWLINE | mutt_which_case (searchbuf))) != 0)
 	{
 	  regerror (err, &SearchRE, buffer, sizeof (buffer));
 	  mutt_error ("%s", buffer);
@@ -1749,20 +1743,18 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	  else
 	    SearchFlag = M_SEARCH;
 	}
-	redraw = REDRAW_INDEX;
+	redraw = REDRAW_BODY;
 	break;
 
       case OP_SEARCH_TOGGLE:
-
 	if (SearchCompiled)
 	{
 	  SearchFlag ^= M_SEARCH;
-	  redraw = REDRAW_INDEX;
+	  redraw = REDRAW_BODY;
 	}
 	break;
 
       case OP_HELP:
-
 	/* don't let the user enter the help-menu from the help screen! */
 	if (! InHelp)
 	{
@@ -1776,19 +1768,52 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_PAGER_HIDE_QUOTED:
-
 	if (has_types)
 	{
 	  hideQuoted = hideQuoted ? 0 : M_HIDE;
 	  if (hideQuoted && lineInfo[topline].type == MT_COLOR_QUOTED)
 	    topline = upNLines (1, lineInfo, topline, hideQuoted);
 	  else
-	    redraw = 1;
+	    redraw = REDRAW_BODY;
+	}
+	break;
+
+      case OP_PAGER_SKIP_QUOTED:
+	if (has_types)
+	{
+	  int dretval = 0;
+	  int new_topline = topline;
+
+	  while ((new_topline < lastLine ||
+		  (0 == (dretval = display_line (fp, &last_pos, &lineInfo,
+			 new_topline, &lastLine, &maxLine, M_TYPES,
+			 &QuoteList, &q_level, &force_redraw, &SearchRE))))
+		 && lineInfo[new_topline].type != MT_COLOR_QUOTED)
+	    new_topline++;
+
+	  if (dretval < 0)
+	  {
+	    mutt_error ("No more quoted text.");
+	    break;
+	  }
+
+	  while ((new_topline < lastLine ||
+		  (0 == (dretval = display_line (fp, &last_pos, &lineInfo,
+			 new_topline, &lastLine, &maxLine, M_TYPES,
+			 &QuoteList, &q_level, &force_redraw, &SearchRE))))
+		 && lineInfo[new_topline].type == MT_COLOR_QUOTED)
+	    new_topline++;
+
+	  if (dretval < 0)
+	  {
+	    mutt_error ("No more unquoted text after quoted text.");
+	    break;	  
+	  }
+	  topline = new_topline;
 	}
 	break;
 
       case OP_PAGER_BOTTOM: /* move to the end of the file */
-
 	if (lineInfo[curline].offset < sb.st_size - 1)
 	{
 	  i = curline;
@@ -1805,13 +1830,11 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_REDRAW:
-
 	clearok (stdscr, TRUE);
 	redraw = REDRAW_FULL;
 	break;
 
       case OP_NULL:
-
 	km_error_key (MENU_PAGER);
 	break;
 
@@ -1821,30 +1844,21 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	 */
 
       case OP_BOUNCE_MESSAGE:
-
-	CHECK_MODE;
-	ci_bounce_message (hdr, &redraw);
-	break;
-
-      case OP_BROWSE_URL:
-
-	CHECK_MODE;
-	mutt_browse_url (hdr);
-	redraw = REDRAW_FULL;
+	CHECK_MODE(IsHeader (extra));
+	ci_bounce_message (extra->hdr, &redraw);
 	break;
 
       case OP_CREATE_ALIAS:
-
-	CHECK_MODE;
-	mutt_create_alias (hdr->env);
+	CHECK_MODE(IsHeader (extra));
+	mutt_create_alias (extra->hdr->env);
+	MAYBE_REDRAW (redraw);
 	break;
 
       case OP_DELETE:
-
-	CHECK_MODE;
+	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
-	mutt_set_flag (Context, hdr, M_DELETE, 1);
-	redraw = REDRAW_STATUS;
+	mutt_set_flag (Context, extra->hdr, M_DELETE, 1);
+	redraw = REDRAW_STATUS | REDRAW_INDEX;
 	if (option (OPTRESOLVE))
 	{
 	  ch = -1;
@@ -1854,14 +1868,11 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 
       case OP_DELETE_THREAD:
       case OP_DELETE_SUBTHREAD:
-
-	CHECK_MODE;
+	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
 
-	if (ch == OP_DELETE_THREAD)
-	  r = mutt_thread_set_flag (hdr, M_DELETE, 1);
-	else
-	  r = mutt_subthread_set_flag (hdr, M_DELETE, 1);
+	r = mutt_thread_set_flag (extra->hdr, M_DELETE, 1,
+				  ch == OP_DELETE_THREAD ? 0 : 1);
 
 	if (r != -1)
 	{
@@ -1875,18 +1886,16 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	  if (!option (OPTRESOLVE) && PagerIndexLines)
 	    redraw = REDRAW_FULL;
 	  else
-	    redraw = REDRAW_STATUS;
+	    redraw = REDRAW_STATUS | REDRAW_INDEX;
 	}
 	break;
 
       case OP_DISPLAY_ADDRESS:
-
-	CHECK_MODE;
-	mutt_display_address (hdr->env->from);
+	CHECK_MODE(IsHeader (extra));
+	mutt_display_address (extra->hdr->env->from);
 	break;
 
       case OP_ENTER_COMMAND:
-
 	old_smart_wrap = option (OPTWRAP);
 	old_markers = option (OPTMARKERS);
 	old_PagerIndexLines = PagerIndexLines;
@@ -1896,7 +1905,7 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	if (option (OPTNEEDRESORT))
 	{
 	  unset_option (OPTNEEDRESORT);
-	  CHECK_MODE;
+	  CHECK_MODE(IsHeader (extra));
 	  set_option (OPTNEEDRESORT);
 	}
 
@@ -1965,11 +1974,10 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_FLAG_MESSAGE:
-	
-	CHECK_MODE;
+	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
-	mutt_set_flag (Context, hdr, M_FLAG, !hdr->flagged);
-	redraw = REDRAW_STATUS;
+	mutt_set_flag (Context, extra->hdr, M_FLAG, !extra->hdr->flagged);
+	redraw = REDRAW_STATUS | REDRAW_INDEX;
 	if (option (OPTRESOLVE))
 	{
 	  ch = -1;
@@ -1977,85 +1985,70 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	}
 	break;
 
-#ifdef _PGPPATH
-      case OP_FORGET_PASSPHRASE:
-
-	mutt_forget_passphrase ();
-	break;
-#endif /* _PGPPATH */
-
       case OP_PIPE:
-
-	CHECK_MODE;
-	mutt_pipe_message (hdr);
+	CHECK_MODE(IsHeader (extra) || IsAttach (extra));
+	if (IsAttach (extra))
+	  mutt_pipe_attachment_list (extra->fp, 0, extra->bdy, 0);
+	else
+	  mutt_pipe_message (extra->hdr);
 	break;
 
       case OP_PRINT:
-	
-	CHECK_MODE;
-	mutt_print_message (hdr);
+	CHECK_MODE(IsHeader (extra));
+	mutt_print_message (extra->hdr);
 	break;
 
       case OP_MAIL:
-
-	CHECK_MODE;
-	ci_send_message (0, NULL, NULL, NULL);
+	CHECK_MODE(IsHeader (extra));
+	ci_send_message (0, NULL, NULL, NULL, NULL);
 	redraw = REDRAW_FULL;
 	break;
 
       case OP_REPLY:
-
-	CHECK_MODE;
-	ci_send_message (SENDREPLY, NULL, NULL, hdr);
+	CHECK_MODE(IsHeader (extra));
+	ci_send_message (SENDREPLY, NULL, NULL, extra->ctx, extra->hdr);
 	redraw = REDRAW_FULL;
 	break;
 
       case OP_RECALL_MESSAGE:
-
-	CHECK_MODE;
-	ci_send_message (SENDPOSTPONED, NULL, NULL, hdr);
+	CHECK_MODE(IsHeader (extra));
+	ci_send_message (SENDPOSTPONED, NULL, NULL, extra->ctx, extra->hdr);
 	redraw = REDRAW_FULL;
 	break;
 
       case OP_GROUP_REPLY:
-
-	CHECK_MODE;
-	ci_send_message (SENDREPLY | SENDGROUPREPLY, NULL, NULL, hdr);
+	CHECK_MODE(IsHeader (extra));
+	ci_send_message (SENDREPLY | SENDGROUPREPLY, NULL, NULL, extra->ctx, extra->hdr);
 	redraw = REDRAW_FULL;
 	break;
 
       case OP_LIST_REPLY:
-
-	CHECK_MODE;
-	ci_send_message (SENDREPLY | SENDLISTREPLY, NULL, NULL, hdr);
+	CHECK_MODE(IsHeader (extra));
+	ci_send_message (SENDREPLY | SENDLISTREPLY, NULL, NULL, extra->ctx, extra->hdr);
 	redraw = REDRAW_FULL;
 	break;
 
       case OP_FORWARD_MESSAGE:
-
-	CHECK_MODE;
-	ci_send_message (SENDFORWARD, NULL, NULL, hdr);
+	CHECK_MODE(IsHeader (extra));
+	ci_send_message (SENDFORWARD, NULL, NULL, extra->ctx, extra->hdr);
 	redraw = REDRAW_FULL;
 	break;
 
       case OP_SAVE:
+	if (IsAttach (extra))
+	{
+	  mutt_save_attachment_list (extra->fp, 0, extra->bdy);
+	  break;
+	}
+	/* fall through */
       case OP_COPY_MESSAGE:
       case OP_DECODE_SAVE:
       case OP_DECODE_COPY:
-
-	CHECK_MODE;
-	j = 0;
-	if (ch == OP_DECODE_COPY)
-	{
-	  ch = OP_COPY_MESSAGE;
-	  j = 1;
-	}
-	else if (ch == OP_DECODE_SAVE)
-	{
-	  ch = OP_SAVE;
-	  j = 1;
-	}
-	if (mutt_save_message (hdr, (ch == OP_SAVE), j, &redraw) == 0 && ch == OP_SAVE)
+	CHECK_MODE(IsHeader (extra));
+	if (mutt_save_message (extra->hdr,
+			       (ch == OP_SAVE || ch == OP_DECODE_SAVE),
+			       (ch == OP_DECODE_SAVE || ch == OP_DECODE_COPY),
+			       &redraw) == 0 && (ch == OP_SAVE || ch == OP_DECODE_SAVE))
 	{
 	  if (option (OPTRESOLVE))
 	  {
@@ -2063,22 +2056,20 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	    rc = OP_MAIN_NEXT_UNDELETED;
 	  }
 	  else
-	    redraw |= REDRAW_STATUS;
+	    redraw |= REDRAW_STATUS | REDRAW_INDEX;
 	}
 	MAYBE_REDRAW (redraw);
 	break;
 
       case OP_SHELL_ESCAPE:
-	
 	mutt_shell_escape ();
 	MAYBE_REDRAW (redraw);
 	break;
 
-      case OP_TAG_MESSAGE:
-
-	CHECK_MODE;
-	mutt_set_flag (Context, hdr, M_TAG, !hdr->tagged);
-	redraw = REDRAW_STATUS;
+      case OP_TAG:
+	CHECK_MODE(IsHeader (extra));
+	mutt_set_flag (Context, extra->hdr, M_TAG, !extra->hdr->tagged);
+	redraw = REDRAW_STATUS | REDRAW_INDEX;
 	if (option (OPTRESOLVE))
 	{
 	  ch = -1;
@@ -2087,16 +2078,15 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_TOGGLE_NEW:
-
-	CHECK_MODE;
+	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
-	if (hdr->read || hdr->old)
-	  mutt_set_flag (Context, hdr, M_NEW, 1);
+	if (extra->hdr->read || extra->hdr->old)
+	  mutt_set_flag (Context, extra->hdr, M_NEW, 1);
 	else if (!first)
-	  mutt_set_flag (Context, hdr, M_READ, 1);
+	  mutt_set_flag (Context, extra->hdr, M_READ, 1);
 	first = 0;
         Context->msgnotreadyet = -1;
-	redraw = REDRAW_STATUS;
+	redraw = REDRAW_STATUS | REDRAW_INDEX;
 	if (option (OPTRESOLVE))
 	{
 	  ch = -1;
@@ -2105,11 +2095,10 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	break;
 
       case OP_UNDELETE:
-
-	CHECK_MODE;
+	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
-	mutt_set_flag (Context, hdr, M_DELETE, 0);
-	redraw = REDRAW_STATUS;
+	mutt_set_flag (Context, extra->hdr, M_DELETE, 0);
+	redraw = REDRAW_STATUS | REDRAW_INDEX;
 	if (option (OPTRESOLVE))
 	{
 	  ch = -1;
@@ -2119,26 +2108,11 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 
       case OP_UNDELETE_THREAD:
       case OP_UNDELETE_SUBTHREAD:
-
-	CHECK_MODE;
+	CHECK_MODE(IsHeader (extra));
 	CHECK_READONLY;
 
-	if ((Sort & SORT_MASK) != SORT_THREADS)
-	{
-	  mutt_error ("Threading is not enabled.");
-	  break;
-	}
-
-	if (ch == OP_UNDELETE_THREAD)
-	  r = mutt_thread_set_flag (hdr, M_DELETE, 0);
-	else
-	{
-	  HEADER *h = hdr;
-
-	  while (h->parent && h->deleted)
-	    h = h->parent;
-	  r = mutt_subthread_set_flag (h, M_DELETE, 0);
-	}
+	r = mutt_thread_set_flag (extra->hdr, M_DELETE, 0,
+				  ch == OP_UNDELETE_THREAD ? 0 : 1);
 
 	if (r != -1)
 	{
@@ -2152,31 +2126,33 @@ mutt_pager (const char *banner, const char *fname, HEADER *hdr, int do_color)
 	  if (!option (OPTRESOLVE) && PagerIndexLines)
 	    redraw = REDRAW_FULL;
 	  else
-	    redraw = REDRAW_STATUS;
+	    redraw = REDRAW_STATUS | REDRAW_INDEX;
 	}
 	break;
 
       case OP_VERSION:
-
-	mutt_message (MuttVersion);
+	mutt_version ();
 	break;
 
       case OP_VIEW_ATTACHMENTS:
-
-	CHECK_MODE;
-	mutt_view_attachments (hdr);
+	CHECK_MODE(IsHeader (extra));
+	mutt_view_attachments (extra->hdr);
 	redraw = REDRAW_FULL;
 	break;
 
-      default:
 
+
+
+
+
+      default:
 	ch = -1;
 	break;
     }
   }
 
   fclose (fp);
-  if (hdr)
+  if (IsHeader (extra))
     Context->msgnotreadyet = -1;
     
   cleanup_quote (&QuoteList);
