@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -18,10 +18,9 @@
 
 #include "mutt.h"
 #include "mutt_regex.h"
+#include "mailbox.h"
+#include "mime.h"
 #include "rfc2047.h"
-#ifdef _PGPPATH
-#include "pgp.h"
-#endif /* _PGPPATH */
 
 #include <string.h>
 #include <ctype.h>
@@ -29,90 +28,56 @@
 #include <stdlib.h>
 
 /* Reads an arbitrarily long header field, and looks ahead for continuation
- * lines.  ``line'' is dynamically increased if more space is required to fit
- * the whole line.
+ * lines.  ``line'' must point to a dynamically allocated string; it is
+ * increased if more space is required to fit the whole line.
  */
-char *mutt_read_rfc822_line (FILE *f, char *line, size_t *linelen)
+static char *read_rfc822_line (FILE *f, char *line, size_t *linelen)
 {
-  size_t offset = 0, l;
-  char buf[LONG_STRING];
-  char ch, *pbuf, *q;
-  int is_cont = 0; /* continuation line? */
-  int is_full;
-
-  if (line)
-    *line = 0;
+  char *buf = line;
+  char ch;
+  size_t offset = 0;
 
   FOREVER
   {
-    if (fgets (buf, sizeof (buf), f) == NULL)
-      return (line);
-
-    /* check to see if we got a full line */
-    l = strlen (buf);
-    is_full = (!l || buf[l - 1] == '\n');
-
-    pbuf = buf;
-    if (is_cont)
+    if (fgets (buf, *linelen - offset, f) == NULL ||	/* end of file or */
+	(ISSPACE (*line) && !offset))			/* end of headers */ 
     {
-      /* unfold the continuation line, as described by MIME */
-      for (; ISSPACE (*pbuf); pbuf++, l--) /* empty */
+      *line = 0;
+      return (line);
+    }
+
+    buf += strlen (buf) - 1;
+    if (*buf == '\n')
+    {
+      /* we did get a full line. remove trailing space */
+      while (ISSPACE (*buf))
+	*buf-- = 0;	/* we cannot come beyond line's beginning because
+			 * it begins with a non-space */
+
+      /* check to see if the next line is a continuation line */
+      if ((ch = fgetc (f)) != ' ' && ch != '\t')
+      {
+	ungetc (ch, f);
+	return (line); /* next line is a separate header field or EOH */
+      }
+
+      /* eat tabs and spaces from the beginning of the continuation line */
+      while ((ch = fgetc (f)) == ' ' || ch == '\t')
 	;
-      if (*pbuf)
-      {
-	*--pbuf = ' '; /* convert to a single space */
-	l++;
-      }
+      ungetc (ch, f);
+      *++buf = ' '; /* string is still terminated because we removed
+		       at least one whitespace char above */
     }
-    is_cont = 0;
 
-    /* if we got a full line, remove trailing space */
-    if (l && pbuf[l - 1] == '\n')
+    buf++;
+    offset = buf - line;
+    if (*linelen < offset + STRING)
     {
-      for (q = pbuf + l - 1 ; q >= pbuf && ISSPACE (*q) ; q--, l--)
-        *q = 0;
+      /* grow the buffer */
+      *linelen += STRING;
+      safe_realloc ((void **) &line, *linelen);
+      buf = line + offset;
     }
-
-    if (offset + l > *linelen)
-    {
-      /* not enough space to hold the (rest of the) line, grow the buffer */
-      if (line)
-      {
-	*linelen = offset + l;
-	safe_realloc ((void **) &line, *linelen + 1);
-      }
-      else
-      {
-	*linelen = l;
-	line = safe_malloc (*linelen + 1);
-      }
-    }
-    else if (!l && !line)
-    {
-      /* Avoid a segfault if the first line we read is blank. */
-      return (NULL);
-    }
-
-    if (l)
-    {
-      memcpy (line + offset, pbuf, l + 1);
-      offset += l;
-    }
-
-    /* check to see if we got a full line */
-    if (!is_full)
-      continue; /* no, read the next part */
-
-    /* a blank line indicates the end of the message header */
-    if (! *line)
-      return (line);
-
-    /* check to see if the next line is a continuation line */
-    ch = fgetc (f);
-    ungetc (ch, f);
-    if (ch != ' ' && ch != '\t')
-      return (line); /* next line is a separate header field or EOH */
-    is_cont = 1;
   }
   /* not reached */
 }
@@ -149,11 +114,11 @@ int mutt_check_encoding (const char *c)
   else if (strncasecmp ("binary", c, sizeof ("binary")-1) == 0)
     return (ENCBINARY);
   else if (strncasecmp ("quoted-printable", c, sizeof ("quoted-printable")-1) == 0)
-    return(ENCQUOTEDPRINTABLE);
-  else if (strncasecmp("base64", c, sizeof("base64")-1) == 0)
-    return(ENCBASE64);
+    return (ENCQUOTEDPRINTABLE);
+  else if (strncasecmp ("base64", c, sizeof("base64")-1) == 0)
+    return (ENCBASE64);
   else
-    return(ENCOTHER);
+    return (ENCOTHER);
 }
 
 static PARAMETER *parse_parameters (const char *s)
@@ -165,81 +130,85 @@ static PARAMETER *parse_parameters (const char *s)
 
   while (*s)
   {
-    if ((p = strchr (s, '=')) == NULL)
+    if ((p = strpbrk (s, "=;")) == NULL)
     {
       dprint(1, (debugfile, "parse_parameters: malformed parameter: %s\n", s));
       return (head); /* just bail out now */
     }
 
-    i = p - s;
-
-    new = mutt_new_parameter ();
-
-    new->attribute = safe_malloc (i + 1);
-    memcpy (new->attribute, s, i);
-    new->attribute[i] = 0;
-
-    s = p + 1; /* skip over the = */
-    SKIPWS (s);
-
-    if (*s == '"')
+    /* if we hit a ; now the parameter has no value, just skip it */
+    if (*p != ';')
     {
-      s++;
-      for (i=0; *s && *s != '"' && i < sizeof (buffer) - 1; i++, s++)
+      i = p - s;
+
+      new = mutt_new_parameter ();
+
+      new->attribute = safe_malloc (i + 1);
+      memcpy (new->attribute, s, i);
+      new->attribute[i] = 0;
+
+      /* remove whitespace from the end of the attribute name */
+      while (ISSPACE (new->attribute[--i]))
+	new->attribute[i] = 0;
+
+      s = p + 1; /* skip over the = */
+      SKIPWS (s);
+
+      if (*s == '"')
       {
-        if (*s == '\\')
+	s++;
+	for (i=0; *s && *s != '"' && i < sizeof (buffer) - 1; i++, s++)
 	{
-          /* Quote the next character */
-          buffer[i] = s[1];
-	  if (!*++s)
-	    break;
-        }
-        else
-          buffer[i] = *s;
+	  if (*s == '\\')
+	  {
+	    /* Quote the next character */
+	    buffer[i] = s[1];
+	    if (!*++s)
+	      break;
+	  }
+	  else
+	    buffer[i] = *s;
+	}
+	buffer[i] = 0;
+	if (*s)
+	  s++; /* skip over the " */
       }
-      buffer[i] = 0;
-      if (*s)
-	s++; /* skip over the " */
+      else
+      {
+	for (i=0; *s && *s != ' ' && *s != ';' && i < sizeof (buffer) - 1; i++, s++)
+	  buffer[i] = *s;
+	buffer[i] = 0;
+      }
+
+      new->value = safe_strdup (buffer);
+
+      /* Add this parameter to the list */
+      if (head)
+      {
+	cur->next = new;
+	cur = cur->next;
+      }
+      else
+	head = cur = new;
     }
     else
     {
-      for (i=0; *s && *s != ' ' && *s != ';' && i < sizeof (buffer) - 1; i++, s++)
-        buffer[i] = *s;
-      buffer[i] = 0;
+      dprint (1, (debugfile, "parse_parameters(): parameter with no value: %s\n", s));
+      s = p;
     }
 
-    new->value = safe_strdup (buffer);
-
-    /* Add this parameter to the list */
-    if (head)
-    {
-      cur->next = new;
-      cur = cur->next;
-    }
-    else
-      head = cur = new;
-
     /* Find the next parameter */
-    if (*s == ';')
-      s++;
-    else
-    {
-      while (*s && *s != ';')
-	s++;
-      if (!*s)
-	break; /* no more parameters! */
-      s++;
-    }
+    if (*s != ';' && (s = strchr (s, ';')) == NULL)
+	break; /* no more parameters */
 
-    /* Move past any leading whitespace */
-    SKIPWS (s);
-
-    /* skip empty parameters */
-    while (*s == ';')
+    do
     {
       s++;
+
+      /* Move past any leading whitespace */
       SKIPWS (s);
     }
+    while (*s == ';'); /* skip empty parameters */
   }    
 
   return (head);
@@ -278,7 +247,8 @@ static void parse_content_type (char *s, BODY *ct)
   if ((pc = strchr(s, ';')) != NULL)
   {
     *pc++ = 0;
-    while (*pc && whitespace(*pc)) pc++;
+    while (*pc && ISSPACE (*pc))
+      pc++;
     ct->parameter = parse_parameters(pc);
 
     /* Some pre-RFC1521 gateways still use the "name=filename" convention */
@@ -290,7 +260,7 @@ static void parse_content_type (char *s, BODY *ct)
   if ((pc = strchr(s, '/')))
   {
     *pc++ = 0;
-    while (*pc && !whitespace (*pc) && *pc != ';')
+    while (*pc && !ISSPACE (*pc) && *pc != ';')
     {
       buffer[i++] = *pc;
       pc++;
@@ -365,19 +335,21 @@ static void parse_content_disposition (char *s, BODY *ct)
 BODY *mutt_read_mime_header (FILE *fp, int digest)
 {
   BODY *p = mutt_new_body();
-  char *c, *line = NULL;
-  size_t linelen = 0;
+  char *c;
+  char *line = safe_malloc (LONG_STRING);
+  size_t linelen = LONG_STRING;
   
   p->hdr_offset = ftell(fp);
 
   p->encoding = ENC7BIT; /* default from RFC1521 */
   p->type = digest ? TYPEMESSAGE : TYPETEXT;
 
-  while ((line = mutt_read_rfc822_line (fp, line, &linelen)) && *line != 0)
+  while (*(line = read_rfc822_line (fp, line, &linelen)) != 0)
   {
     /* Find the value of the current header */
     if ((c = strchr (line, ':')))
     {
+      *c = 0;
       c++;
       SKIPWS (c);
       if (!*c)
@@ -392,26 +364,20 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
       break;
     }
 
-    switch (tolower (line[0]))
+    if (!strncasecmp ("content-", line, 8))
     {
-      case 'c':
-
-	if (!strncasecmp ("ontent-", line + 1, 7))
-	{
-	  if (!strncasecmp ("type", line + 8, 4))
-	    parse_content_type (c, p);
-	  else if (!strncasecmp ("transfer-encoding", line + 8, 17))
-	    p->encoding = mutt_check_encoding (c);
-	  else if (!strncasecmp ("disposition", line + 8, 11))
-	    parse_content_disposition (c, p);
-	  else if (!strncasecmp ("description", line + 8, 11))
-	  {
-	    safe_free ((void **) &p->description);
-	    p->description = safe_strdup (c);
-	    rfc2047_decode (p->description, p->description, strlen (p->description) + 1);
-	  }
-	}
-	break;
+      if (!strcasecmp ("type", line + 8))
+	parse_content_type (c, p);
+      else if (!strcasecmp ("transfer-encoding", line + 8))
+	p->encoding = mutt_check_encoding (c);
+      else if (!strcasecmp ("disposition", line + 8))
+	parse_content_disposition (c, p);
+      else if (!strcasecmp ("description", line + 8))
+      {
+	safe_free ((void **) &p->description);
+	p->description = safe_strdup (c);
+	rfc2047_decode (p->description, p->description, strlen (p->description) + 1);
+      }
     }
   }
   p->offset = ftell (fp); /* Mark the start of the real data */
@@ -420,7 +386,7 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
   else if (p->type == TYPEMESSAGE && !p->subtype)
     p->subtype = safe_strdup ("rfc822");
 
-  safe_free ((void **) &line);
+  free (line);
 
   return (p);
 }
@@ -440,13 +406,15 @@ BODY *mutt_parse_messageRFC822 (FILE *fp, BODY *parent)
 {
   BODY *msg;
 
-  parent->hdr = mutt_new_header();
-  parent->hdr->offset = ftell(fp);
+  parent->hdr = mutt_new_header ();
+  parent->hdr->offset = ftell (fp);
   parent->hdr->env = mutt_read_rfc822_header (fp, parent->hdr);
   msg = parent->hdr->content;
 
-  if (msg->length == -1)
-    msg->length = parent->length - (msg->offset - parent->offset);
+  /* ignore the length given in the content-length since it could be wrong
+     and we already have the info to calculate the correct length */
+  /* if (msg->length == -1) */
+  msg->length = parent->length - (msg->offset - parent->offset);
 
   /* if body of this message is empty, we can end up with a negative length */
   if (msg->length < 0)
@@ -522,7 +490,7 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int di
       }
 
       /* Remove any trailing whitespace, up to the length of the boundary */
-      for (i = len - 1; whitespace (buffer[i]) && i >= blen + 2; i--)
+      for (i = len - 1; ISSPACE (buffer[i]) && i >= blen + 2; i--)
         buffer[i] = 0;
 
       /* Check for the end boundary */
@@ -661,7 +629,7 @@ TimeZones[] =
  * This routine assumes that `h' has been initialized to 0.  the `timezone'
  * field is optional, defaulting to +0000 if missing.
  */
-static int parse_date (char *s, HEADER *h)
+static time_t parse_date (char *s, HEADER *h)
 {
   int count = 0;
   char *t;
@@ -669,17 +637,18 @@ static int parse_date (char *s, HEADER *h)
   struct tm tm;
   int i;
   int tz_offset = 0;
+  int zhours = 0;
+  int zminutes = 0;
+  int zoccident = 0;
   const char *ptz;
   char tzstr[SHORT_STRING];
 
   /* kill the day of the week, if it exists. */
-  if ((t = strchr (s, ',')) != 0)
-  {
+  if ((t = strchr (s, ',')))
     t++;
-    SKIPWS (t);
-  }
   else
     t = s;
+  SKIPWS (t);
 
   memset (&tm, 0, sizeof (tm));
 
@@ -723,7 +692,6 @@ static int parse_date (char *s, HEADER *h)
 	break;
 
       case 4: /* timezone */
-
 	/* sometimes we see things like (MST) or (-0700) so attempt to
 	 * compensate by uncommenting the string if non-RFC822 compliant
 	 */
@@ -733,11 +701,11 @@ static int parse_date (char *s, HEADER *h)
 	{
 	  if (ptz[1] && ptz[2] && ptz[3] && ptz[4])
 	  {
-	    h->zhours = (ptz[1] - '0') * 10 + (ptz[2] - '0');
-	    h->zminutes = (ptz[3] - '0') * 10 + (ptz[4] - '0');
+	    zhours = (ptz[1] - '0') * 10 + (ptz[2] - '0');
+	    zminutes = (ptz[3] - '0') * 10 + (ptz[4] - '0');
 
 	    if (ptz[0] == '-')
-	      h->zoccident = 1;
+	      zoccident = 1;
 	  }
 	}
 	else
@@ -745,9 +713,9 @@ static int parse_date (char *s, HEADER *h)
 	  for (i = 0; TimeZones[i].tzname; i++)
 	    if (!strcasecmp (TimeZones[i].tzname, ptz))
 	    {
-	      h->zhours = TimeZones[i].zhours;
-	      h->zminutes = TimeZones[i].zminutes;
-	      h->zoccident = TimeZones[i].zoccident;
+	      zhours = TimeZones[i].zhours;
+	      zminutes = TimeZones[i].zminutes;
+	      zoccident = TimeZones[i].zoccident;
 	      break;
 	    }
 
@@ -757,12 +725,12 @@ static int parse_date (char *s, HEADER *h)
 	    if ((t = strtok (NULL, " \t")) != NULL)
 	    {
 	      if (!strcasecmp (t, "DST"))
-		h->zhours++;
+		zhours++;
 	    }
 	  }
 	}
-	tz_offset = h->zhours * 3600 + h->zminutes * 60;
-	if (!h->zoccident)
+	tz_offset = zhours * 3600 + zminutes * 60;
+	if (!zoccident)
 	  tz_offset = -tz_offset;
 	break;
     }
@@ -776,9 +744,14 @@ static int parse_date (char *s, HEADER *h)
     return (-1);
   }
 
-  h->date_sent = mutt_mktime (&tm) + tz_offset;
+  if (h)
+  {
+    h->zhours = zhours;
+    h->zminutes = zminutes;
+    h->zoccident = zoccident;
+  }
 
-  return 0;
+  return (mutt_mktime (&tm, 0) + tz_offset);
 }
 
 /* extract the first substring that looks like a message-id */
@@ -797,14 +770,14 @@ static char *extract_message_id (const char *s)
   return (r);
 }
 
-void mutt_parse_mime_message (HEADER *cur)
+void mutt_parse_mime_message (CONTEXT *ctx, HEADER *cur)
 {
   MESSAGE *msg;
 
   if (cur->content->type != TYPEMESSAGE && cur->content->type != TYPEMULTIPART)
     return; /* nothing to do */
 
-  if ((msg = mx_open_message (Context, cur->msgno)))
+  if ((msg = mx_open_message (ctx, cur->msgno)))
   {
     fseek (msg->fp, cur->content->offset, 0);
 
@@ -818,7 +791,6 @@ void mutt_parse_mime_message (HEADER *cur)
       if (!cur->content->parts)
 	cur->content->parts = mutt_parse_messageRFC822 (msg->fp, cur->content);
     }
-      
 
     /* try to recover from parsing error */
     if (!cur->content->parts)
@@ -828,9 +800,6 @@ void mutt_parse_mime_message (HEADER *cur)
       cur->content->subtype = safe_strdup ("plain");
     }
     
-#ifdef _PGPPATH
-    cur->pgp = pgp_query (cur->content);
-#endif /* _PGPPATH */
     mx_close_message (&msg);
   }
 }
@@ -845,21 +814,19 @@ void mutt_parse_mime_message (HEADER *cur)
  *		NULL, then we are reading a postponed message, or called
  *		from mutt_edit_headers() so we should keep a list of the
  *		user-defined headers.
- *
- * keep		1 to add the "mute" headers to the "userhdrs" list;
- *		used when building new headers for decode-save
- *
  */
-ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
+ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr)
 {
   ENVELOPE *e = mutt_new_envelope();
   LIST *last = NULL;
-  char *line = NULL, *p;
-  char in_reply_to[SHORT_STRING] = "";
+  char *line = safe_malloc (LONG_STRING);
+  char *p;
+  char in_reply_to[STRING];
   long loc;
   int matched;
-  int date_found = 0;
-  size_t linelen = 0;
+  size_t linelen = LONG_STRING;
+
+  in_reply_to[0] = 0;
 
   if (hdr)
   {
@@ -873,14 +840,13 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
   }
 
   loc = ftell (f);
-  while ((line = mutt_read_rfc822_line (f, line, &linelen)) != NULL &&
-	 *line != 0)
+  while (*(line = read_rfc822_line (f, line, &linelen)) != 0)
   {
     matched = 0;
 
     if ((p = strpbrk (line, ": \t")) == NULL || *p != ':')
     {
-      char return_path[LONG_STRING] = { 0 };
+      char return_path[LONG_STRING];
       time_t t;
 
       /* some bogus MTAs will quote the original "From " line */
@@ -893,7 +859,7 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
       {
 	/* MH somtimes has the From_ line in the middle of the header! */
 	if (hdr && !hdr->received)
-	  hdr->received = t;
+	  hdr->received = t + mutt_local_tz ();
 	loc = ftell (f);
 	continue;
       }
@@ -902,61 +868,64 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
       break; /* end of header */
     }
 
+    *p = 0;
     p++;
     SKIPWS (p);
+    if (!*p)
+      continue; /* skip empty header fields */
 
     switch (tolower (line[0]))
     {
       case 'a':
-	if (strncasecmp (line+1, "pparently-to:", 13) == 0)
+	if (strcasecmp (line+1, "pparently-to") == 0)
 	{
-	  rfc822_parse_adrlist (&e->to, p, "@");
+	  e->to = rfc822_parse_adrlist (e->to, p);
 	  matched = 1;
 	}
-	else if (strncasecmp (line+1, "pparently-from:", 15) == 0)
+	else if (strcasecmp (line+1, "pparently-from") == 0)
 	{
-	  rfc822_parse_adrlist (&e->from, p, "@");
+	  e->from = rfc822_parse_adrlist (e->from, p);
 	  matched = 1;
 	}
 	break;
 
       case 'b':
-	if (strncasecmp (line+1, "cc:", 3) == 0)
+	if (strcasecmp (line+1, "cc") == 0)
 	{
-	  rfc822_parse_adrlist (&e->bcc, p, "@");
+	  e->bcc = rfc822_parse_adrlist (e->bcc, p);
 	  matched = 1;
 	}
 	break;
 
       case 'c':
-	if (strncasecmp (line+1, "c:", 2) == 0)
+	if (strcasecmp (line+1, "c") == 0)
 	{
-	  rfc822_parse_adrlist (&e->cc, p, "@");
+	  e->cc = rfc822_parse_adrlist (e->cc, p);
 	  matched = 1;
 	}
 	else if (strncasecmp (line + 1, "ontent-", 7) == 0)
 	{
-	  if (strncasecmp (line+8, "type:", 5) == 0)
+	  if (strcasecmp (line+8, "type") == 0)
 	  {
-	    if (hdr && !keep)
+	    if (hdr)
 	      parse_content_type (p, hdr->content);
 	    matched = 1;
 	  }
-	  else if (strncasecmp (line+8, "transfer-encoding:", 18) == 0)
+	  else if (strcasecmp (line+8, "transfer-encoding") == 0)
 	  {
-	    if (hdr && !keep)
+	    if (hdr)
 	      hdr->content->encoding = mutt_check_encoding (p);
 	    matched = 1;
 	  }
-	  else if (strncasecmp (line+8, "length:", 7) == 0)
+	  else if (strcasecmp (line+8, "length") == 0)
 	  {
-	    if (hdr && !keep)
+	    if (hdr)
 	      hdr->content->length = atoi (p);
 	    matched = 1;
 	  }
-	  else if (strncasecmp (line+8, "description:", 12) == 0)
+	  else if (strcasecmp (line+8, "description") == 0)
 	  {
-	    if (hdr && !keep)
+	    if (hdr)
 	    {
 	      safe_free ((void **) &hdr->content->description);
 	      hdr->content->description = safe_strdup (p);
@@ -966,9 +935,9 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
 	    }
 	    matched = 1;
 	  }
-	  else if (strncasecmp (line+8, "disposition:", 5) == 0)
+	  else if (strcasecmp (line+8, "disposition") == 0)
 	  {
-	    if (hdr && !keep)
+	    if (hdr)
 	      parse_content_disposition (p, hdr->content);
 	    matched = 1;
 	  }
@@ -976,88 +945,123 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
 	break;
 
       case 'd':
-	if (!strncasecmp ("ate:", line + 1, 4))
+	if (!strcasecmp ("ate", line + 1))
 	{
-	  if (hdr && parse_date (p, hdr) == 0)
-	    date_found = 1;
+	  if (hdr)
+	    hdr->date_sent = parse_date (p, hdr);
+	  matched = 1;
 	}
 	break;
 
+      case 'e':
+	if (!strcasecmp ("xpires", line + 1) &&
+	    hdr && parse_date (p, hdr) < time (NULL))
+	  hdr->expired = 1;
+	break;
+
       case 'f':
-	if (!strncasecmp ("rom:", line + 1, 4))
+	if (!strcasecmp ("rom", line + 1))
 	{
-	  rfc822_parse_adrlist (&e->from, p, "@");
+	  e->from = rfc822_parse_adrlist (e->from, p);
 	  matched = 1;
 	}
 	break;
 
       case 'i':
-	if (strncasecmp (line+1, "n-reply-to:", 11) == 0)
+	if (!strcasecmp (line+1, "n-reply-to"))
 	{
 	  if (hdr)
+	  {
 	    strfcpy (in_reply_to, p, sizeof (in_reply_to));
+	    rfc2047_decode (in_reply_to, in_reply_to,
+			    sizeof (in_reply_to));
+	  }
 	}
 	break;
 
       case 'l':
-	if (strncasecmp (line+1, "ines:", 5) == 0)
+	if (!strcasecmp (line + 1, "ines"))
 	{
-	  if (hdr && !keep)
-	    hdr->lines = atoi(p);
+	  if (hdr)
+	    hdr->lines = atoi (p);
 	  matched = 1;
 	}
 	break;
 
       case 'm':
-	if (strncasecmp (line+1, "ime-version:", 12) == 0)
+	if (!strcasecmp (line + 1, "ime-version"))
 	{
 	  if (hdr)
 	    hdr->mime = 1;
 	  matched = 1;
 	}
-	else if (strncasecmp (line+1, "essage-id:", 10) == 0)
+	else if (!strcasecmp (line + 1, "essage-id"))
 	{
-	  if (!keep)
-	  {
-	    /* We add a new "Message-Id:" when building a message */
-	    safe_free ((void **)&e->message_id);
-	    e->message_id = extract_message_id (p);
-	  }
+	  /* We add a new "Message-Id:" when building a message */
+	  safe_free ((void **) &e->message_id);
+	  e->message_id = extract_message_id (p);
 	  matched = 1;
 	}
+	else if (!strncasecmp (line + 1, "ail-", 4))
+	{
+	  if (!strcasecmp (line + 5, "reply-to"))
+	  {
+	    /* override the Reply-To: field */
+	    rfc822_free_address (&e->reply_to);
+	    e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
+	    matched = 1;
+	  }
+	  else if (!strcasecmp (line + 5, "followup-to"))
+	  {
+	    e->mail_followup_to = rfc822_parse_adrlist (e->mail_followup_to, p);
+	    matched = 1;
+	  }
+	}
 	break;
 
       case 'r':
-	if (strncasecmp (line+1, "eferences:", 10) == 0)
+	if (!strcasecmp (line + 1, "eferences"))
 	{
 	  mutt_free_list (&e->references);
 	  e->references = mutt_parse_references (p);
 	  matched = 1;
 	}
-	else if (strncasecmp (line+1, "eply-to:", 8) == 0)
+	else if (!strcasecmp (line + 1, "eply-to"))
 	{
-	  rfc822_parse_adrlist (&e->reply_to, p, "@");
+	  e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
 	  matched = 1;
 	}
-	else if (strncasecmp (line+1, "eturn-path:", 11) == 0)
+	else if (!strcasecmp (line + 1, "eturn-path"))
 	{
-	  if (hdr) rfc822_parse_adrlist (&e->return_path, p, "@");
+	  e->return_path = rfc822_parse_adrlist (e->return_path, p);
+	  matched = 1;
+	}
+	else if (!strcasecmp (line + 1, "eceived"))
+	{
+	  if (hdr && !hdr->received)
+	  {
+	    char *d = strchr (p, ';');
+
+	    if (d)
+	      hdr->received = parse_date (d + 1, NULL);
+	  }
 	  matched = 1;
 	}
 	break;
 
       case 's':
-	if (strncasecmp (line+1, "ubject:", 7) == 0)
+	if (!strcasecmp (line + 1, "ubject"))
 	{
-	  if (!e->subject) e->subject = safe_strdup (p);
+	  if (!e->subject)
+	    e->subject = safe_strdup (p);
 	  matched = 1;
 	}
-	else if (strncasecmp (line+1, "ender:", 6) == 0)
+	else if (!strcasecmp (line + 1, "ender"))
 	{
-	  rfc822_parse_adrlist (&e->sender, p, "@");
+	  e->sender = rfc822_parse_adrlist (e->sender, p);
 	  matched = 1;
 	}
-	else if (strncasecmp (line+1, "tatus:", 6) == 0)
+	else if (!strcasecmp (line + 1, "tatus"))
 	{
 	  if (hdr)
 	  {
@@ -1081,18 +1085,21 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
 	  }
 	  matched = 1;
 	}
+	else if ((!strcasecmp ("upersedes", line + 1) ||
+	          !strcasecmp ("upercedes", line + 1)) && hdr)
+	  e->supersedes = safe_strdup (p);
 	break;
 
       case 't':
-	if (strncasecmp (line+1, "o:", 2) == 0)
+	if (strcasecmp (line+1, "o") == 0)
 	{
-	  rfc822_parse_adrlist (&e->to, p, "@");
+	  e->to = rfc822_parse_adrlist (e->to, p);
 	  matched = 1;
 	}
 	break;
 
       case 'x':
-	if (strncasecmp (line+1, "-status:", 8) == 0)
+	if (strcasecmp (line+1, "-status") == 0)
 	{
 	  if (hdr)
 	  {
@@ -1118,27 +1125,6 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
 	  matched = 1;
 	}
 	    
-	if (hdr && keep && strncasecmp (line+1, "-mailer:", 8) != 0)
-	{
-	  /* If we are building a new message we want to save all
-	   * relevant headers in the "userhdrs" list
-	   */
-	  LIST *tmp = e->userhdrs;
-
-	  while (tmp && tmp->next)
-	    tmp = tmp->next;
-	  if (tmp)
-	  {
-	    tmp->next = mutt_new_list();
-	    tmp = tmp->next;
-	  }
-	  else
-	    e->userhdrs = tmp = mutt_new_list();
-
-	  tmp->data = safe_strdup (line);
-	}
-	break;
-
       default:
 	break;
     }
@@ -1156,13 +1142,14 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
       }
       else
 	last = e->userhdrs = mutt_new_list ();
+      line[strlen (line)] = ':';
       last->data = safe_strdup (line);
     }
 
     loc = ftell (f);
   }
 
-  safe_free ((void **) &line);
+  free (line);
 
   if (hdr)
   {
@@ -1172,8 +1159,7 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
     /* if an in-reply-to was given, check to see if it is in the references
      * list already.  if not, add it so we can do a better job of threading.
      */
-    if (!keep && in_reply_to[0] &&
-		    (p = extract_message_id (in_reply_to)) != NULL)
+    if (in_reply_to[0] && (p = extract_message_id (in_reply_to)) != NULL)
     {
       if (!e->references ||
 	  (e->references && strcmp (e->references->data, p) != 0))
@@ -1193,28 +1179,24 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
     rfc2047_decode_adrlist (e->to);
     rfc2047_decode_adrlist (e->cc);
     rfc2047_decode_adrlist (e->reply_to);
+    rfc2047_decode_adrlist (e->mail_followup_to);
     rfc2047_decode_adrlist (e->return_path);
     rfc2047_decode_adrlist (e->sender);
 
     if (e->subject)
     {
+      regmatch_t pmatch[1];
+
       rfc2047_decode (e->subject, e->subject, strlen (e->subject) + 1);
 
-      if (!keep)
-      {
-	regmatch_t pmatch[1];
-
-	if (regexec ((regex_t *) ReplyRegexp.rx, e->subject, 1, pmatch, 0) == 0)
-	  e->real_subj = e->subject + pmatch[0].rm_eo;
-	else
-	  e->real_subj = e->subject;
-      }
+      if (regexec (ReplyRegexp.rx, e->subject, 1, pmatch, 0) == 0)
+	e->real_subj = e->subject + pmatch[0].rm_eo;
+      else
+	e->real_subj = e->subject;
     }
 
-    /* Check for missing or invalid date.  If this is the case, convert the
-     * date the message was received to UTC and use that value.
-     */
-    if (!date_found)
+    /* check for missing or invalid date */
+    if (hdr->date_sent <= 0)
     {
       dprint(1,(debugfile,"read_rfc822_header(): no date found, using received time from msg separator\n"));
       hdr->date_sent = hdr->received;
@@ -1224,7 +1206,7 @@ ENVELOPE *_mutt_read_rfc822_header (FILE *f, HEADER *hdr, int keep)
   return (e);
 }
 
-void mutt_parse_adrlist (ADDRESS **p, const char *s, const char *host)
+ADDRESS *mutt_parse_adrlist (ADDRESS *p, const char *s)
 {
   const char *q;
 
@@ -1238,10 +1220,12 @@ void mutt_parse_adrlist (ADDRESS **p, const char *s, const char *host)
     r = tmp;
     while ((r = strtok (r, " \t")) != NULL)
     {
-      rfc822_parse_adrlist (p, r, host);
+      p = rfc822_parse_adrlist (p, r);
       r = NULL;
     }
   }
   else
-    rfc822_parse_adrlist (p, s, host);
+    p = rfc822_parse_adrlist (p, s);
+  
+  return p;
 }
