@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -21,10 +21,8 @@
  */
 
 #include "mutt.h"
-
-#ifdef USE_POP
-
-#include "mutt_curses.h"
+#include "mailbox.h"
+#include "mx.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -81,7 +79,6 @@ void mutt_fetchPopMail (void)
   char buffer[2048];
   char msgbuf[SHORT_STRING];
   int s, i, msgs, bytes, err = 0;
-  time_t now;
   CONTEXT ctx;
   MESSAGE *msg = NULL;
 
@@ -209,17 +206,12 @@ void mutt_fetchPopMail (void)
       break;
     }
 
-    if ((msg = mx_open_new_message (&ctx, NULL)) == NULL)
+    if ((msg = mx_open_new_message (&ctx, NULL, M_ADD_FROM)) == NULL)
     {
       err = 1;
       break;
     }
 
-    now = time (0);
-    
-    if (msg->magic == M_MMDF || msg->magic == M_MBOX)
-      fprintf (msg->fp, "From %s %s", Username, ctime (&now));
-
     /* Now read the actual message. */
     FOREVER
     {
@@ -317,5 +309,3 @@ fail:
   mutt_error ("Server closed connection!");
   close (s);
 }
-
-#endif /* USE_POP */
