@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,16 +17,32 @@
  */ 
 
 #include "mutt.h"
-#include "mutt_curses.h"
 #include "mutt_menu.h"
-#include "send.h"
 #include "rfc1524.h"
+#include "mime.h"
+#include "mailbox.h"
+#include "mapping.h"
 
 #include <ctype.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/stat.h>
 
+static struct mapping_t PostponeHelp[] = {
+  { "Exit",  OP_EXIT },
+  { "Del",   OP_DELETE },
+  { "Undel", OP_UNDELETE },
+  { "Help",  OP_HELP },
+  { NULL }
+};
+
+
+
+
+
+
+
+
 static short PostCount = 0;
 static time_t LastModify = 0;
 static CONTEXT *PostContext = NULL;
@@ -91,7 +107,6 @@ static HEADER *select_msg (void)
   MUTTMENU *menu;
   int i, done=0, r=-1;
   char helpstr[SHORT_STRING];
-  char tmp[16];
 
   menu = mutt_new_menu ();
   menu->make_entry = post_entry;
@@ -99,17 +114,7 @@ static HEADER *select_msg (void)
   menu->max = PostContext->msgcount;
   menu->title = "Postponed Messages";
   menu->data = PostContext;
-
-  helpstr[0] = 0;
-  mutt_make_help (tmp, sizeof (tmp), "Exit  ", MENU_POST, OP_EXIT);
-  strcat (helpstr, tmp);
-  mutt_make_help (tmp, sizeof (tmp), "Del  ", MENU_POST, OP_DELETE);
-  strcat (helpstr, tmp);
-  mutt_make_help (tmp, sizeof (tmp), "Undel  ", MENU_POST, OP_UNDELETE);
-  strcat (helpstr, tmp);
-  mutt_make_help (tmp, sizeof (tmp), "Help", MENU_POST, OP_HELP);
-  strcat (helpstr, tmp);
-  menu->help = helpstr;
+  menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_POST, PostponeHelp);
 
   while (!done)
   {
@@ -117,7 +122,6 @@ static HEADER *select_msg (void)
     {
       case OP_DELETE:
       case OP_UNDELETE:
-
 	mutt_set_flag (PostContext, PostContext->hdrs[menu->current], M_DELETE, (i == OP_DELETE) ? 1 : 0);
 	PostCount = PostContext->msgcount - PostContext->deleted;
 	if (option (OPTRESOLVE) && menu->current < menu->max - 1)
@@ -137,13 +141,11 @@ static HEADER *select_msg (void)
 	break;
 
       case OP_GENERIC_SELECT_ENTRY:
-
 	r = menu->current;
 	done = 1;
 	break;
 
       case OP_EXIT:
-
 	done = 1;
 	break;
     }
@@ -154,6 +156,8 @@ static HEADER *select_msg (void)
 }
 
 /* args:
+ *      ctx	Context info, used when recalling a message to which
+ *              we reply.
  *	hdr	envelope/attachment info for recalled message
  *	cur	if message was a reply, `cur' is set to the message which
  *		`hdr' is in reply to
@@ -163,28 +167,28 @@ static HEADER *select_msg (void)
  *	0		normal exit
  *	SENDREPLY	recalled message is a reply
  */
-int mutt_get_postponed (HEADER *hdr, HEADER **cur)
+int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur)
 {
   HEADER *h;
-  BODY *bdy = NULL;
   MESSAGE *msg;
   int code = SENDPOSTPONED;
   LIST *tmp;
   LIST *last = NULL;
   LIST *next;
-  char buf[LONG_STRING];
   char file[_POSIX_PATH_MAX];
   char *p;
   int opt_delete;
 
   if ((PostContext = mx_open_mailbox (Postponed, M_NOSORT, NULL)) == NULL)
   {
+    PostCount = 0;
     mutt_error ("No postponed messages.");
     return (-1);
   }
   
-  if (PostContext->msgcount == 0)
+  if (! PostContext->msgcount)
   {
+    PostCount = 0;
     mx_close_mailbox (PostContext);
     safe_free ((void **) &PostContext);
     mutt_error ("No postponed messages.");
@@ -211,13 +215,6 @@ int mutt_get_postponed (HEADER *hdr, HEADER **cur)
   }
 
   fseek (msg->fp, h->offset, 0);
-
-  if (msg->magic == M_MBOX || msg->magic == M_MMDF)
-  {
-    /* eat the message separator */
-    fgets (buf, sizeof (buf), msg->fp);
-  }
-
   hdr->env = mutt_read_rfc822_header (msg->fp, NULL);
 
   if (h->content->type == TYPEMESSAGE || h->content->type == TYPEMULTIPART)
@@ -296,22 +293,21 @@ int mutt_get_postponed (HEADER *hdr, HEADER **cur)
 
   safe_free ((void **) &PostContext);
 
-  tmp = hdr->env->userhdrs;
-  while (tmp)
+  for (tmp = hdr->env->userhdrs; tmp; )
   {
-    if (Context && strncmp ("X-Mutt-References:", tmp->data, 18) == 0)
+    if (ctx && strncmp ("X-Mutt-References:", tmp->data, 18) == 0)
     {
       int j;
 
       p = tmp->data + 18;
       SKIPWS (p);
 
-      for (j=0; j < Context->msgcount; j++)
+      for (j=0; j < ctx->msgcount; j++)
       {
-	if (Context->hdrs[j]->env->message_id &&
-	    strcmp (Context->hdrs[j]->env->message_id, p) == 0)
+	if (ctx->hdrs[j]->env->message_id &&
+	    strcmp (ctx->hdrs[j]->env->message_id, p) == 0)
 	{
-	  *cur = Context->hdrs[j];
+	  *cur = ctx->hdrs[j];
 	  break;
 	}
       }
@@ -328,37 +324,6 @@ int mutt_get_postponed (HEADER *hdr, HEADER **cur)
       if (*cur)
 	code |= SENDREPLY;
     }
-    else if (strncmp ("X-Mutt-Attachment:", tmp->data, 18) == 0)
-    {
-      /* For compatibility with old mutt postponed folders */
-      char *q;
-
-      p = tmp->data + 18;
-      SKIPWS (p);
-
-      if ((q = strchr (p, ' ')) != NULL)
-	*q++ = 0;
-      if (hdr->content)
-      {
-	bdy->next = mutt_make_attach (p);
-	if (bdy->next)
-	  bdy = bdy->next;
-      }
-      else
-	hdr->content = bdy = mutt_make_attach (p);
-      if (bdy)
-	bdy->description = safe_strdup (q);
-
-      /* remove the x-mutt-attachment field */
-      next = tmp->next;
-      if (last)
-	last->next = tmp->next;
-      else
-	hdr->env->userhdrs = tmp->next;
-      tmp->next = NULL;
-      mutt_free_list (&tmp);
-      tmp = next;
-    }
     else
     {
       last = tmp;
@@ -367,3 +332,4 @@ int mutt_get_postponed (HEADER *hdr, HEADER **cur)
   }
   return (code);
 }
+
