@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -102,25 +102,74 @@ int rfc1524_expand_command (BODY *a, char *filename, char *type,
   return needspipe;
 }
 
-int rfc1524_mailcap_parse (BODY *a,
-			   char *filename,
-			   char *type, 
-			   rfc1524_entry *entry,
-			   int opt)
+/* NUL terminates a rfc 1524 field,
+ * returns start of next field or NULL */
+static char *get_field (char *s)
 {
-  FILE *fp = NULL;
-  char buf[LONG_STRING];
-  char basetype[STRING];
   char *ch;
+
+  if (!s)
+    return NULL;
+
+  while ((ch = strpbrk (s, ";\\")) != NULL)
+  {
+    if (*ch == '\\')
+    {
+      s = ch + 1;
+      if (*s)
+	s++;
+    }
+    else
+    {
+      *ch++ = 0;
+      SKIPWS (ch);
+      break;
+    }
+  }
+  mutt_remove_trailing_ws (s);
+  return ch;
+}
+
+static int get_field_text (char *field, char **entry,
+			   char *type, char *filename, int line)
+{
+  field = mutt_skip_whitespace (field);
+  if (*field == '=')
+  {
+    if (entry)
+    {
+      field = mutt_skip_whitespace (++field);
+      safe_free ((void **) entry);
+      *entry = safe_strdup (field);
+    }
+    return 1;
+  }
+  else 
+  {
+    mutt_error ("Improperly formated entry for type %s in \"%s\" line %d",
+		type, filename, line);
+    return 0;
+  }
+}
+
+static int rfc1524_mailcap_parse (BODY *a,
+				  char *filename,
+				  char *type, 
+				  rfc1524_entry *entry,
+				  int opt)
+{
+  FILE *fp;
+  char *buf = NULL;
+  size_t buflen;
+  char *ch;
+  char *field;
   int found = FALSE;
-  int match = TRUE;
-  int wrap = FALSE;
-  int copiousoutput = FALSE;
-  int composecommand = FALSE;
-  int editcommand = FALSE;
-  int printcommand = FALSE;
-  int x = 0, y = 0;
-  int btlen = 0;
+  int copiousoutput;
+  int composecommand;
+  int editcommand;
+  int printcommand;
+  int btlen;
+  int line = 0;
 
   /* rfc1524 mailcap file is of the format:
    * base/type; command; extradefs
@@ -132,287 +181,158 @@ int rfc1524_mailcap_parse (BODY *a,
    * line wraps with a \ at the end of the line
    * # for comments
    */
+
+  /* find length of basetype */
+  if ((ch = strchr (type, '/')) == NULL)
+    return FALSE;
+  btlen = ch - type;
+
   if ((fp = fopen (filename, "r")) != NULL)
   {
-    /* copy base type into basetype */
-    x = 0;
-    while (type[x] && x < sizeof (basetype) && type[x] != '/')
+    while (!found && (buf = mutt_read_line (buf, &buflen, fp, &line)) != NULL)
     {
-      basetype[x] = type[x];
-      x++;
-    }
-    basetype[x] = '\0';
-    dprint (2, (debugfile, "mailcap basetype: %s\n", basetype));
-    btlen = x;
-    while (!found && fgets (buf, sizeof (buf), fp))
-    {
-      /* Strip comments (ie, everything after #, but not \# */
-      if ((ch = strchr (buf, '#')))
-        if ((ch == buf) || (*(ch-1) != '\\'))
-	  *ch = '\0';
-  
-      /* Strip trailing white space */
-      y = strlen (buf);
-      y = (y ? y - 1 : 0);
-      while (y && (buf[y] == ' ' || buf[y] == '\t' || buf[y] == '\n'))
-	buf[y--] = '\0';
-  
-      /* handle line wrap */
-      if (buf[y] == '\\')
-	wrap = TRUE;
-      else
-	wrap = FALSE;
-
-      while (wrap && fgets (buf + strlen (buf) - 1, sizeof (buf) - strlen (buf),fp)) 
-      {
-	if ((ch = strchr (buf,'#')))
-	{
-	  if (*(ch-1) != '\\')
-	    *ch = '\0';
-	}
-	y = strlen (buf);
-	y = (y ? y - 1 : 0);
-	while (buf[y] == ' ' || buf[y] == '\t' || buf[y] == '\n') 
-	  buf[y--] = '\0';
-	if (buf[y] == '\\')
-	  wrap = TRUE; 
-	else
-	  wrap = FALSE; 
-      }
+      /* ignore comments */
+      if (*buf == '#')
+	continue;
       dprint (2, (debugfile, "mailcap entry: %s\n", buf));
 
-      if (!strncasecmp (buf, type, strlen (type)) && (buf[strlen (type)] == ';'))
-	found = TRUE;
-      else
+      /* check type */
+      ch = get_field (buf);
+      if (strcasecmp (buf, type) &&
+	  (strncasecmp (buf, type, btlen) ||
+	   (buf[btlen] != 0 &&			/* implicit wild */
+	    strcmp (buf + btlen, "/*"))))	/* wildsubtype */
+	continue;
+
+      /* next field is the viewcommand */
+      field = ch;
+      ch = get_field (ch);
+      if (entry)
+	entry->command = safe_strdup (field);
+
+      /* parse the optional fields */
+      found = TRUE;
+      copiousoutput = FALSE;
+      composecommand = FALSE;
+      editcommand = FALSE;
+      printcommand = FALSE;
+
+      while (ch)
       {
-	/* Check * case */
-	if (strncmp (buf, basetype, btlen-1) == 0)
+	field = ch;
+	ch = get_field (ch);
+	dprint (2, (debugfile, "field: %s\n", field));
+
+	if (!strcasecmp (field, "needsterminal"))
 	{
-	  /* check * wildcard */ 
-	  if (buf[btlen] && buf[btlen+1] && buf[btlen] == '/' && 
-	      buf[btlen+1] == '*') 
-	    found = TRUE;
-	  /* check implicit wild */
-	  else if (buf[btlen] && buf[btlen] == ';')
-	    found = TRUE;
+	  if (entry)
+	    entry->needsterminal = TRUE;
+	}
+	else if (!strcasecmp (field, "copiousoutput"))
+	{
+	  copiousoutput = TRUE;
+	  if (entry)
+	    entry->copiousoutput = TRUE;
+	}
+	else if (!strncasecmp (field, "composetyped", 12))
+	{
+	  /* this compare most occur before compose to match correctly */
+	  /* this isn't particularly well specified in the rfc, so skip it */
+	}
+	else if (!strncasecmp (field, "compose", 7))
+	{
+	  if (get_field_text (field + 7, entry ? &entry->composecommand : NULL,
+			      type, filename, line))
+	    composecommand = TRUE;
+	}
+	else if (!strncasecmp (field, "print", 5))
+	{
+	  if (get_field_text (field + 5, entry ? &entry->printcommand : NULL,
+			      type, filename, line))
+	    printcommand = TRUE;
+	}
+	else if (!strncasecmp (field, "edit", 4))
+	{
+	  if (get_field_text (field + 4, entry ? &entry->editcommand : NULL,
+			      type, filename, line))
+	    editcommand = TRUE;
+	}
+	else if (!strncasecmp (field, "nametemplate", 12))
+	{
+	  get_field_text (field + 12, entry ? &entry->nametemplate : NULL,
+			  type, filename, line);
+	}
+	else if (!strncasecmp (field, "x-convert", 9))
+	{
+	  get_field_text (field + 9, entry ? &entry->convert : NULL,
+			  type, filename, line);
+	}
+	else if (!strncasecmp (field, "test", 4))
+	{
+	  /* 
+	   * This routine executes the given test command to determine
+	   * if this is the right entry.
+	   */
+	  char *test_command = NULL;
+	  size_t len;
+
+	  if (get_field_text (field + 4, &test_command, type, filename, line)
+	      && test_command)
+	  {
+	    len = strlen (test_command) + STRING;
+	    safe_realloc ((void **) &test_command, len);
+	    rfc1524_expand_command (a, NULL, type, test_command, len);
+	    if (mutt_system (test_command))
+	    {
+	      /* a non-zero exit code means test failed */
+	      found = FALSE;
+	    }
+	    free (test_command);
+	  }
+	}
+      } /* while (ch) */
+
+      if (opt == M_AUTOVIEW)
+      {
+	if (!copiousoutput)
+	  found = FALSE;
+      }
+      else if (opt == M_COMPOSE)
+      {
+	if (!composecommand)
+	  found = FALSE;
+      }
+      else if (opt == M_EDIT)
+      {
+	if (!editcommand)
+	  found = FALSE;
+      }
+      else if (opt == M_PRINT)
+      {
+	if (!printcommand)
+	  found = FALSE;
+      }
+      
+      if (!found)
+      {
+	/* reset */
+	if (entry)
+	{
+	  safe_free ((void **) &entry->command);
+	  safe_free ((void **) &entry->composecommand);
+	  safe_free ((void **) &entry->editcommand);
+	  safe_free ((void **) &entry->printcommand);
+	  safe_free ((void **) &entry->nametemplate);
+	  safe_free ((void **) &entry->convert);
+	  entry->needsterminal = 0;
+	  entry->copiousoutput = 0;
 	}
       }
-
-      /* Parse the found line for the information we need */
-      if (found)
-      {
-	char *tok;
-	int z = 0, i = 0;
-	char token[STRING];
-
-	/* first field is content type, already checked, so skip */
-	while (z < sizeof (buf) && buf[z] && buf[z] != ';')
-	  z++;
-	z++;
-
-	/* next field is the command */
-	while (z < sizeof (buf) && i < sizeof (token) && buf[z] && 
-	      !(buf[z] == ';' && buf[z-1] != '\\'))
-	  token[i++] = buf[z++];
-	token[i] = '\0';
-	z++;
-	tok = skip_whitespace (token);
-	if (entry)
-	  entry->command = safe_strdup (tok);
-
-	/* parse the optional fields */
-	/* next field is the command */
-	i = 0; 
-	while (z < sizeof (buf) && i < sizeof (token) && buf[z] && 
-	      !(buf[z] == ';' && buf[z-1] != '\\')) 
-	  token[i++] = buf[z++];
-	token[i] = '\0';
-	z++;
-	while (token[0])
-	{
-	  tok = skip_whitespace (token);
-	  dprint (2, (debugfile, "token: %s\n", tok));
-	  if (!strcasecmp (tok, "needsterminal"))
-	  {
-	    if (entry)
-	      entry->needsterminal = TRUE;
-	  }
-	  else if (!strcasecmp (tok, "copiousoutput"))
-	  {
-	    copiousoutput = TRUE;
-	    if (entry)
-	      entry->copiousoutput = TRUE;
-	  }
-	  else if (!strncasecmp (tok, "composetyped", 12))
-	  {
-	    /* this compare most occur before compose to match correctly */
-	    /* this isn't particularly well specified in the rfc, so skip it */
-	  }
-	  else if (!strncasecmp (tok, "compose", 7))
-	  {
-	    tok += 7;
-	    tok = skip_whitespace (tok);
-	    if (*tok == '=')
-	    {
-	      composecommand = TRUE;
-	      if (entry)
-	      {
-		tok = skip_whitespace (++tok);
-		safe_free ((void **) &entry->composecommand);
-		entry->composecommand = safe_strdup (tok);
-	      }
-	    }
-	    else 
-	      mutt_error ("Improperly formated mailcap entry for type %s", type);
-	  } 
-	  else if (!strncasecmp (tok, "print", 5))
-	  {
-	    tok+=5;
-	    tok = skip_whitespace (tok);
-	    if (*tok == '=')
-	    {
-	      printcommand = TRUE;
-	      if (entry)
-	      {
-		tok = skip_whitespace (++tok);
-		safe_free ((void **) &entry->printcommand);
-		entry->printcommand = safe_strdup(tok);
-	      }
-	    }
-	    else 
-	      mutt_error("Improperly formated mailcap entry for type %s", type);
-	  }
-	  else if (!strncasecmp (tok, "edit", 4))
-	  {
-	    tok+=4;
-	    tok = skip_whitespace (tok);
-	    if (*tok == '=')
-	    {
-	      editcommand = TRUE;
-	      if (entry)
-	      {
-		tok = skip_whitespace (++tok);
-		safe_free((void **) &entry->editcommand);
-		entry->editcommand = safe_strdup(tok);
-	      }
-	    }
-	    else 
-	      mutt_error ("Improperly formated mailcap entry for type %s", type);
-	  }
-	  else if (!strncasecmp (tok, "nametemplate", 12))
-	  {
-	    tok += 12;
-	    tok = skip_whitespace (tok);
-	    if (*tok == '=')
-	    {
-	      if (entry)
-	      {
-		tok = skip_whitespace (++tok);
-		safe_free ((void **) &entry->nametemplate);
-		entry->nametemplate = safe_strdup (tok);
-	      }
-	    }
-	    else
-	      mutt_error ("Improperly formated mailcap entry for type %s", type);
-	  }
-	  else if (!strncasecmp (tok, "x-convert", 9))
-	  {
-	    tok += 9;
-	    tok = skip_whitespace (tok);
-	    if (*tok == '=')
-	    {
-	      if (entry)
-	      {
-		tok = skip_whitespace (++tok);
-		safe_free ((void **) &entry->convert);
-		entry->convert = safe_strdup (tok);
-	      }
-	    }
-	    else
-	      mutt_error ("Improperly formated mailcap entry for type %s", type);
-	  }
-	  else if (!strncasecmp (tok, "test", 4))
-	  {
-	    /* 
-	     * This routine executes the given test command to determine
-	     * if this is the right entry.
-	     */
-	    char path[_POSIX_PATH_MAX+STRING];
-	    int retval;
-
-            tok += 4;
-            tok = skip_whitespace (tok);
-            if (*tok == '=')
-            {
-              tok = skip_whitespace (++tok);
-	      strfcpy (path, tok, sizeof (path));
-	      rfc1524_expand_command (a, NULL, type, path, sizeof (path));
-	      retval = mutt_system (path);
-	      if (retval)
-	      {
-	        /* a non-zero exit code means test failed */
-		match = FALSE;
-	      }
-              dprint (2, (debugfile,"testing entry: %s returns %d\n", path, retval));
-            }
-            else 
-              mutt_error ("Improperly formated mailcap entry for type %s", type);
-          }
-          i = 0; 
-          while(z < sizeof (buf) && i < sizeof (token) && buf[z] && 
-                !(buf[z] == ';' && buf[z-1] != '\\'))
-            token[i++] = buf[z++];
-          token[i] = '\0';
-          z++;
-        }
-
-        if (opt == M_AUTOVIEW)
-	{
-          if (!copiousoutput)
-	    match = FALSE;
-        }
-	else if (opt == M_COMPOSE)
-	{
-          if (!composecommand)
-	    match = FALSE;
-        }
-	else if (opt == M_EDIT)
-	{
-          if (!editcommand)
-	    match = FALSE;
-        }
-	else if (opt == M_PRINT)
-	{
-          if (!printcommand)
-	    match = FALSE;
-        }
-
-        if (!match)
-	{
-          found = FALSE;
-
-          /* reset */
-          match = TRUE;
-          if (entry)
-          {
-	    safe_free ((void **) &entry->command);
-	    safe_free ((void **) &entry->composecommand);
-	    safe_free ((void **) &entry->editcommand);
-	    safe_free ((void **) &entry->printcommand);
-	    safe_free ((void **) &entry->nametemplate);
-	    safe_free ((void **) &entry->convert);
-	    entry->needsterminal = 0;
-	    entry->copiousoutput = 0;
-	  }
-        }
-      }
-    }
-
-    fclose(fp);
-    return found;
-  }
-  else
-    return FALSE;
+    } /* while (!found && (buf = mutt_read_line ())) */
+    fclose (fp);
+  } /* if ((fp = fopen ())) */
+  safe_free ((void **) &buf);
+  return found;
 }
 
 rfc1524_entry *rfc1524_new_entry()
@@ -649,9 +569,8 @@ int rfc1524_expand_filename (char *nametemplate,
  * the filename doesn't exist yet, but if its used while sending a message,
  * then we need to rename the existing file.
  *
- * This function returns 2 on successful move, 1 on old file doesn't exist,
- * and 0 on failure.  
- * This function returns 0 if the newfile already exists or an error occurs.
+ * This function returns 0 on successful move, 1 on old file doesn't exist,
+ * 2 on new file already exists, and 3 on other failure.
  */
 int mutt_rename_file (char *oldfile, char *newfile)
 {
@@ -660,17 +579,17 @@ int mutt_rename_file (char *oldfile, char *newfile)
   if (access (oldfile, F_OK) != 0)
     return 1;
   if (access (newfile, F_OK) == 0)
-    return 0;
+    return 2;
   if ((ofp = fopen (oldfile,"r")) == NULL)
-    return 0;
-  if ((nfp = fopen (newfile,"w")) == NULL)
+    return 3;
+  if ((nfp = safe_fopen (newfile,"w")) == NULL)
   {
     fclose(ofp);
-    return 0;
+    return 3;
   }
   mutt_copy_stream (ofp,nfp);
   fclose (nfp);
   fclose (ofp);
   mutt_unlink (oldfile);
-  return 2;
+  return 0;
 }
