@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,10 @@
 
 #include "mutt.h"
 #include "rfc2047.h"
-#include "send.h"
+#include "mx.h"
+#include "mime.h"
+#include "mailbox.h"
+#include "copy.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -28,9 +31,16 @@
 #include <sys/stat.h>
 #include <signal.h>
 
+
+
+
+
+
+
+
 #define DISPOSITION(X) X==DISPATTACH?"attachment":"inline"
 
-extern char *tspecials;
+const char MimeSpecials[] = "@.,;<>[]\\\"()?/=";
 
 char B64Chars[64] = {
   'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
@@ -40,6 +50,8 @@ char B64Chars[64] = {
   '8', '9', '+', '/'
 };
 
+static char MsgIdPfx = 'A';
+
 static void transform_to_7bit (BODY *a, FILE *fpin);
 
 static void encode_quoted (FILE * fin, FILE *fout, int istext)
@@ -290,9 +302,10 @@ int mutt_write_mime_header (BODY *a, FILE *f)
   PARAMETER *p;
   char buffer[STRING];
   char *t;
+  char *fn;
   int len;
   int tmplen;
-
+  
   fprintf (f, "Content-Type: %s/%s", TYPE (a->type), a->subtype);
 
   if (a->parameter)
@@ -305,7 +318,7 @@ int mutt_write_mime_header (BODY *a, FILE *f)
       fputc (';', f);
 
       buffer[0] = 0;
-      rfc822_cat (buffer, sizeof (buffer), p->value, tspecials);
+      rfc822_cat (buffer, sizeof (buffer), p->value, MimeSpecials);
 
       tmplen = strlen (buffer) + strlen (p->attribute) + 1;
 
@@ -334,19 +347,23 @@ int mutt_write_mime_header (BODY *a, FILE *f)
   if (a->description)
     fprintf(f, "Content-Description: %s\n", a->description);
 
-  if (a->use_disp && (a->disposition == DISPATTACH || a->filename))
+  if (a->use_disp && (a->disposition == DISPATTACH || a->filename || a->d_filename))
   {
     fprintf (f, "Content-Disposition: %s", DISPOSITION (a->disposition));
-    if (a->filename)
+
+    if(!(fn = a->d_filename))
+      fn = a->filename;
+    
+    if (fn)
     {
       /* Strip off the leading path... */
-      if ((t = strrchr (a->filename, '/')))
+      if ((t = strrchr (fn, '/')))
 	t++;
       else
-	t = a->filename;
+	t = fn;
       
       buffer[0] = 0;
-      rfc822_cat (buffer, sizeof (buffer), t, tspecials);
+      rfc822_cat (buffer, sizeof (buffer), t, MimeSpecials);
       fprintf (f, "; filename=%s", buffer);
     }
 
@@ -354,48 +371,38 @@ int mutt_write_mime_header (BODY *a, FILE *f)
   }
 
   /* Do NOT add the terminator here!!! */
-  return 0;
+  return (ferror (f) ? -1 : 0);
 }
 
 int mutt_write_mime_body (BODY *a, FILE *f)
 {
-  char boundary[SHORT_STRING];
+  char *p, boundary[SHORT_STRING];
   FILE *fpin;
   BODY *t;
-  PARAMETER *p;
 
   if (a->type == TYPEMULTIPART)
   {
     /* First, find the boundary to use */
-    for (p = a->parameter; p ; p = p->next)
+    if (!(p = mutt_get_parameter ("boundary", a->parameter)))
     {
-      if (strcmp ("boundary", p->attribute) == 0)
-      {
-        strfcpy (boundary, p->value, sizeof (boundary));
-        break;
-      }
+      dprint (1, (debugfile, "mutt_write_mime_body(): no boundary parameter found!\n"));
+      return (-1);
     }
+    strfcpy (boundary, p, sizeof (boundary));
 
     for (t = a->parts; t ; t = t->next)
     {
       fprintf (f, "\n--%s\n", boundary);
-      mutt_write_mime_header (t, f);
+      if (mutt_write_mime_header (t, f) == -1)
+	return -1;
       fputc ('\n', f);
       if (mutt_write_mime_body (t, f) == -1)
 	return -1;
     }
-    fprintf(f, "\n--%s--\n", boundary);
-    return 0;
+    fprintf (f, "\n--%s--\n", boundary);
+    return (ferror (f) ? -1 : 0);
   }
 
-#ifdef _PGPPATH
-  /* This is pretty gross, but it's the best solution for now... */
-  if (a->type == TYPEAPPLICATION && strcmp (a->subtype, "pgp-encrypted") == 0)
-  {
-    fputs ("Version: 1\n", f);
-    return 0;
-  }
-#endif /* _PGPPATH */
 
   if ((fpin = fopen (a->filename, "r")) == NULL)
   {
@@ -411,7 +418,7 @@ int mutt_write_mime_body (BODY *a, FILE *f)
     mutt_copy_stream (fpin, f);
   fclose (fpin);
 
-  return 0;
+  return (ferror (f) ? -1 : 0);
 }
 
 #define BOUNDARYLEN 16
@@ -428,7 +435,7 @@ char *mutt_generate_boundary (void)
 }
 
 /* analyze the contents of a file to determine which MIME encoding to use */
-CONTENT *mutt_get_content_info (const char *fname)
+static CONTENT *mutt_get_content_info (const char *fname)
 {
   CONTENT *info;
   FILE *fp;
@@ -633,70 +640,46 @@ void mutt_message_to_7bit (BODY *a, FILE *fp)
   char *line = NULL;
   FILE *fpin = NULL;
   FILE *fpout = NULL;
-  int have_mime_version = 0;
   struct stat sb;
 
   if (!a->filename && fp)
     fpin = fp;
   else if (!a->filename || !(fpin = fopen (a->filename, "r")))
   {
-    mutt_error ("could not open %s", a->filename ? a->filename : "(null)");
+    mutt_error ("Could not open %s", a->filename ? a->filename : "(null)");
     return;
+  } 
+  else
+  {
+    a->offset = 0;
+    if (stat (a->filename, &sb) == -1)
+    {
+      mutt_perror ("stat");
+      fclose (fpin);
+    }
+    a->length = sb.st_size;
   }
 
-  fseek(fpin, a->offset, 0);
-  a->parts = mutt_parse_messageRFC822 (fpin, a);
-  
-  mutt_mktemp(temp);
-  if(!(fpout = safe_fopen (temp, "w+")))
+  mutt_mktemp (temp);
+  if (!(fpout = safe_fopen (temp, "w+")))
   {
     mutt_perror ("fopen");
     goto cleanup;
   }
 
-  transform_to_7bit (a->parts, fpin);
-
   fseek (fpin, a->offset, 0);
-  while ((line = mutt_read_rfc822_line(fpin, line, &linelen)) != NULL
-	 && *line)
-  {
-    if (tolower(*line) == 'c') 
-    {
-      if (!strncasecmp (line+1, "ontent-transfer-encoding:", 
-			sizeof ("ontent-transfer-encoding:") -1 ))
-	continue;
+  a->parts = mutt_parse_messageRFC822 (fpin, a);
 
-      if (!strncasecmp (line+1, "ontent-type:",
-			sizeof ("ontent-type:") - 1))
-	continue;
-
-      if (!strncasecmp (line+1, "ontent-description:",
-			sizeof ("ontent-description:") - 1))
-	continue;
-
-      if (!strncasecmp (line+1, "ontent-disposition:",
-			sizeof ("ontent-disposition:") - 1))
-	continue;
-
-      if (!strncasecmp (line+1, "ontent-length:",
-			sizeof ("ontent-length:") - 1))
-	continue;
-
-    }
-    else if (tolower(*line) == 'm')
-      if (!strncasecmp (line+1, "ime-version:", sizeof ("ime-version:" - 1)))
-	have_mime_version = 1;
-
-    fprintf(fpout, "%s\n", line);
-  }
-
-  if (!have_mime_version)
-    fputs ("MIME-Version: 1.0\n", fpout);
+  transform_to_7bit (a->parts, fpin);
+  
+  mutt_copy_hdr (fpin, fpout, a->offset, a->offset + a->length, 
+		 CH_MIME | CH_NONEWLINE | CH_XMIT, NULL);
 
+  fputs ("Mime-Version: 1.0\n", fpout);
   mutt_write_mime_header (a->parts, fpout);
   fputc ('\n', fpout);
   mutt_write_mime_body (a->parts, fpout);
-
+  
  cleanup:
   safe_free ((void **) &line);
   linelen = 0;
@@ -709,9 +692,9 @@ void mutt_message_to_7bit (BODY *a, FILE *fp)
     return;
   
   a->encoding = ENC7BIT;
+  a->d_filename = a->filename;
   if (a->filename && a->unlink)
     unlink (a->filename);
-  safe_free ((void **)&a->filename);
   a->filename = safe_strdup (temp);
   a->unlink = 1;
   if(stat (a->filename, &sb) == -1) 
@@ -720,7 +703,6 @@ void mutt_message_to_7bit (BODY *a, FILE *fp)
     return;
   }
   a->length = sb.st_size;
-
   mutt_free_body (&a->parts);
 }
 
@@ -740,14 +722,14 @@ static void transform_to_7bit (BODY *a, FILE *fpin)
 
       transform_to_7bit (a->parts, fpin);
     } 
-    else if(a->type == TYPEMESSAGE && strcasecmp (a->subtype, "delivery-status"))
+    else if (a->type == TYPEMESSAGE && strcasecmp (a->subtype, "delivery-status"))
     {
       mutt_message_to_7bit (a, fpin);
     }
     else 
     {
       mutt_mktemp (buff);
-      if((s.fpout = safe_fopen (buff, "w")) == NULL) 
+      if ((s.fpout = safe_fopen (buff, "w")) == NULL) 
       {
 	mutt_perror ("fopen");
 	return;
@@ -755,7 +737,7 @@ static void transform_to_7bit (BODY *a, FILE *fpin)
       s.fpin = fpin;
       mutt_decode_attachment (a, &s);
       fclose (s.fpout);
-      safe_free ((void **)&a->filename); /* shouldn't be necessary */
+      a->d_filename = a->filename;
       a->filename = safe_strdup (buff);
       a->unlink = 1;
       if (stat (a->filename, &sb) == -1) 
@@ -765,6 +747,7 @@ static void transform_to_7bit (BODY *a, FILE *fpin)
       }
       a->length = sb.st_size;
 
+      mutt_update_encoding (a);
       if (a->encoding == ENC8BIT)
 	a->encoding = ENCQUOTEDPRINTABLE;
       else if(a->encoding == ENCBINARY)
@@ -774,7 +757,7 @@ static void transform_to_7bit (BODY *a, FILE *fpin)
 }
 
 /* determine which Content-Transfer-Encoding to use */
-void mutt_set_encoding (BODY *b, CONTENT *info)
+static void mutt_set_encoding (BODY *b, CONTENT *info)
 {
   if (b->type == TYPETEXT)
   {
@@ -785,9 +768,9 @@ void mutt_set_encoding (BODY *b, CONTENT *info)
     else
       b->encoding = ENC7BIT;
   }
-  else if(b->type == TYPEMESSAGE)
+  else if (b->type == TYPEMESSAGE)
   {
-    if(info->lobin || info->hibin)
+    if (info->lobin || info->hibin)
     {
       if (option (OPTALLOW8BIT) && !info->lobin)
 	b->encoding = ENC8BIT; 
@@ -832,15 +815,6 @@ void mutt_update_encoding (BODY *a)
     a->parameter->value = safe_strdup (set_text_charset (info));
   }
 
-#ifdef _PGPPATH
-  /* save the info in case this message is signed.  we will want to do Q-P
-   * encoding if any lines begin with "From " so the signature won't be munged,
-   * for example.
-   */ 
-  safe_free ((void **) &a->content);
-  a->content = info;
-  info = NULL;
-#endif
 
   safe_free ((void **) &info);
 }
@@ -890,15 +864,6 @@ BODY *mutt_make_attach (const char *path)
 
   mutt_set_encoding (att, info);
 
-#ifdef _PGPPATH
-  /*
-   * save the info in case this message is signed.  we will want to do Q-P
-   * encoding if any lines begin with "From " so the signature won't be munged,
-   * for example.
-   */
-  att->content = info;
-  info = NULL;
-#endif
   safe_free ((void **) &info);
 
   return (att);
@@ -908,13 +873,12 @@ static int get_toplevel_encoding (BODY *a)
 {
   int e = ENC7BIT;
 
-  while (a)
+  for (; a; a = a->next)
   {
     if (a->encoding == ENCBINARY)
       return (ENCBINARY);
-    else if (a->encoding == ENC8BIT && e == ENC7BIT)
+    else if (a->encoding == ENC8BIT)
       e = ENC8BIT;
-    a = a->next;
   }
 
   return (e);
@@ -958,14 +922,12 @@ static char *mutt_make_date (char *s)
 }
 
 /* wrapper around mutt_write_address() so we can handle very large
- * recipient lists without needing a huge temporary buffer in memory
- */
-void mutt_write_address_list (ADDRESS *adr, FILE *fp)
+   recipient lists without needing a huge temporary buffer in memory */
+void mutt_write_address_list (ADDRESS *adr, FILE *fp, int linelen)
 {
   ADDRESS *tmp;
   char buf[LONG_STRING];
   int count = 0;
-  int linelen = 4; /* assume the line begins with "To: " or "Cc: " */
   int len;
 
   while (adr)
@@ -994,7 +956,7 @@ void mutt_write_address_list (ADDRESS *adr, FILE *fp)
     }
     fputs (buf, fp);
     adr->next = tmp;
-    if (adr->host && adr->next && adr->next->mailbox)
+    if (!adr->group && adr->next && adr->next->mailbox)
     {
       linelen++;
       fputc (',', fp);
@@ -1066,7 +1028,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach, int mode)
   if (env->to)
   {
     fputs ("To: ", fp);
-    mutt_write_address_list (env->to, fp);
+    mutt_write_address_list (env->to, fp, 4);
   }
   else if (mode > 0)
     fputs ("To: \n", fp);
@@ -1074,7 +1036,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach, int mode)
   if (env->cc)
   {
     fputs ("Cc: ", fp);
-    mutt_write_address_list (env->cc, fp);
+    mutt_write_address_list (env->cc, fp, 4);
   }
   else if (mode > 0)
     fputs ("Cc: \n", fp);
@@ -1082,7 +1044,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach, int mode)
   if (env->bcc)
   {
     fputs ("Bcc: ", fp);
-    mutt_write_address_list (env->bcc, fp);
+    mutt_write_address_list (env->bcc, fp, 5);
   }
   else if (mode > 0)
     fputs ("Bcc: \n", fp);
@@ -1095,11 +1057,17 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach, int mode)
   if (env->reply_to)
   {
     fputs ("Reply-To: ", fp);
-    mutt_write_address_list (env->reply_to, fp);
+    mutt_write_address_list (env->reply_to, fp, 10);
   }
   else if (mode > 0)
     fputs ("Reply-To: \n", fp);
 
+  if (env->mail_followup_to)
+  {
+    fputs ("Mail-Followup-To: ", fp);
+    mutt_write_address_list (env->mail_followup_to, fp, 18);
+  }
+
   if (mode <= 0)
   {
     if (env->references)
@@ -1118,7 +1086,7 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach, int mode)
   if (mode == 0)
   {
     /* Add a vanity header */
-    fprintf (fp, "X-Mailer: %s\n", MuttVersion);
+    fprintf (fp, "X-Mailer: Mutt %s\n", VERSION);
   }
 #endif
 
@@ -1139,7 +1107,7 @@ static void encode_headers (LIST *h)
   char *p;
   size_t len;
 
-  while (h)
+  for (; h; h = h->next)
   {
     if ((p = strchr (h->data, ':')))
     {
@@ -1147,11 +1115,10 @@ static void encode_headers (LIST *h)
       SKIPWS (p);
       snprintf (tmp, sizeof (tmp), "%s: ", h->data);
       len = strlen (tmp);
-      rfc2047_encode_string (tmp + len, (unsigned char *) p, sizeof (tmp) - len);
+      rfc2047_encode_string (tmp + len, sizeof (tmp) - len, (unsigned char *) p);
       safe_free ((void **) &h->data);
       h->data = safe_strdup (tmp);
     }
-    h = h->next;
   }
 }
 
@@ -1165,7 +1132,7 @@ static void encode_descriptions (BODY *b)
   {
     if (t->description)
     {
-      rfc2047_encode_string (tmp, (unsigned char *) t->description, sizeof (tmp));
+      rfc2047_encode_string (tmp, sizeof (tmp), (unsigned char *) t->description);
       safe_free ((void **) &t->description);
       t->description = safe_strdup (tmp);
     }
@@ -1182,10 +1149,11 @@ char *mutt_gen_msgid (void)
 
   now = time (NULL);
   tm = localtime (&now);
-  snprintf (buf, sizeof (buf), "<%d%02d%02d%02d%02d%02d.%05ld@%s>",
-	    tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour,
-	    tm->tm_min, tm->tm_sec, LRAND() % 65535,
+  snprintf (buf, sizeof (buf), "<%d%02d%02d%02d%02d%02d.%c%d@%s>",
+	    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour,
+	    tm->tm_min, tm->tm_sec, MsgIdPfx, getpid (),
 	    Fqdn[0] != '@' ? Fqdn : Hostname);
+  MsgIdPfx = (MsgIdPfx == 'Z') ? 'A' : MsgIdPfx + 1;
   return (safe_strdup (buf));
 }
 
@@ -1212,17 +1180,19 @@ int mutt_send_message (HEADER *msg, const char *fcc)
   rfc2047_encode_adrlist (msg->env->cc);
   rfc2047_encode_adrlist (msg->env->bcc);
   rfc2047_encode_adrlist (msg->env->from);
+  rfc2047_encode_adrlist (msg->env->mail_followup_to);
   if (msg->env->subject)
   {
-    rfc2047_encode_string (buffer, (unsigned char *)msg->env->subject,
-			   sizeof (buffer) - 1);
+    rfc2047_encode_string (buffer, sizeof (buffer) -1,
+			   (unsigned char *)msg->env->subject);
     safe_free ((void **) &msg->env->subject);
     msg->env->subject = safe_strdup (buffer);
   }
   encode_headers (msg->env->userhdrs);
   encode_descriptions (msg->content);
 
-  msg->env->message_id = mutt_gen_msgid ();
+  if (!msg->env->message_id)
+    msg->env->message_id = mutt_gen_msgid ();
 
   /* Write out the message in MIME form. */
   mutt_mktemp (tempfile);
@@ -1233,7 +1203,12 @@ int mutt_send_message (HEADER *msg, const char *fcc)
   fputc ('\n', tempfp); /* tie off the header. */
 
   mutt_write_mime_body (msg->content, tempfp);
-  fclose (tempfp);
+  if (fclose (tempfp) != 0)
+  {
+    mutt_perror (tempfile);
+    unlink (tempfile);
+    return (-1);
+  }
 
   strfcpy (cmd, Sendmail, sizeof (cmd));
   if (DsnNotify[0])
@@ -1250,6 +1225,9 @@ int mutt_send_message (HEADER *msg, const char *fcc)
   if (msg->content->encoding == ENC8BIT && option (OPTUSE8BITMIME))
     strcat (cmd, " -B8BITMIME");
 
+  /* this should really check the return code, but at least under AIX,
+   * mutt_system() always returns -1
+   */
   invoke_sendmail (cmd, tempfile);
 
   /* save a copy of the message, if necessary. */
@@ -1271,29 +1249,16 @@ int mutt_send_message (HEADER *msg, const char *fcc)
   return 0;
 }
 
-static void mutt_rfc822_address (char *s, size_t len, ADDRESS *a)
-{
-  if (a->host)
-  {
-    if (*a->host != '@')
-      snprintf (s, len, "%s@%s", a->mailbox, a->host);
-    else
-      strfcpy (s, a->mailbox, len);
-  }
-  else if (a->mailbox)
-    snprintf (s, len, "%s: ", a->mailbox);
-  else
-    strfcpy (s, ";", len);
-}
-
 static void write_addresses (char *s, size_t len, ADDRESS *a)
 {
   size_t l;
 
   while (a && len > 0)
   {
-    mutt_rfc822_address (s, len, a);
-    l = strlen (s);
+    l = strlen (a->mailbox);
+    if (l > len - 1)
+      l = len - 1;
+    memcpy (s, a->mailbox, l + 1); /* +1 to copy the  trailing \0 */
     len -= l;
     s += l;
     a = a->next;
@@ -1328,13 +1293,13 @@ void mutt_bounce_message (HEADER *h, ADDRESS *to)
     if ((f = safe_fopen (tempfile, "w")) != NULL)
     {
       fseek (msg->fp, h->offset, 0);
-      mutt_copy_header (msg->fp, h, f, CH_XMIT);
+      mutt_copy_header (msg->fp, h, f, CH_XMIT | CH_NONEWLINE, NULL);
       fprintf (f, "Resent-From: %s", Username);
       if (Fqdn[0] != '@')
 	fprintf (f, "@%s", Fqdn);
       fprintf (f, "\nResent-%s", mutt_make_date (date));
       fputs ("Resent-To: ", f);
-      mutt_write_address_list (to, f);
+      mutt_write_address_list (to, f, 11);
       fputc ('\n', f);
       mutt_copy_bytes (msg->fp, f, h->content->length);
       fclose (f);
@@ -1351,11 +1316,17 @@ void mutt_bounce_message (HEADER *h, ADDRESS *to)
 
 ADDRESS *mutt_default_from (void)
 {
-  ADDRESS *adr = mutt_new_address ();
+  ADDRESS *adr = rfc822_new_address ();
 
   adr->personal = safe_strdup (Realname);
-  adr->mailbox = safe_strdup (Username);
-  adr->host = safe_strdup (option (OPTUSEDOMAIN) ? Fqdn : "@");
+  
+  if (option (OPTUSEDOMAIN))
+  {
+    adr->mailbox = safe_malloc (strlen (Username) + strlen (Fqdn) + 2);
+    sprintf (adr->mailbox, "%s@%s", Username, Fqdn);
+  }
+  else
+    adr->mailbox = safe_strdup (Username);
   return (adr);
 }
 
@@ -1373,15 +1344,14 @@ ADDRESS *mutt_remove_duplicates (ADDRESS *addr)
   {
     tmp = top;
     do {
-      if (addr->mailbox && addr->host && tmp->mailbox && tmp->host &&
-	  !strcasecmp (addr->mailbox, tmp->mailbox) &&
-	  !strcasecmp (addr->host, tmp->host))
+      if (addr->mailbox && tmp->mailbox &&
+	  !strcasecmp (addr->mailbox, tmp->mailbox))
       {
 	/* duplicate address, just ignore it */
 	tmp = addr;
 	addr = addr->next;
 	tmp->next = NULL;
-	mutt_free_address (&tmp);
+	rfc822_free_address (&tmp);
       }
       else if (!tmp->next)
       {
@@ -1405,8 +1375,8 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post)
   CONTEXT f;
   MESSAGE *msg;
   char tempfile[_POSIX_PATH_MAX];
-  struct stat st;
   FILE *tempfp = NULL;
+  int r;
 
   if (mx_open_mailbox (path, M_APPEND | M_QUIET, &f) == NULL)
   {
@@ -1430,19 +1400,12 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post)
   }
 
   hdr->read = 1; /* make sure to put it in the `cur' directory (maildir) */
-
-  if ((msg = mx_open_new_message (&f, hdr)) == NULL)
+  if ((msg = mx_open_new_message (&f, hdr, M_ADD_FROM)) == NULL)
   {
     mx_close_mailbox (&f);
     return (-1);
   }
 
-  if (f.magic == M_MMDF || f.magic == M_MBOX)
-  {
-    time_t now = time (NULL);
-    fprintf (msg->fp, "From %s %s", Username, ctime (&now));
-  }
-
   mutt_write_rfc822_header (msg->fp, hdr->env, hdr->content, (post ? -1 : 0));
 
   /* (postponment) if this was a reply of some sort, <msgid> contians the
@@ -1455,8 +1418,12 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post)
     fprintf (msg->fp, "X-Mutt-References: %s\n", msgid);
   fprintf (msg->fp, "Status: RO\n");
 
+    
   if (tempfp)
   {
+    char sasha[LONG_STRING];
+    int lines = 0;
+
     mutt_write_mime_body (hdr->content, tempfp);
 
     /* make sure the last line ends with a newline.  Emacs doesn't ensure
@@ -1471,24 +1438,40 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post)
     }
 
     fflush (tempfp);
-    rewind (tempfp);
-    fstat (fileno (tempfp), &st);
+    if (ferror (tempfp))
+    {
+      dprint (1, (debugfile, "mutt_write_fcc(): %s: write failed.\n", tempfile));
+      fclose (tempfp);
+      unlink (tempfile);
+      mx_close_message (&msg);
+      mx_close_mailbox (&f);
+      return -1;
+    }
 
-    fprintf (msg->fp, "Content-Length: %ld\n\n", (long) st.st_size);
+    /* count the number of lines */
+    rewind (tempfp);
+    while (fgets (sasha, sizeof (sasha), tempfp) != NULL)
+      lines++;
+    fprintf (msg->fp, "Content-Length: %ld\n", (long) ftell (tempfp));
+    fprintf (msg->fp, "Lines: %d\n\n", lines);
 
     /* copy the body and clean up */
-    mutt_copy_stream (tempfp, msg->fp);
-    fclose (tempfp);
-    unlink (tempfile);
+    rewind (tempfp);
+    r = mutt_copy_stream (tempfp, msg->fp);
+    if (fclose (tempfp) != 0)
+      r = -1;
+    /* if there was an error, leave the temp version */
+    if (!r)
+      unlink (tempfile);
   }
   else
   {
     fputc ('\n', msg->fp); /* finish off the header */
-    mutt_write_mime_body (hdr->content, msg->fp);
+    r = mutt_write_mime_body (hdr->content, msg->fp);
   }
 
   mx_close_message (&msg);
   mx_close_mailbox (&f);
 
-  return 0;
+  return r;
 }
