@@ -1,14 +1,3 @@
-#include "config.h"
-
-#ifndef HAVE_SNPRINTF
-
-#include <ctype.h>
-#include <sys/types.h>
-
-/* Define this as a fall through, HAVE_STDARG_H is probably already set */
-
-#define HAVE_VARARGS_H
-
 /**************************************************************
  * Original:
  * Patrick Powell Tue Apr 11 09:48:21 PDT 1995
@@ -23,13 +12,13 @@
  * causing nast effects.
  *
  * More Recently:
- *  Brandon Long (blong@fiction.net) 9/15/96 for mutt 0.43
+ *  Brandon Long <blong@fiction.net> 9/15/96 for mutt 0.43
  *  This was ugly.  It is still ugly.  I opted out of floating point
  *  numbers, but the formatter understands just about everything
  *  from the normal C string format, at least as far as I can tell from
  *  the Solaris 2.5 printf(3S) man page.
  *
- *  Brandon Long (blong@fiction.net) 10/22/97 for mutt 0.87.1
+ *  Brandon Long <blong@fiction.net> 10/22/97 for mutt 0.87.1
  *    Ok, added some minimal floating point support, which means this
  *    probably requires libm on most operating systems.  Don't yet
  *    support the exponent (e,E) and sigfig (g,G).  Also, fmtint()
@@ -39,9 +28,28 @@
  *    original.  Also, there is now a builtin-test, just compile with:
  *           gcc -DTEST_SNPRINTF -o snprintf snprintf.c -lm
  *    and run snprintf for results.
+ * 
+ *  Thomas Roessler <roessler@guug.de> 01/27/98 for mutt 0.89i
+ *    The PGP code was using unsigned hexadecimal formats. 
+ *    Unfortunately, unsigned formats simply didn't work.
+ *
+ *  Michael Elkins <me@cs.hmc.edu> 03/05/98 for mutt 0.90.8
+ *    The original code assumed that both snprintf() and vsnprintf() were
+ *    missing.  Some systems only have snprintf() but not vsnprintf(), so
+ *    the code is now broken down under HAVE_SNPRINTF and HAVE_VSNPRINTF.
  *
  **************************************************************/
 
+#include "config.h"
+
+#if !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF)
+
+#include <ctype.h>
+#include <sys/types.h>
+
+/* Define this as a fall through, HAVE_STDARG_H is probably already set */
+
+#define HAVE_VARARGS_H
 
 /* varargs declarations: */
 
@@ -65,8 +73,8 @@
 # endif
 #endif
 
-int snprintf (char *str, size_t count, const char *fmt, ...);
-int vsnprintf (char *str, size_t count, const char *fmt, va_list arg);
+/*int snprintf (char *str, size_t count, const char *fmt, ...);*/
+/*int vsnprintf (char *str, size_t count, const char *fmt, va_list arg);*/
 
 static void dopr (char *buffer, size_t maxlen, const char *format, 
                   va_list args);
@@ -78,36 +86,6 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
 		   long double fvalue, int min, int max, int flags);
 static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c );
 
-int vsnprintf (char *str, size_t count, const char *fmt, va_list args)
-{
-  str[0] = 0;
-  dopr(str, count, fmt, args);
-  return(strlen(str));
-}
-
-/* VARARGS3 */
-#ifdef HAVE_STDARGS
-int snprintf (char *str,size_t count,const char *fmt,...)
-#else
-int snprintf (va_alist) va_dcl
-#endif
-{
-#ifndef HAVE_STDARGS
-  char *str;
-  size_t count;
-  char *fmt;
-#endif
-  VA_LOCAL_DECL;
-    
-  VA_START (fmt);
-  VA_SHIFT (str, char *);
-  VA_SHIFT (count, size_t );
-  VA_SHIFT (fmt, char *);
-  (void) vsnprintf(str, count, fmt, ap);
-  VA_END;
-  return(strlen(str));
-}
-
 /*
  * dopr(): poor man's version of doprintf
  */
@@ -123,12 +101,13 @@ int snprintf (va_alist) va_dcl
 #define DP_S_DONE    7
 
 /* format flags - Bits */
-#define DP_F_MINUS 1
-#define DP_F_PLUS  2
-#define DP_F_SPACE 4
-#define DP_F_NUM   8
-#define DP_F_ZERO  16
-#define DP_F_UP    32
+#define DP_F_MINUS 	(1 << 0)
+#define DP_F_PLUS  	(1 << 1)
+#define DP_F_SPACE 	(1 << 2)
+#define DP_F_NUM   	(1 << 3)
+#define DP_F_ZERO  	(1 << 4)
+#define DP_F_UP    	(1 << 5)
+#define DP_F_UNSIGNED 	(1 << 6)
 
 /* Conversion Flags */
 #define DP_C_SHORT   1
@@ -274,7 +253,7 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	fmtint (buffer, &currlen, maxlen, value, 10, min, max, flags);
 	break;
       case 'o':
-	flags &= ~DP_F_PLUS;
+	flags |= DP_F_UNSIGNED;
 	if (cflags == DP_C_SHORT)
 	  value = va_arg (args, unsigned short int);
 	else if (cflags == DP_C_LONG)
@@ -284,7 +263,7 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	fmtint (buffer, &currlen, maxlen, value, 8, min, max, flags);
 	break;
       case 'u':
-	flags &= ~DP_F_PLUS;
+	flags |= DP_F_UNSIGNED;
 	if (cflags == DP_C_SHORT)
 	  value = va_arg (args, unsigned short int);
 	else if (cflags == DP_C_LONG)
@@ -296,7 +275,7 @@ static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
       case 'X':
 	flags |= DP_F_UP;
       case 'x':
-	flags &= ~DP_F_PLUS;
+	flags |= DP_F_UNSIGNED;
 	if (cflags == DP_C_SHORT)
 	  value = va_arg (args, unsigned short int);
 	else if (cflags == DP_C_LONG)
@@ -445,17 +424,21 @@ static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
     max = 0;
 
   uvalue = value;
-  if( value < 0 ) {
-    signvalue = '-';
-    uvalue = -value;
-  }
-  else
-    if (flags & DP_F_PLUS)  /* Do a sign (+/i) */
-      signvalue = '+';
+
+  if(!(flags & DP_F_UNSIGNED))
+  {
+    if( value < 0 ) {
+      signvalue = '-';
+      uvalue = -value;
+    }
+    else
+      if (flags & DP_F_PLUS)  /* Do a sign (+/i) */
+	signvalue = '+';
     else
       if (flags & DP_F_SPACE)
 	signvalue = ' ';
-
+  }
+  
   if (flags & DP_F_UP) caps = 1; /* Should characters be upper case? */
 
   do {
@@ -468,7 +451,7 @@ static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
   convert[place] = 0;
 
   zpadlen = max - place;
-  spadlen = min - MAX (max, place) - (signvalue ? 1 : 0); 
+  spadlen = min - MAX (max, place) - (signvalue ? 1 : 0);
   if (zpadlen < 0) zpadlen = 0;
   if (spadlen < 0) spadlen = 0;
   if (flags & DP_F_ZERO)
@@ -692,6 +675,40 @@ static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c)
   if (*currlen < maxlen)
     buffer[(*currlen)++] = c;
 }
+#endif /* !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF) */
+
+#ifndef HAVE_VSNPRINTF
+int vsnprintf (char *str, size_t count, const char *fmt, va_list args)
+{
+  str[0] = 0;
+  dopr(str, count, fmt, args);
+  return(strlen(str));
+}
+#endif /* !HAVE_VSNPRINTF */
+
+#ifndef HAVE_SNPRINTF
+/* VARARGS3 */
+#ifdef HAVE_STDARGS
+int snprintf (char *str,size_t count,const char *fmt,...)
+#else
+int snprintf (va_alist) va_dcl
+#endif
+{
+#ifndef HAVE_STDARGS
+  char *str;
+  size_t count;
+  char *fmt;
+#endif
+  VA_LOCAL_DECL;
+    
+  VA_START (fmt);
+  VA_SHIFT (str, char *);
+  VA_SHIFT (count, size_t );
+  VA_SHIFT (fmt, char *);
+  (void) vsnprintf(str, count, fmt, ap);
+  VA_END;
+  return(strlen(str));
+}
 
 #ifdef TEST_SNPRINTF
 #ifndef LONG_STRING
