@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
  */ 
 
 #include "mutt.h"
+#include "sort.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -25,20 +26,40 @@
 
 #define SORTCODE(x) (Sort & SORT_REVERSE) ? -(x) : x
 
+/* function to use as discriminator when normal sort method is equal */
+static sort_t *AuxSort = NULL;
+
+#define AUXSORT(code,a,b) if (!code && AuxSort && !option(OPTAUXSORT)) { \
+  set_option(OPTAUXSORT); \
+  code = AuxSort(a,b); \
+  unset_option(OPTAUXSORT); \
+}
+
+int compare_score (const void *a, const void *b)
+{
+  HEADER **pa = (HEADER **) a;
+  HEADER **pb = (HEADER **) b;
+  int result = (*pb)->score - (*pa)->score; /* note that this is reverse */
+  AUXSORT(result,a,b);
+  return (SORTCODE (result));
+}
+
 int compare_size (const void *a, const void *b)
 {
   HEADER **pa = (HEADER **) a;
   HEADER **pb = (HEADER **) b;
-
-  return (SORTCODE ((*pa)->content->length - (*pb)->content->length));
+  int result = (*pa)->content->length - (*pb)->content->length;
+  AUXSORT(result,a,b);
+  return (SORTCODE (result));
 }
 
 int compare_date_sent (const void *a, const void *b)
 {
   HEADER **pa = (HEADER **) a;
   HEADER **pb = (HEADER **) b;
-
-  return (SORTCODE ((*pa)->date_sent - (*pb)->date_sent));
+  int result = (*pa)->date_sent - (*pb)->date_sent;
+  AUXSORT(result,a,b);
+  return (SORTCODE (result));
 }
 
 int compare_subject (const void *a, const void *b)
@@ -57,52 +78,63 @@ int compare_subject (const void *a, const void *b)
   else if (!(*pb)->env->real_subj)
     rc = 1;
   else
-    rc = strcmp ((*pa)->env->real_subj, (*pb)->env->real_subj);
+    rc = strcasecmp ((*pa)->env->real_subj, (*pb)->env->real_subj);
+  AUXSORT(rc,a,b);
   return (SORTCODE (rc));
 }
 
+char *mutt_get_name (ADDRESS *a)
+{
+  ADDRESS *ali;
+
+  if (a)
+  {
+    if (option (OPTREVALIAS) && (ali = alias_reverse_lookup (a)) && ali->personal)
+      return ali->personal;
+    else if (a->personal)
+      return a->personal;
+    else if (a->mailbox)
+      return (a->mailbox);
+  }
+  /* don't return NULL to avoid segfault when printing/comparing */
+  return ("");
+}
+
+int compare_to (const void *a, const void *b)
+{
+  HEADER **ppa = (HEADER **) a;
+  HEADER **ppb = (HEADER **) b;
+  char *fa, *fb;
+  int result;
+
+  fa = mutt_get_name ((*ppa)->env->to);
+  fb = mutt_get_name ((*ppb)->env->to);
+  result = strcasecmp (fa, fb);
+  AUXSORT(result,a,b);
+  return (SORTCODE (result));
+}
+
 int compare_from (const void *a, const void *b)
 {
   HEADER **ppa = (HEADER **) a;
   HEADER **ppb = (HEADER **) b;
-  HEADER *pa = *ppa, *pb = *ppb;
-  ADDRESS *adr;
-  char bufa[SHORT_STRING], bufb[SHORT_STRING];
   char *fa, *fb;
+  int result;
 
-  if (option (OPTREVALIAS) && (adr = alias_reverse_lookup (pa->env->from))
-				&& adr->personal)
-    fa = adr->personal;
-  else if (pa->env->from && pa->env->from->personal)
-    fa = pa->env->from->personal;
-  else
-  {
-    bufa[0] = 0;
-    rfc822_write_address (bufa, sizeof (bufa), pa->env->from);
-    fa = bufa;
-  }
-
-  if (option (OPTREVALIAS) && (adr = alias_reverse_lookup (pb->env->from))
-				&& adr->personal)
-    fb = adr->personal;
-  else if (pb->env->from && pb->env->from->personal)
-    fb = pb->env->from->personal;
-  else
-  {
-    bufb[0] = 0;
-    rfc822_write_address (bufb, sizeof (bufb), pb->env->from);
-    fb = bufb;
-  }
-
-  return (SORTCODE (strcmp (fa, fb)));
+  fa = mutt_get_name ((*ppa)->env->from);
+  fb = mutt_get_name ((*ppb)->env->from);
+  result = strcasecmp (fa, fb);
+  AUXSORT(result,a,b);
+  return (SORTCODE (result));
 }
 
 int compare_date_received (const void *a, const void *b)
 {
   HEADER **pa = (HEADER **) a;
   HEADER **pb = (HEADER **) b;
-  
-  return (SORTCODE ((*pa)->received - (*pb)->received));
+  int result = (*pa)->received - (*pb)->received;
+  AUXSORT(result,a,b);
+  return (SORTCODE (result));
 }
 
 int compare_order (const void *a, const void *b)
@@ -110,6 +142,7 @@ int compare_order (const void *a, const void *b)
   HEADER **ha = (HEADER **) a;
   HEADER **hb = (HEADER **) b;
 
+  /* no need to auxsort because you will never have equality here */
   return (SORTCODE ((*ha)->index - (*hb)->index));
 }
 
@@ -129,17 +162,23 @@ sort_t *mutt_get_sort_func (int method)
       return (compare_from);
     case SORT_SIZE:
       return (compare_size);
+    case SORT_TO:
+      return (compare_to);
+    case SORT_SCORE:
+      return (compare_score);
     default:
       return (NULL);
   }
   /* not reached */
 }
 
-void mutt_sort_headers (CONTEXT *ctx)
+void mutt_sort_headers (CONTEXT *ctx, int init)
 {
   int i;
   sort_t *sortfunc;
   
+  unset_option (OPTNEEDRESORT);
+
   if (!ctx)
     return;
 
@@ -150,15 +189,38 @@ void mutt_sort_headers (CONTEXT *ctx)
      * in that routine, so we must make sure to zero the vcount member.
      */
     ctx->vcount = 0;
+    ctx->tree = 0;
     return; /* nothing to do! */
   }
 
   if (!ctx->quiet)
     mutt_message ("Sorting mailbox...");
 
+  /* threads may be bogus, so clear the links */
+  if (init)
+    mutt_clear_threads (ctx);
+
+  if (option (OPTNEEDRESCORE))
+  {
+    for (i = 0; i < ctx->msgcount; i++)
+      mutt_score_message (ctx->hdrs[i]);
+    unset_option (OPTNEEDRESCORE);
+  }
+
   if ((Sort & SORT_MASK) == SORT_THREADS)
-    mutt_sort_threads (ctx);
-  else if ((sortfunc = mutt_get_sort_func (Sort)) == NULL)
+  {
+    AuxSort = NULL;
+    /* if $sort_aux changed after the mailbox is sorted, then all the
+       subthreads need to be resorted */
+    if (option (OPTSORTSUBTHREADS))
+    {
+      ctx->tree = mutt_sort_subthreads (ctx->tree, mutt_get_sort_func (SortAux));
+      unset_option (OPTSORTSUBTHREADS);
+    }
+    mutt_sort_threads (ctx, init);
+  }
+  else if ((sortfunc = mutt_get_sort_func (Sort)) == NULL ||
+	   (AuxSort = mutt_get_sort_func (SortAux)) == NULL)
   {
     mutt_error ("Could not find sorting function! [report this bug]");
     sleep (1);
