@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,12 +17,23 @@
  */ 
 
 #include "mutt.h"
+#include "mutt_menu.h"
 #include "mutt_curses.h"
+#include "sort.h"
 
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
 
+static char *get_sort_str (char *buf, size_t buflen, int method)
+{
+  snprintf (buf, buflen, "%s%s%s",
+	    (method & SORT_REVERSE) ? "reverse-" : "",
+	    (method & SORT_LAST) ? "last-" : "",
+	    mutt_getnamebyvalue (method & SORT_MASK, SortMethods));
+  return buf;
+}
+
 /*
  * %b = number of incoming folders with unread messages [option]
  * %d = number of deleted messages [option]
@@ -34,8 +45,10 @@
  * %M = number of messages shown (virutal message count when limiting) [option]
  * %n = number of new messages [option]
  * %p = number of postponed messages [option]
+ * %P = percent of way through index
  * %r = readonly/wontwrite/changed flag
- * %s = current sorting method
+ * %s = current sorting method ($sort)
+ * %S = current aux sorting method ($sort_aux)
  * %t = # of tagged messages [option]
  * %v = Mutt version
  *
@@ -46,18 +59,19 @@
  *    %?<format_char>?<optional_string>?
  */
 
-void mutt_status_line (char *buf, size_t buflen, const char *p)
+void menu_status_line (char *buf, size_t buflen, MUTTMENU *menu, const char *p)
 {
   char *cp;
   char *wptr = buf;
   char tmp[SHORT_STRING];
+  char tmp2[SHORT_STRING];
   char prefix[SHORT_STRING];
   char fmt[SHORT_STRING];
   char optstring[SHORT_STRING];
   char ch;
   int wlen = 0;
   int optional = 0;
-  int num_post;
+  int count;
   int len;
 
   buflen--; /* save room for the trailing \0 */
@@ -112,7 +126,7 @@ void mutt_status_line (char *buf, size_t buflen, const char *p)
 	  if (!*p)
 	    break; /* bad format */
 	  ch = *p++; /* pad character */
-	  mutt_status_line (tmp, sizeof (tmp), p);
+	  menu_status_line (tmp, sizeof (tmp), menu, p);
 	  len = COLS - wlen - strlen (tmp);
 	  while (len > 0)
 	  {
@@ -141,9 +155,9 @@ void mutt_status_line (char *buf, size_t buflen, const char *p)
 	  if (!optional)
 	  {
 	    snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
-	    snprintf (tmp, sizeof (tmp), fmt, mutt_buffy_check ());
+	    snprintf (tmp, sizeof (tmp), fmt, mutt_buffy_check (0));
 	  }
-	  else if (!mutt_buffy_check ())
+	  else if (!mutt_buffy_check (0))
 	    optional = 0;
 	  break;
 
@@ -192,11 +206,9 @@ void mutt_status_line (char *buf, size_t buflen, const char *p)
 
 	  if (!optional)
 	  {
-	    char size[5];
-
 	    snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	    mutt_pretty_size (size, sizeof (size), Context ? Context->size : 0);
-	    snprintf (tmp, sizeof (tmp), fmt, size);
+	    mutt_pretty_size (tmp2, sizeof (tmp2), Context ? Context->size : 0);
+	    snprintf (tmp, sizeof (tmp), fmt, tmp2);
 	  }
 	  else if (!Context || !Context->size)
 	    optional = 0;
@@ -235,18 +247,43 @@ void mutt_status_line (char *buf, size_t buflen, const char *p)
 	    optional = 0;
 	  break;
 
-	case 'p':
+	case 'o':
 
-	  num_post = mutt_num_postponed ();
 	  if (!optional)
 	  {
 	    snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
-	    snprintf (tmp, sizeof (tmp), fmt, num_post);
+	    snprintf (tmp, sizeof (tmp), fmt, Context ? Context->unread - Context->new : 0);
 	  }
-	  else if (!num_post)
+	  else if (!Context || !(Context->unread - Context->new))
+	    optional = 0;
+	  break;
+	  
+	case 'p':
+
+	  count = mutt_num_postponed ();
+	  if (!optional)
+	  {
+	    snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+	    snprintf (tmp, sizeof (tmp), fmt, count);
+	  }
+	  else if (!count)
 	    optional = 0;
 	  break;
 
+	case 'P':
+	
+	  if (menu->top + menu->pagelen >= menu->max)
+	    cp = menu->top ? "end" : "all";
+	  else
+	  {
+	    count = (100 * (menu->top + menu->pagelen)) / menu->max;
+	    snprintf (tmp2, sizeof (tmp2), "%d%%", count);
+            cp = tmp2;
+	  }
+	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
+	  snprintf (tmp, sizeof (tmp), fmt, cp);
+	  break;
+  
 	case 'r':
 
 	  if (Context)
@@ -258,21 +295,15 @@ void mutt_status_line (char *buf, size_t buflen, const char *p)
 	  break;
 
 	case 's':
-
 	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	  if ((Sort & SORT_MASK) == SORT_THREADS)
-	     snprintf (prefix, sizeof (prefix),
-		       "%sthreads/%s%s%s",
-		       (Sort & SORT_REVERSE) ? "reverse-" : "",
-		       (SortAux & SORT_REVERSE) ? "reverse-" : "",
-		       (SortAux & SORT_LAST) ? "last-" : "",
-		       mutt_getnamebyvalue ((SortAux & SORT_MASK), SortMethods));
-	  else
-	    snprintf (prefix, sizeof (prefix), "%s%s",
-		      (Sort & SORT_REVERSE) ? "reverse-" : "",
-		      mutt_getnamebyvalue ((Sort & SORT_MASK), SortMethods));
-		       
-	  snprintf (tmp, sizeof (tmp), fmt, prefix);
+	  snprintf (tmp, sizeof (tmp), fmt,
+		    get_sort_str (prefix, sizeof (prefix), Sort));
+	  break;
+
+	case 'S':
+	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
+	  snprintf (tmp, sizeof (tmp), fmt,
+		    get_sort_str (prefix, sizeof (prefix), SortAux));
 	  break;
 
 	case 't':
@@ -286,10 +317,21 @@ void mutt_status_line (char *buf, size_t buflen, const char *p)
 	    optional = 0;
 	  break;
 
+	case 'u':
+
+	  if (!optional)
+	  {
+	    snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+	    snprintf (tmp, sizeof (tmp), fmt, Context ? Context->unread : 0);
+	  }
+	  else if (!Context || !Context->unread)
+	    optional = 0;
+	  break;
+	  
 	case 'v':
 
-	  snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	  snprintf (tmp, sizeof (tmp), fmt, MuttVersion);
+	  snprintf (fmt, sizeof (fmt), "Mutt %%s");
+	  snprintf (tmp, sizeof (tmp), fmt, VERSION);
 	  break;
 
 	case 0:
@@ -304,7 +346,7 @@ void mutt_status_line (char *buf, size_t buflen, const char *p)
       }
 
       if (optional)
-	mutt_status_line (tmp, sizeof (tmp), optstring);
+	menu_status_line (tmp, sizeof (tmp), menu, optstring);
 
       if ((len = strlen (tmp)))
       {
