@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,95 +17,26 @@
  */ 
 
 #include "mutt.h"
+#include "sort.h"
 
 #include <string.h>
 #include <ctype.h>
 
 /* This function makes use of the fact that Mutt stores message references in
- * reverse order (i.e., last to first).  This is optiminal since we would like
- * to find the most recent message to which "cur" refers itself.
- */
+   reverse order (i.e., last to first).  This is optiminal since we would like
+   to find the most recent message to which "cur" refers itself.  */
 static HEADER *find_reference (HEADER *cur, CONTEXT *ctx)
 {
   LIST *refs = cur->env->references;
-  int i, j;
+  void *ptr;
 
   for (; refs; refs = refs->next)
-  {
-    /* If the mailbox is already mostly sorted (such as when new mail arrives),
-     * the reference is likely to be just before the current message, so
-     * searching backward from this message first could save some time.
-     *
-     * Note: cur == ctx->hdrs[ cur->msgno ]
-     */
-
-    for (j = 0; j <= 1; j++)
-    {
-      i = cur->msgno;
-
-      if ((j == 0) ^ (ctx->revsort == 0))
-      {
-	while (++i < ctx->msgcount)
-	{
-	  if (ctx->hdrs[i]->env->message_id &&
-	      !strcmp (refs->data, ctx->hdrs[i]->env->message_id))
-	    return (ctx->hdrs[i]);
-	}
-      }
-      else
-      {
-	while (--i >= 0)
-	{
-	  if (ctx->hdrs[i]->env->message_id &&
-	      !strcmp (refs->data, ctx->hdrs[i]->env->message_id))
-	    return (ctx->hdrs[i]);
-	}
-      }
-    }
-  } 
+    if ((ptr = hash_find (ctx->id_hash, refs->data)))
+      return ptr;
 
   return NULL;
 }
 
-static HEADER *find_subject (HEADER *cur, HEADER **tree, int size)
-{
-  HEADER *last = NULL;
-  int i, curno = 0;
-
-  if (cur->env->real_subj &&
-      ((cur->env->real_subj != cur->env->subject) || (!option (OPTSORTRE))))
-  {
-    for (i = 0; i < size; i++)
-    {
-      if (tree[i] == cur)
-      {
-	/* save the current position so we can replace it if we find a
-	 * matching subject
-	 */
-	curno = i;
-      }
-      else if (cur->date_sent >= tree[i]->date_sent &&
-	       (!last || (last->date_sent <= tree[i]->date_sent)) &&
-	       tree[i]->env->real_subj &&
-	       strcmp (cur->env->real_subj, tree[i]->env->real_subj) == 0)
-      {
-	last = tree[i];
-      }
-    }
-  }
-  if (last)
-  {
-    /* since we found a matching subject, remove this message from the list
-     * of toplevel messages and fill the hole with the last message in the
-     * list.  this reduces the number of searches the next time around.
-     * note that `size' gets decremented in the pseudo_threads() function
-     * when this functions returns non-NULL
-     */
-    tree[curno] = tree[size - 1];
-  }
-  return last;
-}
-
 /* Since the graphics characters have a value >255, I have to resort to
  * using escape sequences to pass the information to print_enriched_string().
  *
@@ -234,25 +165,78 @@ static void insert_message (HEADER **tree, HEADER *msg, sort_t *sortFunc)
   msg->next = NULL;
 }
 
-static HEADER *pseudo_threads (HEADER *list, HEADER **array, int size,
-			       sort_t *sortFunc)
+/* returns 1 if `a' is a descendant (child) of thread `b' */
+static int is_descendant (HEADER *a, HEADER *b)
 {
-  HEADER *top = list, *cur, *tmp, *curchild, *nextchild;
+  /* find the top parent of the thread */
+  while (a->parent)
+    a = a->parent;
+  return (a == b);
+}
 
-  while (list)
+/* find the best possible match for a parent mesage based upon subject.
+   if there are multiple matches, the one which was sent the latest, but
+   before the current message, is used. */
+static HEADER *find_subject (CONTEXT *ctx, HEADER *cur)
+{
+  struct hash_elem *ptr;
+  HEADER *tmp, *last = NULL;
+  ENVELOPE *env = cur->env;
+  int hash;
+
+  if (env->real_subj &&
+      ((env->real_subj != env->subject) || (!option (OPTSORTRE))))
   {
-    cur = list;
-    list = list->next;
-    if ((tmp = find_subject (cur, array, size)) != NULL)
+    hash = hash_string ((unsigned char *) env->real_subj, ctx->subj_hash->nelem);
+    for (ptr = ctx->subj_hash->table[hash]; ptr; ptr = ptr->next)
+    {
+      tmp = ptr->data;
+      if (tmp != cur &&			/* don't match the same message */
+	  !tmp->fake_thread &&		/* don't match pseudo threads */
+	  tmp->subject_changed &&	/* only match interesting replies */
+	  !is_descendant (tmp, cur) &&	/* don't match in the same thread */
+	  cur->date_sent >= tmp->date_sent &&
+	  (!last || (last->date_sent <= tmp->date_sent)) &&
+	  tmp->env->real_subj &&
+	  strcmp (env->real_subj, tmp->env->real_subj) == 0)
+      {
+	last = tmp; /* best match so far */
+      }
+    }
+  }
+  return last;
+}
+
+static void unlink_message (HEADER **top, HEADER *cur)
+{
+  if (cur->prev)
+  {
+    cur->prev->next = cur->next;
+
+    if (cur->next)
+      cur->next->prev = cur->prev;
+  }
+  else
+  {
+    if (cur->next)
+      cur->next->prev = NULL;
+    *top = cur->next;
+  }
+}
+
+static void pseudo_threads (CONTEXT *ctx, sort_t *sortFunc)
+{
+  HEADER *tree = ctx->tree;
+  HEADER *top = tree, *cur, *tmp, *curchild, *nextchild;
+
+  while (tree)
+  {
+    cur = tree;
+    tree = tree->next;
+    if ((tmp = find_subject (ctx, cur)) != NULL)
     {
-      size--;
       /* detach this message from it's current location */
-      if (list)
-	list->prev = cur->prev;
-      if (cur != top)
-	cur->prev->next = cur->next;
-      else
-	top = cur->next;
+      unlink_message (&top, cur);
 
       cur->subject_changed = 0;
       cur->fake_thread = 1;
@@ -260,21 +244,14 @@ static HEADER *pseudo_threads (HEADER *list, HEADER **array, int size,
       insert_message (&tmp->child, cur, sortFunc);
       
       /* if the message we're attaching has pseudo-children, they
-       * need to be attached to its parent, so move them up a level.
-       */
-      curchild = cur->child;
-      while (curchild)
+	 need to be attached to its parent, so move them up a level.  */
+      for (curchild = cur->child; curchild; )
       {
 	nextchild = curchild->next;
 	if (curchild->fake_thread)
 	{
 	  /* detach this message from its current location */
-	  if (curchild->next)
-	    curchild->next->prev = curchild->prev;
-	  if (curchild->prev)
-	    curchild->prev->next = curchild->next;
-	  else
-	    cur->child = curchild->next;
+	  unlink_message (&cur->child, curchild);
 	  curchild->parent = tmp;
 	  insert_message (&tmp->child, curchild, sortFunc);
 	}
@@ -282,7 +259,7 @@ static HEADER *pseudo_threads (HEADER *list, HEADER **array, int size,
       }
     }
   }
-  return top;
+  ctx->tree = top;
 }
 
 static HEADER *sort_last (HEADER *top)
@@ -364,16 +341,72 @@ static HEADER *sort_last (HEADER *top)
   return top;
 }
 
-void mutt_sort_threads (CONTEXT *ctx)
+static void move_descendants (HEADER **tree, HEADER *cur, sort_t *usefunc)
+{
+  HEADER *ptr, *tmp = *tree;
+
+  while (tmp)
+  {
+    /* only need to look at the last reference */
+    if (tmp->env->references &&
+	strcmp (tmp->env->references->data, cur->env->message_id) == 0)
+    {
+      /* remove message from current location */
+      unlink_message (tree, tmp);
+
+      tmp->parent = cur;
+      if (cur->env->real_subj && tmp->env->real_subj)
+	tmp->subject_changed = strcmp (tmp->env->real_subj, cur->env->real_subj) ? 1 : 0;
+      else
+	tmp->subject_changed = (cur->env->real_subj || tmp->env->real_subj) ? 1 : 0;
+      tmp->fake_thread = 0; /* real reference */
+
+      ptr = tmp;
+      tmp = tmp->next;
+      insert_message (&cur->child, ptr, usefunc);
+    }
+    else
+      tmp = tmp->next;
+  }
+}
+
+void mutt_clear_threads (CONTEXT *ctx)
+{
+  int i;
+
+  for (i = 0; i < ctx->msgcount; i++)
+  {
+    ctx->hdrs[i]->parent = NULL;
+    ctx->hdrs[i]->next = NULL;
+    ctx->hdrs[i]->prev = NULL;
+    ctx->hdrs[i]->child = NULL;
+    ctx->hdrs[i]->threaded = 0;
+    ctx->hdrs[i]->fake_thread = 0;
+  }
+  ctx->tree = NULL;
+}
+
+HEADER *mutt_sort_subthreads (HEADER *hdr, sort_t *func)
 {
   HEADER *top = NULL;
-  HEADER *tmp;
-  HEADER *cur;
+  HEADER *t;
+  
+  while (hdr)
+  {
+    t = hdr;
+    hdr = hdr->next;
+    insert_message (&top, t, func);
+    if (t->child)
+      t->child = mutt_sort_subthreads (t->child, func);
+  }
+  return top;
+}
+
+void mutt_sort_threads (CONTEXT *ctx, int init)
+{
   sort_t *usefunc = NULL;
-  int i;
-  int oldsort;
-  HEADER **subj_changes = NULL;
-  int subj_index = 0;
+  HEADER *tmp, *CUR;
+  int i, oldsort;
   
   /* set Sort to the secondary method to support the set sort_aux=reverse-*
    * settings.  The sorting functions just look at the value of
@@ -382,10 +415,6 @@ void mutt_sort_threads (CONTEXT *ctx)
   oldsort = Sort;
   Sort = SortAux;
   
-  /* allocate an array for pseudo-threading, if strict_threads is off */
-  if (!option (OPTSTRICTTHREADS))
-    subj_changes = safe_malloc (ctx->msgcount * sizeof (HEADER *));
-
   /* get secondary sorting method.  we can't have threads, so use the date
    * if the user specified it
    */
@@ -396,50 +425,59 @@ void mutt_sort_threads (CONTEXT *ctx)
   if (!(Sort & SORT_LAST))
     usefunc = mutt_get_sort_func (Sort);
 
-  /* this is the only link we need to clear, the rest are cleared elsewhere
-   * as appropriate to reduce the number of operations.
-   */
-  for (i = 0; i < ctx->msgcount; i++)
-    ctx->hdrs[i]->child = NULL;
-
   for (i = 0; i < ctx->msgcount; i++)
   {
-    cur = ctx->hdrs[i];
-    cur->fake_thread = 0;
+    CUR = ctx->hdrs[i];
 
-    if ((tmp = find_reference (cur, ctx)) != NULL)
+    if (CUR->fake_thread)
     {
-      cur->parent = tmp;
-      if (cur->env->real_subj && tmp->env->real_subj)
-	cur->subject_changed = strcmp (tmp->env->real_subj, cur->env->real_subj) ? 1 : 0;
+      /* Move pseudo threads back to the top level thread so that they can
+       * can be moved later if they are descendants of messages that were
+       * just delivered.
+       */
+      CUR->fake_thread = 0;
+      CUR->subject_changed = 1;
+      unlink_message (&CUR->parent->child, CUR);
+      CUR->parent = NULL;
+      insert_message (&ctx->tree, CUR, usefunc);
+    }
+    else if (!CUR->threaded)
+    {
+      if ((tmp = find_reference (CUR, ctx)) != NULL)
+      {
+	CUR->parent = tmp;
+	if (CUR->env->real_subj && tmp->env->real_subj)
+	  CUR->subject_changed = strcmp (tmp->env->real_subj, CUR->env->real_subj) ? 1 : 0;
+	else
+	  CUR->subject_changed = (CUR->env->real_subj || tmp->env->real_subj) ? 1 : 0;
+      }
       else
-	cur->subject_changed = (cur->env->real_subj || tmp->env->real_subj) ? 1 : 0;
-      insert_message (&tmp->child, cur, usefunc);
-    }
-    else
-    {
-      cur->parent = NULL;
-      cur->subject_changed = 1;
-      insert_message (&top, cur, usefunc); /* add to top-level thread */
-    }
+	CUR->subject_changed = 1;
 
-    /* if the subject is different, add it to the lookup table */
-    if (!option (OPTSTRICTTHREADS) && cur->subject_changed)
-      subj_changes[subj_index++] = cur;
+      if (!init)
+      {
+	/* Search the children of `tmp' for decendants of `cur'.  This is only
+	 * done when the mailbox has already been threaded since we don't have
+	 * to worry about the tree being threaded wrong (because of a missing
+	 * parent) during the initial threading.
+	 */
+	if (CUR->env->message_id)
+	  move_descendants (tmp ? &tmp->child : &ctx->tree, CUR, usefunc);
+      }
+      insert_message (tmp ? &tmp->child : &ctx->tree, CUR, usefunc);
+      CUR->threaded = 1;
+    }
   }
 
   if (!option (OPTSTRICTTHREADS))
-  {
-    top = pseudo_threads (top, subj_changes, subj_index, usefunc);
-    safe_free ((void *) &subj_changes); /* free the lookup table */
-  }
+    pseudo_threads (ctx, usefunc);
 
   /* now that the whole tree is put together, we can sort by last-* */
   if (Sort & SORT_LAST)
   {
     for (i = 0; i < ctx->msgcount; i++)
       ctx->hdrs[i]->last_sort = ctx->hdrs[i];
-    top = sort_last (top);
+    ctx->tree = sort_last (ctx->tree);
   }
   
   /* restore the oldsort order. */
@@ -451,7 +489,7 @@ void mutt_sort_threads (CONTEXT *ctx)
    * This, of course, means the auxillary sort has to go forwards
    * because we map it backwards here.
    */
-  linearize_tree (top, ctx->hdrs + (Sort & SORT_REVERSE ? ctx->msgcount - 1 : 0));
+  linearize_tree (ctx->tree, ctx->hdrs + (Sort & SORT_REVERSE ? ctx->msgcount - 1 : 0));
 }
 
 int _mutt_aside_thread (HEADER *hdr, short dir, short subthreads)
